// Copyright 2003 Regents of the University of California

// SETI_BOINC is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2, or (at your option) any later
// version.

// SETI_BOINC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.

// You should have received a copy of the GNU General Public License along
// with SETI_BOINC; see the file COPYING.  If not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

// In addition, as a special exception, the Regents of the University of
// California give permission to link the code of this program with libraries
// that provide specific optimized fast Fourier transform (FFT) functions and
// distribute a linked executable.  You must obey the GNU General Public
// License in all respects for all of the code used other than the FFT library
// itself.  Any modification required to support these libraries must be
// distributed in source code form.  If you modify this file, you may extend
// this exception to your version of the file, but you are not obligated to
// do so. If you do not wish to do so, delete this exception statement from
// your version.

// analyzeFuncs.C
// $Id: analyzeFuncs.cpp,v 1.34.2.26 2006/05/16 21:54:19 korpela Exp $
//

#define DO_SMOOTH

//#include "config.h"
#include "version.h"

#include <cstdio>
#include <iostream>
#include <cstdlib>
#include <cmath>
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif
#include <time.h>

#include "sincos.h"
#include "util.h"
#include "s_util.h"
#include "boinc_api.h"
#ifdef BOINC_APP_GRAPHICS
#include "reduce.h"
#include "graphics_data.h"
#include "graphics_api.h"
#ifdef DYNAMIC_GRAPHICS
#include "graphics_lib.h"
#endif
#endif
#include "diagnostics.h"

#if defined(_PGO_INSTRUMENT)
//Jason: Include for PGO
#pragma message ("-------PGO Instrumentation Enabled-------")
    #include <pgouser.h>
#endif

#ifdef USE_IPP
  #ifdef _WIN64
	#pragma message ("----x64 IPP includes----")
	#include "ipp_x64_static.h"
  #else
	#include "ipp_includes.h"
  #endif	
#elif defined(USE_FFTWF)
	#pragma message ("----FFTW----")
	#include "fftw3.h"
#else
	#pragma message ("----ooura----")
	#include "fft8g.h"
#endif // USE_IPP

#include "cpu_check.h"

#ifdef __SSE3__
#pragma message("---- SSE3 ----")
#include <pmmintrin.h>
#elif defined __SSE2__
#pragma message("---- SSE2 ----")
#include <emmintrin.h>
#elif defined __SSE__
#pragma message("---- SSE ----")
#include <xmmintrin.h>
#endif

#ifdef USE_SSE2
#pragma message("------> Enabling SSE3 emulation by SSE2 ----")
#include "SSE3_emulate.h"
#endif

#define STRIPMAX 100

#include "seti.h"
#include "analyze.h"
#include "analyzeReport.h"
#include "gaussfit.h"
#include "spike.h"
#include "malloc_a.h"
#include "analyzeFuncs.h"
#include "analyzePoT.h"
#include "chirpfft.h"
#include "worker.h"
#include "filesys.h"
#ifdef BOINC_APP_GRAPHICS
#include "gdata.h"
#include "sah_gfx.h"
#endif
#include "progress.h"

//#include <Accelerate/Accelerate.h>
//#include <vecLib/cblas.h>
//#include <ppc_intrinsics.h>

#if defined(__VEC__) || defined(__SSE3__)
// polynomial coefficients for sine / cosine approximation in the chirp routine
#define SS1 (_mm_set_ps(1.5707963268, 1.5707963268, 1.5707963268, 1.5707963268))
#define SS2 (_mm_set_ps(-0.6466386396, -0.6466386396, -0.6466386396, -0.6466386396))
#define SS3 (_mm_set_ps(0.0679105987, 0.0679105987, 0.0679105987, 0.0679105987))
#define SS4 (_mm_set_ps(-0.0011573807, -0.0011573807, -0.0011573807, -0.0011573807))
#define CC1 (_mm_set_ps(-1.2341299769, -1.2341299769, -1.2341299769, -1.2341299769))
#define CC2 (_mm_set_ps(0.2465220241, 0.2465220241, 0.2465220241, 0.2465220241))
#define CC3 (_mm_set_ps(-0.0123926179, -0.0123926179, -0.0123926179, -0.0123926179))

#define SS1F (_mm_set_ps(1.5707963235, 1.5707963235, 1.5707963235, 1.5707963235))
#define SS2F (_mm_set_ps(-0.645963615, -0.645963615, -0.645963615, -0.645963615))
#define SS3F (_mm_set_ps(0.0796819754, 0.0796819754, 0.0796819754, 0.0796819754))
#define SS4F (_mm_set_ps(-0.0046075748, -0.0046075748, -0.0046075748, -0.0046075748))
#define CC1F (_mm_set_ps(-1.2336977925, -1.2336977925, -1.2336977925, -1.2336977925))
#define CC2F (_mm_set_ps(0.2536086171, 0.2536086171, 0.2536086171, 0.2536086171))
#define CC3F (_mm_set_ps(-0.0204391631, -0.0204391631, -0.0204391631, -0.0204391631))

#define FASTER_SINCOS

#define ZERO		(_mm_set_ps(0.0f, 0.0f, 0.0f, 0.0f))
#define ONE         (_mm_set_ps(1.0f, 1.0f, 1.0f, 1.0f))
#define TWO         (_mm_set_ps(2.0f, 2.0f, 2.0f, 2.0f))
#define DONE        (_mm_set_pd(1.0, 1.0))
#define DTWO        (_mm_set_pd(2.0, 2.0))
#define DTHREE      (_mm_set_pd(3.0, 3.0))
#define DFOUR       (_mm_set_pd(4.0, 4.0))
#define DEIGHT      (_mm_set_pd(8.0, 8.0))
#define IZERO		(_mm_set_epi32{0, 0, 0, 0})
#define IONE		(_mm_set_epi32(1, 1, 1, 1))
#define ITWO		(_mm_set_epi32(2, 2, 2, 2))
#define ITHREE		(_mm_set_epi32(3, 3, 3, 3))
#define IFOUR		(_mm_set_epi32(4, 4, 4, 4))
#define IEIGHT		((vUInt32) {8, 8, 8, 8})
#define ISIXTEEN	((vUInt32) {16, 16, 16, 16})
#define IALLBITS	((vUInt32) {0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff})
#define SIGNBITS ((vUInt32) {0x80000000, 0x80000000, 0x80000000, 0x80000000})

#if defined(USE_PPC_G5) || defined(USE_I386_COREDUO) || (defined(USE_I386_XEON) && (defined(USE_I386_CORE2) || defined(USE_SSE3) || defined(USE_SSE2)))
#define HALF_STRIDE
#endif

#ifdef __VEC__
#define TRANSPOSE_BLOCK_SIZE 8
#define TRANSPOSE_VECTOR_LEN 2
#elif defined(__SSE__)
#ifdef HALF_STRIDE
#define TRANSPOSE_BLOCK_SIZE 8
#define TRANSPOSE_VECTOR_LEN 2
#else
#define TRANSPOSE_BLOCK_SIZE 4
#define TRANSPOSE_VECTOR_LEN 1
#endif
#endif

// 2^52 (used by quickRound)
#define TWO_TO_52	4.503599627370496e15
#endif

#ifdef USE_IPP
static int MaxBufSize = 0;
#if 0
static DSPSplitComplex VFftBuf;
#else
static Ipp8u* FftBuf = NULL;
#endif
#endif // USE_IPP

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define INVALID_CHIRP 2e+20

#ifdef USE_PPC_OPTIMIZATIONS
#define USE_TRANSPOSED_POT
#define USE_VECTORIZED_CHIRP
#define USE_VECTORIZED_POWER
#elif defined(USE_I386_OPTIMIZATIONS)
#define USE_TRANSPOSED_POT
#define USE_VECTORIZED_CHIRP
#define USE_VECTORIZED_POWER
// #define USE_SPLIT_COMPLEX
#endif

ChirpFftPair_t* ChirpFftPairs = NULL;

double ProgressUnitSize;
double progress=0, remaining=1;

// These are used to calculate chirped signals
// TrigStep contains trigonometric functions for MinChirpStep over time
// CurrentTrig contains current trigonometric fuctions
//    Tetsuji "Maverick" Rai

// Trigonometric arrays
typedef struct {double Cos, Sin; }
SinCosArray;

typedef struct {
  double *cos;
  double *sin;
} SplitSinCosArray;

SplitSinCosArray SplitTrigStep;
SplitSinCosArray SplitCurrentTrig;

SinCosArray* TrigStep = NULL;     // trigonometric array of MinChirpStep
SinCosArray* CurrentTrig = NULL;  // current chirprate trigonometric array
int CurrentChirpRateInd;          // current chirprate index (absolute value)

void InitTrigArray(int, double, int, double);
void FreeTrigArray(void);
void CalcTrigArray (int len, int ChirpRateInd);

#if 0
void InitSplitTrigArray(int, double, int, double);
void FreeSplitTrigArray(void);
void CalcSplitTrigArray (int len, int ChirpRateInd);
#endif

void intrinsic_cpuid()
{
  	__cpuid(CPUInfo, 0);
    nIds = CPUInfo[0];
    memset(CPUString, 0, sizeof(CPUString));
    *((int*)CPUString) = CPUInfo[1];
    *((int*)(CPUString+4)) = CPUInfo[3];
    *((int*)(CPUString+8)) = CPUInfo[2];

    // Get the information associated with each valid Id
    for (i=0; i<=nIds; ++i)
    {
        __cpuid(CPUInfo, i);
   // Interpret CPU feature information.
        if  (i == 1)
        {
            nSteppingID = CPUInfo[0] & 0xf;
            nModel = (CPUInfo[0] >> 4) & 0xf;
            nFamily = (CPUInfo[0] >> 8) & 0xf;
            nProcessorType = (CPUInfo[0] >> 12) & 0x3;
            nExtendedmodel = (CPUInfo[0] >> 16) & 0xf;
            nExtendedfamily = (CPUInfo[0] >> 20) & 0xff;
            nBrandIndex = CPUInfo[1] & 0xff;
            nCLFLUSHcachelinesize = ((CPUInfo[1] >> 8) & 0xff) * 8;
            nAPICPhysicalID = (CPUInfo[1] >> 24) & 0xff;
            bSSE3NewInstructions = (CPUInfo[2] & 0x1) || false;
            bMONITOR_MWAIT = (CPUInfo[2] & 0x8) || false;
            bCPLQualifiedDebugStore = (CPUInfo[2] & 0x10) || false;
            bThermalMonitor2 = (CPUInfo[2] & 0x100) || false;
            nFeatureInfo = CPUInfo[3];
        }
    }

    // Calling __cpuid with 0x80000000 as the InfoType argument
    // gets the number of valid extended IDs.
    __cpuid(CPUInfo, 0x80000000);
    nExIds = CPUInfo[0];
    memset(CPUBrandString, 0, sizeof(CPUBrandString));

    // Get the information associated with each extended ID.
    for (i=0x80000000; i<=nExIds; ++i)
    {
        __cpuid(CPUInfo, i);
   // Interpret CPU brand string and cache information.
        if  (i == 0x80000002)
            memcpy(CPUBrandString, CPUInfo, sizeof(CPUInfo));
        else if  (i == 0x80000003)
            memcpy(CPUBrandString + 16, CPUInfo, sizeof(CPUInfo));
        else if  (i == 0x80000004)
            memcpy(CPUBrandString + 32, CPUInfo, sizeof(CPUInfo));
        else if  (i == 0x80000006)
        {
            nCacheLineSize = CPUInfo[2] & 0xff;
            nL2Associativity = (CPUInfo[2] >> 12) & 0xf;
            nCacheSizeK = (CPUInfo[2] >> 16) & 0xffff;
 		}
    }
    // Display all the information in user-friendly format.
   if  (nExIds >= 0x80000004)
	   fprintf( stderr, "%10s: %s \n"
            , "CPUID"
            , CPUBrandString);
   int myfreq;
    ippGetCpuFreqMhz(&myfreq);
    fprintf(stderr,"%10s: %d x %d MHz ","Speed",ippGetNumCoresOnDie(), myfreq);
fprintf(stderr,"\n");
  if  (nExIds >= 0x80000006)
    {
		fprintf( stderr, "%10s: L1=%dK L2=%dK\n"
			    , "Cache"
				, nCacheLineSize
				, nCacheSizeK);
	}

  if  (nIds >= 1)
    {
		   if  (nFeatureInfo || bSSE3NewInstructions ||
             bMONITOR_MWAIT || bCPLQualifiedDebugStore ||
             bThermalMonitor2)
        {
if  (bSSE3NewInstructions)
#if defined (USE_SSE41)
fprintf( stderr, "%10s: %s%s%s%s%s%s \n"
            , "Features"
            , "MMX "
            , "SSE "
			, "SSE2 "
			, "SSE3 "
			, "SSSE3 "
            , "SSE4.1 \n");
#elif defined (USE_SSSE3)
fprintf( stderr, "%10s: %s%s%s%s%s \n"
            , "Features"
            , "MMX "
            , "SSE "
			, "SSE2 "
			, "SSE3 "
            , "SSSE3 \n");
#elif defined (USE_SSE3)
fprintf( stderr, "%10s: %s%s%s%s \n"
            , "Features"
            , "MMX "
            , "SSE "
			, "SSE2 "
            , "SSE3 \n");
#elif defined (USE_SSE2)
fprintf( stderr, "%10s: %s%s%s \n"
            , "Features"
            , "MMX "
            , "SSE "
            , "SSE2 \n");
#elif defined (USE_SSE)
fprintf( stderr, "%10s: %s%s \n"
            , "Features"
            , "MMX "
            , "SSE\n");
#elif defined (USE_MMX)
fprintf( stderr, "%10s: %s \n"
            , "Features"
            , "MMX \n");
#endif
            i = 0;
            nIds = 1;
            while (i < (sizeof(szFeatures)/sizeof(const char*)))
            {
                if  (nFeatureInfo & nIds)
                {
//                    printf_s("\t");
                }
                nIds <<= 1;
                ++i;
            }
        }
   }
}

// The main analysis function.  Args:
// state pointer to data, # of points, starting chirp/fftlen
//  Must be called with unchirped data;
//  this function modifies (chirps) the data in place
// swi  parsed WU header

//#define DEBUG
#ifdef DEBUG
int icfft;  // for debug
#endif

double LOAD_STORE_ADJUSTMENT = 2.85;

int seti_analyze (ANALYSIS_STATE& state) {
  sah_complex* DataIn = state.savedWUData;
#ifdef USE_SPLIT_COMPLEX
  DSPSplitComplex SplitDataIn;
#endif
  int NumDataPoints = state.npoints;
#ifdef USE_SPLIT_COMPLEX
  DSPSplitComplex SplitChirpedData;
  DSPSplitComplex SplitWorkData;
#else
  sah_complex* ChirpedData = NULL;
  sah_complex* WorkData = NULL;
#endif
  float* PowerSpectrum = NULL;
  float* tPowerSpectrum = NULL;
  int num_cfft                  = 0;
  int cfft_left;
  double MinChirpStep            = 0.0;
  float chirprate;
// JWS - nonarray last_chirp_ind, added doinplace
//Jason
//  int lastchirpindmax = 8; 
//  int last_chirp_ind[8], chirprateind, dochirp;
//  int lastchirpindmax = 1024;
//  int last_chirp_ind[1024], chirprateind, dochirp;
//  for (unsigned int l = 0; l < lastchirpindmax; l++) {
//    last_chirp_ind[l] = - 1 << 20;
//  }
  int chirprateind, dochirp, last_chirp_ind = -1 << 20;

  double progress_diff, progress_in_cfft, cputime0=0;
  int retval=0;
#ifndef DEBUG
  int icfft;
  int istrip;
#endif
  int NumFfts, ifft, fftlen, NumBlockFfts, iblockfft, chirplen;
  bool dotranspose, doinplace;
  int CurrentSub, CurrentFreqSub;
  int FftNum, FftOrder;
  unsigned long bitfield=swi.analysis_cfg.analysis_fft_lengths;
  unsigned long FftLen;
#ifdef USE_IPP
#ifdef USE_SPLIT_COMPLEX
  IppsFFTSpec_C_32f* SFftSpec[MAX_NUM_FFTS];
  FFTSetup vFftSpec[MAX_NUM_FFTS];
#else
  IppsFFTSpec_C_32fc* FftSpec[MAX_NUM_FFTS];
#endif
  int BufSize;

  ippStaticInit();   // initialization of IPP library
#elif defined(USE_FFTWF)
  // plan space for fftw
  fftwf_plan analysis_plans[MAX_NUM_FFTS];
#else
  // fields need by the ooura fft logic
  int * BitRevTab[MAX_NUM_FFTS];
  float * CoeffTab[MAX_NUM_FFTS];
#endif

  // Initialize LOAD_STORE_ADJUSTMENT
  if (swi.analysis_cfg.credit_rate != 0)
    LOAD_STORE_ADJUSTMENT = swi.analysis_cfg.credit_rate;
  else
    swi.analysis_cfg.credit_rate = LOAD_STORE_ADJUSTMENT;

  // Allocate data array and work area arrays.

#ifdef USE_SPLIT_COMPLEX
  SplitChirpedData.realp = (float *) state.data;
  SplitChirpedData.imagp = SplitChirpedData.realp + NumDataPoints;
#else
  ChirpedData = state.data;
#endif
  PowerSpectrum = (float*) calloc_a(NumDataPoints, sizeof(float), MEM_ALIGN);
  if(PowerSpectrum == NULL) SETIERROR(MALLOC_FAILED, "PowerSpectrum == NULL");
#ifdef USE_TRANSPOSED_POT
  tPowerSpectrum = (float*) calloc_a(NumDataPoints, sizeof(float), MEM_ALIGN);
  if(tPowerSpectrum == NULL) SETIERROR(MALLOC_FAILED, "PowerSpectrum == NULL");
#endif

  // we hates alignment, precious
  float *vmaxNoDC = (float *) malloc_a(4096, MEM_ALIGN);
  unsigned int *vmaxInd = (unsigned int *) malloc_a(4096, MEM_ALIGN);
  float *vmean = (float *) malloc_a(4096, MEM_ALIGN);
  
#ifdef USE_FFTWF
  double sz;
  FILE *wisdom;
  if (wisdom=boinc_fopen("wisdom.sah","r")) {
    char *wiz=(char *)calloc_a(1024,64,MEM_ALIGN);
    int n=0;
    while (wiz && n<64*1024 && !feof(wisdom)) {
      n+=fread(wiz+n,1,80,wisdom);
    }
    fftwf_import_wisdom_from_string(wiz);
    free_a(wiz);
    fclose(wisdom);
  }
#endif

  // boinc_worker_timer();
  FftNum=0;
  FftOrder=0;
  FftLen=1;
#ifdef BOINC_APP_GRAPHICS
  if (!nographics()) strcpy(sah_graphics->status, "Generating FFT Coefficients");
#endif
  while (bitfield != 0) {
    if (bitfield & 1) {
      swi.analysis_fft_lengths[FftNum]=FftLen;
      swi.analysis_fft_orders[FftNum]=FftOrder;
#ifdef USE_IPP
      int order = 0;
      for (int tmp = FftLen; !(tmp & 1); order++) tmp >>= 1;
#ifdef USE_SPLIT_COMPLEX
#if 0
      vFftSpec[FftNum] = vDSP_create_fftsetup(order, 2);
#else
      int BufSize;
      if (ippsFFTInitAlloc_C_32f(&SFftSpec[FftNum], order,
                                  IPP_FFT_NODIV_BY_ANY, ippAlgHintFast)) {
        SETIERROR (MALLOC_FAILED, "ippsFFTInitAlloc failed");
      }
      ippsFFTGetBufSize_C_32f(SFftSpec[FftNum], &BufSize);
      if (BufSize > MaxBufSize) MaxBufSize = BufSize;
#endif
#else
      int BufSize;
      if (ippsFFTInitAlloc_C_32fc(&FftSpec[FftNum], order,
                                  IPP_FFT_NODIV_BY_ANY, ippAlgHintFast)) {
        SETIERROR (MALLOC_FAILED, "ippsFFTInitAlloc failed");
      }
      ippsFFTGetBufSize_C_32fc(FftSpec[FftNum], &BufSize);
      if (BufSize > MaxBufSize) MaxBufSize = BufSize;
#endif
#elif !defined(USE_FFTWF)
      // See docs in fft8g.C for sizing guidelines for BitRevTab and CoeffTab.
      BitRevTab[FftNum] = (int*) calloc_a(3+(int)sqrt((float)swi.analysis_fft_lengths[FftNum]), sizeof(int), MEM_ALIGN);
      if(BitRevTab[FftNum] == NULL)  SETIERROR(MALLOC_FAILED, "BitRevTab[FftNum] == NULL");
      BitRevTab[FftNum][0] = 0;
#else


      WorkData = (sah_complex *)malloc_a(FftLen * sizeof(sah_complex),MEM_ALIGN);
      sah_complex *scratch=(sah_complex *)malloc_a(FftLen*sizeof(sah_complex),MEM_ALIGN);
      if ((WorkData == NULL) || (scratch==NULL)) {
        SETIERROR(MALLOC_FAILED, "WorkData == NULL || scratch == NULL");
      }
      // TODO: Deallocate these at the end of the function
      analysis_plans[FftNum] = fftwf_plan_dft_1d(FftLen, scratch, WorkData, FFTW_BACKWARD, FFTW_MEASURE|FFTW_PRESERVE_INPUT);
#endif
      FftNum++;
#ifdef USE_FFTWF
      free_a(scratch);
      free_a(WorkData);
#endif /* USE_FFTWF */

    }
    FftLen*=2;
    FftOrder++;
    bitfield>>=1;
  }

  ////Jason:  let's make a lookup for the PulsePoTLen & Overlaps
  int PulsePoTLengths[20], PulseOverlaps[20];
  for (int iNFord=3; iNFord<18; iNFord++) 
  {
		GetPulsePoTLen(1 << iNFord, &PulsePoTLengths[iNFord], &PulseOverlaps[iNFord]);
//		fprintf(stderr,"Created LU, NumFFTs %d, fftlen %d, Nfftord %d, pulspotLen %d, PulseOverlap %d\n",
//			(1 << iNFord), NumDataPoints/(1 << iNFord), iNFord, PulsePoTLengths[iNFord],PulseOverlaps[iNFord]);
  }

#ifdef USE_FFTWF
  wisdom=boinc_fopen("wisdom.sah","w");
  if (wisdom) {
    char *wiz=fftwf_export_wisdom_to_string();
    if (wiz) {
      fwrite(wiz,strlen(wiz),1,wisdom);
    }
    fclose(wisdom);
  }
#endif
  swi.num_fft_lengths=FftNum;
  // boinc_worker_timer();
  
#ifdef USE_FFTWF
  boinc_install_signal_handlers();
#endif
#ifdef USE_IPP
#if 0
  VFftBuf.realp = (float *) malloc_a(16384, MEM_ALIGN);
  VFftBuf.imagp = (float *) malloc_a(16384, MEM_ALIGN);
#else
  if (MaxBufSize) {
    FftBuf = (Ipp8u*) malloc_a (MaxBufSize, MEM_ALIGN);
    if (FftBuf == NULL) SETIERROR (MALLOC_FAILED, "FftBuf == NULL");
  }
#endif
#elif !defined(USE_FFTWF)
  for(FftNum = 0; FftNum < swi.num_fft_lengths; FftNum++) {
    CoeffTab[FftNum] = (float*) calloc_a(swi.analysis_fft_lengths[FftNum]/2, sizeof(float), MEM_ALIGN);
    if(CoeffTab[FftNum] == NULL) SETIERROR(MALLOC_FAILED, "CoeffTab[FftNum] == NULL");
  }
#endif

//  // Allocate WorkData array the size of the biggest FFT we'll do
//  // TODO: Deallocate this at the end of the function
//  WorkData = (sah_complex *)malloc_a(FftLen/2 * sizeof(sah_complex),MEM_ALIGN);
  // Allocate WorkData array large enough to hold 8 of any FFT
  // TODO: Deallocate this at the end of the function
#ifdef USE_SPLIT_COMPLEX
  SplitWorkData.realp = (float *) malloc_a(NumDataPoints * sizeof(float), MEM_ALIGN);
  SplitWorkData.imagp = (float *) malloc_a(NumDataPoints * sizeof(float), MEM_ALIGN);
#else
  WorkData = (sah_complex *)malloc_a(NumDataPoints * sizeof(sah_complex),MEM_ALIGN);
  if (WorkData == NULL) {
    SETIERROR(MALLOC_FAILED, "WorkData == NULL");
  }
#endif
  
  // Smooth Baseline

#ifdef DO_SMOOTH
#ifdef BOINC_APP_GRAPHICS
  if (!nographics()) strcpy(sah_graphics->status, "Doing Baseline Smoothing");
#endif
  retval = v_BaseLineSmooth(
             DataIn, NumDataPoints, swi.analysis_cfg.bsmooth_boxcar_length,
             swi.analysis_cfg.bsmooth_chunk_size
           );
  if (retval) SETIERROR(retval,"from v_BaseLineSmooth");
#endif

#ifdef USE_SPLIT_COMPLEX
  // convert interleaved complex to split complex
  SplitDataIn.realp = (float *)malloc_a(NumDataPoints * sizeof(float), MEM_ALIGN);
  SplitDataIn.imagp = (float *)malloc_a(NumDataPoints * sizeof(float), MEM_ALIGN);
  vDSP_ctoz((const DSPComplex *) DataIn, 2, &SplitDataIn, 1, NumDataPoints);
  
//  printf("SplitDataIn = {%p, %p}\n", SplitDataIn.realp, SplitDataIn.imagp);
#endif

  // gernerate table of chirp/fft pairs (we may read table from file if testing)
  if(cfft_file != NULL)
    num_cfft = ReadCFftFile(&ChirpFftPairs, &MinChirpStep);
  else
    num_cfft = GenChirpFftPairs(&ChirpFftPairs, &MinChirpStep);
  if(num_cfft == MALLOC_FAILED) {
    SETIERROR(MALLOC_FAILED, "num_cfft == MALLOC_FAILED");
  }
  if(state.icfft == 0) {
#if defined(_WIN32) && !defined(_WIN64)
  fprintf(stderr,"Windows optimized S@H Enhanced application by Alex Kan\n");
  #if defined(__SSE41__)
       #if defined(USE_I386_XEON)
	      fprintf(stderr,"Version info: SSE4.1x (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
	      fprintf(stderr,"SSE4.1x Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #else
		  fprintf(stderr,"Version info: SSE4.1 (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
		  fprintf(stderr,"SSE4.1 Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #endif
  #elif defined(USE_I386_CORE2)
       #if defined(USE_I386_XEON)
	      fprintf(stderr,"Version info: SSSE3x (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
	      fprintf(stderr,"SSSE3x Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #else
	      fprintf(stderr,"Version info: SSSE3 (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
	      fprintf(stderr,"SSSE3 Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
       #endif
  #elif defined(USE_SSE3)
       #if defined(USE_I386_XEON)
          fprintf(stderr,"Version info: SSE3x (AMD/Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
          fprintf(stderr,"SSE3x Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #else
          fprintf(stderr,"Version info: SSE3 (AMD/Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
          fprintf(stderr,"SSE3 Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
       #endif
  #elif defined(USE_SSE2)
       #if defined(USE_I386_XEON)
          fprintf(stderr,"Version info: SSE2x (AMD/Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
          fprintf(stderr,"SSE2x Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #else
          fprintf(stderr,"Version info: SSE2 (AMD/Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
          fprintf(stderr,"SSE2 Win32 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
       #endif
  #endif
  #if defined(_PGO_INSTRUMENT)
	 fprintf(stderr, " **** PGO Instrumented Binary **** \n\n");
  #endif
#elif defined(_WIN64)
  fprintf(stderr,"Windows optimized S@H Enhanced application by Alex Kan\n");
  #if defined(__SSE41__)
       #if defined(USE_I386_XEON)
	      fprintf(stderr,"Version info: SSE4.1x (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
	      fprintf(stderr,"SSE4.1x Win64 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #else
		  fprintf(stderr,"Version info: SSE4.1 (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
		  fprintf(stderr,"SSE4.1 Win64 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #endif
  #elif defined(USE_I386_CORE2)
       #if defined(USE_I386_XEON)
	      fprintf(stderr,"Version info: SSSE3x (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
	      fprintf(stderr,"SSSE3x Win64 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
	   #else
	      fprintf(stderr,"Version info: SSSE3 (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
	      fprintf(stderr,"SSSE3 Win64 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
       #endif
  #elif defined(USE_SSE3)
      fprintf(stderr,"Version info: SSE3 (AMD/Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n");
      fprintf(stderr,"SSE3 Win64 Build %d , Ported by : Jason G, Raistmer, JDWhale\n\n", APP_BUILD_NO );
  #endif
  #if defined(_PGO_INSTRUMENT)
	 fprintf(stderr, " **** PGO Instrumented Binary **** \n\n");
  #endif
#else
  fprintf(stderr,"OS X optimized S@H Enhanced application by Alex Kan\n");
#ifdef USE_PPC_G4
#ifdef BOINC_APP_GRAPHICS
  fprintf(stderr,"Version info: OS X Altivec (G4/G5, G4-optimized v6.1) V5.13 by Alex Kan\n\n");
#else
  fprintf(stderr,"Version info: OS X Altivec (G4/G5, G4-optimized v8-nographics) V5.13 by Alex Kan\n\n");
#endif
#elif defined(USE_PPC_G5)
#ifdef BOINC_APP_GRAPHICS
  fprintf(stderr,"Version info: OS X Altivec (G4/G5, G5-optimized v6.1) V5.13 by Alex Kan\n\n");
#else
  fprintf(stderr,"Version info: OS X Altivec (G4/G5, G5-optimized v8-nographics) V5.13 by Alex Kan\n\n");
#endif
#elif defined(USE_I386_COREDUO)
#ifdef BOINC_APP_GRAPHICS
  fprintf(stderr,"Version info: OS X SSE3 (Intel, Core Duo-optimized v7.2) V5.13 by Alex Kan\n\n");
#else
  fprintf(stderr,"Version info: OS X SSE3 (Intel, Core Duo-optimized v8-nographics) V5.13 by Alex Kan\n\n");
#endif
#elif defined(USE_I386_CORE2)
#ifdef BOINC_APP_GRAPHICS
  fprintf(stderr,"Version info: OS X SSSE3 (Intel, Core 2-optimized v7.2) V5.13 by Alex Kan\n\n");
#else
#ifdef USE_I386_XEON
  fprintf(stderr,"Version info: OS X SSSE3 (Intel, Xeon-optimized v8-nographics) V5.13 by Alex Kan\n\n");
#else
  fprintf(stderr,"Version info: OS X SSSE3 (Intel, Core 2-optimized v8-nographics) V5.13 by Alex Kan\n\n");
#endif
#endif
#endif
#endif 
  intrinsic_cpuid();
  fprintf(stderr,"Work Unit Info:\n");
  fprintf(stderr,"...............\n");
  fprintf(stderr,"Credit multiplier is :  %.2f\n", LOAD_STORE_ADJUSTMENT);
  fprintf(stderr,"WU true angle range is :  %f\n", swi.angle_range);
  }else{//if(state.icfft == 0)
	fprintf( stderr, "Restarted at %.2f percent.\n", progress * 100);
  }
  fflush(stderr);

  // Get together various values that we'll need to analyse power over time
  ComputePoTInfo(num_cfft, NumDataPoints);

  // used to calculate percent done
  //ProgressUnitSize = GetProgressUnitSize(NumDataPoints, num_cfft, swi);
  ProgressUnitSize = GetProgressUnitSize(NumDataPoints, num_cfft);
  //#define DUMP_CHIRP
#ifdef DUMP_CHIRP
  // dump chirp/fft pairs and exit.
  fprintf(stderr, "size  = %d MinChirpStep = %f\n", num_cfft, MinChirpStep);
  for(icfft = 0; icfft < num_cfft; icfft++) {
    fprintf(stderr,"%6d %15.11f %6d %6d %d %d\n",
            icfft,
            ChirpFftPairs[icfft].ChirpRate,
            ChirpFftPairs[icfft].ChirpRateInd,
            ChirpFftPairs[icfft].FftLen,
            ChirpFftPairs[icfft].GaussFit,
            ChirpFftPairs[icfft].PulseFind
           );
  }
  fflush(stderr); exit(0);
#endif

#ifndef USE_VECTORIZED_CHIRP
  // Initialize TrigArray's if we have the memor....
  if ((app_init_data.host_info.m_nbytes != 0)  &&
      (app_init_data.host_info.m_nbytes >= (double)(64*1024*1024))) {
    InitTrigArray (NumDataPoints, MinChirpStep,
                   ChirpFftPairs[state.icfft].ChirpRateInd,
                   swi.subband_sample_rate);
  }
#endif

//  InitSplitTrigArray (NumDataPoints, MinChirpStep, ChirpFftPairs[state.icfft].ChirpRateInd, swi.subband_sample_rate);

  boinc_wu_cpu_time(cputime0);
  reset_units();
  double chirp_units=0;
  
//  {
//	int PulsePoTLen, PulseOverlap;
//	int skips = 0;
//	int spikeGroups = 0;
//	int spikeGroupLength = 0;
//	int singletonSpikeGroups = 0;
//	int maxSpikeGroupLength = 0;
//	for (int q = 0; q < num_cfft; q++) {
//	  GetPulsePoTLen(NumDataPoints/ChirpFftPairs[q].FftLen, &PulsePoTLen, &PulseOverlap);
//	  dotranspose = !(PulsePoTLen > PoTInfo.TripletMax || PulsePoTLen < PoTInfo.TripletMin) ||
//		ChirpFftPairs[q].GaussFit || ChirpFftPairs[q].PulseFind;
//	  if (!dotranspose) {
//		skips++;
//		spikeGroupLength++;
//	  } else if (spikeGroupLength) {
//		spikeGroups++;
//		if (spikeGroupLength > maxSpikeGroupLength) maxSpikeGroupLength = spikeGroupLength;
//		if (spikeGroupLength == 1) singletonSpikeGroups++;
//		spikeGroupLength = 0;
//	  }
//	}
//	
//	if (spikeGroupLength) {
//	  spikeGroups++;
//	  if (spikeGroupLength > maxSpikeGroupLength) maxSpikeGroupLength = spikeGroupLength;
//	  if(spikeGroupLength == 1) singletonSpikeGroups++;
//	}
//	
//	printf("tpot hit rate is %f\n", (float)skips/num_cfft);
//	printf("average spike-finding group length is %f\n", (float)skips/spikeGroups);
//	printf("singleton spike-finding group rate is %f\n", (float)singletonSpikeGroups/spikeGroups);
//	printf("maximum spike-finding group length is %d\n", maxSpikeGroupLength);
//	printf("sequential sweep reduction %f\n", (float)(skips-spikeGroups)/num_cfft);
//	
//	exit(0);
//  }

  // Loop through chirp/fft pairs - this is the top level analysis loop.
  double last_ptime=0;
  int rollovers=0;
  double clock_max=0;
  
#if 1
  icfft = state.icfft;
  while (icfft < num_cfft) {
    unsigned int stripLength = 0;
	unsigned int stripLimit;
	unsigned int maxFFTlen;
	unsigned int istripCount;

	//Jason: Moved from below to use stripLimit
    /* Noisy WUs finish early -- do the first few pairs in the same order as other apps to avoid validation issues pertaining to exact result report order. */
	if (icfft < 80 ) stripLimit = 1;
	else stripLimit = STRIPMAX;

	//Hoping to be able to change this in the future by changing hardwired istrips.
//	maxFFTlen = 131072;
	maxFFTlen = 32768;
//	maxFFTlen = 4096;

    do {
      int PulsePoTLen, PulseOverlap;
	  unsigned int NForder;
      
      fftlen = ChirpFftPairs[icfft+stripLength].FftLen;
      NumFfts = NumDataPoints / fftlen;
	  for ( NForder=17;NForder>2;NForder--)
	     if ( NumFfts == 1 << NForder ) break;
	  PulsePoTLen = PulsePoTLengths[NForder];
	  PulseOverlap = PulseOverlaps[NForder];
//	  fprintf(stderr,"FFT Length: %d  , NF Order: %d, NumFFTs %d, PulsePoTLen %d, PulseOverlap %d \n",
//		  fftlen, NForder, NumFfts, PulsePoTLen, PulseOverlap);
//	  GetPulsePoTLen(NumFfts, &PulsePoTLen, &PulseOverlap);
      dotranspose = !(PulsePoTLen > PoTInfo.TripletMax || PulsePoTLen < PoTInfo.TripletMin) || ChirpFftPairs[icfft+stripLength].GaussFit || ChirpFftPairs[icfft+stripLength].PulseFind;
      
      if (!dotranspose) 
	  {
		  stripLength++;
		  if (fftlen > maxFFTlen) maxFFTlen = fftlen;
	  }
    } while ( !dotranspose && stripLength < stripLimit && icfft + stripLength < num_cfft );
    
    /* Noisy WUs finish early -- do the first few pairs in the same order as other apps to avoid validation issues pertaining to exact result report order. */
   // if (icfft < 80 && stripLength > 1) stripLength = 1; //Jason: Moving this up to use StripLimit.

    boinc_fpops_cumulative(analysis_state.FLOP_counter*LOAD_STORE_ADJUSTMENT + SETUP_FLOPS);
    remaining = 1.0 - (double) icfft / num_cfft;
    
    if (stripLength) {
      /* No power-over-time analysis for the next few chirp-FFT pairs -- do strip-based spike-finding */
      int icsfft;
      int stripstartchirpind = last_chirp_ind; // JWS - each strip starts at same index
      
//      printf("strip-based spike-finding for cfft pairs [%d %d]\n", icfft, icfft + stripLength - 1);

	  istripCount = swi.nsamples / maxFFTlen;  //Disabled pending deeper investigation
//	  istripCount = 8;
      for (istrip = 0; istrip < istripCount; istrip++) {
        last_chirp_ind = stripstartchirpind; // JWS - each strip starts at same index
        for (icsfft = icfft; icsfft < icfft + stripLength; icsfft++) {
          fftlen = ChirpFftPairs[icsfft].FftLen;
          chirprate = ChirpFftPairs[icsfft].ChirpRate;
          chirprateind = ChirpFftPairs[icsfft].ChirpRateInd;
          
          if (dochirp = (chirprateind != last_chirp_ind)) {
            if (istrip == 0) {
              progress += (double) (ProgressUnitSize * CHIRP_PROGRESS_FACTOR);
              chirp_units += (double) (ProgressUnitSize * CHIRP_PROGRESS_FACTOR);
              progress = std::min(progress, 1.0);
            }
          }

          last_chirp_ind = chirprateind;

          // If the next chirp/fft pair has a different chirp index
          // or we're at the end, FFTs can be done in place.
          if (icsfft < num_cfft - 1) doinplace = (chirprateind != ChirpFftPairs[icsfft+1].ChirpRateInd);
            else doinplace = true;
//		  doinplace = (fftlen == 131072); 
          
          state.icfft = icsfft;
          
          /* Find index into FFT length table for the current FFT length. */
          for (FftNum = 0; FftNum < swi.num_fft_lengths; FftNum++) {
            if (swi.analysis_fft_lengths[FftNum] == fftlen) {
              break;
            }
          }
          
          /* If PoT freq bin is non-negative, we are in PoT analysis for this cfft pair and should not re-output an "ogh" line. */
          if(state.PoT_freq_bin == -1) {
            retval = result_group_start();
            if (retval) SETIERROR(retval,"from result_group_start");
          }

// JWS
//          NumFfts = (NumDataPoints / 8) / fftlen;
          NumFfts = maxFFTlen / fftlen;
//          NumBlockFfts = fftlen < 32768 ? 1 << (15 - swi.analysis_fft_orders[FftNum]) : 1;
          // If we're going to chirp, the length must be <= maxFFTlen, and should be
          // at least some minimum for efficiency.
          // TODO: optimize the minimum by target capabilities
          if (!dochirp || maxFFTlen < 4096) NumBlockFfts = 1;
          else NumBlockFfts = (fftlen < 4096) ? (4096/fftlen) : 1;
          chirplen = fftlen * NumBlockFfts;
          
          ifft = NumFfts * istrip;
          for (iblockfft = NumFfts * istrip; iblockfft < NumFfts * (istrip + 1); iblockfft += NumBlockFfts) {
            if (dochirp) {
//              printf(" chirping points %d through %d (chirprateind=%d)\n", iblockfft * fftlen, iblockfft * fftlen + chirplen - 1, chirprateind);
              
#ifdef USE_PPC_G4
              retval = v_vChirpDataG4(DataIn, ChirpedData, chirprateind, chirprate, iblockfft * fftlen, chirplen, swi.subband_sample_rate);
#elif defined(USE_PPC_G5)
              retval = v_vChirpDataG5(DataIn, ChirpedData, chirprateind, chirprate, iblockfft * fftlen, chirplen, swi.subband_sample_rate);
#else
              retval = v_vChirpData(DataIn, ChirpedData, chirprateind, chirprate, iblockfft * fftlen, chirplen, swi.subband_sample_rate);
#endif      
            }
          
            for (; ifft < iblockfft + NumBlockFfts; ifft++) {
              float mean, max;
              unsigned int ind;
              
              CurrentSub = fftlen * ifft;
              CurrentFreqSub = fftlen * (ifft & (TRANSPOSE_BLOCK_SIZE - 1));
              state.FLOP_counter += 5 * (double) fftlen * log((double) fftlen) / log(2.0);
              
//              printf("  FFTing points %d through %d with icsfft %d\n", CurrentSub, CurrentSub + fftlen - 1, icsfft);
              
      #ifdef USE_IPP
              /* If the next pair will chirp, FFTs are allowed to destroy their input */
              if (doinplace) {
                ippsFFTInv_CToC_32fc_I((Ipp32fc*)&ChirpedData[CurrentSub], FftSpec[FftNum], FftBuf);
              } else {
                ippsFFTInv_CToC_32fc((Ipp32fc*)&ChirpedData[CurrentSub], (Ipp32fc*)WorkData, FftSpec[FftNum], FftBuf);
              }
#elif defined(USE_FFTWF)
              fftwf_execute_dft(analysis_plans[FftNum], &ChirpedData[CurrentSub], WorkData);
      #else
              // replace time with freq - ooura FFT
              cdft(fftlen*2, 1, WorkData, BitRevTab[FftNum], CoeffTab[FftNum]);
      #endif
      
              // replace freq with power
              state.FLOP_counter+=(double)fftlen;

#ifdef USE_IPP
              if (doinplace) {
                v_vGetPowerSpectrumMeanMax(&ChirpedData[CurrentSub], &PowerSpectrum[CurrentSub], fftlen, &max, &ind, &mean);
              } else 
#endif
              {
                v_vGetPowerSpectrumMeanMax(WorkData, &PowerSpectrum[CurrentSub], fftlen, &max, &ind, &mean);
              }
              
              if(state.PoT_freq_bin == -1) {
              	state.FLOP_counter+=(double)fftlen;
              	retval = FindSpikes(&PowerSpectrum[CurrentSub],
              						fftlen, ifft, swi, &max, &ind, &mean);

              	progress += SpikeProgressUnits(fftlen)*ProgressUnitSize/(NumFfts*istripCount);
              	if (retval) SETIERROR(retval,"from FindSpikes");
              }

              progress = std::min(progress,1.0);
              
              remaining = 1.0 - ((double) icfft + (stripLength * istrip + (icsfft - icfft)) / (double)istripCount) / num_cfft;
              fraction_done(progress,remaining);
            }
          }
        }
      }
      
      if (icsfft == (num_cfft - 1)) {
        progress = 1;
        remaining = 0;
        fraction_done(progress, remaining);
      }
      
      /* XXX: this should be safe if we know we're skipping PoT analysis, but I should check to be sure */
      state.icfft = icsfft + 1;
      
      retval = checkpoint();
      if (retval) SETIERROR(retval, "from checkpoint() in seti_analyze()");
    } else {
      /* Power-over-time analysis needed -- do standard transposed-PoT routine */
      chirprate = ChirpFftPairs[icfft].ChirpRate;
      chirprateind = ChirpFftPairs[icfft].ChirpRateInd;

      if (dochirp = (chirprateind != last_chirp_ind)) {
        progress += (double) (ProgressUnitSize * CHIRP_PROGRESS_FACTOR);
        chirp_units += (double) (ProgressUnitSize * CHIRP_PROGRESS_FACTOR);
        progress = std::min(progress, 1.0);
      }

// JWS
   //   for (unsigned int l = 0; l < 8; l++) {
//      for (unsigned int l = 0; l < lastchirpindmax; l++) {
//        last_chirp_ind[l] = chirprateind;
//      }
      last_chirp_ind = chirprateind;

      // If the next chirp/fft pair has a different chirp index
      // or we're at the end, FFTs can be done in place.
      if (icfft < num_cfft - 1) doinplace = (chirprateind != ChirpFftPairs[icfft+1].ChirpRateInd);
       else doinplace = true;
//      doinplace = (fftlen == 131072); 

      state.icfft = icfft;
      
      /* Find index into FFT length table for the current FFT length. */
      for (FftNum = 0; FftNum < swi.num_fft_lengths; FftNum++) {
        if (swi.analysis_fft_lengths[FftNum] == fftlen) {
          break;
        }
      }
      
      /* If PoT freq bin is non-negative, we are in PoT analysis for this cfft pair and should not re-output an "ogh" line. */
      if(state.PoT_freq_bin == -1) {
        retval = result_group_start();
        if (retval) SETIERROR(retval,"from result_group_start");
      }
      
      NumBlockFfts = fftlen < 32768 ? 1 << (15 - swi.analysis_fft_orders[FftNum]) : 1;
//    NumBlockFfts = fftlen < 16384 ? 1 << (14 - swi.analysis_fft_orders[FftNum]) : 1;
//    NumBlockFfts = fftlen < 8192 ? 1 << (13 - swi.analysis_fft_orders[FftNum]) : 1;
//    NumBlockFfts = fftlen < 4096 ? 1 << (12 - swi.analysis_fft_orders[FftNum]) : 1;
//    NumBlockFfts = fftlen < 2048 ? 1 << (11 - swi.analysis_fft_orders[FftNum]) : 1;
//    NumBlockFfts = fftlen < 1024 ? 1 << (10 - swi.analysis_fft_orders[FftNum]) : 1;
      
      ifft = 0;
      chirplen = fftlen * NumBlockFfts;
      for (iblockfft = 0; iblockfft < NumFfts; iblockfft += NumBlockFfts) {
        if (dochirp) {
#ifdef USE_PPC_G4
          retval = v_vChirpDataG4(DataIn, ChirpedData, chirprateind, chirprate, iblockfft * fftlen, chirplen, swi.subband_sample_rate);
#elif defined(USE_PPC_G5)
          retval = v_vChirpDataG5(DataIn, ChirpedData, chirprateind, chirprate, iblockfft * fftlen, chirplen, swi.subband_sample_rate);
#else
		  retval = v_vpChirpData(DataIn, ChirpedData, chirprateind, chirprate, iblockfft * fftlen, chirplen, swi.subband_sample_rate);         	
#endif
        }

        for (; ifft < iblockfft + NumBlockFfts; ifft++) {
          float mean, max;
          unsigned int ind;
          
          CurrentSub = fftlen * ifft;
#ifdef HALF_STRIDE
          CurrentFreqSub = fftlen * (ifft & (TRANSPOSE_BLOCK_SIZE - 1)) / 2;
#else
          CurrentFreqSub = fftlen * (ifft & (TRANSPOSE_BLOCK_SIZE - 1));
#endif
          state.FLOP_counter += 5 * (double) fftlen * log((double) fftlen) / log(2.0);
          
#ifdef USE_IPP
          /* If the next pair will chirp, FFTs are allowed to destroy their input */
          if (doinplace) {
            ippsFFTInv_CToC_32fc_I((Ipp32fc*)&ChirpedData[CurrentSub], FftSpec[FftNum], FftBuf);
          } else {
            ippsFFTInv_CToC_32fc((Ipp32fc*)&ChirpedData[CurrentSub], (Ipp32fc*) &WorkData[CurrentFreqSub], FftSpec[FftNum], FftBuf);
          }
#elif defined(USE_FFTWF)
          fftwf_execute_dft(analysis_plans[FftNum], &ChirpedData[CurrentSub], &WorkData[CurrentFreqSub]);
#else
          cdft(fftlen*2, 1, WorkData, BitRevTab[FftNum], CoeffTab[FftNum]);
#endif
         
          /* replace freq with power */
          state.FLOP_counter += (double) fftlen;
          
#ifdef HALF_STRIDE
          /* compute power spectrum in-place (by destroying the first half of the frequency data) */
#ifdef USE_IPP
          if (doinplace) {
            v_vGetPowerSpectrumMeanMaxWrite(&ChirpedData[CurrentSub], &PowerSpectrum[CurrentSub], fftlen, &max, &ind, &mean);
          } else
#endif
          {
            v_vGetPowerSpectrumMeanMaxWrite(&WorkData[CurrentFreqSub], &PowerSpectrum[CurrentSub], fftlen, &max, &ind, &mean);
          }
          
          if (state.PoT_freq_bin == -1) {
            state.FLOP_counter += (double) fftlen;
            retval = FindSpikes(&PowerSpectrum[CurrentSub], fftlen, ifft, swi, &max, &ind, &mean);
            progress += SpikeProgressUnits(fftlen) * ProgressUnitSize/NumFfts;
            if (retval) SETIERROR(retval, "from FindSpikes");
          }
          
          if ((ifft & (TRANSPOSE_BLOCK_SIZE - 1)) == (TRANSPOSE_BLOCK_SIZE - 1)) {
#ifdef __VEC__
            v_vTranspose8((float *)WorkData, fftlen * sizeof(float), fftlen, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], NumFfts);
#elif defined(__SSE__)
//            if (fftlen == 131072) {
            if (doinplace) {
              ippmTranspose_m_32f((Ipp32f*)&ChirpedData[fftlen * (ifft & ~(TRANSPOSE_BLOCK_SIZE-1))], fftlen * sizeof(sah_complex), sizeof(float), fftlen, TRANSPOSE_BLOCK_SIZE, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], NumFfts * sizeof(float), sizeof(float));
            } else {
              ippmTranspose_m_32f((Ipp32f*)WorkData, fftlen * sizeof(float), sizeof(float), fftlen, TRANSPOSE_BLOCK_SIZE, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], NumFfts * sizeof(float), sizeof(float));
            }
#endif
          }
#else
          if ((ifft & (TRANSPOSE_BLOCK_SIZE - 1)) == (TRANSPOSE_BLOCK_SIZE - 1)) {
            unsigned int clfft;
            
#ifdef __VEC__
            v_vTransposePowerMeanMax8(WorkData, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], fftlen, NumFfts, (vFloat *) vmaxNoDC, (vUInt32 *) vmaxInd, (vFloat *) vmean);              
#elif defined(__SSE__)
//            if (fftlen == 131072) {
            if (doinplace) {
              v_vTransposePowerMeanMax4(&ChirpedData[fftlen * (ifft & ~(TRANSPOSE_BLOCK_SIZE - 1))], &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], fftlen, NumFfts, vmaxNoDC, vmaxInd, vmean);
            } else {
              v_vTransposePowerMeanMax4(WorkData, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], fftlen, NumFfts, vmaxNoDC, vmaxInd, vmean);
            }
#endif

            if (state.PoT_freq_bin == -1) {
              state.FLOP_counter += (double) TRANSPOSE_BLOCK_SIZE * fftlen;
              for (clfft = 0; clfft < TRANSPOSE_BLOCK_SIZE; clfft++) {
              	retval = FindSpikes(&PowerSpectrum[fftlen * ((ifft & ~(TRANSPOSE_BLOCK_SIZE-1)) + clfft)], fftlen, (ifft & ~(TRANSPOSE_BLOCK_SIZE-1)) + clfft, swi, &vmaxNoDC[clfft], &vmaxInd[clfft], &vmean[clfft]);
              }
              
              progress += TRANSPOSE_BLOCK_SIZE * SpikeProgressUnits(fftlen) * ProgressUnitSize/NumFfts;
            }
          }
#endif

          progress = std::min(progress, 1.0);
          remaining = 1.0 - (double) (icfft+1) / num_cfft;
          fraction_done(progress, remaining);
        }
      }
      
      retval = analyze_pot(PowerSpectrum, tPowerSpectrum, NumDataPoints, ChirpFftPairs[icfft]);
      if (retval) SETIERROR(retval, "from analyze_pot");

      if (icfft == (num_cfft - 1)) {
        progress = 1;
        remaining = 0;
        fraction_done(progress, remaining);
      }
      
      retval = checkpoint();
      if (retval) SETIERROR(retval, "from checkpoint() in seti_analyze()");
    }
        
    if (stripLength)
      icfft += stripLength;
    else
      icfft++;
      
    istrip = -1;
  }
#else
  for (icfft = state.icfft; icfft < num_cfft; icfft++) {
    fftlen    = ChirpFftPairs[icfft].FftLen;
    chirprate = ChirpFftPairs[icfft].ChirpRate;
    chirprateind = ChirpFftPairs[icfft].ChirpRateInd;
    boinc_fpops_cumulative(analysis_state.FLOP_counter*LOAD_STORE_ADJUSTMENT + SETUP_FLOPS);

    // boinc_worker_timer();
#ifdef DEBUG
    double ptime=static_cast<double>((unsigned)clock())/CLOCKS_PER_SEC+
          clock_max*rollovers;
    clock_max=std::max(last_ptime-ptime,clock_max);
    if (ptime<last_ptime) {
      rollovers++;
      ptime=static_cast<double>((unsigned)clock())/CLOCKS_PER_SEC+
           clock_max*rollovers;
    }
    last_ptime=ptime;

    fprintf(stderr,"%f %f %f %f %f %f %f %f %f\n",
      ptime,
      progress,
      ((double)icfft)/num_cfft,
      analysis_state.FLOP_counter,
      triplet_units,
      pulse_units,
      spike_units,
      gauss_units,
      chirp_units
    );
    fflush(stderr);

    double cputime=0;
    boinc_wu_cpu_time(cputime);
    cputime-=cputime0;
#endif

    remaining=1.0-(double)icfft/num_cfft;
	
    if(dochirp = (chirprateind != last_chirp_ind)) {
#ifdef BOINC_APP_GRAPHICS
      if (!nographics()) strcpy(sah_graphics->status, "Chirping data");
#endif

#ifndef USE_VECTORIZED_CHIRP
      retval = v_ChirpData(
                 DataIn,
                 ChirpedData,
                 chirprateind,
                 chirprate,
                 NumDataPoints,
                 swi.subband_sample_rate
               );
#endif
      
      if (retval) SETIERROR(retval, "from v_ChirpData()");

      progress += (double)(ProgressUnitSize * CHIRP_PROGRESS_FACTOR);
      chirp_units+=(double)(ProgressUnitSize * CHIRP_PROGRESS_FACTOR);
      progress = std::min(progress,1.0);
    }

    //    last_chirp = chirprate;
    last_chirp_ind = chirprateind;

    // Process this FFT length.
    // For a given FFT length (at a given chirp), we construct
    // PowerSpectrum[] which is a "waterfall" array of power spectra,
    // each fftlen long on the frequency axis and sample time long
    // on the time axis.
    // As we go along, we check each spectra for spikes.

    state.icfft = icfft;     // update analysis state

    // Find index into FFT length table for the current
    // FFT length.  This will be the same index needed
    // for ooura's coeffecient and bit reverse tables.
    for(FftNum = 0; FftNum < swi.num_fft_lengths; FftNum++) {
      if(swi.analysis_fft_lengths[FftNum] == fftlen) {
        break;
      }
    }

#ifdef BOINC_APP_GRAPHICS
    if (!nographics()) {
      sah_graphics->fft_info.chirp_rate = chirprate;
      sah_graphics->fft_info.fft_len = fftlen;
      strcpy(sah_graphics->status, "Computing Fast Fourier Transform");
    }
#endif

    // If PoT freq bin is non-negative, we are into PoT analysis
    // for this cfft pair and should not re-output an "ogh" line.
    if(state.PoT_freq_bin == -1) {
      retval = result_group_start();
      if (retval) SETIERROR(retval,"from result_group_start");
    }

    // Number of FFTs for this length
    NumFfts   = NumDataPoints / fftlen;
#ifdef USE_VECTORIZED_CHIRP
    NumBlockFfts = fftlen < 32768 ? 1<<(15-swi.analysis_fft_orders[FftNum]) : 1;
#else
    NumBlockFfts = 1;
#endif
	
	// do we need the transpose?
	{
	  int PulsePoTLen, PulseOverlap;
	  GetPulsePoTLen(NumFfts, &PulsePoTLen, &PulseOverlap);
	  dotranspose = !(PulsePoTLen > PoTInfo.TripletMax || PulsePoTLen < PoTInfo.TripletMin) ||
		ChirpFftPairs[icfft].GaussFit || ChirpFftPairs[icfft].PulseFind;
	}

#ifdef BOINC_APP_GRAPHICS
    GRAPH_BUFFER* gbp=0;
    if (!nographics()) {
      gbp = sah_graphics->get_generate_buffer(true);
      if (gbp) {
        gbp->rarray.init_data(fftlen, NumFfts);
      }
    }
#endif
    
    ifft = 0;
    chirplen = fftlen*NumBlockFfts;
    for (iblockfft = 0; iblockfft < NumFfts; iblockfft += NumBlockFfts) {
#ifdef USE_VECTORIZED_CHIRP
      if(dochirp) {
#ifdef USE_PPC_G4
        retval = v_vChirpDataG4(
                                DataIn,
                                ChirpedData,
                                chirprateind,
                                chirprate,
                                iblockfft*fftlen,
                                chirplen,
                                swi.subband_sample_rate
                                );
#elif defined(USE_PPC_G5)
        retval = v_vChirpDataG5(
                                DataIn,
                                ChirpedData,
                                chirprateind,
                                chirprate,
                                iblockfft*fftlen,
                                chirplen,
                                swi.subband_sample_rate
                                );
#else
#if 0
        if (chirprateind > 0) {
          printf("%d: %f\n", chirprateind, chirprate / MinChirpStep);
        }
#endif

        retval = v_vChirpData(
#ifdef USE_SPLIT_COMPLEX
                              &SplitDataIn,
                              &SplitChirpedData,
#else
                              DataIn,
                              ChirpedData,
#endif
                              chirprateind,
                              chirprate,
                              iblockfft*fftlen,
                              chirplen,
                              swi.subband_sample_rate
                              );
#endif
      }
#endif
      
	  if (!dotranspose) {
      for (; ifft < iblockfft+NumBlockFfts; ifft++) {
        float mean;
        float max;
        unsigned int ind;
        
        CurrentSub = fftlen * ifft;
        state.FLOP_counter+=5*(double)fftlen*log((double)fftlen)/log(2.0);
  
#ifdef USE_IPP
#ifdef USE_SPLIT_COMPLEX
//        printf("ippsFFTInv_CToC_32f(%p, %p, %p, %p)\n", SplitChirpedData.realp + CurrentSub, SplitChirpedData.imagp + CurrentSub, SplitWorkData.realp, SplitWorkData.imagp);
#if 0
        DSPSplitComplex SubChirpedData;
        SubChirpedData.realp = SplitChirpedData.realp + CurrentSub;
        SubChirpedData.imagp = SplitChirpedData.imagp + CurrentSub;

        vDSP_fft_zopt(vFftSpec[FftNum], &SubChirpedData, 1, &SplitWorkData, 1, &VFftBuf, swi.analysis_fft_orders[FftNum], FFT_INVERSE);
#else
        ippsFFTInv_CToC_32f((const Ipp32f *) (SplitChirpedData.realp + CurrentSub), (const Ipp32f *) (SplitChirpedData.imagp + CurrentSub), (Ipp32f *) SplitWorkData.realp, (Ipp32f *) SplitWorkData.imagp, SFftSpec[FftNum], FftBuf);
#endif
#else
        ippsFFTInv_CToC_32fc((Ipp32fc*)&ChirpedData[CurrentSub], (Ipp32fc*)WorkData,
      					   FftSpec[FftNum], FftBuf);
#endif
#elif defined(USE_FFTWF)
        fftwf_execute_dft(analysis_plans[FftNum], &ChirpedData[CurrentSub], WorkData);
#else
        // replace time with freq - ooura FFT
        cdft(fftlen*2, 1, WorkData, BitRevTab[FftNum], CoeffTab[FftNum]);
#endif
  
        // replace freq with power
        state.FLOP_counter+=(double)fftlen;
        
#ifdef USE_SPLIT_COMPLEX
        v_vGetPowerSpectrumMeanMax( &SplitWorkData,
      							  &PowerSpectrum[CurrentSub],
      							  fftlen, &max, &ind, &mean);	
#else
        v_vGetPowerSpectrumMeanMax( WorkData,
      							  &PowerSpectrum[CurrentSub],
      							  fftlen, &max, &ind, &mean);	
#endif
  
        if(state.PoT_freq_bin == -1) {
        	state.FLOP_counter+=(double)fftlen;
        	retval = FindSpikes(&PowerSpectrum[CurrentSub],
        						fftlen, ifft, swi, &max, &ind, &mean);
	
        	progress += SpikeProgressUnits(fftlen)*ProgressUnitSize/NumFfts;
        	if (retval) SETIERROR(retval,"from FindSpikes");
        }
  
        progress = std::min(progress,1.0);
#ifdef BOINC_APP_GRAPHICS
        if (!nographics()) {
        	if (gbp) {
        	  gbp->rarray.add_source_row(PowerSpectrum+fftlen*ifft);
        	}
        	sah_graphics->local_progress = (((float)ifft+1)/NumFfts);
        }
#endif
        remaining=1.0-(double)(icfft+1)/num_cfft;
        fraction_done(progress,remaining);
        // jeffc
        //fprintf(stderr, "S fft len %d  progress = %12.10f\n", fftlen, progress);
      }
	  } else {
  		for (; ifft < iblockfft+NumBlockFfts; ifft++) {
  		  CurrentSub = fftlen * ifft;
  		  CurrentFreqSub = fftlen * (ifft & (TRANSPOSE_BLOCK_SIZE-1));
  		  state.FLOP_counter+=5*(double)fftlen*log((double)fftlen)/log(2.0);
		  
#ifdef USE_IPP
#ifdef USE_SPLIT_COMPLEX
        DSPSplitComplex SubWorkData;
        SubWorkData.realp = SplitWorkData.realp + CurrentFreqSub;
        SubWorkData.imagp = SplitWorkData.imagp + CurrentFreqSub;

#if 0
        DSPSplitComplex SubChirpedData;
        SubChirpedData.realp = SplitChirpedData.realp + CurrentSub;
        SubChirpedData.imagp = SplitChirpedData.imagp + CurrentSub;

        vDSP_fft_zopt(vFftSpec[FftNum], &SubChirpedData, 1, &SubWorkData, 1, &VFftBuf, swi.analysis_fft_orders[FftNum], FFT_INVERSE);
#else
//        printf("ippsFFTInv_CToC_32f(%p, %p, %p, %p)\n", SplitChirpedData.realp + CurrentSub, SplitChirpedData.imagp + CurrentSub, SubWorkData.realp, SubWorkData.imagp);
        ippsFFTInv_CToC_32f((const Ipp32f *) (SplitChirpedData.realp + CurrentSub), (const Ipp32f *) (SplitChirpedData.imagp + CurrentSub), (Ipp32f *) SubWorkData.realp, (Ipp32f *) SubWorkData.imagp, SFftSpec[FftNum], FftBuf);
#endif
#else
  		  ippsFFTInv_CToC_32fc((Ipp32fc*)&ChirpedData[CurrentSub], (Ipp32fc*) &WorkData[CurrentFreqSub],
  							   FftSpec[FftNum], FftBuf);
#endif
#elif defined(USE_FFTWF)
  		  fftwf_execute_dft(analysis_plans[FftNum], &ChirpedData[CurrentSub], &WorkData[CurrentFreqSub]);
#else
  		  // replace time with freq - ooura FFT
  		  cdft(fftlen*2, 1, WorkData, BitRevTab[FftNum], CoeffTab[FftNum]);
#endif		  
		  
  		  // replace freq with power
  		  state.FLOP_counter+=(double)fftlen;

  		  if ((ifft & (TRANSPOSE_BLOCK_SIZE-1)) == (TRANSPOSE_BLOCK_SIZE-1)) {
          // this is the last FFT in a transpose block - transpose and do spike-finding
          // XXX: stupid graphics wants the power spectrum, so we may not be able to omit the non-transposed data
          unsigned int clfft;

#ifdef __VEC__
          v_vTransposePowerMeanMax8(WorkData, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], fftlen, NumFfts, maxNoDC.v, maxInd.v, mean.v);
#elif defined(__SSE__)
#ifdef USE_SPLIT_COMPLEX
          v_vTransposePowerMeanMax4(&SplitWorkData, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], fftlen, NumFfts, vmaxNoDC, vmaxInd, vmean);
#else
          v_vTransposePowerMeanMax4(WorkData, &tPowerSpectrum[ifft & ~(TRANSPOSE_BLOCK_SIZE-1)], fftlen, NumFfts, vmaxNoDC, vmaxInd, vmean);
#endif
#endif
          if(state.PoT_freq_bin == -1) {
            state.FLOP_counter+=(double)TRANSPOSE_BLOCK_SIZE*fftlen;
            for (clfft = 0; clfft < TRANSPOSE_BLOCK_SIZE; clfft++) {
          	retval = FindSpikes(
          						&PowerSpectrum[fftlen * ((ifft & ~(TRANSPOSE_BLOCK_SIZE-1)) + clfft)],
          						fftlen,
          						(ifft & ~(TRANSPOSE_BLOCK_SIZE-1)) + clfft,
          						swi,
          						&vmaxNoDC[clfft],
          						&vmaxInd[clfft],
          						&vmean[clfft]
          						);
            }
            progress += TRANSPOSE_BLOCK_SIZE*SpikeProgressUnits(fftlen)*ProgressUnitSize/NumFfts;
          }
  		  }
		  
  		  progress = std::min(progress,1.0);
#ifdef BOINC_APP_GRAPHICS
  		  if (!nographics()) {
  			if (gbp) {
  			  gbp->rarray.add_source_row(PowerSpectrum+fftlen*ifft);
  			}
  			sah_graphics->local_progress = (((float)ifft+1)/NumFfts);
  		  }
#endif
  		  remaining=1.0-(double)(icfft+1)/num_cfft;
  		  fraction_done(progress,remaining);
  		  // jeffc
  		  //fprintf(stderr, "S fft len %d  progress = %12.10f\n", fftlen, progress);
  		}
	  }
	  
//      for (; ifft < iblockfft+NumBlockFfts; ifft++) {
//        CurrentSub = fftlen * ifft;
//		CurrentFreqSub = fftlen * ifft
//
//        state.FLOP_counter+=5*(double)fftlen*log((double)fftlen)/log(2.0);
//#ifdef USE_IPP
//        ippsFFTInv_CToC_32fc((Ipp32fc*)&ChirpedData[CurrentSub], (Ipp32fc*)WorkData,
//                             FftSpec[FftNum], FftBuf);
//#elif defined(USE_FFTWF)
//        fftwf_execute_dft(analysis_plans[FftNum], &ChirpedData[CurrentSub], &WorkData[CurrentSub]);
//#else
//        // replace time with freq - ooura FFT
//        cdft(fftlen*2, 1, WorkData, BitRevTab[FftNum], CoeffTab[FftNum]);
//#endif
//
//        // replace freq with power
//        state.FLOP_counter+=(double)fftlen;
//
//		if (!dotranspose) {
////#if defined(__VEC__) && defined(USE_VECTORIZED_POWER)
//		  v_vGetPowerSpectrumMeanMax( &WorkData[CurrentSub],
//									  &PowerSpectrum[CurrentSub],
//									  fftlen,
//									  &max,
//									  &ind,
//									  &mean
//									  );
////#elif defined(__SSE3__) && defined(USE_VECTORIZED_POWER)
////        v_vGetPowerSpectrum( WorkData,
////                             &PowerSpectrum[CurrentSub],
////                             fftlen
////                             );
////#else
////        v_GetPowerSpectrum( WorkData,
////                            &PowerSpectrum[CurrentSub],
////                            fftlen
////                          );
////#endif
//        // any ETIs ?!
//        // If PoT freq bin is non-negative, we are into PoT analysis
//        // for this cfft pair and need not redo spike finding.
//		  if(state.PoT_freq_bin == -1) {
//			state.FLOP_counter+=(double)fftlen;
////#if defined(__VEC__) && defined(USE_VECTORIZED_POWER)
//			retval = FindSpikes(
//								&PowerSpectrum[CurrentSub],
//								fftlen,
//								ifft,
//								swi,
//								&max,
//								&ind,
//								&mean
//								);
////#else
////			retval = FindSpikes(
////								&PowerSpectrum[CurrentSub],
////								fftlen,
////								ifft,
////								swi,
////								NULL,
////								NULL,
////								NULL
////								);
////#endif
//			progress += SpikeProgressUnits(fftlen)*ProgressUnitSize/NumFfts;
//			if (retval) SETIERROR(retval,"from FindSpikes");
//		  }
//		} else if ((ifft & 0x7) == 7) {
//		  // this is the last FFT in a block of 8 - transpose and do spike-finding
//		  // XXX: stupid graphics wants the power spectrum, so we may not be able to omit the non-transposed data
//		  union {
//			vector float v[2];
//			float s[8];
//		  } maxNoDC;
//		  union {
//			vector unsigned int v[2];
//			unsigned int s[8];
//		  } maxInd;
//		  union {
//			vector float v[2];
//			float s[8];
//		  } mean;
//		  unsigned int clfft;
//		  
//		  v_vTransposePowerMeanMax8(&WorkData[fftlen * (ifft & ~0x7)], &tPowerSpectrum[ifft & ~0x7], fftlen, NumFfts, maxNoDC.v, maxInd.v, mean.v);
//		  if(state.PoT_freq_bin == -1) {
//			state.FLOP_counter+=(double)8*fftlen;
//			for (clfft = 0; clfft < 8; clfft++) {
//			  retval = FindSpikes(
//								  &PowerSpectrum[fftlen * ((ifft & ~0x7) + clfft)],
//								  fftlen,
//								  (ifft & ~0x7) + clfft,
//								  swi,
//								  &maxNoDC.s[clfft],
//								  &maxInd.s[clfft],
//								  &mean.s[clfft]
//								  );
//			}
//			progress += 8*SpikeProgressUnits(fftlen)*ProgressUnitSize/NumFfts;
//		  }
//		}		  
		
        //progress = ((float)icfft)/num_cfft + ((float)ifft)/(NumFfts*num_cfft);
//	  progress = std::min(progress,1.0);
//  #ifdef BOINC_APP_GRAPHICS
//        if (!nographics()) {
//          if (gbp) {
//            gbp->rarray.add_source_row(PowerSpectrum+fftlen*ifft);
//          }
//          sah_graphics->local_progress = (((float)ifft+1)/NumFfts);
//        }
//  #endif
//        remaining=1.0-(double)(icfft+1)/num_cfft;
//        fraction_done(progress,remaining);
//        // jeffc
//        //fprintf(stderr, "S fft len %d  progress = %12.10f\n", fftlen, progress);
//      } // loop through chirped data array
    }

#ifdef BOINC_APP_GRAPHICS
    if (!nographics()) {
      if (gbp) {
        sah_graphics->generate_done(gbp);
      }
    }
#endif
    fraction_done(progress,remaining);
    // jeffc
    //fprintf(stderr, "Sdone fft len %d  progress = %12.10f\n", fftlen, progress);
    
#ifdef USE_TRANSPOSED_POT
//    // transpose PoT matrix to make memory accesses nicer
//    vSgetmo(NumFfts, fftlen,
//            (vFloat *) PowerSpectrum, (vFloat *) tPowerSpectrum);
#endif

    // Analyze Power over Time.  May return quickly if this FFT
    // length and/or this WUs slew rate places the data block
    // outside PoT analysis limits.
    // Counting flops is done inside analyze_pot
    retval = analyze_pot(PowerSpectrum, tPowerSpectrum, NumDataPoints, ChirpFftPairs[icfft]);
    if (retval) SETIERROR(retval,"from analyze_pot");

#ifdef BOINC_APP_GRAPHICS
    // switch the display back to "best of" signals
    //
    if (!nographics()) {
      sah_graphics->gi.copy(best_gauss, true);
      sah_graphics->pi.copy(best_pulse, true);
      sah_graphics->ti.copy(best_triplet, true);
    }
#endif
    // Force progress to 100% before calling result_group_end() to store
    //  100% in state file so it will survive exit & relaunch
    if (icfft == (num_cfft-1)) {
      progress = 1;
      remaining = 0;
      fraction_done(progress,remaining);
    }
    retval = checkpoint();
    if (retval) SETIERROR(retval,"from checkpoint() in seti_analyse()");

  } // loop over chirp/fftlen pairs
#endif
  // Return the "best of" signals.  This may include duplicates of
  // already reported interesting signals.
  if(best_spike->score) {
    retval = outfile.printf("%s", best_spike->s.print_xml(0,0,1,"best_spike").c_str());
    if (retval < 0) {
      SETIERROR(WRITE_FAILED,"from outfile.printf (best spike) in seti_analyze()");
    }

  }
  if(best_gauss->score) {
    retval = outfile.printf("%s", best_gauss->g.print_xml(0,0,1,"best_gaussian").c_str());
    if (retval < 0) {
      SETIERROR(WRITE_FAILED,"from outfile.printf (best gaussian) in seti_analyze()");
    }
  }
  if(best_pulse->score) {
    retval = outfile.printf("%s", best_pulse->p.print_xml(0,0,1,"best_pulse").c_str());
    if (retval < 0) {
      SETIERROR(WRITE_FAILED,"from outfile.printf (best pulse) in seti_analyze()");
    }
  }
  if(best_triplet->score) {
    retval = outfile.printf("%s", best_triplet->t.print_xml(0,0,1,"best_triplet").c_str());
    if (retval < 0) {
      SETIERROR(WRITE_FAILED,"from outfile.printf (best triplet) in seti_analyze()");
    }
  }

#ifdef BOINC_APP_GRAPHICS
  if (!nographics()) strcpy(sah_graphics->status, "Work unit done");
#endif
  
  final_report(); // flop and signal counts to stderr

  // jeffc
  //retval = outfile.flush();
  retval = checkpoint();  // force a final checkpoint
  xml_indent(-2);
  outfile.printf("</result>");
  outfile.close();
  //if (retval) SETIERROR(WRITE_FAILED,"from outfile.fflush in seti_analyze()");
  
  if (vmaxNoDC) free_a(vmaxNoDC);
  if (vmaxInd) free_a(vmaxInd);
  if (vmean) free_a(vmean);
  
  if (PowerSpectrum) free_a(PowerSpectrum);

#ifdef USE_IPP
  for (FftNum = 0; FftNum < swi.num_fft_lengths; FftNum++) {
#ifdef USE_SPLIT_COMPLEX
#if 0
    if (vFftSpec[FftNum]) vDSP_destroy_fftsetup(vFftSpec[FftNum]);
#else
    if (SFftSpec[FftNum]) ippsFFTFree_C_32f (SFftSpec[FftNum]);
#endif
#else
    if (FftSpec[FftNum]) ippsFFTFree_C_32fc (FftSpec[FftNum]);
#endif
  }
#if 0
  if (VFftBuf.realp) free_a(VFftBuf.realp);
  if (VFftBuf.imagp) free_a(VFftBuf.imagp);
  VFftBuf.realp = NULL;
  VFftBuf.imagp = NULL;
#else
  if (FftBuf) free_a(FftBuf);
  FftBuf = NULL;
#endif
#elif !defined(USE_FFTWF)
  for(FftNum = 0; FftNum < swi.num_fft_lengths; FftNum++) {
    if (BitRevTab[FftNum]) free_a(BitRevTab[FftNum]);
  }
  for(FftNum = 0; FftNum < swi.num_fft_lengths; FftNum++) {
    if (CoeffTab[FftNum]) free_a(CoeffTab[FftNum]);
  }
#endif

// XXX: need to free split-complex arrays here
#ifdef USE_SPLIT_COMPLEX
  free_a(SplitWorkData.realp);
  free_a(SplitWorkData.imagp);
#else
  if (WorkData) free_a(WorkData);
  WorkData = NULL;
#endif

  if (ChirpFftPairs) free(ChirpFftPairs);
#ifndef USE_VECTORIZED_CHIRP
  if ((app_init_data.host_info.m_nbytes != 0)  &&
      (app_init_data.host_info.m_nbytes >= (double)(64*1024*1024))) {
    FreeTrigArray();
  }
#endif
//  FreeSplitTrigArray();

#if defined(_PGO_INSTRUMENT)
//Jason: PGO
  _PGOPTI_Prof_Dump_And_Reset();
#endif

  return retval;
}


int v_BaseLineSmooth(
  sah_complex* DataIn,
  int NumDataPoints,
  int BoxCarLength,
  int NumPointsInChunk
) {

  // We use a sliding boxcar method for baseline smoothing.  Input data
  // is the time domain.  It is transformed (using a separate array)
  // into the frequency domain.  After baseline smoothing is done
  // in the frequency domain, it is transformed back into the time domain.

  int h, i, j, k;
  sah_complex* DataInChunk;
  static sah_complex *DataOutChunk=0;
  static float* PowerSpectrum=0;
  float Total, LocalMean, ScaleFactor;
  int NumTimeChunks, TimeChunk, Endpoint;
  static int OldNumPointsInChunk = 0;
#ifdef USE_IPP
  static IppsFFTSpec_C_32fc* FftSpec = NULL;
#elif defined(USE_FFTWF)
  static fftwf_plan backward_transform, forward_transform;
#endif /* USE_FFTWF */

#ifndef USE_FFTWF
  static int * BitRevTab = NULL;
  static float * CoeffTab = NULL;
#endif /* USE_FFTWF */

  NumTimeChunks = NumDataPoints / NumPointsInChunk;


  // If we keep doing transforms that are the same length, don't reinitialize plans
  if (NumPointsInChunk != OldNumPointsInChunk) {
    if (OldNumPointsInChunk != 0) {

#ifdef USE_IPP
      if (FftSpec) ippsFFTFree_C_32fc (FftSpec);
#elif defined(USE_FFTWF)
      if (backward_transform) free_a(backward_transform);
      if (forward_transform) free_a(forward_transform);
#else
      if (BitRevTab) free_a(BitRevTab);
      if (CoeffTab) free_a(CoeffTab);
#endif
      if (PowerSpectrum) free_a(PowerSpectrum);
      if (DataOutChunk) free_a(DataOutChunk);
    }
    PowerSpectrum = (float*) calloc_a(NumPointsInChunk, sizeof(float), MEM_ALIGN);
    if (PowerSpectrum == NULL) {
      printf("Could not allocate Power Spectrum array in v_BaseLineSmooth()\n");
      exit(1);
    }
    // Do the transforms in the DataOutChunk, since DataInChunk won't
    // necessarily be aligned correctly (and we won't get SIMD)
    // TODO: automatically make DataInChunk aligned correctly so we
    //       don't need the memcpy. This may already be done if
    //       NumPointsInChunk*sizeof(sah_complex) is a multiple of MEM_ALIGN
    DataOutChunk = (sah_complex *)malloc_a(NumPointsInChunk * sizeof(sah_complex),MEM_ALIGN);
    OldNumPointsInChunk = NumPointsInChunk;

#ifdef USE_IPP
    int order = 0;
    for (int tmp = NumPointsInChunk; !(tmp & 1); order++) tmp >>= 1;
    ippsFFTInitAlloc_C_32fc (&FftSpec, order, IPP_FFT_NODIV_BY_ANY,
                             ippAlgHintAccurate);
#elif defined(USE_FFTWF)
    sah_complex *scratch = (sah_complex *)malloc_a(NumPointsInChunk * sizeof(sah_complex),MEM_ALIGN);

    backward_transform = fftwf_plan_dft_1d(NumPointsInChunk, scratch, DataOutChunk, FFTW_BACKWARD, FFTW_MEASURE|FFTW_PRESERVE_INPUT);
    forward_transform = fftwf_plan_dft_1d(NumPointsInChunk, DataOutChunk, DataOutChunk, FFTW_FORWARD, FFTW_MEASURE);
    free_a(scratch);
#else
    BitRevTab = (int*) calloc_a(3+(int)sqrt((float)NumPointsInChunk/2), sizeof(int), MEM_ALIGN);
    if(BitRevTab == NULL) return MALLOC_FAILED;

    CoeffTab = (float*) calloc_a(NumPointsInChunk/2, sizeof(float), MEM_ALIGN);
    if(CoeffTab == NULL) return MALLOC_FAILED;
    // flag to tell cdft() to init it's work areas
    // already done since we used calloc_a();
    // BitRevTab[0] = 0;
#endif

  }

  for (TimeChunk = 0; TimeChunk < NumTimeChunks; TimeChunk++) {
#ifdef BOINC_APP_GRAPHICS
    if (!nographics()) sah_graphics->local_progress = (((float)TimeChunk)/NumTimeChunks);
#endif

    DataInChunk = &(DataIn[TimeChunk*NumPointsInChunk]);
#ifndef USE_FFTWF
    memcpy( DataOutChunk, DataInChunk, (int)(NumPointsInChunk*sizeof(sah_complex)) );
#endif

    // transform to freq
#ifdef USE_IPP
    ippsFFTInv_CToC_32fc ((Ipp32fc*)DataOutChunk, (Ipp32fc*)DataOutChunk,
                          FftSpec, NULL);
#elif !defined(USE_FFTWF)
    cdft(NumPointsInChunk*2, 1, DataOutChunk, BitRevTab, CoeffTab);
#else
    fftwf_execute_dft(backward_transform,DataInChunk,DataOutChunk);
#endif

    v_GetPowerSpectrum(
      DataOutChunk, PowerSpectrum, NumPointsInChunk
    );

    // Begin: normalize in freq. domain via sliding boxcar

    Endpoint = NumPointsInChunk / 2;

    i = Endpoint;         // start i at lowest negative freq;
    // this is low point in first boxcar
    j = i + BoxCarLength / 2;        // start j midpoint in first boxcar
    k = i + BoxCarLength;            // start k at end of first boxcar;
    // thus a boxcar is i----j----k
    Total = 0;
    for (h = i; h < k; h++) {            // Get mean for first boxcar
      Total += PowerSpectrum[h];
    }
    LocalMean = Total / BoxCarLength;
    ScaleFactor = (float)(sqrt(LocalMean / 2));

    // normalize 1st half of 1st boxcar
    for (h = i; h < j; h++) {
      DataOutChunk[h][0] /= ScaleFactor;
      DataOutChunk[h][1] /= ScaleFactor;
    }

    for (; k != Endpoint; i++, j++, k++) {  // sliding boxcar
      if (k == NumPointsInChunk) {    // take care of wrapping
        k = 0;
      }
      if (j == NumPointsInChunk) {
        j = 0;
      }
      if (i == NumPointsInChunk) {
        i = 0;
      }

      LocalMean = LocalMean
                  - PowerSpectrum[i] / BoxCarLength
                  + PowerSpectrum[k] / BoxCarLength;
      ScaleFactor = (float)(sqrt(LocalMean / 2));

      DataOutChunk[j][0] /= ScaleFactor;
      DataOutChunk[j][1] /= ScaleFactor;

    } // End sliding boxcar

    // normalize final half of final boxcar
    for (h = j; h < k; h++) {
      DataOutChunk[h][0] /= ScaleFactor;
      DataOutChunk[h][1] /= ScaleFactor;
    }

    // End: normalize in freq. domain via sliding boxcar

    // transform back to time
#ifdef USE_IPP
    ippsFFTFwd_CToC_32fc((Ipp32fc*)DataOutChunk, (Ipp32fc*)DataOutChunk,
                         FftSpec, NULL);
#elif !defined(USE_FFTWF)
    cdft(NumPointsInChunk*2, -1, DataOutChunk, BitRevTab, CoeffTab);
#else
    fftwf_execute(forward_transform);
#endif
    analysis_state.FLOP_counter+=10.0*NumPointsInChunk*log((double)NumPointsInChunk)/log(2.0)+10.0*NumPointsInChunk;
    // return powers to normal
    for (i = 0; i < NumPointsInChunk; i++) {
      DataInChunk[i][0] = DataOutChunk[i][0]/NumPointsInChunk;
      DataInChunk[i][1] = DataOutChunk[i][1]/NumPointsInChunk;
    }
  }

  return 0;
}


void v_GetPowerSpectrum(
  sah_complex* FreqData,
  float* PowerSpectrum,
  int NumDataPoints
) {
  int i;

#ifdef __INTEL_COMPILER
#pragma message ("---using ICC---")
#pragma vector aligned
  __assume_aligned (FreqData, MEM_ALIGN);
#endif
  analysis_state.FLOP_counter+=3.0*NumDataPoints;
  for (i = 0; i < NumDataPoints; i++) {
    PowerSpectrum[i] = FreqData[i][0] * FreqData[i][0]
                       + FreqData[i][1] * FreqData[i][1];
  }
}


#ifdef __VEC__
void v_vGetPowerSpectrum(
  sah_complex* FreqData,
  float* PowerSpectrum,
  int NumDataPoints
) {
  int i = 0, vEnd;
  const vector unsigned char real = (vector unsigned char) (0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
  const vector unsigned char imag = (vector unsigned char) (4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
  analysis_state.FLOP_counter+=3.0*NumDataPoints;

  vEnd = NumDataPoints - (NumDataPoints & 15);
  for (i = 0; i < vEnd; i += 16) {
    const float *f = (const float *) (FreqData + i);
    float *p = PowerSpectrum + i;
    vector float f1, f2, f3, f4, f5, f6, f7, f8;
    vector float re1, im1, re2, im2, re3, im3, re4, im4;
    vector float p1, p2, p3, p4;
    
#ifdef USE_PPC_G4
    // prefetch next 512 bytes into L1 cache
    vec_dstt(f, 0x10020100, 0);
#endif

    f1 = vec_ld(0, f);
    f2 = vec_ld(16, f);
    f3 = vec_ld(32, f);
    f4 = vec_ld(48, f);
    f5 = vec_ld(64, f);
    f6 = vec_ld(80, f);
    f7 = vec_ld(96, f);
    f8 = vec_ld(112, f);
    re1 = vec_perm(f1, f2, real);
    im1 = vec_perm(f1, f2, imag);
    re2 = vec_perm(f3, f4, real);
    im2 = vec_perm(f3, f4, imag);
    re3 = vec_perm(f5, f6, real);
    im3 = vec_perm(f5, f6, imag);
    re4 = vec_perm(f7, f8, real);
    im4 = vec_perm(f7, f8, imag);

#ifdef USE_PPC_G4
    // zero out a spot for our power data in the cache
    __dcbz(0, p);
    __dcbz(32, p);
#endif

    p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
    p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
    p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
    p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
    vec_st(p1, 0, p);
    vec_st(p2, 16, p);
    vec_st(p3, 32, p);
    vec_st(p4, 48, p);
  }  
  
  vEnd = NumDataPoints - (NumDataPoints & 3);
  for (; i < vEnd; i += 4) {
    const float *f = (const float *) (FreqData + i);
    float *p = PowerSpectrum + i;
    vector float f1, f2;
    vector float re1, im1;
    vector float p1;
    
    f1 = vec_ld(0, f);
    f2 = vec_ld(16, f);
    re1 = vec_perm(f1, f2, real);
    im1 = vec_perm(f1, f2, imag);
    p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
    vec_st(p1, 0, p);    
  }  

  // handle tail elements, although in practice this never happens
  for (; i < NumDataPoints; i++) {
    PowerSpectrum[i] = FreqData[i][0] * FreqData[i][0]
    + FreqData[i][1] * FreqData[i][1];
  }  
}

void v_vGetPowerSpectrumMeanMax(
  sah_complex* FreqData,
  float* PowerSpectrum,
  int NumDataPoints,
  float* pMaxNoDC,
  unsigned int* pMaxInd,
  float* pMean
) {
  int i = 0, vEnd;
  const vector unsigned char real = (vector unsigned char) (0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
  const vector unsigned char imag = (vector unsigned char) (4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
  analysis_state.FLOP_counter+=3.0*NumDataPoints;
  vector unsigned int ind1, ind2, ind3, ind4, ind;
  vector float max1, max2, max3, max4, max;
  vector float sum1, sum2, sum3, sum4, sum;
  vector unsigned int j1, j2, j3, j4;
  vector unsigned int cmp;
  float meanScale;
  
  if (NumDataPoints == 8) {
    // explicitly handle the first 8 elements to deal with the no-DC max
    const float *f = (const float *) FreqData;
    float *p = PowerSpectrum;
    vector float f1, f2, f3, f4;
    vector float re1, im1, re2, im2;
    vector float p1, p2;

    meanScale = 0.125f;
	
    f1 = vec_ld(0, f);
    f2 = vec_ld(16, f);
    f3 = vec_ld(32, f);
    f4 = vec_ld(48, f);
  	re1 = vec_perm(f1, f2, real);
    im1 = vec_perm(f1, f2, imag);
    re2 = vec_perm(f3, f4, real);
    im2 = vec_perm(f3, f4, imag);
  	p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
    p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));

  	max1 = vec_and(p1, (vFloat) ((vector unsigned int) {0, 0xffffffff, 0xffffffff, 0xffffffff}));
  	max2 = p2;
  	ind1 = ((vector unsigned int) {0, 1, 2, 3});
  	ind2 = ((vector unsigned int) {4, 5, 6, 7});
	
	// reduce to 1 vector
	sum = vec_add(p1, p2);
	max = vec_max(max1, max2);
	ind = vec_sel(ind1, ind2, vec_cmpgt(max2, max1));
		
//	vec_st(p1, 0, p);
//	vec_st(p2, 16, p);
  } else {
    // set up 16 data points for other cases
    const float *f = (const float *) FreqData;
    float *p = PowerSpectrum;
    vector float f1, f2, f3, f4, f5, f6, f7, f8;
    vector float re1, im1, re2, im2, re3, im3, re4, im4;
    vector float p1, p2, p3, p4;

    meanScale = 1.0f / NumDataPoints;

    f1 = vec_ld(0, f);
    f2 = vec_ld(16, f);
    f3 = vec_ld(32, f);
    f4 = vec_ld(48, f);
    f5 = vec_ld(64, f);
    f6 = vec_ld(80, f);
    f7 = vec_ld(96, f);
    f8 = vec_ld(112, f);

    re1 = vec_perm(f1, f2, real);
    im1 = vec_perm(f1, f2, imag);
    re2 = vec_perm(f3, f4, real);
    im2 = vec_perm(f3, f4, imag);
    re3 = vec_perm(f5, f6, real);
    im3 = vec_perm(f5, f6, imag);
    re4 = vec_perm(f7, f8, real);
    im4 = vec_perm(f7, f8, imag);	
    p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
    p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
    p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
    p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));

    sum1 = p1;
    sum2 = p2;
    sum3 = p3;
    sum4 = p4;

    max1 = vec_and(p1, (vFloat) ((vector unsigned int) {0, 0xffffffff, 0xffffffff, 0xffffffff}));
    max2 = p2;
    max3 = p3;
    max4 = p4;
    j1 = ind1 = ((vector unsigned int) {0, 1, 2, 3});
    j2 = ind2 = ((vector unsigned int) {4, 5, 6, 7});
    j3 = ind3 = ((vector unsigned int) {8, 9, 10, 11});
    j4 = ind4 = ((vector unsigned int) {12, 13, 14, 15});
	
//	vec_st(p1, 0, p);
//	vec_st(p2, 16, p);
//	vec_st(p3, 32, p);
//	vec_st(p4, 48, p);	

  	j1 = vec_add(j1, ISIXTEEN);
  	j2 = vec_add(j2, ISIXTEEN);
  	j3 = vec_add(j3, ISIXTEEN);
  	j4 = vec_add(j4, ISIXTEEN);	
	
  	vEnd = NumDataPoints & ~0xf;
	for (i = 16; i < vEnd; i += 16) {
	  f = (const float *) (FreqData + i);
	  p = PowerSpectrum + i;
	  
#ifdef USE_PPC_G4
	  // prefetch next 512 bytes into L1 cache
	  vec_dstt(f, 0x10020100, 0);
#endif
	  
	  f1 = vec_ld(0, f);
	  f2 = vec_ld(16, f);
	  f3 = vec_ld(32, f);
	  f4 = vec_ld(48, f);
	  f5 = vec_ld(64, f);
	  f6 = vec_ld(80, f);
	  f7 = vec_ld(96, f);
	  f8 = vec_ld(112, f);
	  re1 = vec_perm(f1, f2, real);
	  im1 = vec_perm(f1, f2, imag);
	  re2 = vec_perm(f3, f4, real);
	  im2 = vec_perm(f3, f4, imag);
	  re3 = vec_perm(f5, f6, real);
	  im3 = vec_perm(f5, f6, imag);
	  re4 = vec_perm(f7, f8, real);
	  im4 = vec_perm(f7, f8, imag);
	  
#ifdef USE_PPC_G4
//	  // zero out a spot for our power data in the cache
//	  __dcbz(0, p);
//	  __dcbz(32, p);
#endif
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  sum1 = vec_add(sum1, p1);
	  sum2 = vec_add(sum2, p2);
	  sum3 = vec_add(sum3, p3);
	  sum4 = vec_add(sum4, p4);
	  
	  ind1 = vec_sel(ind1, j1, vec_cmpgt(p1, max1));
	  ind2 = vec_sel(ind2, j2, vec_cmpgt(p2, max2));
	  ind3 = vec_sel(ind3, j3, vec_cmpgt(p3, max3));
	  ind4 = vec_sel(ind4, j4, vec_cmpgt(p4, max4));
	  max1 = vec_max(max1, p1);
	  max2 = vec_max(max2, p2);
	  max3 = vec_max(max3, p3);
	  max4 = vec_max(max4, p4);
										  
	  j1 = vec_add(j1, ISIXTEEN);
	  j2 = vec_add(j2, ISIXTEEN);
	  j3 = vec_add(j3, ISIXTEEN);
	  j4 = vec_add(j4, ISIXTEEN);

//	  vec_st(p1, 0, p);
//	  vec_st(p2, 16, p);
//	  vec_st(p3, 32, p);
//	  vec_st(p4, 48, p);
	}  
	
	vEnd = NumDataPoints & ~0x3;
	for (; i < vEnd; i += 4) {
	  f = (const float *) (FreqData + i);
	  p = PowerSpectrum + i;
	  
	  f1 = vec_ld(0, f);
	  f2 = vec_ld(16, f);
	  re1 = vec_perm(f1, f2, real);
	  im1 = vec_perm(f1, f2, imag);
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  
	  sum1 = vec_add(sum1, p1);
	  ind1 = vec_sel(ind1, j1, vec_cmpgt(p1, max1));
	  max1 = vec_max(max1, p1);
	  j1 = vec_add(j1, IFOUR);
	  
//	  vec_st(p1, 0, p);    
	}
	
	// reduce to 1 vector
	sum = vec_add(vec_add(sum1, sum2), vec_add(sum3, sum4));
  cmp = (vector unsigned int) vec_or(vec_cmpgt(max2, max1),
  								   vec_and(vec_cmpeq(max2, max1),
  										   vec_cmplt(ind2, ind1)));
	max1 = vec_max(max1, max2);
	ind1 = vec_sel(ind1, ind2, cmp);

	cmp = (vector unsigned int) vec_or(vec_cmpgt(max4, max3),
										vec_and(vec_cmpeq(max4, max3),
												vec_cmplt(ind4, ind3)));
	max2 = vec_max(max3, max4);
	ind2 = vec_sel(ind3, ind4, cmp);

	cmp = (vector unsigned int) vec_or(vec_cmpgt(max2, max1),
								   vec_and(vec_cmpeq(max2, max1),
										   vec_cmplt(ind2, ind1)));
	max = vec_max(max1, max2);
	ind = vec_sel(ind1, ind2, cmp);
  }
  
  // perform reductions
  float *vpSum = (float *) &sum;
  float ssum = ((vpSum[0] + vpSum[1]) + (vpSum[2] + vpSum[3])) * meanScale;
  
  cmp = (vector unsigned int) vec_or(vec_cmpgt(max, vec_sld(max, max, 8)),
									 vec_and(vec_cmpeq(max, vec_sld(max, max, 8)),
											 vec_cmplt(ind, vec_sld(ind, ind, 8))));
  max = vec_max(max, vec_sld(max, max, 8));
  ind = vec_sel(vec_sld(ind, ind, 8), ind, cmp);
  
  cmp = (vector unsigned int) vec_or(vec_cmpgt(max, vec_sld(max, max, 4)),
									 vec_and(vec_cmpeq(max, vec_sld(max, max, 4)),
											 vec_cmplt(ind, vec_sld(ind, ind, 4))));
  {
	// make the codegen happy so it doesn't completely destroy my index
	union {
	  vector unsigned int v;
	  unsigned int s[4];
	} vind;
	
	vector float vmax = vec_max(max, vec_sld(max, max, 4));
	float *vpMax = (float *) &vmax;
	vind.v = vec_sel(vec_sld(ind, ind, 4), ind, cmp);
	
	float smax = *vpMax;
	unsigned int sind = vind.s[0];
	
	*pMean = ssum;
	*pMaxNoDC = smax;
	*pMaxInd = sind;
  }
}

void v_vGetPowerSpectrumMeanMaxWrite(
  sah_complex* FreqData,
  float* PowerSpectrum,
  int NumDataPoints,
  float* pMaxNoDC,
  unsigned int* pMaxInd,
  float* pMean
) {
  int i = 0, vEnd;
  const vector unsigned char real = (vector unsigned char) (0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
  const vector unsigned char imag = (vector unsigned char) (4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
  analysis_state.FLOP_counter+=3.0*NumDataPoints;
  vector unsigned int ind1, ind2, ind3, ind4, ind;
  vector float max1, max2, max3, max4, max;
  vector float sum1, sum2, sum3, sum4, sum;
  vector unsigned int j1, j2, j3, j4;
  vector unsigned int cmp;
  float meanScale;
  
  if (NumDataPoints == 8) {
    // explicitly handle the first 8 elements to deal with the no-DC max
    const float *f = (const float *) FreqData;
//    float *p = PowerSpectrum;
    float *p = (float *) FreqData;
    vector float f1, f2, f3, f4;
    vector float re1, im1, re2, im2;
    vector float p1, p2;

    meanScale = 0.125f;
	
    f1 = vec_ld(0, f);
    f2 = vec_ld(16, f);
    f3 = vec_ld(32, f);
    f4 = vec_ld(48, f);
  	re1 = vec_perm(f1, f2, real);
    im1 = vec_perm(f1, f2, imag);
    re2 = vec_perm(f3, f4, real);
    im2 = vec_perm(f3, f4, imag);
  	p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
    p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));

  	max1 = vec_and(p1, (vFloat) ((vector unsigned int) {0, 0xffffffff, 0xffffffff, 0xffffffff}));
  	max2 = p2;
  	ind1 = ((vector unsigned int) {0, 1, 2, 3});
  	ind2 = ((vector unsigned int) {4, 5, 6, 7});
	
	// reduce to 1 vector
	sum = vec_add(p1, p2);
	max = vec_max(max1, max2);
	ind = vec_sel(ind1, ind2, vec_cmpgt(max2, max1));
		
	vec_st(p1, 0, p);
	vec_st(p2, 16, p);
  } else {
    // set up 16 data points for other cases
    const float *f = (const float *) FreqData;
//    float *p = PowerSpectrum;
    float *p = (float *) FreqData;
    vector float f1, f2, f3, f4, f5, f6, f7, f8;
    vector float re1, im1, re2, im2, re3, im3, re4, im4;
    vector float p1, p2, p3, p4;

    meanScale = 1.0f / NumDataPoints;

    f1 = vec_ld(0, f);
    f2 = vec_ld(16, f);
    f3 = vec_ld(32, f);
    f4 = vec_ld(48, f);
    f5 = vec_ld(64, f);
    f6 = vec_ld(80, f);
    f7 = vec_ld(96, f);
    f8 = vec_ld(112, f);

    re1 = vec_perm(f1, f2, real);
    im1 = vec_perm(f1, f2, imag);
    re2 = vec_perm(f3, f4, real);
    im2 = vec_perm(f3, f4, imag);
    re3 = vec_perm(f5, f6, real);
    im3 = vec_perm(f5, f6, imag);
    re4 = vec_perm(f7, f8, real);
    im4 = vec_perm(f7, f8, imag);	
    p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
    p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
    p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
    p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));

    sum1 = p1;
    sum2 = p2;
    sum3 = p3;
    sum4 = p4;

    max1 = vec_and(p1, (vFloat) ((vector unsigned int) {0, 0xffffffff, 0xffffffff, 0xffffffff}));
    max2 = p2;
    max3 = p3;
    max4 = p4;
    j1 = ind1 = ((vector unsigned int) {0, 1, 2, 3});
    j2 = ind2 = ((vector unsigned int) {4, 5, 6, 7});
    j3 = ind3 = ((vector unsigned int) {8, 9, 10, 11});
    j4 = ind4 = ((vector unsigned int) {12, 13, 14, 15});
	
	vec_st(p1, 0, p);
	vec_st(p2, 16, p);
	vec_st(p3, 32, p);
	vec_st(p4, 48, p);	

  	j1 = vec_add(j1, ISIXTEEN);
  	j2 = vec_add(j2, ISIXTEEN);
  	j3 = vec_add(j3, ISIXTEEN);
  	j4 = vec_add(j4, ISIXTEEN);	
	
  	vEnd = NumDataPoints & ~0xf;
	for (i = 16; i < vEnd; i += 16) {
	  f = (const float *) (FreqData + i);
//	  p = PowerSpectrum + i;
	  p = ((float *)FreqData) + i;
	  
#ifdef USE_PPC_G4
	  // prefetch next 512 bytes into L1 cache
	  vec_dstt(f, 0x10020100, 0);
#endif
	  
	  f1 = vec_ld(0, f);
	  f2 = vec_ld(16, f);
	  f3 = vec_ld(32, f);
	  f4 = vec_ld(48, f);
	  f5 = vec_ld(64, f);
	  f6 = vec_ld(80, f);
	  f7 = vec_ld(96, f);
	  f8 = vec_ld(112, f);
	  re1 = vec_perm(f1, f2, real);
	  im1 = vec_perm(f1, f2, imag);
	  re2 = vec_perm(f3, f4, real);
	  im2 = vec_perm(f3, f4, imag);
	  re3 = vec_perm(f5, f6, real);
	  im3 = vec_perm(f5, f6, imag);
	  re4 = vec_perm(f7, f8, real);
	  im4 = vec_perm(f7, f8, imag);
	  
#ifdef USE_PPC_G4
	  // zero out a spot for our power data in the cache
	  __dcbz(0, p);
	  __dcbz(32, p);
#endif
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  sum1 = vec_add(sum1, p1);
	  sum2 = vec_add(sum2, p2);
	  sum3 = vec_add(sum3, p3);
	  sum4 = vec_add(sum4, p4);
	  
	  ind1 = vec_sel(ind1, j1, vec_cmpgt(p1, max1));
	  ind2 = vec_sel(ind2, j2, vec_cmpgt(p2, max2));
	  ind3 = vec_sel(ind3, j3, vec_cmpgt(p3, max3));
	  ind4 = vec_sel(ind4, j4, vec_cmpgt(p4, max4));
	  max1 = vec_max(max1, p1);
	  max2 = vec_max(max2, p2);
	  max3 = vec_max(max3, p3);
	  max4 = vec_max(max4, p4);
										  
	  j1 = vec_add(j1, ISIXTEEN);
	  j2 = vec_add(j2, ISIXTEEN);
	  j3 = vec_add(j3, ISIXTEEN);
	  j4 = vec_add(j4, ISIXTEEN);

	  vec_st(p1, 0, p);
	  vec_st(p2, 16, p);
	  vec_st(p3, 32, p);
	  vec_st(p4, 48, p);
	}  
	
	vEnd = NumDataPoints & ~0x3;
	for (; i < vEnd; i += 4) {
	  f = (const float *) (FreqData + i);
//	  p = PowerSpectrum + i;
	  p = ((float *) FreqData) + i;
	  
	  f1 = vec_ld(0, f);
	  f2 = vec_ld(16, f);
	  re1 = vec_perm(f1, f2, real);
	  im1 = vec_perm(f1, f2, imag);
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  
	  sum1 = vec_add(sum1, p1);
	  ind1 = vec_sel(ind1, j1, vec_cmpgt(p1, max1));
	  max1 = vec_max(max1, p1);
	  j1 = vec_add(j1, IFOUR);
	  
	  vec_st(p1, 0, p);
	}
	
	// reduce to 1 vector
	sum = vec_add(vec_add(sum1, sum2), vec_add(sum3, sum4));
  cmp = (vector unsigned int) vec_or(vec_cmpgt(max2, max1),
  								   vec_and(vec_cmpeq(max2, max1),
  										   vec_cmplt(ind2, ind1)));
	max1 = vec_max(max1, max2);
	ind1 = vec_sel(ind1, ind2, cmp);

	cmp = (vector unsigned int) vec_or(vec_cmpgt(max4, max3),
										vec_and(vec_cmpeq(max4, max3),
												vec_cmplt(ind4, ind3)));
	max2 = vec_max(max3, max4);
	ind2 = vec_sel(ind3, ind4, cmp);

	cmp = (vector unsigned int) vec_or(vec_cmpgt(max2, max1),
								   vec_and(vec_cmpeq(max2, max1),
										   vec_cmplt(ind2, ind1)));
	max = vec_max(max1, max2);
	ind = vec_sel(ind1, ind2, cmp);
  }
  
  // perform reductions
  float *vpSum = (float *) &sum;
  float ssum = ((vpSum[0] + vpSum[1]) + (vpSum[2] + vpSum[3])) * meanScale;
  
  cmp = (vector unsigned int) vec_or(vec_cmpgt(max, vec_sld(max, max, 8)),
									 vec_and(vec_cmpeq(max, vec_sld(max, max, 8)),
											 vec_cmplt(ind, vec_sld(ind, ind, 8))));
  max = vec_max(max, vec_sld(max, max, 8));
  ind = vec_sel(vec_sld(ind, ind, 8), ind, cmp);
  
  cmp = (vector unsigned int) vec_or(vec_cmpgt(max, vec_sld(max, max, 4)),
									 vec_and(vec_cmpeq(max, vec_sld(max, max, 4)),
											 vec_cmplt(ind, vec_sld(ind, ind, 4))));
  {
	// make the codegen happy so it doesn't completely destroy my index
	union {
	  vector unsigned int v;
	  unsigned int s[4];
	} vind;
	
	vector float vmax = vec_max(max, vec_sld(max, max, 4));
	float *vpMax = (float *) &vmax;
	vind.v = vec_sel(vec_sld(ind, ind, 4), ind, cmp);
	
	float smax = *vpMax;
	unsigned int sind = vind.s[0];
	
	*pMean = ssum;
	*pMaxNoDC = smax;
	*pMaxInd = sind;
  }
}

inline vector float vec_fpsplat(float f_Constant) {
  vector unsigned char vuc_Splat;
  vector float vf_Constant;
  
  vuc_Splat = vec_lvsl(0, &f_Constant);
  vf_Constant = vec_lde(0, &f_Constant);
  vuc_Splat = (vector unsigned char) vec_splat((vector float) vuc_Splat, 0);
  vf_Constant = vec_perm(vf_Constant, vf_Constant, vuc_Splat);
  
  return vf_Constant;
}

void v_vTranspose8(
  const float *powerSpectrum,
  int inStride,
  int fftlen,
  float *tPowerSpectrum,
  int numffts
) {
  const float *p;
  float *t;
  vector float p1, p2, p3, p4;
  vector float t1, t2, t3, t4;
  vector float p13_1, p13_2, p24_1, p24_2;
  int outStride = numffts * sizeof(float);
  int i;
  
  for (i = 0; i < fftlen; i += 8) {
    p = powerSpectrum + i;
    t = tPowerSpectrum + i*numffts;
    
    /////////////
    // upper left
    // TODO: insert prefetches here?
    p1 = vec_ld(0, p);
    p2 = vec_ld(inStride, p);
    p3 = vec_ld(inStride*2, p);
    p4 = vec_ld(inStride*3, p);
    
    // transpose step 1
	  p13_1 = vec_mergeh(p1, p3);
	  p13_2 = vec_mergel(p1, p3);
	  p24_1 = vec_mergeh(p2, p4);
	  p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  t1 = vec_mergeh(p13_1, p24_1);
	  t2 = vec_mergel(p13_1, p24_1);
	  t3 = vec_mergeh(p13_2, p24_2);
	  t4 = vec_mergel(p13_2, p24_2);
	  
	  // store
    vec_st(t1, 0, t);
    vec_st(t2, outStride, t);
    vec_st(t3, outStride*2, t);
    vec_st(t4, outStride*3, t);
    
    //////////////
    // upper right
    // TODO: insert prefetches here?
    p1 = vec_ld(16, p);
    p2 = vec_ld(inStride + 16, p);
    p3 = vec_ld(inStride*2 + 16, p);
    p4 = vec_ld(inStride*3 + 16, p);

    // transpose step 1
	  p13_1 = vec_mergeh(p1, p3);
	  p13_2 = vec_mergel(p1, p3);
	  p24_1 = vec_mergeh(p2, p4);
	  p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  t1 = vec_mergeh(p13_1, p24_1);
	  t2 = vec_mergel(p13_1, p24_1);
	  t3 = vec_mergeh(p13_2, p24_2);
	  t4 = vec_mergel(p13_2, p24_2);
	  
	  // store
    vec_st(t1, outStride*4, t);
    vec_st(t2, outStride*5, t);
    vec_st(t3, outStride*6, t);
    vec_st(t4, outStride*7, t);
    
    /////////////
    // lower left
    p1 = vec_ld(inStride*4, p);
    p2 = vec_ld(inStride*5, p);
    p3 = vec_ld(inStride*6, p);
    p4 = vec_ld(inStride*7, p);
    
    // transpose step 1
	  p13_1 = vec_mergeh(p1, p3);
	  p13_2 = vec_mergel(p1, p3);
	  p24_1 = vec_mergeh(p2, p4);
	  p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  t1 = vec_mergeh(p13_1, p24_1);
	  t2 = vec_mergel(p13_1, p24_1);
	  t3 = vec_mergeh(p13_2, p24_2);
	  t4 = vec_mergel(p13_2, p24_2);
	  
	  // store
    vec_st(t1, 16, t);
    vec_st(t2, outStride + 16, t);
    vec_st(t3, outStride*2 + 16, t);
    vec_st(t4, outStride*3 + 16, t);
    
    //////////////
    // lower right
    p1 = vec_ld(inStride*4 + 16, p);
    p2 = vec_ld(inStride*5 + 16, p);
    p3 = vec_ld(inStride*6 + 16, p);
    p4 = vec_ld(inStride*7 + 16, p);
    
    // transpose step 1
	  p13_1 = vec_mergeh(p1, p3);
	  p13_2 = vec_mergel(p1, p3);
	  p24_1 = vec_mergeh(p2, p4);
	  p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  t1 = vec_mergeh(p13_1, p24_1);
	  t2 = vec_mergel(p13_1, p24_1);
	  t3 = vec_mergeh(p13_2, p24_2);
	  t4 = vec_mergel(p13_2, p24_2);
	  
	  // store
    vec_st(t1, outStride*4 + 16, t);
    vec_st(t2, outStride*5 + 16, t);
    vec_st(t3, outStride*6 + 16, t);
    vec_st(t4, outStride*7 + 16, t);
  }
}

void v_vTransposePowerMeanMax8(
  sah_complex* FreqData,
  float* tPowerSpectrum,
  int fftlen,
  int numffts,
  vector float *pMaxNoDC,
  vector unsigned int *pMaxInd,
  vector float *pMean
) {
  const vector unsigned char real = (vector unsigned char) (0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
  const vector unsigned char imag = (vector unsigned char) (4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
  analysis_state.FLOP_counter+=24.0*fftlen;

  vector unsigned int ind1234, ind5678;
  vector float max1234, max5678;
  vector float sum1234, sum5678;
  vector unsigned int vj1, vj2, vj3, vj4;
  vector unsigned int cmp;
  vector float meanScale = vec_fpsplat(1.0f / fftlen);
  
  // initialize reduction variables
  max1234 = max5678 = ZERO;
  ind1234 = ind5678 = IZERO;
  sum1234 = sum5678 = ZERO;
  vj1 = (vector unsigned int) (0, 1, 2, 3);
  vj2 = (vector unsigned int) (4, 5, 6, 7);
  
#define PREFETCH_BYTES 128
  
  // we are apparently on the cusp of register spillage, so eat the code size cost to handle the DC term :(
  {
	int i = 0;
	const float *f = (const float *) (FreqData + i);
	float *t = tPowerSpectrum + i*numffts;
	vector float f1_1, f1_2, f2_1, f2_2, f3_1, f3_2, f4_1, f4_2;
	vector float re1, im1, re2, im2, re3, im3, re4, im4;
	vector float p1, p2, p3, p4;
	
#ifdef USE_PPC_G4
	// zero the first half of the cachelines used in an 8x8 block
	__dcbz(0, t);
	__dcbz(numffts*sizeof(float), t);
	__dcbz(numffts*sizeof(float)*2, t);
	__dcbz(numffts*sizeof(float)*3, t);
#endif
	
	// upper left quadrant
	{
	  f1_1 = vec_ld(0, f);
	  f1_2 = vec_ld(16, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex), f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex) + 16, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*2, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*2 + 16, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*3, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*3 + 16, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
#ifdef USE_PPC_G4
	  // prefetch a few blocks ahead
	  // don't use vec_dst, because our access pattern is too strange
	  __dcbt(fftlen*sizeof(sah_complex)*0+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*0+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*1+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*1+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*2+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*2+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*3+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*3+PREFETCH_BYTES+32, f);
#endif
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p13_1 = vec_mergeh(p1, p3);
	  vector float p13_2 = vec_mergel(p1, p3);
	  vector float p24_1 = vec_mergeh(p2, p4);
	  vector float p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p13_1, p24_1);
	  vector float t2 = vec_mergel(p13_1, p24_1);
	  vector float t3 = vec_mergeh(p13_2, p24_2);
	  vector float t4 = vec_mergel(p13_2, p24_2);
	  
	  // update packed max and sum
	  sum1234 = vec_add(sum1234, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind1234 = vec_sel(ind1234, vec_sel(vec_splat(vj1, 1),
										 vec_sel(vec_splat(vj1, 2), vec_splat(vj1, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), t2)),
						vec_cmpgt(vec_max(t2, vec_max(t3, t4)), max1234));
	  max1234 = vec_max(max1234, vec_max(t2, vec_max(t3, t4)));
	  
	  vec_st(t1, 0, t);
	  vec_st(t2, numffts*sizeof(float), t);
	  vec_st(t3, numffts*sizeof(float)*2, t);
	  vec_st(t4, numffts*sizeof(float)*3, t);
	}

#ifdef USE_PPC_G4
	// zero the second half of the cachelines used in an 8x8 block
	__dcbz(numffts*sizeof(float)*4, t);
	__dcbz(numffts*sizeof(float)*5, t);
	__dcbz(numffts*sizeof(float)*6, t);
	__dcbz(numffts*sizeof(float)*7, t);
#endif	
	
	// upper right quadrant
	{
	  f1_1 = vec_ld(32, f);
	  f1_2 = vec_ld(48, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex) + 32, f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex) + 48, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*2 + 32, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*2 + 48, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*3 + 32, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*3 + 48, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p13_1 = vec_mergeh(p1, p3);
	  vector float p13_2 = vec_mergel(p1, p3);
	  vector float p24_1 = vec_mergeh(p2, p4);
	  vector float p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p13_1, p24_1);
	  vector float t2 = vec_mergel(p13_1, p24_1);
	  vector float t3 = vec_mergeh(p13_2, p24_2);
	  vector float t4 = vec_mergel(p13_2, p24_2);
	  
	  // update packed max and sum
	  sum1234 = vec_add(sum1234, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind1234 = vec_sel(ind1234, vec_sel(vec_sel(vec_splat(vj2, 0), vec_splat(vj2, 1), vec_cmpgt(t2, t1)),
										 vec_sel(vec_splat(vj2, 2), vec_splat(vj2, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), vec_max(t1, t2))),
						vec_cmpgt(vec_max(vec_max(t1, t2), vec_max(t3, t4)), max1234));
	  max1234 = vec_max(max1234, vec_max(vec_max(t1, t2), vec_max(t3, t4)));
	  
	  vec_st(t1, numffts*sizeof(float)*4, t);
	  vec_st(t2, numffts*sizeof(float)*5, t);
	  vec_st(t3, numffts*sizeof(float)*6, t);
	  vec_st(t4, numffts*sizeof(float)*7, t);	  
	}
	
	// lower left quadrant
	{
	  f1_1 = vec_ld(fftlen*sizeof(sah_complex)*4, f);
	  f1_2 = vec_ld(fftlen*sizeof(sah_complex)*4 + 16, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex)*5, f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex)*5 + 16, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*6, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*6 + 16, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*7, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*7 + 16, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
#ifdef USE_PPC_G4
	  // prefetch a few blocks ahead
	  // don't use vec_dst, because our access pattern is too strange
	  __dcbt(fftlen*sizeof(sah_complex)*4+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*4+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*5+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*5+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*6+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*6+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*7+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*7+PREFETCH_BYTES+32, f);
#endif
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p57_1 = vec_mergeh(p1, p3);
	  vector float p57_2 = vec_mergel(p1, p3);
	  vector float p68_1 = vec_mergeh(p2, p4);
	  vector float p68_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p57_1, p68_1);
	  vector float t2 = vec_mergel(p57_1, p68_1);
	  vector float t3 = vec_mergeh(p57_2, p68_2);
	  vector float t4 = vec_mergel(p57_2, p68_2);
	  
	  // update packed max and sum
	  sum5678 = vec_add(sum5678, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind5678 = vec_sel(ind5678, vec_sel(vec_splat(vj1, 1),
										 vec_sel(vec_splat(vj1, 2), vec_splat(vj1, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), t2)),
						vec_cmpgt(vec_max(t2, vec_max(t3, t4)), max5678));
	  max5678 = vec_max(max5678, vec_max(t2, vec_max(t3, t4)));	  	  
	  
	  vec_st(t1, 16, t);
	  vec_st(t2, numffts*sizeof(float) + 16, t);
	  vec_st(t3, numffts*sizeof(float)*2 + 16, t);
	  vec_st(t4, numffts*sizeof(float)*3 + 16, t);
	}
	
	// lower right quadrant
	{
	  f1_1 = vec_ld(fftlen*sizeof(sah_complex)*4 + 32, f);
	  f1_2 = vec_ld(fftlen*sizeof(sah_complex)*4 + 48, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex)*5 + 32, f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex)*5 + 48, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*6 + 32, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*6 + 48, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*7 + 32, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*7 + 48, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p57_1 = vec_mergeh(p1, p3);
	  vector float p57_2 = vec_mergel(p1, p3);
	  vector float p68_1 = vec_mergeh(p2, p4);
	  vector float p68_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p57_1, p68_1);
	  vector float t2 = vec_mergel(p57_1, p68_1);
	  vector float t3 = vec_mergeh(p57_2, p68_2);
	  vector float t4 = vec_mergel(p57_2, p68_2);
	  
	  // update packed max and sum
	  sum5678 = vec_add(sum5678, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind5678 = vec_sel(ind5678, vec_sel(vec_sel(vec_splat(vj2, 0), vec_splat(vj2, 1), vec_cmpgt(t2, t1)),
										 vec_sel(vec_splat(vj2, 2), vec_splat(vj2, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), vec_max(t1, t2))),
						vec_cmpgt(vec_max(vec_max(t1, t2), vec_max(t3, t4)), max5678));
	  max5678 = vec_max(max5678, vec_max(vec_max(t1, t2), vec_max(t3, t4)));	  
	  
	  vec_st(t1, numffts*sizeof(float)*4 + 16, t);
	  vec_st(t2, numffts*sizeof(float)*5 + 16, t);
	  vec_st(t3, numffts*sizeof(float)*6 + 16, t);
	  vec_st(t4, numffts*sizeof(float)*7 + 16, t);	  
	}
	
	vj1 = vec_add(vj1, IEIGHT);
	vj2 = vec_add(vj2, IEIGHT);
  }
  
  // makes the relatively safe assumption that FFTs all have length >= 8
  for (int i = 8; i < fftlen; i += 8) {
	const float *f = (const float *) (FreqData + i);
	float *t = tPowerSpectrum + i*numffts;
	vector float f1_1, f1_2, f2_1, f2_2, f3_1, f3_2, f4_1, f4_2;
	vector float re1, im1, re2, im2, re3, im3, re4, im4;
	vector float p1, p2, p3, p4;
	
#ifdef USE_PPC_G4
	// zero the cachelines used in an 8x8 block
	__dcbz(0, t);
	__dcbz(numffts*sizeof(float), t);
	__dcbz(numffts*sizeof(float)*2, t);
	__dcbz(numffts*sizeof(float)*3, t);
#endif
	
	// upper left quadrant
	{
	  f1_1 = vec_ld(0, f);
	  f1_2 = vec_ld(16, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex), f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex) + 16, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*2, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*2 + 16, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*3, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*3 + 16, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
#ifdef USE_PPC_G4
	  // prefetch a few blocks ahead
	  // don't use vec_dst, because our access pattern is too strange
	  __dcbt(fftlen*sizeof(sah_complex)*0+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*0+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*1+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*1+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*2+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*2+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*3+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*3+PREFETCH_BYTES+32, f);
#endif
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p13_1 = vec_mergeh(p1, p3);
	  vector float p13_2 = vec_mergel(p1, p3);
	  vector float p24_1 = vec_mergeh(p2, p4);
	  vector float p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p13_1, p24_1);
	  vector float t2 = vec_mergel(p13_1, p24_1);
	  vector float t3 = vec_mergeh(p13_2, p24_2);
	  vector float t4 = vec_mergel(p13_2, p24_2);
	  
	  // update packed max and sum
	  sum1234 = vec_add(sum1234, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind1234 = vec_sel(ind1234, vec_sel(vec_sel(vec_splat(vj1, 0), vec_splat(vj1, 1), vec_cmpgt(t2, t1)),
										 vec_sel(vec_splat(vj1, 2), vec_splat(vj1, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), vec_max(t1, t2))),
						vec_cmpgt(vec_max(vec_max(t1, t2), vec_max(t3, t4)), max1234));
	  max1234 = vec_max(max1234, vec_max(vec_max(t1, t2), vec_max(t3, t4)));
	  
	  vec_st(t1, 0, t);
	  vec_st(t2, numffts*sizeof(float), t);
	  vec_st(t3, numffts*sizeof(float)*2, t);
	  vec_st(t4, numffts*sizeof(float)*3, t);
	}
	
#ifdef USE_PPC_G4
	// zero the second half of the cachelines used in an 8x8 block
	__dcbz(numffts*sizeof(float)*4, t);
	__dcbz(numffts*sizeof(float)*5, t);
	__dcbz(numffts*sizeof(float)*6, t);
	__dcbz(numffts*sizeof(float)*7, t);
#endif	
	
	// upper right quadrant
	{
	  f1_1 = vec_ld(32, f);
	  f1_2 = vec_ld(48, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex) + 32, f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex) + 48, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*2 + 32, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*2 + 48, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*3 + 32, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*3 + 48, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p13_1 = vec_mergeh(p1, p3);
	  vector float p13_2 = vec_mergel(p1, p3);
	  vector float p24_1 = vec_mergeh(p2, p4);
	  vector float p24_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p13_1, p24_1);
	  vector float t2 = vec_mergel(p13_1, p24_1);
	  vector float t3 = vec_mergeh(p13_2, p24_2);
	  vector float t4 = vec_mergel(p13_2, p24_2);
	  
	  // update packed max and sum
	  sum1234 = vec_add(sum1234, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind1234 = vec_sel(ind1234, vec_sel(vec_sel(vec_splat(vj2, 0), vec_splat(vj2, 1), vec_cmpgt(t2, t1)),
										 vec_sel(vec_splat(vj2, 2), vec_splat(vj2, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), vec_max(t1, t2))),
						vec_cmpgt(vec_max(vec_max(t1, t2), vec_max(t3, t4)), max1234));
	  max1234 = vec_max(max1234, vec_max(vec_max(t1, t2), vec_max(t3, t4)));

	  vec_st(t1, numffts*sizeof(float)*4, t);
	  vec_st(t2, numffts*sizeof(float)*5, t);
	  vec_st(t3, numffts*sizeof(float)*6, t);
	  vec_st(t4, numffts*sizeof(float)*7, t);	  
	}
	
	// lower left quadrant
	{
	  f1_1 = vec_ld(fftlen*sizeof(sah_complex)*4, f);
	  f1_2 = vec_ld(fftlen*sizeof(sah_complex)*4 + 16, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex)*5, f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex)*5 + 16, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*6, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*6 + 16, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*7, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*7 + 16, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
#ifdef USE_PPC_G4
	  // prefetch a few blocks ahead
	  // don't use vec_dst, because our access pattern is too strange
	  __dcbt(fftlen*sizeof(sah_complex)*4+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*4+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*5+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*5+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*6+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*6+PREFETCH_BYTES+32, f);
	  __dcbt(fftlen*sizeof(sah_complex)*7+PREFETCH_BYTES, f);
	  __dcbt(fftlen*sizeof(sah_complex)*7+PREFETCH_BYTES+32, f);
#endif
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p57_1 = vec_mergeh(p1, p3);
	  vector float p57_2 = vec_mergel(p1, p3);
	  vector float p68_1 = vec_mergeh(p2, p4);
	  vector float p68_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p57_1, p68_1);
	  vector float t2 = vec_mergel(p57_1, p68_1);
	  vector float t3 = vec_mergeh(p57_2, p68_2);
	  vector float t4 = vec_mergel(p57_2, p68_2);
	  
	  // update packed max and sum
	  sum5678 = vec_add(sum5678, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind5678 = vec_sel(ind5678, vec_sel(vec_sel(vec_splat(vj1, 0), vec_splat(vj1, 1), vec_cmpgt(t2, t1)),
										 vec_sel(vec_splat(vj1, 2), vec_splat(vj1, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), vec_max(t1, t2))),
						vec_cmpgt(vec_max(vec_max(t1, t2), vec_max(t3, t4)), max5678));
	  max5678 = vec_max(max5678, vec_max(vec_max(t1, t2), vec_max(t3, t4)));	  	  
	  
	  vec_st(t1, 16, t);
	  vec_st(t2, numffts*sizeof(float) + 16, t);
	  vec_st(t3, numffts*sizeof(float)*2 + 16, t);
	  vec_st(t4, numffts*sizeof(float)*3 + 16, t);
	}
	
	// lower right quadrant
	{
	  f1_1 = vec_ld(fftlen*sizeof(sah_complex)*4 + 32, f);
	  f1_2 = vec_ld(fftlen*sizeof(sah_complex)*4 + 48, f);
	  f2_1 = vec_ld(fftlen*sizeof(sah_complex)*5 + 32, f);
	  f2_2 = vec_ld(fftlen*sizeof(sah_complex)*5 + 48, f);
	  f3_1 = vec_ld(fftlen*sizeof(sah_complex)*6 + 32, f);
	  f3_2 = vec_ld(fftlen*sizeof(sah_complex)*6 + 48, f);
	  f4_1 = vec_ld(fftlen*sizeof(sah_complex)*7 + 32, f);
	  f4_2 = vec_ld(fftlen*sizeof(sah_complex)*7 + 48, f);
	  
	  re1 = vec_perm(f1_1, f1_2, real);
	  im1 = vec_perm(f1_1, f1_2, imag);
	  re2 = vec_perm(f2_1, f2_2, real);
	  im2 = vec_perm(f2_1, f2_2, imag);
	  re3 = vec_perm(f3_1, f3_2, real);
	  im3 = vec_perm(f3_1, f3_2, imag);
	  re4 = vec_perm(f4_1, f4_2, real);
	  im4 = vec_perm(f4_1, f4_2, imag);
	  
	  p1 = vec_madd(re1, re1, vec_madd(im1, im1, ZERO));
	  p2 = vec_madd(re2, re2, vec_madd(im2, im2, ZERO));
	  p3 = vec_madd(re3, re3, vec_madd(im3, im3, ZERO));
	  p4 = vec_madd(re4, re4, vec_madd(im4, im4, ZERO));
	  
	  // transpose step 1
	  vector float p57_1 = vec_mergeh(p1, p3);
	  vector float p57_2 = vec_mergel(p1, p3);
	  vector float p68_1 = vec_mergeh(p2, p4);
	  vector float p68_2 = vec_mergel(p2, p4);
	  
	  // transpose step 2
	  vector float t1 = vec_mergeh(p57_1, p68_1);
	  vector float t2 = vec_mergel(p57_1, p68_1);
	  vector float t3 = vec_mergeh(p57_2, p68_2);
	  vector float t4 = vec_mergel(p57_2, p68_2);
	  
	  // update packed max and sum
	  sum5678 = vec_add(sum5678, vec_add(vec_add(t1, t2), vec_add(t3, t4)));
	  ind5678 = vec_sel(ind5678, vec_sel(vec_sel(vec_splat(vj2, 0), vec_splat(vj2, 1), vec_cmpgt(t2, t1)),
										 vec_sel(vec_splat(vj2, 2), vec_splat(vj2, 3), vec_cmpgt(t4, t3)),
										 vec_cmpgt(vec_max(t3, t4), vec_max(t1, t2))),
						vec_cmpgt(vec_max(vec_max(t1, t2), vec_max(t3, t4)), max5678));
	  max5678 = vec_max(max5678, vec_max(vec_max(t1, t2), vec_max(t3, t4)));	  
	  
	  vec_st(t1, numffts*sizeof(float)*4 + 16, t);
	  vec_st(t2, numffts*sizeof(float)*5 + 16, t);
	  vec_st(t3, numffts*sizeof(float)*6 + 16, t);
	  vec_st(t4, numffts*sizeof(float)*7 + 16, t);	  
	}
	
	vj1 = vec_add(vj1, IEIGHT);
	vj2 = vec_add(vj2, IEIGHT);
  }
  
  pMean[0] = vec_madd(meanScale, sum1234, ZERO);
  pMean[1] = vec_madd(meanScale, sum5678, ZERO);
  pMaxNoDC[0] = max1234;
  pMaxNoDC[1] = max5678;
  pMaxInd[0] = ind1234;
  pMaxInd[1] = ind5678;
}

#elif defined(__SSE3__)
void v_vGetPowerSpectrum(
  sah_complex* FreqData,
  float* PowerSpectrum,
  int NumDataPoints
) {
  int i, vEnd;
  analysis_state.FLOP_counter+=3.0*NumDataPoints;
  
  vEnd = NumDataPoints - (NumDataPoints & 3);
  for (i = 0; i < vEnd; i += 4) {
    const float *f = (const float *) (FreqData + i);
    float *p = PowerSpectrum + i;
    vFloat f1, f2;
    vFloat p1;
    
    f1 = _mm_load_ps(f);
    f2 = _mm_load_ps(f+4);
    p1 = _mm_hadd_ps(_mm_mul_ps(f1, f1), _mm_mul_ps(f2, f2));
    _mm_store_ps(p, p1);
  }
  
  // handle tail elements, although in practice this never happens
  for (; i < NumDataPoints; i++) {
    PowerSpectrum[i] = FreqData[i][0] * FreqData[i][0]
    + FreqData[i][1] * FreqData[i][1];
  }  
}

void v_vGetPowerSpectrumMeanMax(
#ifdef USE_SPLIT_COMPLEX
  DSPSplitComplex *FreqData,
#else
  sah_complex* FreqData,
#endif
  float* PowerSpectrum,
  int NumDataPoints,
  float* pMaxNoDC,
  unsigned int* pMaxInd,
  float* pMean
) {
  int j, vEnd;
  vFloat max;
  vFloat sum;
  vUInt32 ind;
  vUInt32 vj;
  vFloat cmp;
  analysis_state.FLOP_counter+=3.0*NumDataPoints;
  
  vEnd = NumDataPoints - (NumDataPoints & 3);
  
#ifdef USE_SPLIT_COMPLEX
  const float *FreqDataReal = (const float *) (FreqData->realp);
  const float *FreqDataImag = (const float *) (FreqData->imagp);
#endif
  
  // explicitly handle first four elements to exclude DC term
  {
    vFloat p;
#ifdef USE_SPLIT_COMPLEX
    const float *pr = FreqDataReal;
    const float *pi = FreqDataImag;
    vFloat r, i;
    
    r = _mm_load_ps(pr);
    i = _mm_load_ps(pi);
    p = _mm_add_ps(_mm_mul_ps(r, r), _mm_mul_ps(i, i));
#else
    const float *f = (const float *) FreqData;
    vFloat f1, f2;
    
    f1 = _mm_load_ps(f);
    f2 = _mm_load_ps(f+4);
    p = _mm_hadd_ps(_mm_mul_ps(f1, f1), _mm_mul_ps(f2, f2));
#endif
    
    vj = _mm_set_epi32(3, 2, 1, 0);
    ind = vj;
    max = _mm_and_ps(p, (vFloat) _mm_set_epi32(-1, -1, -1, 0));
    sum = p;
    
	vj = _mm_add_epi32(vj, _mm_set_epi32(4,4,4,4) );
  }
  
  for (j = 4; j < vEnd; j += 4) {
    vFloat p;
#ifdef USE_SPLIT_COMPLEX
    const float *pr = FreqDataReal + j;
    const float *pi = FreqDataImag + j;
    vFloat r, i;
    
    r = _mm_load_ps(pr);
    i = _mm_load_ps(pi);
    p = _mm_add_ps(_mm_mul_ps(r, r), _mm_mul_ps(i, i));
#else
    const float *f = (const float *) (FreqData + j);
    vFloat f1, f2;
    
    f1 = _mm_load_ps(f);
    f2 = _mm_load_ps(f+4);
    p = _mm_hadd_ps(_mm_mul_ps(f1, f1), _mm_mul_ps(f2, f2));
#endif

    cmp = _mm_cmpgt_ps(p, max);
    max = _mm_max_ps(p, max);
    ind = _mm_or_si128(_mm_and_si128(vj, (vUInt32) cmp),
                       _mm_andnot_si128((vUInt32) cmp, ind));
    sum = _mm_add_ps(sum, p);
    
    vj = _mm_add_epi32(vj, _mm_set_epi32(4,4,4,4));
  }

  // reduce
  cmp = _mm_or_ps(_mm_cmpgt_ps(max, _mm_movehl_ps(max, max)),
                  _mm_and_ps(_mm_cmpeq_ps(max, _mm_movehl_ps(max, max)),
                             (vFloat) _mm_cmplt_epi32(ind, (vUInt32) _mm_movehl_ps((vFloat) ind, (vFloat) ind))));
  max = _mm_or_ps(_mm_and_ps(max, cmp),
                  _mm_andnot_ps(cmp, _mm_movehl_ps(max, max)));
  ind = _mm_or_si128(_mm_and_si128(ind, (vUInt32) cmp),
                     _mm_andnot_si128((vUInt32) cmp, (vUInt32) _mm_movehl_ps((vFloat) ind, (vFloat) ind)));

  cmp = _mm_or_ps(_mm_cmpgt_ps(max, _mm_movehdup_ps(max)),
                  _mm_and_ps(_mm_cmpeq_ps(max, _mm_movehdup_ps(max)),
                             (vFloat) _mm_cmplt_epi32(ind, (vUInt32) _mm_movehdup_ps((vFloat) ind))));
  max = _mm_or_ps(_mm_and_ps(max, cmp),
                  _mm_andnot_ps(cmp, _mm_movehdup_ps(max)));
  ind = _mm_or_si128(_mm_and_si128(ind, (vUInt32) cmp),
                     _mm_andnot_si128((vUInt32) cmp, (vUInt32) _mm_movehdup_ps((vFloat) ind)));
  _mm_store_ss(pMaxNoDC, max);
//  _mm_store_ss((float *) pMaxInd, (vFloat) ind); // XXX: is there a less hacky way to do this?
  *pMaxInd = _mm_cvtsi128_si32(ind);

  sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
  sum = _mm_add_ps(sum, _mm_movehdup_ps(sum));
  sum = _mm_mul_ss(sum, _mm_set1_ps(1.0f / NumDataPoints));
  _mm_store_ss(pMean, sum);
}

void v_vGetPowerSpectrumMeanMaxWrite(
                                sah_complex* FreqData,
                                float* PowerSpectrum,
                                int NumDataPoints,
                                float* pMaxNoDC,
                                unsigned int* pMaxInd,
                                float* pMean
                                ) {
  int j, vEnd;
  vFloat max;
  vFloat sum;
  vUInt32 ind;
  vUInt32 vj;
  vFloat cmp;
  analysis_state.FLOP_counter+=3.0*NumDataPoints;
  
  vEnd = NumDataPoints - (NumDataPoints & 3);
  
  // explicitly handle first four elements to exclude DC term
  {
    vFloat p;
    const float *f = (const float *) FreqData;
    float *pp = (float *) FreqData;
    vFloat f1, f2;
    
    f1 = _mm_load_ps(f);
    f2 = _mm_load_ps(f+4);
    p = _mm_hadd_ps(_mm_mul_ps(f1, f1), _mm_mul_ps(f2, f2));
    
    vj = _mm_set_epi32(3, 2, 1, 0);
    ind = vj;
    max = _mm_and_ps(p, (vFloat) _mm_set_epi32(-1, -1, -1, 0));
    sum = p;
    
    _mm_store_ps(pp, p);
    
    vj = _mm_add_epi32(vj, _mm_set_epi32(4,4,4,4));
  }
  
  for (j = 4; j < vEnd; j += 4) {
    vFloat p;
    const float *f = (const float *) (FreqData + j);
    float *pp = ((float *) FreqData) + j;
    vFloat f1, f2;
    
    f1 = _mm_load_ps(f);
    f2 = _mm_load_ps(f+4);
    p = _mm_hadd_ps(_mm_mul_ps(f1, f1), _mm_mul_ps(f2, f2));
    
    cmp = _mm_cmpgt_ps(p, max);
    max = _mm_max_ps(p, max);
    ind = _mm_or_si128(_mm_and_si128(vj, (vUInt32) cmp),
                       _mm_andnot_si128((vUInt32) cmp, ind));
    sum = _mm_add_ps(sum, p);
    
    _mm_store_ps(pp, p);
    
    vj = _mm_add_epi32(vj, _mm_set_epi32(4,4,4,4));
  }
  
  // reduce
  cmp = _mm_or_ps(_mm_cmpgt_ps(max, _mm_movehl_ps(max, max)),
                  _mm_and_ps(_mm_cmpeq_ps(max, _mm_movehl_ps(max, max)),
                             (vFloat) _mm_cmplt_epi32(ind, (vUInt32) _mm_movehl_ps((vFloat) ind, (vFloat) ind))));
  max = _mm_or_ps(_mm_and_ps(max, cmp),
                  _mm_andnot_ps(cmp, _mm_movehl_ps(max, max)));
  ind = _mm_or_si128(_mm_and_si128(ind, (vUInt32) cmp),
                     _mm_andnot_si128((vUInt32) cmp, (vUInt32) _mm_movehl_ps((vFloat) ind, (vFloat) ind)));
  
  cmp = _mm_or_ps(_mm_cmpgt_ps(max, _mm_movehdup_ps(max)),
                  _mm_and_ps(_mm_cmpeq_ps(max, _mm_movehdup_ps(max)),
                             (vFloat) _mm_cmplt_epi32(ind, (vUInt32) _mm_movehdup_ps((vFloat) ind))));
  max = _mm_or_ps(_mm_and_ps(max, cmp),
                  _mm_andnot_ps(cmp, _mm_movehdup_ps(max)));
  ind = _mm_or_si128(_mm_and_si128(ind, (vUInt32) cmp),
                     _mm_andnot_si128((vUInt32) cmp, (vUInt32) _mm_movehdup_ps((vFloat) ind)));
  _mm_store_ss(pMaxNoDC, max);
  //  _mm_store_ss((float *) pMaxInd, (vFloat) ind); // XXX: is there a less hacky way to do this?
  *pMaxInd = _mm_cvtsi128_si32(ind);
  
  sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
  sum = _mm_add_ps(sum, _mm_movehdup_ps(sum));
  sum = _mm_mul_ss(sum, _mm_set1_ps(1.0f / NumDataPoints));
  _mm_store_ss(pMean, sum);
}

inline vSInt32 _mm_sel_si128(vSInt32 a, vSInt32 b, vSInt32 mask) {
  b = _mm_and_si128(b, mask);
  a = _mm_andnot_si128(mask, a);
  return _mm_or_si128(a, b);
}

void v_vTransposePowerMeanMax4(
#ifdef USE_SPLIT_COMPLEX
  DSPSplitComplex *FreqData,
#else
  sah_complex* FreqData,
#endif
  float* tPowerSpectrum,
  int fftlen,
  int numffts,
  float *pMaxNoDC,
  unsigned int *pMaxInd,
  float *pMean
) {
  vUInt32 ind1234;
  vUInt32 vj = _mm_setzero_si128();
  vFloat max1234;
  vFloat sum1234;
  vFloat meanScale = _mm_set1_ps(1.0f / fftlen);
  analysis_state.FLOP_counter+=12.0*fftlen;
  
#ifdef USE_SPLIT_COMPLEX
  const float *FreqDataReal = (const float *) (FreqData->realp);
  const float *FreqDataImag = (const float *) (FreqData->imagp);
#endif

  // explicitly handle first four elements to exclude DC term
  {
    vFloat p1, p2, p3, p4;
#ifdef USE_SPLIT_COMPLEX
    const float *r = FreqDataReal;
    const float *i = FreqDataImag;
    float *t = tPowerSpectrum;
    vFloat r1, r2, r3, r4, i1, i2, i3, i4;
    
    r1 = _mm_load_ps(r);
    i1 = _mm_load_ps(i);
    r2 = _mm_load_ps(r+1*fftlen);
    i2 = _mm_load_ps(i+1*fftlen);
    r3 = _mm_load_ps(r+2*fftlen);
    i3 = _mm_load_ps(i+2*fftlen);
    r4 = _mm_load_ps(r+3*fftlen);
    i4 = _mm_load_ps(i+3*fftlen);
    
    p1 = _mm_add_ps(_mm_mul_ps(r1, r1), _mm_mul_ps(i1, i1));
    p2 = _mm_add_ps(_mm_mul_ps(r2, r2), _mm_mul_ps(i2, i2));
    p3 = _mm_add_ps(_mm_mul_ps(r3, r3), _mm_mul_ps(i3, i3));
    p4 = _mm_add_ps(_mm_mul_ps(r4, r4), _mm_mul_ps(i4, i4));
#else
    const float *f = (const float *) FreqData;
    float *t = tPowerSpectrum;
    vFloat f1_1, f1_2, f2_1, f2_2, f3_1, f3_2, f4_1, f4_2;
    
    f1_1 = _mm_load_ps(f);
    f1_2 = _mm_load_ps(f+4);
    f2_1 = _mm_load_ps(f+2*fftlen);
    f2_2 = _mm_load_ps(f+2*fftlen+4);
    f3_1 = _mm_load_ps(f+4*fftlen);
    f3_2 = _mm_load_ps(f+4*fftlen+4);
    f4_1 = _mm_load_ps(f+6*fftlen);
    f4_2 = _mm_load_ps(f+6*fftlen+4);
    
    p1 = _mm_hadd_ps(_mm_mul_ps(f1_1, f1_1), _mm_mul_ps(f1_2, f1_2));
    p2 = _mm_hadd_ps(_mm_mul_ps(f2_1, f2_1), _mm_mul_ps(f2_2, f2_2));
    p3 = _mm_hadd_ps(_mm_mul_ps(f3_1, f3_1), _mm_mul_ps(f3_2, f3_2));
    p4 = _mm_hadd_ps(_mm_mul_ps(f4_1, f4_1), _mm_mul_ps(f4_2, f4_2));
#endif
    
    // transpose step 1
    vFloat p13_1 = _mm_unpackhi_ps(p1, p3);
    vFloat p13_2 = _mm_unpacklo_ps(p1, p3);
    vFloat p24_1 = _mm_unpackhi_ps(p2, p4);
    vFloat p24_2 = _mm_unpacklo_ps(p2, p4);
    
    // transpose step 2 - little-endian, grr
    vFloat t4 = _mm_unpackhi_ps(p13_1, p24_1);
    vFloat t3 = _mm_unpacklo_ps(p13_1, p24_1);
    vFloat t2 = _mm_unpackhi_ps(p13_2, p24_2);
    vFloat t1 = _mm_unpacklo_ps(p13_2, p24_2);
    
    // update packed max and sum
    sum1234 = _mm_add_ps(_mm_add_ps(t1, t2), _mm_add_ps(t3, t4));
    ind1234 = _mm_sel_si128(_mm_add_epi32(vj, IONE),
                            _mm_sel_si128(_mm_add_epi32(vj, ITWO), _mm_add_epi32(vj, ITHREE), (vUInt32) _mm_cmpgt_ps(t4, t3)),
                            (vUInt32) _mm_cmpgt_ps(_mm_max_ps(t3, t4), t2));
    max1234 = _mm_max_ps(t2, _mm_max_ps(t3, t4));
    
    _mm_store_ps(t, t1);
    _mm_store_ps(t+numffts, t2);
    _mm_store_ps(t+2*numffts, t3);
    _mm_store_ps(t+3*numffts, t4);
//    _mm_stream_ps(t, t1);
//    _mm_stream_ps(t+numffts, t2);
//    _mm_stream_ps(t+2*numffts, t3);
//    _mm_stream_ps(t+3*numffts, t4);
    
    vj = _mm_add_epi32(vj, IFOUR);
  }

  for (int j = 4; j < fftlen; j += 4) {
    vFloat p1, p2, p3, p4;
#ifdef USE_SPLIT_COMPLEX
    const float *r = FreqDataReal + j;
    const float *i = FreqDataImag + j;
    float *t = tPowerSpectrum + j*numffts;
    vFloat r1, r2, r3, r4, i1, i2, i3, i4;
    
    r1 = _mm_load_ps(r);
    i1 = _mm_load_ps(i);
    r2 = _mm_load_ps(r+1*fftlen);
    i2 = _mm_load_ps(i+1*fftlen);
    r3 = _mm_load_ps(r+2*fftlen);
    i3 = _mm_load_ps(i+2*fftlen);
    r4 = _mm_load_ps(r+3*fftlen);
    i4 = _mm_load_ps(i+3*fftlen);
    
    p1 = _mm_add_ps(_mm_mul_ps(r1, r1), _mm_mul_ps(i1, i1));
    p2 = _mm_add_ps(_mm_mul_ps(r2, r2), _mm_mul_ps(i2, i2));
    p3 = _mm_add_ps(_mm_mul_ps(r3, r3), _mm_mul_ps(i3, i3));
    p4 = _mm_add_ps(_mm_mul_ps(r4, r4), _mm_mul_ps(i4, i4));
#else
    const float *f = (const float *) (FreqData + j);
    float *t = tPowerSpectrum + j*numffts;
    vFloat f1_1, f1_2, f2_1, f2_2, f3_1, f3_2, f4_1, f4_2;
    
    f1_1 = _mm_load_ps(f);
    f1_2 = _mm_load_ps(f+4);
    f2_1 = _mm_load_ps(f+2*fftlen);
    f2_2 = _mm_load_ps(f+2*fftlen+4);
    f3_1 = _mm_load_ps(f+4*fftlen);
    f3_2 = _mm_load_ps(f+4*fftlen+4);
    f4_1 = _mm_load_ps(f+6*fftlen);
    f4_2 = _mm_load_ps(f+6*fftlen+4);
    
    p1 = _mm_hadd_ps(_mm_mul_ps(f1_1, f1_1), _mm_mul_ps(f1_2, f1_2));
    p2 = _mm_hadd_ps(_mm_mul_ps(f2_1, f2_1), _mm_mul_ps(f2_2, f2_2));
    p3 = _mm_hadd_ps(_mm_mul_ps(f3_1, f3_1), _mm_mul_ps(f3_2, f3_2));
    p4 = _mm_hadd_ps(_mm_mul_ps(f4_1, f4_1), _mm_mul_ps(f4_2, f4_2));
#endif
    
    // transpose step 1
    vFloat p13_1 = _mm_unpackhi_ps(p1, p3);
    vFloat p13_2 = _mm_unpacklo_ps(p1, p3);
    vFloat p24_1 = _mm_unpackhi_ps(p2, p4);
    vFloat p24_2 = _mm_unpacklo_ps(p2, p4);
    
    // transpose step 2 - grr, little-endian
    vFloat t4 = _mm_unpackhi_ps(p13_1, p24_1);
    vFloat t3 = _mm_unpacklo_ps(p13_1, p24_1);
    vFloat t2 = _mm_unpackhi_ps(p13_2, p24_2);
    vFloat t1 = _mm_unpacklo_ps(p13_2, p24_2);
    
    // update packed max and sum
    sum1234 = _mm_add_ps(sum1234, _mm_add_ps(_mm_add_ps(t1, t2), _mm_add_ps(t3, t4)));
    ind1234 = _mm_sel_si128(ind1234,
                            _mm_sel_si128(_mm_sel_si128(vj, _mm_add_epi32(vj, IONE), (vUInt32) _mm_cmpgt_ps(t2, t1)),
                                          _mm_sel_si128(_mm_add_epi32(vj, ITWO), _mm_add_epi32(vj, ITHREE), (vUInt32) _mm_cmpgt_ps(t4, t3)),
                                          (vUInt32) _mm_cmpgt_ps(_mm_max_ps(t3, t4), _mm_max_ps(t1, t2))),
                            (vUInt32) _mm_cmpgt_ps(_mm_max_ps(_mm_max_ps(t1, t2), _mm_max_ps(t3, t4)), max1234));
    max1234 = _mm_max_ps(max1234, _mm_max_ps(_mm_max_ps(t1, t2), _mm_max_ps(t3, t4)));
    
    _mm_store_ps(t, t1);
    _mm_store_ps(t+numffts, t2);
    _mm_store_ps(t+2*numffts, t3);
    _mm_store_ps(t+3*numffts, t4);
//    _mm_stream_ps(t, t1);
//    _mm_stream_ps(t+numffts, t2);
//    _mm_stream_ps(t+2*numffts, t3);
//    _mm_stream_ps(t+3*numffts, t4);
    
    vj = _mm_add_epi32(vj, IFOUR);
  }
  
  _mm_store_ps(pMean, _mm_mul_ps(meanScale, sum1234));
  _mm_store_ps(pMaxNoDC, max1234);
  _mm_store_si128((__m128i *) pMaxInd, ind1234);
}
#endif


// chirp_rate is in Hz per second

#ifdef __VEC__
inline vector float vec_recip(vector float v) {
  // obtain estimate
  vector float estimate = vec_re( v );
  // one round of Newton-Raphson
  return vec_madd( vec_nmsub( estimate, v, (vector float) (1.0) ), estimate, estimate );
}  


// quickRound
//
// Quickly round a value to the nearest integer. This is done by adding a large
// value (which depends on the format of an IEEE double precision variable) and
// then subtracting it again to extract the fractional part of the variable.
//
// If x is positive, roundVal must be TWO_TO_52.
// If x is negative, roundVal must be -TWO_TO_52.
//
// NOTE: This function does not do any fancy IEEE compliant arithmetic (like
// handling NaN etc. or preserving the sign of +/-0).
//
// NOTE: Make sure that you don't use any compiler flags that will cancel
// out the operations.
inline double
quickRound(double x, double roundVal)
{
  return (x + roundVal) - roundVal;
}


// chirp
//
// Increase the frequency of the input signal over time.
//
// Note that the input angle is calculated in double-precision. The angle
// becomes so large that single-precision arithmetic is insufficient for the
// accuracy that we need. We can only convert to single-precision after
// reducing the range to (-0.5, 0.5).
//
// The algorithm used to generate the sine / cosine values is based on that
// described in "A Fast, Vectorizable Algorithm for Producing Single-Precision
// Sine-Cosine Pairs" by Marcus H. Mendenhall. The paper is available at:
//     http://arxiv.org/pdf/cs.MS/0406049
//
// Differences from the implementation in the paper:
// 1. The arguments are not scaled by 1/(2 pi) because the original chirp
//    algorithm multiplies the chirp rate by (2 pi).
// 2. The slower version of the algorithm has been used (as if FASTER_SINCOS
//    had not been defined for the code in the paper).
// 3. The loop has been unrolled once so the variables have been renamed to make
//    things more readable.
// 4. The "struct phase" data type is not used.
// 5. The rounding of the input value to the range (-0.5, 0.5) is done in
//    double-precision as mentioned above, and not via Altivec.
int v_vChirpData(
  sah_complex* cx_DataArray,
  sah_complex* cx_ChirpDataArray,
  int chirp_rate_ind,
  double chirp_rate,
  int  offset,
  int  ul_NumDataPoints,
  double sample_rate
) {
// int offset, int numPoints, double chirpRate
  int i;
  
  if (chirp_rate_ind == 0) {
    memcpy(cx_ChirpDataArray+offset,
           cx_DataArray+offset,
           (int)ul_NumDataPoints * sizeof(sah_complex)
          );
  } else {
    int vEnd;  
    const vector unsigned char real = (vector unsigned char) (0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
	const vector unsigned char imag = (vector unsigned char) (4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
    double rate = chirp_rate * 0.5 / (sample_rate * sample_rate);
    double roundVal = rate >= 0.0 ? TWO_TO_52 : -TWO_TO_52;
    
    // main vectorised loop
    vEnd = (offset+ul_NumDataPoints) - ((offset+ul_NumDataPoints) & 7);
    for (i = offset; i < vEnd; i += 8) {
      const float *d = (const float *) (cx_DataArray + i);
      float *c = (float *) (cx_ChirpDataArray + i);
      double a1 = i;
      double a2 = i + 1.0;
      double a3 = i + 2.0;
      double a4 = i + 3.0;
      double a5 = i + 4.0;
      double a6 = i + 5.0;
      double a7 = i + 6.0;
      double a8 = i + 7.0;
      float v[8] __attribute__ ((aligned (16)));
      vector float d1, d2, d3, d4;
      vector float cd1, cd2, cd3, cd4;
      vector float dre1, dre2;
      vector float dim1, dim2;
      vector float cre1, cre2;
      vector float cim1, cim2;
      vector float x1, x2;
      vector float y1, y2;
      vector float s1, s2;
      vector float c1, c2;
      vector float m1, m2;
      
      // load the signal to be chirped
      d1 = vec_ld(0, d);
      d2 = vec_ld(16, d);
      d3 = vec_ld(32, d);
      d4 = vec_ld(48, d);
      
      // calculate the input angle
      a1 *= a1 * rate; a2 *= a2 * rate;
      a3 *= a3 * rate; a4 *= a4 * rate;
      a5 *= a5 * rate; a6 *= a6 * rate;
      a7 *= a7 * rate; a8 *= a8 * rate;
      
      // reduce the angle to the range (-0.5, 0.5) and store to
      // memory so we can load it into the vector unit
      v[0] = a1 - quickRound(a1, roundVal);
      v[1] = a2 - quickRound(a2, roundVal);
      v[2] = a3 - quickRound(a3, roundVal);
      v[3] = a4 - quickRound(a4, roundVal);
      v[4] = a5 - quickRound(a5, roundVal);
      v[5] = a6 - quickRound(a6, roundVal);
      v[6] = a7 - quickRound(a7, roundVal);
      v[7] = a8 - quickRound(a8, roundVal);
      
      // load angle into the vector unit
      x1 = vec_ld(0, v);
      x2 = vec_ld(16, v);
      
      // square to the range [0, 0.25)
      y1 = vec_madd(x1, x1, ZERO);
      y2 = vec_madd(x2, x2, ZERO);
      
      // perform the initial polynomial approximations
      s1 = vec_madd(x1,
                    vec_madd(y1,
                             vec_madd(y1,
                                      vec_madd(y1, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      s2 = vec_madd(x2,
                    vec_madd(y2,
                             vec_madd(y2,
                                      vec_madd(y2, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      c1 = vec_madd(y1,
                    vec_madd(y1, vec_madd(y1, CC3, CC2), CC1),
                    ONE);
      c2 = vec_madd(y2,
                    vec_madd(y2, vec_madd(y2, CC3, CC2), CC1),
                    ONE);
      
      // permute the loaded data into separate real and complex vectors
      dre1 = vec_perm(d1, d2, real);
      dim1 = vec_perm(d1, d2, imag);
      dre2 = vec_perm(d3, d4, real);
      dim2 = vec_perm(d3, d4, imag);
      
      // perform first angle doubling
      x1 = vec_nmsub(s1, s1, vec_madd(c1, c1, ZERO));
      x2 = vec_nmsub(s2, s2, vec_madd(c2, c2, ZERO));
      y1 = vec_madd(TWO, vec_madd(s1, c1, ZERO), ZERO);
      y2 = vec_madd(TWO, vec_madd(s2, c2, ZERO), ZERO);
      
      // calculate scaling factor to correct the magnitude
//      m1 = vec_nmsub(y1, y1, vec_nmsub(x1, x1, TWO));
//      m2 = vec_nmsub(y2, y2, vec_nmsub(x2, x2, TWO));
      m1 = vec_recip(vec_madd(y1, y1, vec_madd(x1, x1, ZERO)));
      m2 = vec_recip(vec_madd(y2, y2, vec_madd(x2, x2, ZERO)));
      
      // perform second angle doubling
      c1 = vec_nmsub(y1, y1, vec_madd(x1, x1, ZERO));
      c2 = vec_nmsub(y2, y2, vec_madd(x2, x2, ZERO));
      s1 = vec_madd(TWO, vec_madd(y1, x1, ZERO), ZERO);
      s2 = vec_madd(TWO, vec_madd(y2, x2, ZERO), ZERO);
      
      // correct the magnitude (final sine / cosine approximations)
      s1 = vec_madd(s1, m1, ZERO);
      s2 = vec_madd(s2, m2, ZERO);
      c1 = vec_madd(c1, m1, ZERO);
      c2 = vec_madd(c2, m2, ZERO);
      
      // chirp the data
      cre1 = vec_nmsub(dim1, s1, vec_madd(dre1, c1, ZERO));
      cre2 = vec_nmsub(dim2, s2, vec_madd(dre2, c2, ZERO));
      cim1 = vec_madd(dim1, c1, vec_madd(dre1, s1, ZERO));
      cim2 = vec_madd(dim2, c2, vec_madd(dre2, s2, ZERO));
      
      // permute into interleaved form
      cd1 = vec_mergeh(cre1,cim1);
      cd2 = vec_mergel(cre1,cim1);
      cd3 = vec_mergeh(cre2,cim2);
      cd4 = vec_mergel(cre2,cim2);
      
      // store chirped values
      vec_st(cd1, 0, c);
      vec_st(cd2, 16, c);
      vec_st(cd3, 32, c);
      vec_st(cd4, 48, c);
	}
    
    // handle tail elements with scalar code
    for (; i < offset+ul_NumDataPoints; ++i) {
      double angle = rate * i * i * 0.5;
      double s = sin(angle);
      double c = cos(angle);
      float re = cx_DataArray[i][0];
      float im = cx_DataArray[i][1];
      
      cx_ChirpDataArray[i][0] = re * c - im * s;
      cx_ChirpDataArray[i][1] = re * s + im * c;
    }
    analysis_state.FLOP_counter+=12.0*ul_NumDataPoints;
  }
  return 0;
}


int v_vChirpDataG4(
                 sah_complex* cx_DataArray,
                 sah_complex* cx_ChirpDataArray,
                 int chirp_rate_ind,
                 double chirp_rate,
                 int  offset,
                 int  ul_NumDataPoints,
                 double sample_rate
                 ) {
  // int offset, int numPoints, double chirpRate
  int i;
  double fi;
  
  if (chirp_rate_ind == 0) {
    memcpy(cx_ChirpDataArray+offset,
           cx_DataArray+offset,
           (int)ul_NumDataPoints * sizeof(sah_complex)
           );
  } else {
    int vEnd;  
    const vector unsigned char real = (vector unsigned char) (0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
	const vector unsigned char imag = (vector unsigned char) (4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
    double rate = chirp_rate * 0.5 / (sample_rate * sample_rate);
    double roundVal = rate >= 0.0 ? TWO_TO_52 : -TWO_TO_52;
    
    // main vectorised loop
    vEnd = (offset+ul_NumDataPoints) - ((offset+ul_NumDataPoints) & 7);
    for (i = offset, fi = offset; i < vEnd; i += 8, fi += 8.0) {
      const float *d = (const float *) (cx_DataArray + i);
      float *c = (float *) (cx_ChirpDataArray + i);
      double a1 = fi;
      double a2 = fi + 1.0;
      double a3 = fi + 2.0;
      double a4 = fi + 3.0;
      double a5 = fi + 4.0;
      double a6 = fi + 5.0;
      double a7 = fi + 6.0;
      double a8 = fi + 7.0;
      float v[8] __attribute__ ((aligned (16)));
      vector float d1, d2, d3, d4;
      vector float cd1, cd2, cd3, cd4;
      vector float dre1, dre2;
      vector float dim1, dim2;
      vector float cre1, cre2;
      vector float cim1, cim2;
      vector float x1, x2;
      vector float y1, y2;
      vector float s1, s2;
      vector float c1, c2;
      vector float m1, m2;      
      
      // prefetch next 256 bytes into L1 cache
      vec_dstt(d, 0x10010100, 0);
      
      // load the signal to be chirped
      d1 = vec_ldl(0, d);
      d2 = vec_ldl(16, d);
      d3 = vec_ldl(32, d);
      d4 = vec_ldl(48, d);
      
      // calculate the input angle
      a1 *= a1 * rate; a2 *= a2 * rate;
      a3 *= a3 * rate; a4 *= a4 * rate;
      a5 *= a5 * rate; a6 *= a6 * rate;
      a7 *= a7 * rate; a8 *= a8 * rate;
      
      // reduce the angle to the range (-0.5, 0.5) and store to
      // memory so we can load it into the vector unit
      v[0] = a1 - quickRound(a1, roundVal);
      v[1] = a2 - quickRound(a2, roundVal);
      v[2] = a3 - quickRound(a3, roundVal);
      v[3] = a4 - quickRound(a4, roundVal);
      v[4] = a5 - quickRound(a5, roundVal);
      v[5] = a6 - quickRound(a6, roundVal);
      v[6] = a7 - quickRound(a7, roundVal);
      v[7] = a8 - quickRound(a8, roundVal);
      
      // load angle into the vector unit
      x1 = vec_ld(0, v);
      x2 = vec_ld(16, v);
      
      // square to the range [0, 0.25)
      y1 = vec_madd(x1, x1, ZERO);
      y2 = vec_madd(x2, x2, ZERO);
      
      // perform the initial polynomial approximations
      s1 = vec_madd(x1,
                    vec_madd(y1,
                             vec_madd(y1,
                                      vec_madd(y1, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      s2 = vec_madd(x2,
                    vec_madd(y2,
                             vec_madd(y2,
                                      vec_madd(y2, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      c1 = vec_madd(y1,
                    vec_madd(y1, vec_madd(y1, CC3, CC2), CC1),
                    ONE);
      c2 = vec_madd(y2,
                    vec_madd(y2, vec_madd(y2, CC3, CC2), CC1),
                    ONE);
      
      // permute the loaded data into separate real and complex vectors
      dre1 = vec_perm(d1, d2, real);
      dim1 = vec_perm(d1, d2, imag);
      dre2 = vec_perm(d3, d4, real);
      dim2 = vec_perm(d3, d4, imag);
      
      // perform first angle doubling
      x1 = vec_nmsub(s1, s1, vec_madd(c1, c1, ZERO));
      x2 = vec_nmsub(s2, s2, vec_madd(c2, c2, ZERO));
      y1 = vec_madd(TWO, vec_madd(s1, c1, ZERO), ZERO);
      y2 = vec_madd(TWO, vec_madd(s2, c2, ZERO), ZERO);
      
      // calculate scaling factor to correct the magnitude
      //      m1 = vec_nmsub(y1, y1, vec_nmsub(x1, x1, TWO));
      //      m2 = vec_nmsub(y2, y2, vec_nmsub(x2, x2, TWO));
      m1 = vec_recip(vec_madd(y1, y1, vec_madd(x1, x1, ZERO)));
      m2 = vec_recip(vec_madd(y2, y2, vec_madd(x2, x2, ZERO)));
      
      // zero out a spot for our chirped data in the cache
      __dcbz(0, c);
      __dcbz(32, c);

      // perform second angle doubling
      c1 = vec_nmsub(y1, y1, vec_madd(x1, x1, ZERO));
      c2 = vec_nmsub(y2, y2, vec_madd(x2, x2, ZERO));
      s1 = vec_madd(TWO, vec_madd(y1, x1, ZERO), ZERO);
      s2 = vec_madd(TWO, vec_madd(y2, x2, ZERO), ZERO);
      
      // correct the magnitude (final sine / cosine approximations)
      s1 = vec_madd(s1, m1, ZERO);
      s2 = vec_madd(s2, m2, ZERO);
      c1 = vec_madd(c1, m1, ZERO);
      c2 = vec_madd(c2, m2, ZERO);
      
      // chirp the data
      cre1 = vec_nmsub(dim1, s1, vec_madd(dre1, c1, ZERO));
      cre2 = vec_nmsub(dim2, s2, vec_madd(dre2, c2, ZERO));
      cim1 = vec_madd(dim1, c1, vec_madd(dre1, s1, ZERO));
      cim2 = vec_madd(dim2, c2, vec_madd(dre2, s2, ZERO));
      
      // permute into interleaved form
      cd1 = vec_mergeh(cre1,cim1);
      cd2 = vec_mergel(cre1,cim1);
      cd3 = vec_mergeh(cre2,cim2);
      cd4 = vec_mergel(cre2,cim2);
      
      // store chirped values
      vec_st(cd1, 0, c);
      vec_st(cd2, 16, c);
      vec_st(cd3, 32, c);
      vec_st(cd4, 48, c);
	}
    
    // handle tail elements with scalar code
    for (; i < offset+ul_NumDataPoints; ++i) {
      double angle = rate * i * i * 0.5;
      double s = sin(angle);
      double c = cos(angle);
      float re = cx_DataArray[i][0];
      float im = cx_DataArray[i][1];
      
      cx_ChirpDataArray[i][0] = re * c - im * s;
      cx_ChirpDataArray[i][1] = re * s + im * c;
    }
    analysis_state.FLOP_counter+=12.0*ul_NumDataPoints;
  }
  return 0;
}


int v_vChirpDataG5(
                 sah_complex* cx_DataArray,
                 sah_complex* cx_ChirpDataArray,
                 int chirp_rate_ind,
                 double chirp_rate,
                 int  offset,
                 int  ul_NumDataPoints,
                 double sample_rate
                 ) {
  // int offset, int numPoints, double chirpRate
  int i;
  double fi;
  
  if (chirp_rate_ind == 0) {
    memcpy(cx_ChirpDataArray+offset,
           cx_DataArray+offset,
           (int)ul_NumDataPoints * sizeof(sah_complex)
           );
  } else {
    int vEnd;  
    const vector unsigned char real = (vector unsigned char) (0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
	const vector unsigned char imag = (vector unsigned char) (4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
    double rate = chirp_rate * 0.5 / (sample_rate * sample_rate);
    double roundVal = rate >= 0.0 ? TWO_TO_52 : -TWO_TO_52;
    
    // main vectorised loop
    vEnd = (offset+ul_NumDataPoints) - ((offset+ul_NumDataPoints) & 15);
    for (i = offset, fi = offset; i < vEnd; i += 16, fi += 16.0) {
      const float *d = (const float *) (cx_DataArray + i);
      float *c = (float *) (cx_ChirpDataArray + i);
      double a1 = fi;
      double a2 = fi + 1.0;
      double a3 = fi + 2.0;
      double a4 = fi + 3.0;
      double a5 = fi + 4.0;
      double a6 = fi + 5.0;
      double a7 = fi + 6.0;
      double a8 = fi + 7.0;
      double a9 = fi + 8.0;
      double a10 = fi + 9.0;
      double a11 = fi + 10.0;
      double a12 = fi + 11.0;
      double a13 = fi + 12.0;
      double a14 = fi + 13.0;
      double a15 = fi + 14.0;
      double a16 = fi + 15.0;
      float v[16] __attribute__ ((aligned (16)));
      vector float d1, d2, d3, d4, d5, d6, d7, d8;
      vector float cd1, cd2, cd3, cd4, cd5, cd6, cd7, cd8;
      vector float dre1, dre2, dre3, dre4;
      vector float dim1, dim2, dim3, dim4;
      vector float cre1, cre2, cre3, cre4;
      vector float cim1, cim2, cim3, cim4;
      vector float x1, x2, x3, x4;
      vector float y1, y2, y3, y4;
      vector float s1, s2, s3, s4;
      vector float c1, c2, c3, c4;
      vector float m1, m2, m3, m4;
      
      // load the signal to be chirped
      d1 = vec_ld(0, d);
      d2 = vec_ld(16, d);
      d3 = vec_ld(32, d);
      d4 = vec_ld(48, d);
      d5 = vec_ld(64, d);
      d6 = vec_ld(80, d);
      d7 = vec_ld(96, d);
      d8 = vec_ld(112, d);
      
      // calculate the input angle
      a1 *= a1 * rate; a2 *= a2 * rate;
      a3 *= a3 * rate; a4 *= a4 * rate;
      a5 *= a5 * rate; a6 *= a6 * rate;
      a7 *= a7 * rate; a8 *= a8 * rate;
      a9 *= a9 * rate; a10 *= a10 * rate;
      a11 *= a11 * rate; a12 *= a12 * rate;
      a13 *= a13 * rate; a14 *= a14 * rate;
      a15 *= a15 * rate; a16 *= a16 * rate;
      
      // reduce the angle to the range (-0.5, 0.5) and store to
      // memory so we can load it into the vector unit
      v[0] = a1 - quickRound(a1, roundVal);
      v[1] = a2 - quickRound(a2, roundVal);
      v[2] = a3 - quickRound(a3, roundVal);
      v[3] = a4 - quickRound(a4, roundVal);
      v[4] = a5 - quickRound(a5, roundVal);
      v[5] = a6 - quickRound(a6, roundVal);
      v[6] = a7 - quickRound(a7, roundVal);
      v[7] = a8 - quickRound(a8, roundVal);
      v[8] = a9 - quickRound(a9, roundVal);
      v[9] = a10 - quickRound(a10, roundVal);
      v[10] = a11 - quickRound(a11, roundVal);
      v[11] = a12 - quickRound(a12, roundVal);
      v[12] = a13 - quickRound(a13, roundVal);
      v[13] = a14 - quickRound(a14, roundVal);
      v[14] = a15 - quickRound(a15, roundVal);
      v[15] = a16 - quickRound(a16, roundVal);
      
      // load angle into the vector unit
      x1 = vec_ld(0, v);
      x2 = vec_ld(16, v);
      x3 = vec_ld(32, v);
      x4 = vec_ld(48, v);
      
      // square to the range [0, 0.25)
      y1 = vec_madd(x1, x1, ZERO);
      y2 = vec_madd(x2, x2, ZERO);
      y3 = vec_madd(x3, x3, ZERO);
      y4 = vec_madd(x4, x4, ZERO);
      
      // perform the initial polynomial approximations
      s1 = vec_madd(x1,
                    vec_madd(y1,
                             vec_madd(y1,
                                      vec_madd(y1, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      s2 = vec_madd(x2,
                    vec_madd(y2,
                             vec_madd(y2,
                                      vec_madd(y2, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      c1 = vec_madd(y1,
                    vec_madd(y1, vec_madd(y1, CC3, CC2), CC1),
                    ONE);
      c2 = vec_madd(y2,
                    vec_madd(y2, vec_madd(y2, CC3, CC2), CC1),
                    ONE);
      s3 = vec_madd(x3,
                    vec_madd(y3,
                             vec_madd(y3,
                                      vec_madd(y3, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      s4 = vec_madd(x4,
                    vec_madd(y4,
                             vec_madd(y4,
                                      vec_madd(y4, SS4, SS3),
                                      SS2),
                             SS1),
                    ZERO);
      c3 = vec_madd(y3,
                    vec_madd(y3, vec_madd(y3, CC3, CC2), CC1),
                    ONE);
      c4 = vec_madd(y4,
                    vec_madd(y4, vec_madd(y4, CC3, CC2), CC1),
                    ONE);
      
      // permute the loaded data into separate real and complex vectors
      dre1 = vec_perm(d1, d2, real);
      dim1 = vec_perm(d1, d2, imag);
      dre2 = vec_perm(d3, d4, real);
      dim2 = vec_perm(d3, d4, imag);
      dre3 = vec_perm(d5, d6, real);
      dim3 = vec_perm(d5, d6, imag);
      dre4 = vec_perm(d7, d8, real);
      dim4 = vec_perm(d7, d8, imag);
      
      // perform first angle doubling
      x1 = vec_nmsub(s1, s1, vec_madd(c1, c1, ZERO));
      x2 = vec_nmsub(s2, s2, vec_madd(c2, c2, ZERO));
      y1 = vec_madd(TWO, vec_madd(s1, c1, ZERO), ZERO);
      y2 = vec_madd(TWO, vec_madd(s2, c2, ZERO), ZERO);
      x3 = vec_nmsub(s3, s3, vec_madd(c3, c3, ZERO));
      x4 = vec_nmsub(s4, s4, vec_madd(c4, c4, ZERO));
      y3 = vec_madd(TWO, vec_madd(s3, c3, ZERO), ZERO);
      y4 = vec_madd(TWO, vec_madd(s4, c4, ZERO), ZERO);
      
      // calculate scaling factor to correct the magnitude
      //      m1 = vec_nmsub(y1, y1, vec_nmsub(x1, x1, TWO));
      //      m2 = vec_nmsub(y2, y2, vec_nmsub(x2, x2, TWO));
      m1 = vec_recip(vec_madd(y1, y1, vec_madd(x1, x1, ZERO)));
      m2 = vec_recip(vec_madd(y2, y2, vec_madd(x2, x2, ZERO)));
      m3 = vec_recip(vec_madd(y3, y3, vec_madd(x3, x3, ZERO)));
      m4 = vec_recip(vec_madd(y4, y4, vec_madd(x4, x4, ZERO)));
      
      // perform second angle doubling
      c1 = vec_nmsub(y1, y1, vec_madd(x1, x1, ZERO));
      c2 = vec_nmsub(y2, y2, vec_madd(x2, x2, ZERO));
      s1 = vec_madd(TWO, vec_madd(y1, x1, ZERO), ZERO);
      s2 = vec_madd(TWO, vec_madd(y2, x2, ZERO), ZERO);
      c3 = vec_nmsub(y3, y3, vec_madd(x3, x3, ZERO));
      c4 = vec_nmsub(y4, y4, vec_madd(x4, x4, ZERO));
      s3 = vec_madd(TWO, vec_madd(y3, x3, ZERO), ZERO);
      s4 = vec_madd(TWO, vec_madd(y4, x4, ZERO), ZERO);
      
      // correct the magnitude (final sine / cosine approximations)
      s1 = vec_madd(s1, m1, ZERO);
      s2 = vec_madd(s2, m2, ZERO);
      c1 = vec_madd(c1, m1, ZERO);
      c2 = vec_madd(c2, m2, ZERO);
      s3 = vec_madd(s3, m3, ZERO);
      s4 = vec_madd(s4, m4, ZERO);
      c3 = vec_madd(c3, m3, ZERO);
      c4 = vec_madd(c4, m4, ZERO);
      
      // chirp the data
      cre1 = vec_nmsub(dim1, s1, vec_madd(dre1, c1, ZERO));
      cre2 = vec_nmsub(dim2, s2, vec_madd(dre2, c2, ZERO));
      cim1 = vec_madd(dim1, c1, vec_madd(dre1, s1, ZERO));
      cim2 = vec_madd(dim2, c2, vec_madd(dre2, s2, ZERO));
      cre3 = vec_nmsub(dim3, s3, vec_madd(dre3, c3, ZERO));
      cre4 = vec_nmsub(dim4, s4, vec_madd(dre4, c4, ZERO));
      cim3 = vec_madd(dim3, c3, vec_madd(dre3, s3, ZERO));
      cim4 = vec_madd(dim4, c4, vec_madd(dre4, s4, ZERO));
      
      // permute into interleaved form
      cd1 = vec_mergeh(cre1,cim1);
      cd2 = vec_mergel(cre1,cim1);
      cd3 = vec_mergeh(cre2,cim2);
      cd4 = vec_mergel(cre2,cim2);
      cd5 = vec_mergeh(cre3,cim3);
      cd6 = vec_mergel(cre3,cim3);
      cd7 = vec_mergeh(cre4,cim4);
      cd8 = vec_mergel(cre4,cim4);
      
      // store chirped values
      vec_st(cd1, 0, c);
      vec_st(cd2, 16, c);
      vec_st(cd3, 32, c);
      vec_st(cd4, 48, c);
      vec_st(cd5, 64, c);
      vec_st(cd6, 80, c);
      vec_st(cd7, 96, c);
      vec_st(cd8, 112, c);
	}
    
    // handle tail elements with scalar code
    for (; i < offset+ul_NumDataPoints; ++i) {
      double angle = rate * i * i * 0.5;
      double s = sin(angle);
      double c = cos(angle);
      float re = cx_DataArray[i][0];
      float im = cx_DataArray[i][1];
      
      cx_ChirpDataArray[i][0] = re * c - im * s;
      cx_ChirpDataArray[i][1] = re * s + im * c;
    }
    analysis_state.FLOP_counter+=12.0*ul_NumDataPoints;
  }
  return 0;
}


#elif defined(__SSE3__) && defined(USE_VECTORIZED_CHIRP)
inline vFloat __attribute__((__always_inline__)) vec_recip(vFloat v) {
  // obtain estimate
  vFloat estimate = _mm_rcp_ps( v );
  // one round of Newton-Raphson
  return _mm_add_ps(_mm_mul_ps(_mm_sub_ps(ONE, _mm_mul_ps(estimate, v)), estimate), estimate);
}  


int v_vChirpData(
#ifdef USE_SPLIT_COMPLEX
                 DSPSplitComplex *cx_DataArray,
                 DSPSplitComplex *cx_ChirpDataArray,
#else
                 sah_complex* cx_DataArray,
                 sah_complex* cx_ChirpDataArray,
#endif
                 int chirp_rate_ind,
                 double chirp_rate,
                 int offset,
                 int ul_NumDataPoints,
                 double sample_rate
                 ) {
  // int offset, int numPoints, double chirpRate
  unsigned int i, j;
  
  if (chirp_rate_ind == 0) {
#ifdef USE_SPLIT_COMPLEX
    memcpy(cx_ChirpDataArray->realp + offset, cx_DataArray->realp + offset, ul_NumDataPoints * sizeof(float));
    memcpy(cx_ChirpDataArray->imagp + offset, cx_DataArray->imagp + offset, ul_NumDataPoints * sizeof(float));    
#else
    memcpy(cx_ChirpDataArray+offset,
           cx_DataArray+offset,
           (int)ul_NumDataPoints * sizeof(sah_complex)
           );
#endif
  } else {
    int vEnd;  
    double srate = chirp_rate * 0.5 / (sample_rate * sample_rate);
    vDouble rate = _mm_set1_pd(chirp_rate * 0.5 / (sample_rate * sample_rate));
    vDouble roundVal = _mm_set1_pd(srate >= 0.0 ? TWO_TO_52 : -TWO_TO_52);
    
#ifdef USE_SPLIT_COMPLEX
    const float *cx_DataArrayReal = (const float *) (cx_DataArray->realp);
    const float *cx_DataArrayImag = (const float *) (cx_DataArray->imagp);
    float *cx_ChirpDataArrayReal = cx_ChirpDataArray->realp;
    float *cx_ChirpDataArrayImag = cx_ChirpDataArray->imagp;
#endif
    
    vEnd = (offset+ul_NumDataPoints) - ((offset+ul_NumDataPoints) & 3);
    
    // main vectorised loop
    vDouble di = _mm_set1_pd(offset);
#ifdef USE_SPLIT_COMPLEX
    vDouble di1 = _mm_add_pd(_mm_set_pd(1.0, 0.0), di);
    vDouble di2 = _mm_add_pd(_mm_set_pd(3.0, 2.0), di);
#else
    vDouble di1 = _mm_add_pd(_mm_set_pd(2.0, 0.0), di);
    vDouble di2 = _mm_add_pd(_mm_set_pd(3.0, 1.0), di);
#endif

    for (i = offset; i < vEnd; i += 4) {
#ifdef USE_SPLIT_COMPLEX
      const float *pdr = cx_DataArrayReal + i;
      const float *pdi = cx_DataArrayImag + i;
      float *pcdr = cx_ChirpDataArrayReal + i;
      float *pcdi = cx_ChirpDataArrayImag + i;
#else
      const float *d = (const float *) (cx_DataArray + i);
      float *cd = (float *) (cx_ChirpDataArray + i);
#endif
//       vDouble di = _mm_set1_pd(i);
//       vDouble a1 = _mm_add_pd(_mm_set_pd(1.0, 0.0), di);
//       vDouble a2 = _mm_add_pd(_mm_set_pd(3.0, 2.0), di);
      vDouble a1, a2;
      
#ifdef USE_SPLIT_COMPLEX
      vFloat dr, di;
      vFloat cdr, cdi;
#else
      vFloat d1, d2;
      vFloat cd1, cd2;
      vFloat td1, td2;
#endif
      vFloat x;
      vFloat y;
      vFloat z;
      vFloat s;
      vFloat c;
      vFloat m;
      
      // load the signal to be chirped
#ifdef USE_SPLIT_COMPLEX
      dr = _mm_load_ps(pdr);
      di = _mm_load_ps(pdi);
#else
      d1 = _mm_load_ps(d);
      d2 = _mm_load_ps(d+4);
#endif
      
      // calculate the input angle
      a1 = _mm_mul_pd(_mm_mul_pd(di1, di1), rate);
      a2 = _mm_mul_pd(_mm_mul_pd(di2, di2), rate);

      di1 = _mm_add_pd(di1, DFOUR);
      di2 = _mm_add_pd(di2, DFOUR);
      
      // reduce the angle to the range (-0.5, 0.5)
      a1 = _mm_sub_pd(a1, _mm_sub_pd(_mm_add_pd(a1, roundVal), roundVal));
      a2 = _mm_sub_pd(a2, _mm_sub_pd(_mm_add_pd(a2, roundVal), roundVal));
      
      // convert pair of packed double into packed single
      x = _mm_movelh_ps(_mm_cvtpd_ps(a1), _mm_cvtpd_ps(a2));
      
      // square to the range [0, 0.25)
      y = _mm_mul_ps(x, x);
      
      // perform the initial polynomial approximations
//       // Horner's method
//       s = _mm_mul_ps(_mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, SS4),
//                                                                             SS3),
//                                                                  y),
//                                                       SS2),
//                                            y),
//                                 SS1),
//                      x);
//       c = _mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, CC3),
//                                                                  CC2),
//                                                       y),
//                                            CC1),
//                                 y),
//                      ONE);

      // Estrin's method
      z = _mm_mul_ps(y, y);
#ifdef FASTER_SINCOS
      s = _mm_mul_ps(_mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, SS4F),
						      SS3F),
					   z),
				_mm_add_ps(_mm_mul_ps(y, SS2F),
					   SS1F)),
		     x);
      c = _mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, CC3F),
					   CC2F),
				z),
		     _mm_add_ps(_mm_mul_ps(y, CC1F),
				ONE));
#else
      s = _mm_mul_ps(_mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, SS4),
						      SS3),
					   z),
				_mm_add_ps(_mm_mul_ps(y, SS2),
					   SS1)),
		     x);
      c = _mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, CC3),
					   CC2),
				z),
		     _mm_add_ps(_mm_mul_ps(y, CC1),
				ONE));
#endif      

      // perform first angle doubling
      x = _mm_sub_ps(_mm_mul_ps(c, c), _mm_mul_ps(s, s));
      y = _mm_mul_ps(_mm_mul_ps(s, c), TWO);
      
      // calculate scaling factor to correct the magnitude
#ifdef FASTER_SINCOS
//      m = _mm_sub_ps(TWO, _mm_add_ps(_mm_mul_ps(x, x), _mm_mul_ps(y, y)));
	  m = _mm_sub_ps(_mm_sub_ps(TWO, _mm_mul_ps(x, x)), _mm_mul_ps(y, y));
#else
      m = vec_recip(_mm_add_ps(_mm_mul_ps(x, x), _mm_mul_ps(y, y)));
#endif
      
      // perform second angle doubling
      c = _mm_sub_ps(_mm_mul_ps(x, x), _mm_mul_ps(y, y));
      s = _mm_mul_ps(_mm_mul_ps(y, x), TWO);
      
      // correct the magnitude (final sine / cosine approximations)
      s = _mm_mul_ps(s, m);
      c = _mm_mul_ps(c, m);
      
#ifdef USE_SPLIT_COMPLEX
      // chirp the data
      cdr = _mm_sub_ps(_mm_mul_ps(dr, c), _mm_mul_ps(di, s));
      cdi = _mm_add_ps(_mm_mul_ps(dr, s), _mm_mul_ps(di, c));
      
      // store chirped values
      _mm_store_ps(pcdr, cdr);
      _mm_store_ps(pcdi, cdi);
#else
      // chirp the data
      cd1 = _mm_moveldup_ps(c);
      cd2 = _mm_movehdup_ps(c);
      cd1 = _mm_mul_ps(cd1, d1);
      cd2 = _mm_mul_ps(cd2, d2);
      d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
      d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
      td1 = _mm_moveldup_ps(s);
      td2 = _mm_movehdup_ps(s);
      td1 = _mm_mul_ps(td1, d1);
      td2 = _mm_mul_ps(td2, d2);
      cd1 = _mm_addsub_ps(cd1, td1);
      cd2 = _mm_addsub_ps(cd2, td2);
      
      // store chirped values
      _mm_store_ps(cd, cd1);
      _mm_store_ps(cd+4, cd2);
#endif
	}
    
    // handle tail elements with scalar code
    for (; i < offset+ul_NumDataPoints; ++i) {
      double angle = srate * i * i * 0.5;
      double s = sin(angle);
      double c = cos(angle);

#ifdef USE_SPLIT_COMPLEX
      float re = cx_DataArray->realp[i];
      float im = cx_DataArray->imagp[i];
      
      cx_ChirpDataArray->realp[i] = re * c - im * s;
      cx_ChirpDataArray->imagp[i] = re * s + im * c;
#else
      float re = cx_DataArray[i][0];
      float im = cx_DataArray[i][1];
      
      cx_ChirpDataArray[i][0] = re * c - im * s;
      cx_ChirpDataArray[i][1] = re * s + im * c;
#endif
    }
    analysis_state.FLOP_counter+=12.0*ul_NumDataPoints;
  }
  return 0;
}

int v_vpChirpData(
                 sah_complex* cx_DataArray,
                 sah_complex* cx_ChirpDataArray,
                 int chirp_rate_ind,
                 double chirp_rate,
                 int offset,
                 int ul_NumDataPoints,
                 double sample_rate
                 ) {
  // int offset, int numPoints, double chirpRate
  unsigned int i, j;
  
  if (chirp_rate_ind == 0) {
    memcpy(cx_ChirpDataArray+offset,
           cx_DataArray+offset,
           (int)ul_NumDataPoints * sizeof(sah_complex)
           );
  } else {
    int vEnd;  
    double srate = chirp_rate * 0.5 / (sample_rate * sample_rate);
    vDouble rate = _mm_set1_pd(chirp_rate * 0.5 / (sample_rate * sample_rate));
    vDouble roundVal = _mm_set1_pd(srate >= 0.0 ? TWO_TO_52 : -TWO_TO_52);
    
    vEnd = (offset+ul_NumDataPoints) - ((offset+ul_NumDataPoints) & 7);
    
    // main vectorised loop
    vDouble di = _mm_set1_pd(offset);
    vDouble di1 = _mm_add_pd(_mm_set_pd(2.0, 0.0), di);
    vDouble di2 = _mm_add_pd(_mm_set_pd(3.0, 1.0), di);

    for (i = offset; i < vEnd; i += 8) {
      const float *d = (const float *) (cx_DataArray + i);
      float *cd = (float *) (cx_ChirpDataArray + i);
//       vDouble di = _mm_set1_pd(i);
//       vDouble a1 = _mm_add_pd(_mm_set_pd(1.0, 0.0), di);
//       vDouble a2 = _mm_add_pd(_mm_set_pd(3.0, 2.0), di);
      vDouble a1, a2;
      
      vFloat d1, d2;
      vFloat cd1, cd2;
      vFloat td1, td2;
      vFloat x;
      vFloat y;
      vFloat z;
      vFloat s;
      vFloat c;
      vFloat m;
      
      // prefetch 1 loop iteration ahead
      _mm_prefetch((char *) (d+16), _MM_HINT_NTA);
      
      // load the signal to be chirped
      d1 = _mm_load_ps(d);
      d2 = _mm_load_ps(d+4);
      
      // calculate the input angle
      a1 = _mm_mul_pd(_mm_mul_pd(di1, di1), rate);
      a2 = _mm_mul_pd(_mm_mul_pd(di2, di2), rate);

      di1 = _mm_add_pd(di1, DFOUR);
      di2 = _mm_add_pd(di2, DFOUR);
      
      // reduce the angle to the range (-0.5, 0.5)
      a1 = _mm_sub_pd(a1, _mm_sub_pd(_mm_add_pd(a1, roundVal), roundVal));
      a2 = _mm_sub_pd(a2, _mm_sub_pd(_mm_add_pd(a2, roundVal), roundVal));
      
      // convert pair of packed double into packed single
      x = _mm_movelh_ps(_mm_cvtpd_ps(a1), _mm_cvtpd_ps(a2));
      
      // square to the range [0, 0.25)
      y = _mm_mul_ps(x, x);
      
      // perform the initial polynomial approximations

      // Estrin's method
      z = _mm_mul_ps(y, y);
      s = _mm_mul_ps(_mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, SS4F),
						      SS3F),
					   z),
				_mm_add_ps(_mm_mul_ps(y, SS2F),
					   SS1F)),
		     x);
      c = _mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, CC3F),
					   CC2F),
				z),
		     _mm_add_ps(_mm_mul_ps(y, CC1F),
				ONE));

      // perform first angle doubling
      x = _mm_sub_ps(_mm_mul_ps(c, c), _mm_mul_ps(s, s));
      y = _mm_mul_ps(_mm_mul_ps(s, c), TWO);
      
      // calculate scaling factor to correct the magnitude
	  m = _mm_sub_ps(_mm_sub_ps(TWO, _mm_mul_ps(x, x)), _mm_mul_ps(y, y));
      
      // perform second angle doubling
      c = _mm_sub_ps(_mm_mul_ps(x, x), _mm_mul_ps(y, y));
      s = _mm_mul_ps(_mm_mul_ps(y, x), TWO);
      
      // correct the magnitude (final sine / cosine approximations)
      s = _mm_mul_ps(s, m);
      c = _mm_mul_ps(c, m);
      
      // chirp the data
      cd1 = _mm_moveldup_ps(c);
      cd2 = _mm_movehdup_ps(c);
      cd1 = _mm_mul_ps(cd1, d1);
      cd2 = _mm_mul_ps(cd2, d2);
      d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
      d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
      td1 = _mm_moveldup_ps(s);
      td2 = _mm_movehdup_ps(s);
      td1 = _mm_mul_ps(td1, d1);
      td2 = _mm_mul_ps(td2, d2);
      cd1 = _mm_addsub_ps(cd1, td1);
      cd2 = _mm_addsub_ps(cd2, td2);
      
      // store chirped values
      _mm_store_ps(cd, cd1);
      _mm_store_ps(cd+4, cd2);
      
      // load the signal to be chirped
      d1 = _mm_load_ps(d+8);
      d2 = _mm_load_ps(d+12);
      
      // calculate the input angle
      a1 = _mm_mul_pd(_mm_mul_pd(di1, di1), rate);
      a2 = _mm_mul_pd(_mm_mul_pd(di2, di2), rate);
      
      di1 = _mm_add_pd(di1, DFOUR);
      di2 = _mm_add_pd(di2, DFOUR);
      
      // reduce the angle to the range (-0.5, 0.5)
      a1 = _mm_sub_pd(a1, _mm_sub_pd(_mm_add_pd(a1, roundVal), roundVal));
      a2 = _mm_sub_pd(a2, _mm_sub_pd(_mm_add_pd(a2, roundVal), roundVal));
      
      // convert pair of packed double into packed single
      x = _mm_movelh_ps(_mm_cvtpd_ps(a1), _mm_cvtpd_ps(a2));
      
      // square to the range [0, 0.25)
      y = _mm_mul_ps(x, x);
      
      // perform the initial polynomial approximations
      
      // Estrin's method
      z = _mm_mul_ps(y, y);
      s = _mm_mul_ps(_mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, SS4F),
                                                      SS3F),
                                           z),
                                _mm_add_ps(_mm_mul_ps(y, SS2F),
                                           SS1F)),
                     x);
      c = _mm_add_ps(_mm_mul_ps(_mm_add_ps(_mm_mul_ps(y, CC3F),
                                           CC2F),
                                z),
                     _mm_add_ps(_mm_mul_ps(y, CC1F),
                                ONE));
      
      // perform first angle doubling
      x = _mm_sub_ps(_mm_mul_ps(c, c), _mm_mul_ps(s, s));
      y = _mm_mul_ps(_mm_mul_ps(s, c), TWO);
      
      // calculate scaling factor to correct the magnitude
	  m = _mm_sub_ps(_mm_sub_ps(TWO, _mm_mul_ps(x, x)), _mm_mul_ps(y, y));
      
      // perform second angle doubling
      c = _mm_sub_ps(_mm_mul_ps(x, x), _mm_mul_ps(y, y));
      s = _mm_mul_ps(_mm_mul_ps(y, x), TWO);
      
      // correct the magnitude (final sine / cosine approximations)
      s = _mm_mul_ps(s, m);
      c = _mm_mul_ps(c, m);
      
      // chirp the data
      cd1 = _mm_moveldup_ps(c);
      cd2 = _mm_movehdup_ps(c);
      cd1 = _mm_mul_ps(cd1, d1);
      cd2 = _mm_mul_ps(cd2, d2);
      d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
      d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
      td1 = _mm_moveldup_ps(s);
      td2 = _mm_movehdup_ps(s);
      td1 = _mm_mul_ps(td1, d1);
      td2 = _mm_mul_ps(td2, d2);
      cd1 = _mm_addsub_ps(cd1, td1);
      cd2 = _mm_addsub_ps(cd2, td2);
      
      // store chirped values
      _mm_store_ps(cd+8, cd1);
      _mm_store_ps(cd+12, cd2);
	}
    
    // handle tail elements with scalar code
    for (; i < offset+ul_NumDataPoints; ++i) {
      double angle = srate * i * i * 0.5;
      double s = sin(angle);
      double c = cos(angle);

#ifdef USE_SPLIT_COMPLEX
      float re = cx_DataArray->realp[i];
      float im = cx_DataArray->imagp[i];
      
      cx_ChirpDataArray->realp[i] = re * c - im * s;
      cx_ChirpDataArray->imagp[i] = re * s + im * c;
#else
      float re = cx_DataArray[i][0];
      float im = cx_DataArray[i][1];
      
      cx_ChirpDataArray[i][0] = re * c - im * s;
      cx_ChirpDataArray[i][1] = re * s + im * c;
#endif
    }
    analysis_state.FLOP_counter+=12.0*ul_NumDataPoints;
  }
  return 0;
}
#endif

#ifndef USE_INTEL_OPT_CODE
int v_ChirpData(
  sah_complex* cx_DataArray,
  sah_complex* cx_ChirpDataArray,
  int chirp_rate_ind,
  double chirp_rate,
  int  ul_NumDataPoints,
  double sample_rate
) {
  int i;

#ifdef DEBUG
  fprintf(stderr, "icfft = %6d  crate index = %6d\n", icfft, chirp_rate_ind);
  fflush(stderr);
#endif

  if (chirp_rate_ind == 0) {
    memcpy(cx_ChirpDataArray,
           cx_DataArray,
           (int)ul_NumDataPoints * sizeof(sah_complex)
          );    // NOTE INT CAST
  } else {

    // what we do depends on how much memory we have...
    // If we have more than 64MB, we'll cache the chirp table.  If not
    // we'll calculate it each time.
    bool CacheChirpCalc=((app_init_data.host_info.m_nbytes != 0)  &&
                         (app_init_data.host_info.m_nbytes >= (double)(64*1024*1024)));

    // calculate trigonometric array
    // this function returns w/o doing nothing when sign of chirp_rate_ind
    // reverses.  so we have to take care of it.

    if (CacheChirpCalc) {
      CalcTrigArray(ul_NumDataPoints, chirp_rate_ind);
      for (i = 0; i < ul_NumDataPoints; i++) {
        float c, d, real, imag;

        c = CurrentTrig[i].Cos;
        d = (chirp_rate_ind >0)? CurrentTrig[i].Sin : -CurrentTrig[i].Sin;

        // Sometimes chirping is done in place.
        // We don't want to overwrite data prematurely.
        real = cx_DataArray[i][0] * c - cx_DataArray[i][1] * d;
        imag = cx_DataArray[i][0] * d + cx_DataArray[i][1] * c;
        cx_ChirpDataArray[i][0] = real;
        cx_ChirpDataArray[i][1] = imag;
      }
    } else {
      for (i = 0; i < ul_NumDataPoints; i++) {
        float c, d, real, imag;        

        double dd,cc;
        double time=static_cast<double>(i)/sample_rate;
          // since ang is getting moded by 2pi, we calculate "ang mod 2pi"
          // before the call to sincos() inorder to reduce roundoff error.
          // (Bug submitted by Tetsuji "Maverick" Rai)
        double ang  = 0.5*chirp_rate*time*time;
        ang -= floor(ang);
        ang *= M_PI*2;
        sincos(ang,&dd,&cc);
        c=cc;
        d=dd;

        // Sometimes chirping is done in place.
        // We don't want to overwrite data prematurely.
        real = cx_DataArray[i][0] * c - cx_DataArray[i][1] * d;
        imag = cx_DataArray[i][0] * d + cx_DataArray[i][1] * c;
        cx_ChirpDataArray[i][0] = real;
        cx_ChirpDataArray[i][1] = imag;
      }
    }

    analysis_state.FLOP_counter+=12.0*ul_NumDataPoints;
  }
  return 0;
}
#endif // USE_INTEL_OPT_CODE

#if 0
int v_sChirpData(
#ifdef USE_SPLIT_COMPLEX
  DSPSplitComplex *cx_DataArray,
  DSPSplitComplex *cx_ChirpDataArray,
#else
  sah_complex* cx_DataArray,
  sah_complex* cx_ChirpDataArray,
#endif
  int chirp_rate_ind,
  double chirp_rate,
  int  offset,
  int  ul_NumDataPoints,
  double sample_rate
) {
#ifdef DEBUG
  fprintf(stderr, "icfft = %6d  crate index = %6d\n", icfft, chirp_rate_ind);
  fflush(stderr);
#endif

  if (chirp_rate_ind == 0) {
#ifdef USE_SPLIT_COMPLEX
    memcpy(cx_ChirpDataArray->realp + offset, cx_DataArray->realp + offset, ul_NumDataPoints * sizeof(float));
    memcpy(cx_ChirpDataArray->imagp + offset, cx_DataArray->imagp + offset, ul_NumDataPoints * sizeof(float));    
#else
    memcpy(cx_ChirpDataArray + offset,
           cx_DataArray + offset,
           (int)ul_NumDataPoints * sizeof(sah_complex)
          );    // NOTE INT CAST
#endif
  } else {

    // calculate trigonometric array
    // this function returns w/o doing nothing when sign of chirp_rate_ind
    // reverses.  so we have to take care of it.
    
    int j;
    vUInt32 signFlip = (chirp_rate_ind < 0) ? SIGNBITS : IZERO;
    int increment = abs(chirp_rate_ind) - CurrentChirpRateInd;
    
    switch (abs(chirp_rate_ind)-CurrentChirpRateInd) {
      case 0:
        for (j = offset; j < offset+ul_NumDataPoints; j += 4) {
          const float *d = (const float *) (cx_DataArray + j);
          float *cd = (float *) (cx_ChirpDataArray + j);
          const double *real = (const double *) (SplitCurrentTrig.cos + j);
          const double *imag = (const double *) (SplitCurrentTrig.sin + j);
          
          vFloat d1 = _mm_load_ps(d);
          vFloat d2 = _mm_load_ps(d+4);
          
          vFloat cd1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(real)), _mm_cvtpd_ps(_mm_loaddup_pd(real+1)));
          vFloat cd2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(real+2)), _mm_cvtpd_ps(_mm_loaddup_pd(real+3)));
          cd1 = _mm_mul_ps(cd1, d1);
          cd2 = _mm_mul_ps(cd2, d2);
          d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
          d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
          vFloat td1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(imag)), _mm_cvtpd_ps(_mm_loaddup_pd(imag+1)));
          vFloat td2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(imag+2)), _mm_cvtpd_ps(_mm_loaddup_pd(imag+3)));
          td1 = _mm_xor_ps(td1, (vFloat) signFlip);
          td2 = _mm_xor_ps(td2, (vFloat) signFlip);
          td1 = _mm_mul_ps(td1, d1);
          td2 = _mm_mul_ps(td2, d2);
          cd1 = _mm_addsub_ps(cd1, td1);
          cd2 = _mm_addsub_ps(cd2, td2);
          
          _mm_store_ps(cd, cd1);
          _mm_store_ps(cd+4, cd2);
        }
        break;
      case 1:
        for (j = offset; j < offset+ul_NumDataPoints; j += 4) {
          const float *d = (const float *) (cx_DataArray + j);
          float *cd = (float *) (cx_ChirpDataArray + j);
          double *real = (SplitCurrentTrig.cos + j);
          double *imag = (SplitCurrentTrig.sin + j);
          const double *increal = (const double *) (SplitTrigStep.cos + j);
          const double *incimag = (const double *) (SplitTrigStep.sin + j);
          
          // load and increment trig array
          vDouble ur1, ui1, ur2, ui2;
          {
            vDouble r1 = _mm_load_pd(real);
            vDouble i1 = _mm_load_pd(imag);
            vDouble ir1 = _mm_load_pd(increal);
            vDouble ii1 = _mm_load_pd(incimag);
            ur1 = _mm_sub_pd(_mm_mul_pd(r1, ir1), _mm_mul_pd(i1, ii1));
            ui1 = _mm_add_pd(_mm_mul_pd(r1, ii1), _mm_mul_pd(i1, ir1));
            _mm_store_pd(real, ur1);
            _mm_store_pd(imag, ui1);
          }
          
          {
            vDouble r2 = _mm_load_pd(real+2);
            vDouble i2 = _mm_load_pd(imag+2);
            vDouble ir2 = _mm_load_pd(increal+2);
            vDouble ii2 = _mm_load_pd(incimag+2);
            ur2 = _mm_sub_pd(_mm_mul_pd(r2, ir2), _mm_mul_pd(i2, ii2));
            ui2 = _mm_add_pd(_mm_mul_pd(r2, ii2), _mm_mul_pd(i2, ir2));
            _mm_store_pd(real+2, ur2);
            _mm_store_pd(imag+2, ui2);                      
          }
          
          vFloat d1 = _mm_load_ps(d);
          vFloat d2 = _mm_load_ps(d+4);

          vFloat cd1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur1, ur1)));
          vFloat cd2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur2, ur2)));
          cd1 = _mm_mul_ps(cd1, d1);
          cd2 = _mm_mul_ps(cd2, d2);
          d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
          d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
          vFloat td1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui1, ui1)));
          vFloat td2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui2, ui2)));
          td1 = _mm_xor_ps(td1, (vFloat) signFlip);
          td2 = _mm_xor_ps(td2, (vFloat) signFlip);
          td1 = _mm_mul_ps(td1, d1);
          td2 = _mm_mul_ps(td2, d2);
          cd1 = _mm_addsub_ps(cd1, td1);
          cd2 = _mm_addsub_ps(cd2, td2);
          
          _mm_store_ps(cd, cd1);
          _mm_store_ps(cd+4, cd2);
        }
        if (offset + ul_NumDataPoints == 1048576) CurrentChirpRateInd = abs(chirp_rate_ind);
        break;
      case 2:
        for (j = offset; j < offset+ul_NumDataPoints; j += 4) {
          const float *d = (const float *) (cx_DataArray + j);
          float *cd = (float *) (cx_ChirpDataArray + j);
          double *real = (SplitCurrentTrig.cos + j);
          double *imag = (SplitCurrentTrig.sin + j);
          const double *increal = (const double *) (SplitTrigStep.cos + j);
          const double *incimag = (const double *) (SplitTrigStep.sin + j);
        
          // load and increment trig array
          vDouble ur1, ui1, ur2, ui2;
          {
            vDouble r1 = _mm_load_pd(real);
            vDouble i1 = _mm_load_pd(imag);
            vDouble ir1 = _mm_load_pd(increal);
            vDouble ii1 = _mm_load_pd(incimag);
            vDouble i2r1 = _mm_sub_pd(_mm_mul_pd(ir1, ir1), _mm_mul_pd(ii1, ii1));
            vDouble i2i1 = _mm_mul_pd(_mm_mul_pd(ir1, ii1), DTWO);
            ur1 = _mm_sub_pd(_mm_mul_pd(r1, i2r1), _mm_mul_pd(i1, i2i1));
            ui1 = _mm_add_pd(_mm_mul_pd(r1, i2i1), _mm_mul_pd(i1, i2r1));
            _mm_store_pd(real, ur1);
            _mm_store_pd(imag, ui1);
          }

          {
            vDouble r2 = _mm_load_pd(real+2);
            vDouble i2 = _mm_load_pd(imag+2);
            vDouble ir2 = _mm_load_pd(increal+2);
            vDouble ii2 = _mm_load_pd(incimag+2);
            vDouble i2r2 = _mm_sub_pd(_mm_mul_pd(ir2, ir2), _mm_mul_pd(ii2, ii2));
            vDouble i2i2 = _mm_mul_pd(_mm_mul_pd(ir2, ii2), DTWO);
            ur2 = _mm_sub_pd(_mm_mul_pd(r2, i2r2), _mm_mul_pd(i2, i2i2));
            ui2 = _mm_add_pd(_mm_mul_pd(r2, i2i2), _mm_mul_pd(i2, i2r2));
            _mm_store_pd(real+2, ur2);
            _mm_store_pd(imag+2, ui2);          
          }
          
          vFloat d1 = _mm_load_ps(d);
          vFloat d2 = _mm_load_ps(d+4);

          vFloat cd1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur1, ur1)));
          vFloat cd2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur2, ur2)));
          cd1 = _mm_mul_ps(cd1, d1);
          cd2 = _mm_mul_ps(cd2, d2);
          d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
          d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
          vFloat td1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui1, ui1)));
          vFloat td2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui2, ui2)));
          td1 = _mm_xor_ps(td1, (vFloat) signFlip);
          td2 = _mm_xor_ps(td2, (vFloat) signFlip);
          td1 = _mm_mul_ps(td1, d1);
          td2 = _mm_mul_ps(td2, d2);
          cd1 = _mm_addsub_ps(cd1, td1);
          cd2 = _mm_addsub_ps(cd2, td2);
          
          _mm_store_ps(cd, cd1);
          _mm_store_ps(cd+4, cd2);
        }
        if (offset + ul_NumDataPoints == 1048576) CurrentChirpRateInd = abs(chirp_rate_ind);
        break;
      case 3:
        // cos(3x) = 4 cos^3x - 3 sin x
        // sin(3x) = 3 sin x - 4 sin^3 x
        for (j = offset; j < offset+ul_NumDataPoints; j += 4) {
          const float *d = (const float *) (cx_DataArray + j);
          float *cd = (float *) (cx_ChirpDataArray + j);
          double *real = (SplitCurrentTrig.cos + j);
          double *imag = (SplitCurrentTrig.sin + j);
          const double *increal = (const double *) (SplitTrigStep.cos + j);
          const double *incimag = (const double *) (SplitTrigStep.sin + j);
        
          // load and increment trig array
          vDouble r1 = _mm_load_pd(real);
          vDouble i1 = _mm_load_pd(imag);
          vDouble ir1 = _mm_load_pd(increal);
          vDouble ii1 = _mm_load_pd(incimag);
          vDouble i3r1 = _mm_sub_pd(_mm_mul_pd(_mm_mul_pd(ir1, DFOUR), _mm_mul_pd(ir1, ir1)), _mm_mul_pd(ir1, DTHREE));
          vDouble i3i1 = _mm_sub_pd(_mm_mul_pd(ii1, DTHREE), _mm_mul_pd(_mm_mul_pd(ii1, DFOUR), _mm_mul_pd(ii1, ii1)));
          vDouble ur1 = _mm_sub_pd(_mm_mul_pd(r1, i3r1), _mm_mul_pd(i1, i3i1));
          vDouble ui1 = _mm_add_pd(_mm_mul_pd(r1, i3i1), _mm_mul_pd(i1, i3r1));
          _mm_store_pd(real, ur1);
          _mm_store_pd(imag, ui1);
        
          vDouble r2 = _mm_load_pd(real+2);
          vDouble i2 = _mm_load_pd(imag+2);
          vDouble ir2 = _mm_load_pd(increal+2);
          vDouble ii2 = _mm_load_pd(incimag+2);
          vDouble i3r2 = _mm_sub_pd(_mm_mul_pd(_mm_mul_pd(ir2, DFOUR), _mm_mul_pd(ir2, ir2)), _mm_mul_pd(ir2, DTHREE));
          vDouble i3i2 = _mm_sub_pd(_mm_mul_pd(ii2, DTHREE), _mm_mul_pd(_mm_mul_pd(ii2, DFOUR), _mm_mul_pd(ii2, ii2)));
          vDouble ur2 = _mm_sub_pd(_mm_mul_pd(r2, i3r2), _mm_mul_pd(i2, i3i2));
          vDouble ui2 = _mm_add_pd(_mm_mul_pd(r2, i3i2), _mm_mul_pd(i2, i3r2));
          _mm_store_pd(real+2, ur2);
          _mm_store_pd(imag+2, ui2);          
        
          vFloat d1 = _mm_load_ps(d);
          vFloat d2 = _mm_load_ps(d+4);

          vFloat cd1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur1, ur1)));
          vFloat cd2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur2, ur2)));
          cd1 = _mm_mul_ps(cd1, d1);
          cd2 = _mm_mul_ps(cd2, d2);
          d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
          d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
          vFloat td1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui1, ui1)));
          vFloat td2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui2, ui2)));
          td1 = _mm_xor_ps(td1, (vFloat) signFlip);
          td2 = _mm_xor_ps(td2, (vFloat) signFlip);
          td1 = _mm_mul_ps(td1, d1);
          td2 = _mm_mul_ps(td2, d2);
          cd1 = _mm_addsub_ps(cd1, td1);
          cd2 = _mm_addsub_ps(cd2, td2);
          
          _mm_store_ps(cd, cd1);
          _mm_store_ps(cd+4, cd2);
        }
        if (offset + ul_NumDataPoints == 1048576) CurrentChirpRateInd = abs(chirp_rate_ind);
        break;
      case 4:
        // cos(4x) = 8 cos^4 x - 8 cos^2 x + 1
        // sin(4x) = sin x (8 cos^3 x - 4 cos x)
        for (j = offset; j < offset+ul_NumDataPoints; j += 4) {
          const float *d = (const float *) (cx_DataArray + j);
          float *cd = (float *) (cx_ChirpDataArray + j);
          double *real = (SplitCurrentTrig.cos + j);
          double *imag = (SplitCurrentTrig.sin + j);
          const double *increal = (const double *) (SplitTrigStep.cos + j);
          const double *incimag = (const double *) (SplitTrigStep.sin + j);
        
          // load and increment trig array
          vDouble r1 = _mm_load_pd(real);
          vDouble i1 = _mm_load_pd(imag);
          vDouble ir1 = _mm_load_pd(increal);
          vDouble ii1 = _mm_load_pd(incimag);
          vDouble i4r1 = _mm_add_pd(_mm_mul_pd(_mm_sub_pd(_mm_mul_pd(_mm_mul_pd(ir1, ir1), _mm_mul_pd(ir1, ir1)), _mm_mul_pd(ir1, ir1)), DEIGHT), DONE);
          vDouble i4i1 = _mm_mul_pd(_mm_sub_pd(_mm_mul_pd(_mm_mul_pd(ir1, DEIGHT), _mm_mul_pd(ir1, ir1)), _mm_mul_pd(ir1, DFOUR)), ii1);
          vDouble ur1 = _mm_sub_pd(_mm_mul_pd(r1, i4r1), _mm_mul_pd(i1, i4i1));
          vDouble ui1 = _mm_add_pd(_mm_mul_pd(r1, i4i1), _mm_mul_pd(i1, i4r1));
          _mm_store_pd(real, ur1);
          _mm_store_pd(imag, ui1);
        
          vDouble r2 = _mm_load_pd(real+2);
          vDouble i2 = _mm_load_pd(imag+2);
          vDouble ir2 = _mm_load_pd(increal+2);
          vDouble ii2 = _mm_load_pd(incimag+2);
          vDouble i4r2 = _mm_add_pd(_mm_mul_pd(_mm_sub_pd(_mm_mul_pd(_mm_mul_pd(ir2, ir2), _mm_mul_pd(ir2, ir2)), _mm_mul_pd(ir2, ir2)), DEIGHT), DONE);
          vDouble i4i2 = _mm_mul_pd(_mm_sub_pd(_mm_mul_pd(_mm_mul_pd(ir2, DEIGHT), _mm_mul_pd(ir2, ir2)), _mm_mul_pd(ir2, DFOUR)), ii2);
          vDouble ur2 = _mm_sub_pd(_mm_mul_pd(r2, i4r2), _mm_mul_pd(i2, i4i2));
          vDouble ui2 = _mm_add_pd(_mm_mul_pd(r2, i4i2), _mm_mul_pd(i2, i4r2));
          _mm_store_pd(real+2, ur2);
          _mm_store_pd(imag+2, ui2);          
        
          vFloat d1 = _mm_load_ps(d);
          vFloat d2 = _mm_load_ps(d+4);

          vFloat cd1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur1, ur1)));
          vFloat cd2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ur2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ur2, ur2)));
          cd1 = _mm_mul_ps(cd1, d1);
          cd2 = _mm_mul_ps(cd2, d2);
          d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
          d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
          vFloat td1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui1)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui1, ui1)));
          vFloat td2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_movedup_pd(ui2)), _mm_cvtpd_ps(_mm_unpackhi_pd(ui2, ui2)));
          td1 = _mm_xor_ps(td1, (vFloat) signFlip);
          td2 = _mm_xor_ps(td2, (vFloat) signFlip);
          td1 = _mm_mul_ps(td1, d1);
          td2 = _mm_mul_ps(td2, d2);
          cd1 = _mm_addsub_ps(cd1, td1);
          cd2 = _mm_addsub_ps(cd2, td2);
          
          _mm_store_ps(cd, cd1);
          _mm_store_ps(cd+4, cd2);
        }
        if (offset + ul_NumDataPoints == 1048576) CurrentChirpRateInd = abs(chirp_rate_ind);
        break;
      case 5:
        // fall through for now, but do this one someday if I need the speed
      default:
        // in theory, we'll only come through here once per chirp group, since CalcSplitTrigArray will update everything
        CalcSplitTrigArray(ul_NumDataPoints, chirp_rate_ind);
        for (j = offset; j < offset+ul_NumDataPoints; j += 4) {
          const float *d = (const float *) (cx_DataArray + j);
          float *cd = (float *) (cx_ChirpDataArray + j);
          const double *real = (const double *) (SplitCurrentTrig.cos + j);
          const double *imag = (const double *) (SplitCurrentTrig.sin + j);
          
          vFloat d1 = _mm_load_ps(d);
          vFloat d2 = _mm_load_ps(d+4);
          
          vFloat cd1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(real)), _mm_cvtpd_ps(_mm_loaddup_pd(real+1)));
          vFloat cd2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(real+2)), _mm_cvtpd_ps(_mm_loaddup_pd(real+3)));
          cd1 = _mm_mul_ps(cd1, d1);
          cd2 = _mm_mul_ps(cd2, d2);
          d1 = (vFloat) _mm_shuffle_epi32((vSInt32) d1, 0xb1);
          d2 = (vFloat) _mm_shuffle_epi32((vSInt32) d2, 0xb1);
          vFloat td1 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(imag)), _mm_cvtpd_ps(_mm_loaddup_pd(imag+1)));
          vFloat td2 = _mm_movelh_ps(_mm_cvtpd_ps(_mm_loaddup_pd(imag+2)), _mm_cvtpd_ps(_mm_loaddup_pd(imag+3)));
          td1 = _mm_xor_ps(td1, (vFloat) signFlip);
          td2 = _mm_xor_ps(td2, (vFloat) signFlip);
          td1 = _mm_mul_ps(td1, d1);
          td2 = _mm_mul_ps(td2, d2);
          cd1 = _mm_addsub_ps(cd1, td1);
          cd2 = _mm_addsub_ps(cd2, td2);
          
          _mm_store_ps(cd, cd1);
          _mm_store_ps(cd+4, cd2);
        }
        break;
    }

#if 0
    // verify accuracy of computed chirp values
    for (j = offset; j < offset + ul_NumDataPoints; j++) {
      float c, d, real, imag;        

      double dd,cc;
      double time=static_cast<double>(j)/sample_rate;
      // since ang is getting moded by 2pi, we calculate "ang mod 2pi"
      // before the call to sincos() inorder to reduce roundoff error.
      // (Bug submitted by Tetsuji "Maverick" Rai)
      double ang  = 0.5*chirp_rate*time*time;
      ang -= floor(ang);
      ang *= M_PI*2;
      sincos(ang,&dd,&cc);
      
      c=cc;
      d=dd;

      if (chirp_rate_ind < 0) dd *= -1.0;

      // Sometimes chirping is done in place.
      // We don't want to overwrite data prematurely.
      real = cx_DataArray[j][0] * c - cx_DataArray[j][1] * d;
      imag = cx_DataArray[j][0] * d + cx_DataArray[j][1] * c;

      float sreal = cx_ChirpDataArray[j][0];
      float simag = cx_ChirpDataArray[j][1];
            
      if (real != 0.0 && fabs(sreal - real) / real > 0.00001 || imag != 0.0 && fabs(simag - imag) / imag > 0.00001) {
        printf("Bad chirp pair [%g %g]@%d (should have been [%g %g] at chirp index %d (delta %d))\n", sreal, simag, j, real, imag, chirp_rate_ind, increment);
        exit(0);
      }
    }
#endif    

    analysis_state.FLOP_counter+=12.0*ul_NumDataPoints;
  }
  return 0;
}
#endif

// Trigonometric arrays functions
// These functions makes trigonometric arrays very quickly and
// will speed up v_ChirpData()
//   by Tetsuji "Maverick" Rai

// initialize TrigStep and CurrentTrig

void InitTrigArray(int len, double ChirpStep, int InitChirpRateInd,
                   double SampleRate) {
  int i;
  double ang, Coef;

  TrigStep = (SinCosArray*) malloc_a (len * sizeof(SinCosArray), MEM_ALIGN);
  if (TrigStep == NULL) SETIERROR(MALLOC_FAILED, "TrigStep == NULL");

  CurrentTrig = (SinCosArray*) malloc_a (len * sizeof(SinCosArray), MEM_ALIGN);
  if (CurrentTrig == NULL) SETIERROR(MALLOC_FAILED, "CurrentTrig == NULL");

  // Make ChirpStep array

  Coef = ChirpStep / (SampleRate*SampleRate);

  for (i = 0; i < len; i++) {
    // since ang is getting cast to a float, we calculate "ang mod 2pi"
    // before the call to sincosf() inorder to reduce roundoff error.
    // (Bug submitted by Tetsuji "Maverick" Rai)
    // addition: now it's used as double float, but this bug fix
    // is still preferable
    ang = 0.5*(double)i*(double)i*Coef;
    ang -= floor(ang);
    ang *= 2*M_PI;
    sincos(ang, &TrigStep[i].Sin, &TrigStep[i].Cos);
  }

  // Set initial trigonometric array

  if ((CurrentChirpRateInd = abs(InitChirpRateInd)) != 0) {
    Coef = CurrentChirpRateInd*ChirpStep / (SampleRate*SampleRate);

    for (i = 0; i < len; i++) {
      ang = 0.5*(double)i*(double)i*Coef;
      ang -= floor(ang);
      ang *= 2*M_PI;
      sincos(ang, &CurrentTrig[i].Sin, &CurrentTrig[i].Cos);
    }
  } else {
    // if it starts from the beginning, it's quite simple
    for (i = 0; i < len; i++) {
      CurrentTrig[i].Sin = 0.;
      CurrentTrig[i].Cos = 1.0;
    }
  }
}

#if 0
void InitSplitTrigArray(int len, double ChirpStep, int InitChirpRateInd,
                   double SampleRate) {
  int i;
  double ang, Coef;

  SplitTrigStep.cos = (double *) malloc_a(len * sizeof(double), MEM_ALIGN);
  SplitTrigStep.sin = (double *) malloc_a(len * sizeof(double), MEM_ALIGN);
  
  SplitCurrentTrig.cos = (double *) malloc_a(len * sizeof(double), MEM_ALIGN);
  SplitCurrentTrig.sin = (double *) malloc_a(len * sizeof(double), MEM_ALIGN);
  
  // Make ChirpStep array

  Coef = ChirpStep / (SampleRate*SampleRate);

  for (i = 0; i < len; i++) {
    // since ang is getting cast to a float, we calculate "ang mod 2pi"
    // before the call to sincosf() inorder to reduce roundoff error.
    // (Bug submitted by Tetsuji "Maverick" Rai)
    // addition: now it's used as double float, but this bug fix
    // is still preferable
    ang = 0.5*(double)i*(double)i*Coef;
    ang -= floor(ang);
    ang *= 2*M_PI;
    sincos(ang, &SplitTrigStep.sin[i], &SplitTrigStep.cos[i]);
  }

  // Set initial trigonometric array

  if ((CurrentChirpRateInd = abs(InitChirpRateInd)) != 0) {
    Coef = CurrentChirpRateInd*ChirpStep / (SampleRate*SampleRate);

    for (i = 0; i < len; i++) {
      ang = 0.5*(double)i*(double)i*Coef;
      ang -= floor(ang);
      ang *= 2*M_PI;
      sincos(ang, &SplitCurrentTrig.sin[i], &SplitCurrentTrig.cos[i]);
    }
  } else {
    // if it starts from the beginning, it's quite simple
    for (i = 0; i < len; i++) {
      SplitCurrentTrig.sin[i] = 0.;
      SplitCurrentTrig.cos[i] = 1.0;
    }
  }
}
#endif

// calculate next CurrentTrig

void CalcTrigArray (int len, int ChirpRateInd) {
  int TempCRateInd = abs(ChirpRateInd);
  int i, j;

  // skip automatically when TempCRateInd == CurrentChirpRateInd
  //   (it happens when sign of chirprate reverses)
  // Otherwise
  //     sin(x+step) = sin(x)*cos(step) + cos(x)*sin(step)
  //     cos(x+step) = cos(x)*cos(step) - sin(x)*sin(step)

  // In most cases index increases by 1, and in the later phase of a WU
  // by 4 at most in reference WU, but it's 2-3 times faster than
  // sincos() function on P4 with gcc or icc.  When index increases by 1,
  // this is about 10 times as fast as sincos()
#ifdef DEBUG
  fprintf(stderr, " New ind = %6d (abs(%6d))  Previous = %6d\n",
          ChirpRateInd, TempCRateInd, CurrentChirpRateInd);
  fflush(stderr);
#endif

  for (i = CurrentChirpRateInd; i < TempCRateInd; i++) {
    for (j = 0; j < len; j++) {
      double SinX, CosX, SinS, CosS;

      SinX = CurrentTrig[j].Sin;
      CosX = CurrentTrig[j].Cos;
      SinS = TrigStep[j].Sin;
      CosS = TrigStep[j].Cos;

      CurrentTrig[j].Sin = SinX * CosS + CosX * SinS;
      CurrentTrig[j].Cos = CosX * CosS - SinX * SinS;
    }
  }

  CurrentChirpRateInd = TempCRateInd;
}

#if 0
void CalcSplitTrigArray (int len, int ChirpRateInd) {
  int TempCRateInd = abs(ChirpRateInd);
  int i, j, k;

  for (i = len - 32768; i >= 0; i -= 32768) {
    for (j = CurrentChirpRateInd; j < TempCRateInd; j++) {
      for (k = i; k < i+32768; k++) {
        double SinX, CosX, SinS, CosS;

        SinX = SplitCurrentTrig.sin[j];
        CosX = SplitCurrentTrig.cos[j];
        SinS = SplitTrigStep.sin[j];
        CosS = SplitTrigStep.cos[j];

        SplitCurrentTrig.sin[j] = SinX * CosS + CosX * SinS;
        SplitCurrentTrig.cos[j] = CosX * CosS - SinX * SinS;
      }
    }    
  }

  CurrentChirpRateInd = TempCRateInd;
}
#endif

// free TrigStep and CurrentTrig

void FreeTrigArray() {
  if (TrigStep) free_a(TrigStep);
  TrigStep = NULL;

  if (CurrentTrig) free_a(CurrentTrig);
  CurrentTrig = NULL;
}

#if 0
void FreeSplitTrigArray() {
  printf("freeing split trig arrays\n");
  
  if (SplitTrigStep.cos) free_a(SplitTrigStep.cos);
  SplitTrigStep.cos = NULL;

  if (SplitTrigStep.sin) free_a(SplitTrigStep.sin);
  SplitTrigStep.sin = NULL;

  if (SplitCurrentTrig.cos) free_a(SplitCurrentTrig.cos);
  SplitCurrentTrig.cos = NULL;

  if (SplitCurrentTrig.sin) free_a(SplitCurrentTrig.sin);
  SplitCurrentTrig.sin = NULL;
}
#endif
