// Copyright 2003 Regents of the University of California

// SETI_BOINC is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2, or (at your option) any later
// version.

// SETI_BOINC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.

// Copyright 2003 Regents of the University of California

// SETI_BOINC is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2, or (at your option) any later
// version.

// SETI_BOINC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.

// You should have received a copy of the GNU General Public License along
// with SETI_BOINC; see the file COPYING.  If not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// In addition, as a special exception, the Regents of the University of
// California give permission to link the code of this program with libraries
// that provide specific optimized fast Fourier transform (FFT) functions and
// distribute a linked executable.  You must obey the GNU General Public 
// License in all respects for all of the code used other than the FFT library
// itself.  Any modification required to support these libraries must be
// distributed in source code form.  If you modify this file, you may extend 
// this exception to your version of the file, but you are not obligated to 
// do so. If you do not wish to do so, delete this exception statement from 
// your version.


/*
 * $Id: pulsefind.cpp,v 1.16.2.12 2006/05/16 21:54:19 korpela Exp $
 *
 */

#include "config.h"

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <climits>
#include <cstring>
#include <ctime>

#include "diagnostics.h"
#include "util.h"
#include "s_util.h"
#include "boinc_api.h"
#ifdef BOINC_APP_GRAPHICS
#include "graphics_api.h"
#ifdef DYNAMIC_GRAPHICS
#include "graphics_lib.h"
#endif
#endif

#ifdef __SSE3__
#pragma message("---- SSE3 ----")
#include <pmmintrin.h>
#elif defined __SSE2__
#pragma message("---- SSE2 ----")
#include <emmintrin.h>
#elif defined __SSE__
#pragma message("---- SSE ----")
#include <xmmintrin.h>
#endif

#ifdef USE_SSE2
#pragma message("------> Enabling SSE3 emulation by SSE2 ----")
#include "SSE3_emulate.h"
#endif

#ifdef USE_I386_CORE2
#include <tmmintrin.h>
#endif

#include "seti.h"
#include "seti_header.h"
#include "analyzePoT.h"
#include "analyzeReport.h"
#include "worker.h"
#include "malloc_a.h"
#include "lcgamm.h"

#include "analyzefuncs.h"

//#define DEBUG_TRIPLET_VERBOSE
//#define DEBUG_PULSE_VERBOSE
//#define DEBUG_PULSE_BOUNDS

#ifndef M_PI
#define M_PI (3.14159265358979323846)
#endif
#ifndef M_SQRT2
#define M_SQRT2 (1.41421356237309504880)
#endif
#ifndef M_SQRT3
#define M_SQRT3 (1.73205080756887719318)
#endif
#ifndef M_SQRT5
#define M_SQRT5 (2.23606797749978980505)
#endif
#ifndef M_SQRT1_2
#define M_SQRT1_2 (0.70710678118654752440)
#endif
#ifndef M_SQRT1_3
#define M_SQRT1_3 (0.57735026918962584208)
#endif
#ifndef M_SQRT1_5
#define M_SQRT1_5 (0.44721359549995792770)
#endif

//#include <Accelerate/Accelerate.h>

#define REL_ERR(x, ref) fabs((x - ref) / ref)

#if defined(__VEC__) || defined(__SSE2__)
// Altivec constants used by the foldArrayBy{2,3,4,5} functions
#define ZERO		((vFloat) {0.0f, 0.0f, 0.0f, 0.0f})
#define RECIP_TWO	((vFloat) {1.0f/2.0f, 1.0f/2.0f, 1.0f/2.0f, 1.0f/2.0f})
#define RECIP_THREE	((vFloat) {1.0f/3.0f, 1.0f/3.0f, 1.0f/3.0f, 1.0f/3.0f})
#define RECIP_FOUR	((vFloat) {1.0f/4.0f, 1.0f/4.0f, 1.0f/4.0f, 1.0f/4.0f})
#define RECIP_FIVE	((vFloat) {1.0f/5.0f, 1.0f/5.0f, 1.0f/5.0f, 1.0f/5.0f})
#define IFOUR		(_mm_set_epi32(4, 4, 4, 4))
#endif

#ifdef USE_PPC_OPTIMIZATIONS
#define USE_VECTORIZED_PULSE
#define USE_VECTORIZED_TRIPLET
#elif defined(USE_I386_OPTIMIZATIONS)
#define USE_VECTORIZED_PULSE
#define USE_VECTORIZED_TRIPLET
#endif

#ifdef __VEC__
inline vector float vec_fpsplat(float f_Constant) {
  vector unsigned char vuc_Splat;
  vector float vf_Constant;
  
  vuc_Splat = vec_lvsl(0, &f_Constant);
  vf_Constant = vec_lde(0, &f_Constant);
  vuc_Splat = (vector unsigned char) vec_splat((vector float) vuc_Splat, 0);
  vf_Constant = vec_perm(vf_Constant, vf_Constant, vuc_Splat);
  
  return vf_Constant;
}
#endif

/**********************
 *
 *   find_triplets - Analyzes the input signal for a triplet signal.
 *   
 *   Written by Eric Heien.
 */
int find_triplets( const float *power, int len_power, float triplet_thresh, int time_bin, int freq_bin ) {
  static int    *binsAboveThreshold;
  int            i,n,numBinsAboveThreshold=0,p,q;
  float        midpoint,mean_power=0,peak_power,period;
#if defined(__VEC__) && defined(USE_VECTORIZED_TRIPLET)
  vFloat vtempmean_power;
#endif
#if (defined(__VEC__) || defined(__SSE3__)) && defined(USE_VECTORIZED_TRIPLET)
  vFloat vmean_power;
#endif
  
  if (!binsAboveThreshold) {
    binsAboveThreshold=(int *)malloc_a(PoTInfo.TripletMax*sizeof(int),MEM_ALIGN);
    if (!binsAboveThreshold) SETIERROR(MALLOC_FAILED, "!binsAboveThreshold");
  }

  /* Get all the bins that are above the threshold, and find the power array mean value */
#ifdef DEBUG_TRIPLET_VERBOSE
  fprintf(stderr, "In find_triplets()...   PulsePotLen = %d   triplet_thresh = %f   TOffset = %d   PoT = %d\n",  len_power,  triplet_thresh, time_bin, freq_bin);
#endif

#if defined(__VEC__) && defined(USE_VECTORIZED_TRIPLET)
  int vEnd = len_power - (len_power & 3);
  vtempmean_power = ZERO;
  for (i = 0; i < vEnd ; i+=4) {
    const float *p = (const float *) (power + i);
    vector float p1 = vec_ld(0, p);
    
    vtempmean_power = vec_add(vtempmean_power, p1);
  }
  vmean_power = vtempmean_power;
  for (;i<len_power;i++) {
    mean_power += power[i];
  }
  float *vpmean_power = (float *) &vmean_power;
  mean_power += (vpmean_power[0] + vpmean_power[1]) + (vpmean_power[2] + vpmean_power[3]);
  mean_power /= (float)len_power;
#elif defined(__SSE3__) && defined(USE_VECTORIZED_TRIPLET)
  int vEnd = len_power - (len_power & 3);
  vmean_power = _mm_setzero_ps();
  for (i = 0; i < vEnd ; i+=4) {
    vmean_power = _mm_add_ps(vmean_power, _mm_load_ps(power+i));
  }
//  vmean_power = _mm_hadd_ps(vmean_power, vmean_power);
//  vmean_power = _mm_hadd_ps(vmean_power, vmean_power);
  vmean_power = _mm_add_ps(vmean_power, _mm_movehl_ps(vmean_power, vmean_power));
  vmean_power = _mm_add_ss(vmean_power, _mm_movehdup_ps(vmean_power));
//  _mm_store_ss(&mean_power, vmean_power);
  mean_power = _mm_cvtss_f32(vmean_power);
  for (;i<len_power;i++) {
    mean_power += power[i];
  }
  mean_power /= (float)len_power;  
#elif defined(USE_VECTORIZED_TRIPLET)
  vDSP_meanv((float *) power, 1, &mean_power, len_power);
#else
  for( i=0;i<len_power;i++ ) {
    mean_power += power[i];
  }
  mean_power /= (float)len_power;
#endif
  triplet_thresh*=mean_power;
#if defined(__SSE2__) && defined(USE_VECTORIZED_TRIPLET)
  vFloat vTripletThresh = _mm_set1_ps(triplet_thresh);
  vUInt32 vi = _mm_set_epi32(3, 2, 1, 0);
  vEnd = len_power & ~0x3;
  for (i = 0; i < vEnd; i += 4) {
    vFloat vPower = _mm_load_ps(power+i);
    vFloat vcmp = _mm_cmpge_ps(vPower, vTripletThresh);
    if (_mm_movemask_ps(vcmp)) {
      _mm_store_si128((__m128i *) (binsAboveThreshold + numBinsAboveThreshold), vi);
      numBinsAboveThreshold += 4;
    }
    vi = _mm_add_epi32(vi, IFOUR);
  }
  for (; i < len_power; i++) {
    if (power[i] >= triplet_thresh) {
      binsAboveThreshold[numBinsAboveThreshold] = i;
      numBinsAboveThreshold++;
    }
  }
#elif defined(__VEC__) && defined(USE_VECTORIZED_TRIPLET)
  vector float vTripletThresh = vec_fpsplat(triplet_thresh);
  vUInt32 vi = {0, 1, 2, 3};
  vEnd = len_power & ~0x3;
  for (i = 0; i < vEnd; i += 4) {
    const float *p = (const float *) (power + i);
    vFloat vPower = vec_ld(0, p);
    if (vec_any_ge(vPower, vTripletThresh)) {
      vec_st(vi, 0, (vUInt32 *) (binsAboveThreshold + numBinsAboveThreshold));
      numBinsAboveThreshold += 4;
    }
    vi = vec_add(vi, IFOUR);
  }
  for (; i < len_power; i++) {
    if (power[i] >= triplet_thresh) {
      binsAboveThreshold[numBinsAboveThreshold] = i;
      numBinsAboveThreshold++;
    }
  }
#else
  for ( i=0;i<len_power;i++ ) {
    if( power[i] >= triplet_thresh ) {
      binsAboveThreshold[numBinsAboveThreshold] = i;
      numBinsAboveThreshold++;
    }
  }
#endif
  analysis_state.FLOP_counter+=10.0+len_power;

  /* Check each bin combination for a triplet */
  if (numBinsAboveThreshold>2) {
    for( i=0;i<numBinsAboveThreshold-1;i++ ) {
      if (power[binsAboveThreshold[i]] < triplet_thresh) continue;
      
      for( n=i+1;n<numBinsAboveThreshold;n++ ) {
        if (power[binsAboveThreshold[n]] < triplet_thresh) continue;
        
        midpoint = (binsAboveThreshold[i]+binsAboveThreshold[n])/2.0f;
        period = (float)fabs((binsAboveThreshold[i]-binsAboveThreshold[n])/2.0f);

        /* Get the peak power of this triplet */
        peak_power = power[binsAboveThreshold[i]];

        if( power[binsAboveThreshold[n]] > peak_power )
          peak_power = power[binsAboveThreshold[n]];

        p = binsAboveThreshold[i];
        while( (power[p] >= triplet_thresh) && (p <= (static_cast<int>(floor(midpoint)))) ) {    /* Check if there is a pulse "off" in between init and midpoint */
          p++;
        }

        q = static_cast<int>(floor(midpoint))+1;
        while( (power[q] >= triplet_thresh) && (q <= binsAboveThreshold[n])) {    /* Check if there is a pulse "off" in between midpoint and end */
          q++;
        }

        if( p >= static_cast<int>(floor(midpoint)) || q >= binsAboveThreshold[n]) {
          /* if this pulse doesn't have an "off" between all the three spikes, it's dropped */
        } else if( (midpoint - floor(midpoint)) > 0.1f ) {    /* if it's spread among two bins */
          if( power[(int)midpoint] >= triplet_thresh ) {
            if( power[(int)midpoint] > peak_power )
              peak_power = power[(int)midpoint];

            ReportTripletEvent( peak_power/mean_power, mean_power, period, midpoint,time_bin, freq_bin, len_power, power, 1 );
          }

          if( power[(int)(midpoint+1.0f)] >= triplet_thresh ) {
            if( power[(int)(midpoint+1.0f)] > peak_power )
              peak_power = power[(int)(midpoint+1.0f)];

            ReportTripletEvent( peak_power/mean_power, mean_power, period, midpoint,time_bin, freq_bin, len_power, power, 1 );
          }
        } else {            /* otherwise just check the single midpoint bin */
          if( power[(int)midpoint] >= triplet_thresh ) {
            if( power[(int)midpoint] > peak_power )
              peak_power = power[(int)midpoint];

            ReportTripletEvent( peak_power/mean_power, mean_power, period, midpoint,time_bin, freq_bin, len_power, power, 1 );
          }
        }
      }
    }
  }
  analysis_state.FLOP_counter+=(10.0*numBinsAboveThreshold*numBinsAboveThreshold);
  return (0);
}

static struct tftab {
  int n;
  float y;
}
*t_funct_tab;

void init_t_funct(void) {
  if (!t_funct_tab) {
    int tablen = (PoTInfo.PulseMax+2)/3 +
    3*(int)(log((float)PoTInfo.PulseMax)/log(2.0)-3) - 1;
    t_funct_tab=(struct tftab *)malloc_a(tablen*sizeof(struct tftab),MEM_ALIGN);
    if (!t_funct_tab) SETIERROR(MALLOC_FAILED, "!t_funct_tab");
    memset(t_funct_tab, 0, tablen*sizeof(struct tftab));
  }  
}

void update_t_funct(int m, int n, int x) {
  t_funct_tab[x].n=n;
  t_funct_tab[x].y=
    invert_lcgf((float)(-PoTInfo.PulseThresh-log((float)m)),(float)n,
                               (float)1e-4)/n;
}

/**********************
 *
 * t_funct - Caching routine for calls to invert_lcgf, return cache value if present
 *
 */
float t_funct(int m, int n, int x) {
  if (t_funct_tab[x].n!=n) {
    t_funct_tab[x].n=n;
    t_funct_tab[x].y=
      invert_lcgf((float)(-PoTInfo.PulseThresh-log((float)m)),(float)n,
                  (float)1e-4)/n;
  }
  return t_funct_tab[x].y;
}

#ifdef __VEC__
inline vector unsigned int vec_uisplat(unsigned int ui_Constant) {
  vector unsigned char vuc_Splat;
  vector unsigned int vui_Constant;
  
  vuc_Splat = vec_lvsl(0, &ui_Constant);
  vui_Constant = vec_lde(0, &ui_Constant);
  vuc_Splat = (vector unsigned char) vec_splat((vector unsigned int) vuc_Splat, 0);
  vui_Constant = vec_perm(vui_Constant, vui_Constant, vuc_Splat);
  
  return vui_Constant;
}

#if 0
/**********************
*
* foldArrayBy2 - Perform "folds" on 2 adjacent parts of the power over time
*   array. Each element in the "folded" array is the average of the 2 elements
*   in the power over time array. Array is folded in-place.
*
* Assumes 16-byte alignment of input array, but offsets within the array may be
*   arbitrary.
*
*/
static float
foldArrayBy2(float *PoT, int end, int off0, int off1, int offst) {
  float max = 0.0f;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq1 = vec_ld(0, PoT + off0);
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msqz;
  vector unsigned char mask1 = vec_add(vec_lvsl(-1L, PoT + off0), vec_splat_u8(1));
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char maskst = vec_lvsr(0, PoT + offst);
  int i = 0, ist = 0, index = 0;
  
//  int d_elemsWritten = 0;
  
  vEnd = end - (end & 0x3);
  if (vEnd > 0) {
    // handle unaligned output pointers
    if (((intptr_t) (PoT + offst)) & 0xf) {
      float *pst = PoT + offst;
      vector float lsq1 = vec_ld(15, PoT + off0);
      vector float lsq2 = vec_ld(15, PoT + off1);
      vector float x1 = vec_perm(msq1, lsq1, mask1);
      vector float x2 = vec_perm(msq2, lsq2, mask2);
      vector float z = vec_madd(vec_add(x1, x2), RECIP_TWO, ZERO);
      vector float zst = vec_perm(z, z, maskst);
      
      // store up to the first 16-byte aligned address
      if (((intptr_t) pst) & sizeof(unsigned char)) {
        vec_ste((vector unsigned char) zst, index, (unsigned char *) (PoT + offst));
        index += sizeof(unsigned char);
      }
      if (((intptr_t) pst + index) & sizeof(unsigned short)) {
        vec_ste((vector unsigned short) zst, index, (unsigned short *) (PoT + offst));
        index += sizeof(unsigned short);
      }
      while (((intptr_t) pst + index) & (sizeof(zst) - sizeof(unsigned int))) {
        vec_ste((vector unsigned int) zst, index, (unsigned int *) (PoT + offst));
        index += sizeof(unsigned int);
      }
      
      msq1 = lsq1;
      msq2 = lsq2;
      msqz = z;
      
      tempMaxV = vec_max(tempMaxV, z);
      i += 4;
    }

    // main loop
    for (; i < vEnd; i += 4, ist += 4) {
      const float *p = PoT + i;
      float *pst = PoT + ist;
      vector float lsq1 = vec_ld(15, p + off0);
      vector float lsq2 = vec_ld(15, p + off1);
      vector float x1 = vec_perm(msq1, lsq1, mask1);
      vector float x2 = vec_perm(msq2, lsq2, mask2);
      vector float lsqz = vec_madd(vec_add(x1, x2), RECIP_TWO, ZERO);
      vector float zst = vec_perm(msqz, lsqz, maskst);
      
      vec_st(zst, index, pst + offst);

      msq1 = lsq1;
      msq2 = lsq2;
      msqz = lsqz;
      
      tempMaxV = vec_max(tempMaxV, lsqz);
    }
    
    // handle unstored partial vector
    if (index > 0) {
      vector float zst = vec_perm(msqz, msqz, maskst);
      while (index <= sizeof(zst) - sizeof(unsigned int)) {
        vec_ste((vector unsigned int) zst, index, (unsigned int *) (PoT + offst + ist));
        index += sizeof(unsigned int);
      }
      if (index <= sizeof(zst) - sizeof(unsigned short)) {
        vec_ste((vector unsigned short) zst, index, (unsigned short *) (PoT + offst + ist));
        index += sizeof(unsigned int);
      }
      if (index <= sizeof(zst) - sizeof(unsigned char)) {
        vec_ste((vector unsigned char) zst, index, (unsigned char *) (PoT + offst + ist));
      }
      
      ist += 4;
    }
  }
  
  // handle tail elements
  for (; i < end; ++i) {
    float x = (PoT[i + off0] + PoT[i + off1]) / 2.0f;
    
    PoT[i + offst] = x;
    if (x > max)
      max = x;
  }

  // take the maximum across max and maxV and return it
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(maxV, vec_sld(maxV, maxV, 4));
  float *maxVp = (float *) &maxV;
  if (*maxVp > max) max = *maxVp;
  
  return max;  
}


static float
foldArrayBy2SP(float *PoT, int end, int off0, int off1, int offst) {
  float max;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq1 = vec_ld(0, PoT + off0);
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msqz;
  vector unsigned char mask1 = vec_add(vec_lvsl(-1L, PoT + off0), vec_splat_u8(1));
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char maskst = vec_lvsr(0, PoT + offst);
  int i = 0, ist = 0, index = 0;
  
  //  int d_elemsWritten = 0;
  
  vEnd = end - (end & 0x3);
  if (vEnd > 0) {
    // handle unaligned output pointers
    if (((intptr_t) (PoT + offst)) & 0xf) {
      float *pst = PoT + offst;
      vector float lsq1 = vec_ld(15, PoT + off0);
      vector float lsq2 = vec_ld(15, PoT + off1);
      vector float x1 = vec_perm(msq1, lsq1, mask1);
      vector float x2 = vec_perm(msq2, lsq2, mask2);
      vector float z = vec_madd(vec_add(x1, x2), RECIP_TWO, ZERO);
      vector float zst = vec_perm(z, z, maskst);
      
      // store up to the first 16-byte aligned address
      if (((intptr_t) pst) & sizeof(unsigned char)) {
        vec_ste((vector unsigned char) zst, index, (unsigned char *) (PoT + offst));
        index += sizeof(unsigned char);
      }
      if (((intptr_t) pst + index) & sizeof(unsigned short)) {
        vec_ste((vector unsigned short) zst, index, (unsigned short *) (PoT + offst));
        index += sizeof(unsigned short);
      }
      while (((intptr_t) pst + index) & (sizeof(zst) - sizeof(unsigned int))) {
        vec_ste((vector unsigned int) zst, index, (unsigned int *) (PoT + offst));
        index += sizeof(unsigned int);
      }
      
      msq1 = lsq1;
      msq2 = lsq2;
      msqz = z;
      
      tempMaxV = vec_max(tempMaxV, z);
      i += 4;
    }

    const float *p1 = PoT + off0 + i, *p2 = PoT + off1 + i;
    float *pst = PoT + offst + ist;
    vector float lsq1, lsq2, x1, x2, xs, lsqz, zst;
    
#define F2_STAGE1 lsq1 = vec_ld(15, p1); lsq2 = vec_ld(15, p2); i += 4; p1 += 4; p2 += 4;
#define F2_STAGE2 x1 = vec_perm(msq1, lsq1, mask1); x2 = vec_perm(msq2, lsq2, mask2); msq1 = lsq1; msq2 = lsq2; xs = vec_add(x1, x2);
#define F2_STAGE3 lsqz = vec_madd(xs, RECIP_TWO, ZERO);
#define F2_STAGE4 tempMaxV = vec_max(tempMaxV, lsqz); zst = vec_perm(msqz, lsqz, maskst); msqz = lsqz; vec_st(zst, index, pst); ist += 4; pst += 4;
    
    // software pipelined main loop
    if (vEnd-i >= 12) {
      F2_STAGE1;
      F2_STAGE2; F2_STAGE1;
      F2_STAGE3; F2_STAGE2; F2_STAGE1;
      while (i < vEnd) {
        F2_STAGE4; F2_STAGE3; F2_STAGE2; F2_STAGE1;
      }
      F2_STAGE4; F2_STAGE3; F2_STAGE2;
      F2_STAGE4; F2_STAGE3;
      F2_STAGE4;
    }
    
    // main loop
    while (i < vEnd) {
      F2_STAGE1; F2_STAGE2; F2_STAGE3; F2_STAGE4;
    }
    
#undef F2_STAGE1
#undef F2_STAGE2
#undef F2_STAGE3
#undef F2_STAGE4
    
    // handle unstored partial vector
    if (index > 0) {
      vector float zst = vec_perm(msqz, msqz, maskst);
      while (index <= sizeof(zst) - sizeof(unsigned int)) {
        vec_ste((vector unsigned int) zst, index, (unsigned int *) (PoT + offst + ist));
        index += sizeof(unsigned int);
      }
      if (index <= sizeof(zst) - sizeof(unsigned short)) {
        vec_ste((vector unsigned short) zst, index, (unsigned short *) (PoT + offst + ist));
        index += sizeof(unsigned int);
      }
      if (index <= sizeof(zst) - sizeof(unsigned char)) {
        vec_ste((vector unsigned char) zst, index, (unsigned char *) (PoT + offst + ist));
      }
      
      ist += 4;
    }
  }
  
  // take the maximum across max and maxV and return it
  tempMaxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 4));
  float *maxVp = (float *) &maxV;
  max = *maxVp;
  
  // handle tail elements
  for (; i < end; ++i) {
    float x = (PoT[i + off0] + PoT[i + off1]) / 2.0f;
    
    PoT[i + offst] = x;
    if (x > max)
      max = x;
  }
  
  return max;  
}
#endif

#if 1
static float
foldArrayBy2SPA(float *PoT, int end, int off0, int off1, int offst) {
	vector float max, maxa, maxb;
	vector float lsq2a, lsq2b, msq2;
	vector unsigned char mask2;
	vector float x1a, x1b, x2a, x2b, zsta, zstb;
	vector unsigned int taillim, tailmaska, tailmaskb;
	const vector unsigned int tailelema = {0, 1, 2, 3};
	const vector unsigned int tailelemb = {4, 5, 6, 7};
	int i = 0;
	
	const float *s;
	unsigned int o0a, o1a;
	unsigned int o0b, o1b;
	float *d;
	
	s = (const float *) PoT;
	o0a = off0 * sizeof(float);
	o0b = off0 * sizeof(float) + 16;
	o1a = off1 * sizeof(float) + 15;
	o1b = off1 * sizeof(float) + 31;
	d = PoT + offst;
	
	maxa = vec_xor(maxa, maxa);
	maxb = vec_xor(maxb, maxb);
	msq2 = vec_ld(0, PoT + off1);
	mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
	taillim = vec_uisplat(((end - 1) & 0x7) + 1);
	
#if 0
	#define STAGE_ADVANCE 4
    #define F2_STAGE1 x1 = vec_ld(o0, s); lsq2 = vec_ld(o1, s); i += 4; s += 4;
    #define F2_STAGE2 x2 = vec_perm(msq2, lsq2, mask2); msq2 = lsq2; zst = vec_add(x1, x2);
    #define F2_STAGE3 max = vec_max(max, zst); vec_st(zst, 0, d); d += 4;
#endif
	
	#define STAGE_ADVANCE 8
	#define F2_STAGE1 lsq2a = vec_ld(o1a, s); lsq2b = vec_ld(o1b, s); x1a = vec_ld(o0a, s); x1b = vec_ld(o0b, s); i += 8; s += 8;
	#define F2_STAGE2 x2a = vec_perm(msq2, lsq2a, mask2); x2b = vec_perm(lsq2a, lsq2b, mask2); msq2 = lsq2b; zsta = vec_add(x1a, x2a); zstb = vec_add(x1b, x2b);
	#define F2_STAGE3 maxa = vec_max(maxa, zsta); maxb = vec_max(maxb, zstb); vec_st(zsta, 0, d); vec_st(zstb, 16, d); d += 8;
	    
	if (end >= 2*STAGE_ADVANCE) {
		F2_STAGE1;
		F2_STAGE2; F2_STAGE1;
		while (i < end) {
			F2_STAGE3; F2_STAGE2; F2_STAGE1;
		}
		F2_STAGE3; F2_STAGE2;
	} else {
		while (i < end-STAGE_ADVANCE) {
			F2_STAGE1; F2_STAGE2; F2_STAGE3;
		}
		F2_STAGE1; F2_STAGE2;
	}
	
	tailmaska = (vector unsigned int) vec_cmplt(tailelema, taillim);
	tailmaskb = (vector unsigned int) vec_cmplt(tailelemb, taillim);
	zsta = vec_and(zsta, (vector float) tailmaska);
	zstb = vec_and(zstb, (vector float) tailmaskb);
	maxa = vec_max(maxa, zsta);
	maxb = vec_max(maxb, zstb);
	vec_st(zsta, 0, d);
	vec_st(zstb, 16, d);
	
	maxa = vec_max(maxa, maxb);
	maxa = vec_max(maxa, vec_sld(maxa, maxa, 8));
	max = vec_max(maxa, vec_sld(maxa, maxa, 4));
	
	float *maxp = (float *) &max;
	return *maxp;
}

static float
foldArrayBy2SPAL(float *PoT, int end, int off0, int off1, int offst) {
	vector float max, maxa, maxb;
	vector float x1a, x1b, x2a, x2b, zsta, zstb;
	vector unsigned int taillim, tailmaska, tailmaskb;
	const vector unsigned int tailelema = {0, 1, 2, 3};
	const vector unsigned int tailelemb = {4, 5, 6, 7};
	int i = 0;
	
	const float *s;
	unsigned int o0a, o1a;
	unsigned int o0b, o1b;
	float *d;
	
	s = (const float *) PoT;
	o0a = off0 * sizeof(float);
	o0b = off0 * sizeof(float) + 16;
	o1a = off1 * sizeof(float);
	o1b = off1 * sizeof(float) + 16;
	d = PoT + offst;
	
	maxa = vec_xor(maxa, maxa);
	maxb = vec_xor(maxb, maxb);
	taillim = vec_uisplat(((end - 1) & 0x7) + 1);
	
#if 0
	#define STAGE_ADVANCE 4
    #define F2_STAGE1 x1 = vec_ld(o0, s); lsq2 = vec_ld(o1, s); i += 4; s += 4;
    #define F2_STAGE2 x2 = vec_perm(msq2, lsq2, mask2); msq2 = lsq2; zst = vec_add(x1, x2);
    #define F2_STAGE3 max = vec_max(max, zst); vec_st(zst, 0, d); d += 4;
#endif
	
	#define STAGE_ADVANCE 8
	#define F2A_STAGE1 x1a = vec_ld(o0a, s); x1b = vec_ld(o0b, s); x2a = vec_ld(o1a, s); x2b = vec_ld(o1b, s); i += 8; s += 8;
	#define F2A_STAGE2 zsta = vec_add(x1a, x2a); zstb = vec_add(x1b, x2b);
	#define F2A_STAGE3 maxa = vec_max(maxa, zsta); maxb = vec_max(maxb, zstb); vec_st(zsta, 0, d); vec_st(zstb, 16, d); d += 8;
	    
	if (end >= 2*STAGE_ADVANCE) {
		F2A_STAGE1;
		F2A_STAGE2; F2A_STAGE1;
		while (i < end) {
			F2A_STAGE3; F2A_STAGE2; F2A_STAGE1;
		}
		F2A_STAGE3; F2A_STAGE2;
	} else {
		while (i < end-STAGE_ADVANCE) {
			F2A_STAGE1; F2A_STAGE2; F2A_STAGE3;
		}
		F2A_STAGE1; F2A_STAGE2;
	}
	
	tailmaska = (vector unsigned int) vec_cmplt(tailelema, taillim);
	tailmaskb = (vector unsigned int) vec_cmplt(tailelemb, taillim);
	zsta = vec_and(zsta, (vector float) tailmaska);
	zstb = vec_and(zstb, (vector float) tailmaskb);
	maxa = vec_max(maxa, zsta);
	maxb = vec_max(maxb, zstb);
	vec_st(zsta, 0, d);
	vec_st(zstb, 16, d);
	
	maxa = vec_max(maxa, maxb);
	maxa = vec_max(maxa, vec_sld(maxa, maxa, 8));
	max = vec_max(maxa, vec_sld(maxa, maxa, 4));
	
	float *maxp = (float *) &max;
	return *maxp;
}
#endif

#if 0 && OLD_FOLDARRAY
static float
foldArrayBy2SPA(float *PoT, int end, int off0, int off1, int offst) {
  float max;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msqz;
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned int tailelem = {0, 1, 2, 3};
  vector unsigned int taillim = vec_uisplat(((end-1)&0x3) + 1);
  vector unsigned int tailmask;
  int i = 0, index = 0;

//  if (end < 60) printf("end = %d\n", end);
  
  //  int d_elemsWritten = 0;
  const float *p1 = PoT + off0, *p2 = PoT + off1;
  float *pst = PoT + offst;
  vector float lsq2, x1, x2, zst;
      
#define F2_STAGE1 x1 = vec_ld(0, p1); lsq2 = vec_ld(15, p2); i += 4; p1 += 4; p2 += 4;
#define F2_STAGE2 x2 = vec_perm(msq2, lsq2, mask2); msq2 = lsq2; zst = vec_add(x1, x2);
#define F2_STAGE3 tempMaxV = vec_max(tempMaxV, zst); vec_st(zst, index, pst); pst += 4;
    
  // software pipelined main loop - omit last stage of tail element
  if (end-i >= 8) {
    F2_STAGE1;
    F2_STAGE2; F2_STAGE1;
    while (i < end) {
      F2_STAGE3; F2_STAGE2; F2_STAGE1;
    }
    F2_STAGE3; F2_STAGE2;
//    F2_STAGE3;
  } else {    
    // main loop
    while (i < end-4) {
      F2_STAGE1; F2_STAGE2; F2_STAGE3;
    }
    F2_STAGE1; F2_STAGE2;
  }
  
#undef F2_STAGE1
#undef F2_STAGE2
#undef F2_STAGE3
  
  tailmask = (vector unsigned int) vec_cmplt(tailelem, taillim);
  zst = vec_and(zst, (vector float) tailmask);
  tempMaxV = vec_max(tempMaxV, zst);
  vec_st(zst, index, pst);
  
  // take the maximum across max and maxV and return it
  tempMaxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 4));
  float *maxVp = (float *) &maxV;
  max = *maxVp;
  
  return max;  
}

static float
foldArrayBy2SPAL(float *PoT, int end, int off0, int off1, int offst) {
  float max;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector unsigned int tailelem = {0, 1, 2, 3};
  vector unsigned int taillim = vec_uisplat(((end-1)&0x3) + 1);
  vector unsigned int tailmask;
  int i = 0, index = 0;
  
  //  int d_elemsWritten = 0;
  const float *p1 = PoT + off0 + i, *p2 = PoT + off1 + i;
  float *pst = PoT + offst + i;
  vector float lsq1, lsq2, x1, x2, lsqz, zst;
  
#define F2_STAGE1 x1 = vec_ld(0, p1); x2 = vec_ld(0, p2); i += 4; p1 += 4; p2 += 4;
#define F2_STAGE2 zst = vec_add(x1, x2);
#define F2_STAGE3 tempMaxV = vec_max(tempMaxV, zst); vec_st(zst, index, pst); pst += 4;
  
  // software pipelined main loop - omit last stage of tail element
  if (end-i >= 8) {
    F2_STAGE1;
    F2_STAGE2; F2_STAGE1;
    while (i < end) {
      F2_STAGE3; F2_STAGE2; F2_STAGE1;
    }
    F2_STAGE3; F2_STAGE2;
    //    F2_STAGE3;
  } else {    
    // main loop
    while (i < end-4) {
      F2_STAGE1; F2_STAGE2; F2_STAGE3;
    }
    F2_STAGE1; F2_STAGE2;
  }
  
#undef F2_STAGE1
#undef F2_STAGE2
#undef F2_STAGE3
  
  tailmask = (vector unsigned int) vec_cmplt(tailelem, taillim);
  zst = vec_and(zst, (vector float) tailmask);
  tempMaxV = vec_max(tempMaxV, zst);
  vec_st(zst, index, pst);
  
  // take the maximum across max and maxV and return it
  tempMaxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 4));
  float *maxVp = (float *) &maxV;
  max = *maxVp;
  
  return max;  
}
#endif

#if 0
/**********************
*
* foldArrayBy3 - Perform "folds" on 3 adjacent parts of the power over time
*   array. Each element in the "folded" array is the average of the 3 elements
*   in the power over time array.
*
* Assumes 16-byte alignment of input and output arrays.
*
*/
static float
foldArrayBy3(const float *PoT, int end, int off1, int off2, float *foldedPoT) {
  float max = 0.0f;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msq3 = vec_ld(0, PoT + off2);
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
  int i;
  
  vEnd = end - (end & 0x3);
  for (i = 0; i < vEnd; i += 4) {
    const float *p = PoT + i;
    vector float x1 = vec_ld(0, p);
    vector float lsq2 = vec_ld(15, p + off1);
    vector float lsq3 = vec_ld(15, p + off2);
    vector float x2 = vec_perm(msq2, lsq2, mask2);
    vector float x3 = vec_perm(msq3, lsq3, mask3);
    vector float z = vec_madd(vec_add(vec_add(x1, x2), x3), RECIP_THREE, ZERO);
    
    msq2 = lsq2;
    msq3 = lsq3;
    
    tempMaxV = vec_max(tempMaxV, z);
    vec_st(z, 0, foldedPoT + i);
  }
  
  // handle tail elements
  for (; i < end; ++i) {
    float x = (PoT[i] + PoT[i + off1] + PoT[i + off2]) / 3.0f;
    
    foldedPoT[i] = x;
    if (x > max)
      max = x;
  }
  
  // take the maximum across max and maxV and return it
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(maxV, vec_sld(maxV, maxV, 4));
  float *maxVp = (float *) &maxV;
  if (*maxVp > max) max = *maxVp;
  
  return max;
}
#endif

#if 1
static float
foldArrayBy3SP(const float *PoT, int end, int off1, int off2, float *foldedPoT) {
	vector float max, maxa, maxb;
	vector float lsq2a, lsq2b, msq2, lsq3a, lsq3b, msq3;
	vector unsigned char mask2, mask3;
	vector float x1a, x1b, x2a, x2b, x3a, x3b, xs12a, xs12b, za, zb;
	vector unsigned int taillim, tailmaska, tailmaskb;
	const vector unsigned int tailelema = {0, 1, 2, 3};
	const vector unsigned int tailelemb = {4, 5, 6, 7};
	int i = 0;
	
	const float *sa, *sb;
	unsigned int o1a, o2a;
	unsigned int o1b, o2b;
	float *d;
	
	sa = sb = PoT;
	o1a = off1 * sizeof(float) + 15;
	o1b = off1 * sizeof(float) + 31;
	o2a = off2 * sizeof(float) + 15;
	o2b = off2 * sizeof(float) + 31;
	d = foldedPoT;
	
	maxa = vec_xor(maxa, maxa);
	maxb = vec_xor(maxb, maxb);
	msq2 = vec_ld(0, PoT + off1);
	msq3 = vec_ld(0, PoT + off2);
	mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
	mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
	taillim = vec_uisplat(((end - 1) & 0x7) + 1);
	
	#define STAGE_ADVANCE 8
	#define F3_STAGE1 lsq2a = vec_ld(o1a, sa); lsq2b = vec_ld(o1b, sa); i += 8; sa += 8;
	#define F3_STAGE2 x1a = vec_ld(0, sb); x1b = vec_ld(16, sb); x2a = vec_perm(msq2, lsq2a, mask2); x2b = vec_perm(lsq2a, lsq2b, mask2); msq2 = lsq2b; lsq3a = vec_ld(o2a, sb); lsq3b = vec_ld(o2b, sb); sb += 8;
	#define F3_STAGE3 xs12a = vec_add(x1a, x2a); xs12b = vec_add(x1b, x2b); x3a = vec_perm(msq3, lsq3a, mask3); x3b = vec_perm(lsq3a, lsq3b, mask3); msq3 = lsq3b;
	#define F3_STAGE4 za = vec_add(xs12a, x3a); zb = vec_add(xs12b, x3b);
	#define F3_STAGE5 maxa = vec_max(maxa, za); maxb = vec_max(maxb, zb); vec_st(za, 0, d); vec_st(zb, 16, d); d += 8;
  
  if (end >= 4*STAGE_ADVANCE) {
		F3_STAGE1;
		F3_STAGE2; F3_STAGE1;
		F3_STAGE3; F3_STAGE2; F3_STAGE1;
		F3_STAGE4; F3_STAGE3; F3_STAGE2; F3_STAGE1;
		while (i < end) {
			F3_STAGE5; F3_STAGE4; F3_STAGE3; F3_STAGE2; F3_STAGE1;
		}
		F3_STAGE5; F3_STAGE4; F3_STAGE3; F3_STAGE2;
							 F3_STAGE5; F3_STAGE4; F3_STAGE3;
													F3_STAGE5; F3_STAGE4;
	} else {
		while (i < end-STAGE_ADVANCE) {
			F3_STAGE1; F3_STAGE2; F3_STAGE3; F3_STAGE4; F3_STAGE5;
		}
		F3_STAGE1; F3_STAGE2; F3_STAGE3; F3_STAGE4;
	}
	
	tailmaska = (vector unsigned int) vec_cmplt(tailelema, taillim);
	tailmaskb = (vector unsigned int) vec_cmplt(tailelemb, taillim);
	za = vec_and(za, (vector float) tailmaska);
	zb = vec_and(zb, (vector float) tailmaskb);
	maxa = vec_max(maxa, za);
	maxb = vec_max(maxb, zb);
	vec_st(za, 0, d);
	vec_st(zb, 16, d);
	
	maxa = vec_max(maxa, maxb);
	maxa = vec_max(maxa, vec_sld(maxa, maxa, 8));
	max = vec_max(maxa, vec_sld(maxa, maxa, 4));
	
	float *maxp = (float *) &max;
	return *maxp;
}
#endif


#if 0 && OLD_FOLDARRAY
static float
foldArrayBy3SP(const float *PoT, int end, int off1, int off2, float *foldedPoT) {
  float max;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msq3 = vec_ld(0, PoT + off2);
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
  vector unsigned int tailelem = {0, 1, 2, 3};
  vector unsigned int taillim = vec_uisplat(((end-1)&0x3) + 1);
  vector unsigned int tailmask;
  int i = 0;

  const float *p1 = PoT, *p2 = PoT + off1, *p3 = PoT + off2;
  float *pst = foldedPoT;
  vector float lsq2, lsq3, x1, x2, x3, xs12, z;
  
#define F3_STAGE1 lsq2 = vec_ld(15, p2); i += 4; p2 += 4; x1 = vec_ld(0, p1); x2 = vec_perm(msq2, lsq2, mask2); lsq3 = vec_ld(15, p3); p1 += 4; msq2 = lsq2; p3 += 4; xs12 = vec_add(x1, x2); x3 = vec_perm(msq3, lsq3, mask3); msq3 = lsq3;
#define F3_STAGE2 z = vec_add(xs12, x3);
#define F3_STAGE3 tempMaxV = vec_max(tempMaxV, z); vec_st(z, 0, pst); pst += 4;
  
  // software pipelined main loop
  if (end-i >= 8) {
    F3_STAGE1;
    F3_STAGE2; F3_STAGE1;
    while (i < end) {
      F3_STAGE3; F3_STAGE2; F3_STAGE1;
    }
                  F3_STAGE3; F3_STAGE2;
//                           F3_STAGE3;
  } else {
    // main loop
    while (i < end-4) {
      F3_STAGE1; F3_STAGE2; F3_STAGE3;
    }
    F3_STAGE1; F3_STAGE2;
  }
  
#undef F3_STAGE1
#undef F3_STAGE2
#undef F3_STAGE3
  
  tailmask = (vector unsigned int) vec_cmplt(tailelem, taillim);
  z = vec_and(z, (vector float) tailmask);
  tempMaxV = vec_max(tempMaxV, z);
  vec_st(z, 0, pst);
  
  // take the maximum across max and maxV and return it
  tempMaxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 4));
  float *maxVp = (float *) &maxV;
  max = *maxVp;
  
  return max;
}
#endif


#if 0
/**********************
*
* foldArrayBy4 - Perform "folds" on 4 adjacent parts of the power over time
*   array. Each element in the "folded" array is the average of the 4 elements
*   in the power over time array.
*
* Assumes 16-byte alignment of input and output arrays.
*
*/
static float
foldArrayBy4(const float *PoT, int end, int off1, int off2, int off3,
             float *foldedPoT) {
  float max = 0.0f;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msq3 = vec_ld(0, PoT + off2);
  vector float msq4 = vec_ld(0, PoT + off3);
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
  vector unsigned char mask4 = vec_add(vec_lvsl(-1L, PoT + off3), vec_splat_u8(1));
  int i;
  
  vEnd = end - (end & 0x3);
  for (i = 0; i < vEnd; i += 4) {
    const float *p = PoT + i;
    vector float x1 = vec_ld(0, p);
    vector float lsq2 = vec_ld(15, p + off1);
    vector float lsq3 = vec_ld(15, p + off2);
    vector float lsq4 = vec_ld(15, p + off3);
    vector float x2 = vec_perm(msq2, lsq2, mask2);
    vector float x3 = vec_perm(msq3, lsq3, mask3);
    vector float x4 = vec_perm(msq4, lsq4, mask4);
    vector float z = vec_madd(vec_add(vec_add(x1, x2), vec_add(x3, x4)), RECIP_FOUR, ZERO);
    
    msq2 = lsq2;
    msq3 = lsq3;
    msq4 = lsq4;
    
    tempMaxV = vec_max(tempMaxV, z);
    vec_st(z, 0, foldedPoT + i);
  }
  
  // handle tail elements
  for (; i < end; ++i) {
    float x = (PoT[i] + PoT[i + off1] + PoT[i + off2] + PoT[i + off3]) / 4.0f;
    
    foldedPoT[i] = x;
    if (x > max)
      max = x;
  }
  
  // take the maximum across max and maxV and return it
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(maxV, vec_sld(maxV, maxV, 4));
  float *maxVp = (float *) &maxV;
  if (*maxVp > max) max = *maxVp;

  return max;
}
#endif

#if 1
static float
foldArrayBy4SP(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {
	vector float max, maxa, maxb;
	vector float lsq2a, lsq2b, msq2, lsq3a, lsq3b, msq3, lsq4a, lsq4b, msq4;
	vector unsigned char mask2, mask3, mask4;
	vector float x1a, x1b, x2a, x2b, x3a, x3b, x4a, x4b, xs12a, xs12b, xs34a, xs34b, za, zb;
	vector unsigned int taillim, tailmaska, tailmaskb;
	const vector unsigned int tailelema = {0, 1, 2, 3};
	const vector unsigned int tailelemb = {4, 5, 6, 7};
	int i = 0;
	
	const float *s;
	unsigned int o1a, o2a, o3a;
	unsigned int o1b, o2b, o3b;
	float *d;
	
	s = PoT;
	o1a = off1 * sizeof(float) + 15;
	o1b = off1 * sizeof(float) + 31;
	o2a = off2 * sizeof(float) + 15;
	o2b = off2 * sizeof(float) + 31;
	o3a = off3 * sizeof(float) + 15;
	o3b = off3 * sizeof(float) + 31;
	d = foldedPoT;
	
	maxa = vec_xor(maxa, maxa);
	maxb = vec_xor(maxb, maxb);
	msq2 = vec_ld(0, PoT + off1);
	msq3 = vec_ld(0, PoT + off2);
	msq4 = vec_ld(0, PoT + off3);
	mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
	mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
	mask4 = vec_add(vec_lvsl(-1L, PoT + off3), vec_splat_u8(1));
	taillim = vec_uisplat(((end - 1) & 0x7) + 1);
	
	#define STAGE_ADVANCE 8
	#define F4_STAGE1 x1a = vec_ld(0, s); x1b = vec_ld(16, s); lsq2a = vec_ld(o1a, s); lsq2b = vec_ld(o1b, s); lsq3a = vec_ld(o2a, s); lsq3b = vec_ld(o2b, s); lsq4a = vec_ld(o3a, s); lsq4b = vec_ld(o3b, s); i += 8; s += 8;
	#define F4_STAGE2 x2a = vec_perm(msq2, lsq2a, mask2); x2b = vec_perm(lsq2a, lsq2b, mask2); msq2 = lsq2b; x3a = vec_perm(msq3, lsq3a, mask3); x3b = vec_perm(lsq3a, lsq3b, mask3); msq3 = lsq3b; x4a = vec_perm(msq4, lsq4a, mask4); x4b = vec_perm(lsq4a, lsq4b, mask4); msq4 = lsq4b; xs12a = vec_add(x1a, x2a); xs12b = vec_add(x1b, x2b); xs34a = vec_add(x3a, x4a); xs34b = vec_add(x3b, x4b);
	#define F4_STAGE3 za = vec_add(xs12a, xs34a); zb = vec_add(xs12b, xs34b);
	#define F4_STAGE4 maxa = vec_max(maxa, za); maxb = vec_max(maxb, zb); vec_st(za, 0, d); vec_st(zb, 16, d); d += 8;
	
  if (end >= 3*STAGE_ADVANCE) {
		F4_STAGE1;
		F4_STAGE2; F4_STAGE1;
		F4_STAGE3; F4_STAGE2; F4_STAGE1;
		while (i < end) {
			F4_STAGE4; F4_STAGE3; F4_STAGE2; F4_STAGE1;
		}
		F4_STAGE4; F4_STAGE3; F4_STAGE2;
							 F4_STAGE4; F4_STAGE3;
	} else {
		while (i < end-STAGE_ADVANCE) {
			F4_STAGE1; F4_STAGE2; F4_STAGE3; F4_STAGE4;
		}
		F4_STAGE1; F4_STAGE2; F4_STAGE3;
	}
	
	tailmaska = (vector unsigned int) vec_cmplt(tailelema, taillim);
	tailmaskb = (vector unsigned int) vec_cmplt(tailelemb, taillim);
	za = vec_and(za, (vector float) tailmaska);
	zb = vec_and(zb, (vector float) tailmaskb);
	maxa = vec_max(maxa, za);
	maxb = vec_max(maxb, zb);
	vec_st(za, 0, d);
	vec_st(zb, 16, d);
	
	maxa = vec_max(maxa, maxb);
	maxa = vec_max(maxa, vec_sld(maxa, maxa, 8));
	max = vec_max(maxa, vec_sld(maxa, maxa, 4));
	
	float *maxp = (float *) &max;
	return *maxp;
}
#endif

#if 0 && OLD_FOLDARRAY
static float
foldArrayBy4SP(const float *PoT, int end, int off1, int off2, int off3,
             float *foldedPoT) {
  float max;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msq3 = vec_ld(0, PoT + off2);
  vector float msq4 = vec_ld(0, PoT + off3);
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
  vector unsigned char mask4 = vec_add(vec_lvsl(-1L, PoT + off3), vec_splat_u8(1));
  vector unsigned int tailelem = {0, 1, 2, 3};
  vector unsigned int taillim = vec_uisplat(((end-1)&0x3) + 1);
  vector unsigned int tailmask;
  int i = 0;

  const float *p1 = PoT, *p2 = PoT + off1, *p3 = PoT + off2, *p4 = PoT + off3;
  float *pst = foldedPoT;
  vector float lsq2, lsq3, lsq4, x1, x2, x3, x4, xs12, xs34, z;
  
#define F4_STAGE1 lsq2 = vec_ld(15, p2); lsq3 = vec_ld(15, p3); lsq4 = vec_ld(15, p4); i += 4; p2 += 4; p3 += 4; p4 += 4; x1 = vec_ld(0, p1); x2 = vec_perm(msq2, lsq2, mask2); x3 = vec_perm(msq3, lsq3, mask3); x4 = vec_perm(msq4, lsq4, mask4); p1 += 4; msq2 = lsq2; msq3 = lsq3; msq4 = lsq4; xs12 = vec_add(x1, x2); xs34 = vec_add(x3, x4);
#define F4_STAGE2 z = vec_add(xs12, xs34);
#define F4_STAGE3 tempMaxV = vec_max(tempMaxV, z); vec_st(z, 0, pst); pst += 4;
  
  // software pipelined main loop
  if (end-i >= 8) {
    F4_STAGE1;
    F4_STAGE2; F4_STAGE1;
    while (i < end) {
      F4_STAGE3; F4_STAGE2; F4_STAGE1;
    }
                 F4_STAGE3; F4_STAGE2;
//                          F4_STAGE3;
  } else {  
    // main loop
    while (i < end-4) {
      F4_STAGE1; F4_STAGE2; F4_STAGE3;
    }
    F4_STAGE1; F4_STAGE2;
  }
    
#undef F4_STAGE1
#undef F4_STAGE2
#undef F4_STAGE3
  
  tailmask = (vector unsigned int) vec_cmplt(tailelem, taillim);
  z = vec_and(z, (vector float) tailmask);
  tempMaxV = vec_max(tempMaxV, z);
  vec_st(z, 0, pst);
  
  // take the maximum across max and maxV and return it
  tempMaxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 4));
  float *maxVp = (float *) &maxV;
  max = *maxVp;
  
  return max;
}
#endif

#if 0
/**********************
*
* foldArrayBy5 - Perform "folds" on 5 adjacent parts of the power over time
*   array. Each element in the "folded" array is the average of the 5 elements
*   in the power over time array.
*
* Assumes 16-byte alignment of input and output arrays.
*
*/
static float
foldArrayBy5(const float *PoT, int end, int off1, int off2, int off3,
             int off4, float *foldedPoT) {
  float max = 0.0f;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msq3 = vec_ld(0, PoT + off2);
  vector float msq4 = vec_ld(0, PoT + off3);
  vector float msq5 = vec_ld(0, PoT + off4);
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
  vector unsigned char mask4 = vec_add(vec_lvsl(-1L, PoT + off3), vec_splat_u8(1));
  vector unsigned char mask5 = vec_add(vec_lvsl(-1L, PoT + off4), vec_splat_u8(1));
  int i;
  
  vEnd = end - (end & 0x3);
  for (i = 0; i < vEnd; i += 4) {
    const float *p = PoT + i;
    vector float x1 = vec_ld(0, p);
    vector float lsq2 = vec_ld(15, p + off1);
    vector float lsq3 = vec_ld(15, p + off2);
    vector float lsq4 = vec_ld(15, p + off3);
    vector float lsq5 = vec_ld(15, p + off4);
    vector float x2 = vec_perm(msq2, lsq2, mask2);
    vector float x3 = vec_perm(msq3, lsq3, mask3);
    vector float x4 = vec_perm(msq4, lsq4, mask4);
    vector float x5 = vec_perm(msq5, lsq5, mask5);
    vector float z = vec_madd(vec_add(vec_add(vec_add(x1, x2), vec_add(x3, x4)), x5), RECIP_FIVE, ZERO);
    
    msq2 = lsq2;
    msq3 = lsq3;
    msq4 = lsq4;
    msq5 = lsq5;
    
    tempMaxV = vec_max(tempMaxV, z);
    vec_st(z, 0, foldedPoT + i);
  }
  
  // handle tail elements
  for (; i < end; ++i) {
    float x = (PoT[i] + PoT[i + off1] + PoT[i + off2] + PoT[i + off3] + PoT[i + off4]) / 5.0f;
    
    foldedPoT[i] = x;
    if (x > max)
      max = x;
  }
  
  // take the maximum across max and maxV and return it
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(maxV, vec_sld(maxV, maxV, 4));
  float *maxVp = (float *) &maxV;
  if (*maxVp > max) max = *maxVp;
  
  return max;
}
#endif

#if 1
static float
foldArrayBy5SP(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {
	vector float max, maxa;
//	vector float maxb;
	vector float lsq2a, msq2, lsq3a, msq3, lsq4a, msq4, lsq5a, msq5;
//	vector float lsq2b, lsq3b, lsq4b, lsq5b;
	vector unsigned char mask2, mask3, mask4, mask5;
	vector float x1a, x2a, x3a, x4a, x5a, xs12a, xs34a, xs1234a, za;
//	vector float x1b, x2b, x3b, x4b, x5b, xs12b, xs34b, xs1234b, zb;
	vector unsigned int taillim, tailmaska;
//	vector unsigned int tailmaskb;
	const vector unsigned int tailelema = {0, 1, 2, 3};
//	const vector unsigned int tailelemb = {4, 5, 6, 7};
	int i = 0;
	
	const float *sa, *sb, *sc;
	unsigned int o1a, o2a, o3a, o4a;
//	unsigned int o1b, o2b, o3b, o4b;
	float *d;
	
	sa = sb = sc = PoT;
	o1a = off1 * sizeof(float) + 15;
	o2a = off2 * sizeof(float) + 15;
	o3a = off3 * sizeof(float) + 15;
	o4a = off4 * sizeof(float) + 15;
	d = foldedPoT;

	maxa = vec_xor(maxa, maxa);
	msq2 = vec_ld(0, PoT + off1);
	msq3 = vec_ld(0, PoT + off2);
	msq4 = vec_ld(0, PoT + off3);
	msq5 = vec_ld(0, PoT + off4);
	mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
	mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
	mask4 = vec_add(vec_lvsl(-1L, PoT + off3), vec_splat_u8(1));
	mask5 = vec_add(vec_lvsl(-1L, PoT + off4), vec_splat_u8(1));

	#define STAGE_ADVANCE 4
	#define F5_STAGE1 lsq2a = vec_ld(o1a, sa); lsq3a = vec_ld(o2a, sa); lsq4a = vec_ld(o3a, sa); i += 4; sa += 4;
	#define F5_STAGE2 x1a = vec_ld(0, sb); x2a = vec_perm(msq2, lsq2a, mask2); msq2 = lsq2a; x3a = vec_perm(msq3, lsq3a, mask3); msq3 = lsq3a; x4a = vec_perm(msq4, lsq4a, mask4); msq4 = lsq4a; sb += 4;
	#define F5_STAGE3 xs12a = vec_add(x1a, x2a); xs34a = vec_add(x3a, x4a); lsq5a = vec_ld(o4a, sc); sc += 4;
	#define F5_STAGE4 xs1234a = vec_add(xs12a, xs34a); x5a = vec_perm(msq5, lsq5a, mask5); msq5 = lsq5a;
	#define F5_STAGE5 za = vec_add(xs1234a, x5a);
	#define F5_STAGE6 maxa = vec_max(maxa, za); vec_st(za, 0, d); d += 4;
	
	#define F5U_STAGE1 lsq2a = vec_ld(o1a, sa); lsq2b = vec_ld(o1b, sa); lsq3a = vec_ld(o2a, sa); lsq3b = vec_ld(o2b, sa); lsq4a = vec_ld(o3a, sa); lsq4b = vec_ld(o3b, sa); i += 8; sa += 8;
	#define F5U_STAGE2 x1a = vec_ld(0, sb); x1b = vec_ld(16, sb); x2a = vec_perm(msq2, lsq2a, mask2); x2b = vec_perm(lsq2a, lsq2b, mask2); msq2 = lsq2b; x3a = vec_perm(msq3, lsq3a, mask3); x3b = vec_perm(lsq3a, lsq3b, mask3); msq3 = lsq3b; x4a = vec_perm(msq4, lsq4a, mask4); x4b = vec_perm(lsq4a, lsq4b, mask4); msq4 = lsq4b; sb += 8;
	#define F5U_STAGE3 xs12a = vec_add(x1a, x2a); xs12b = vec_add(x1b, x2b); xs34a = vec_add(x3a, x4a); xs34b = vec_add(x3b, x4b); lsq5a = vec_ld(o4a, sc); lsq5b = vec_ld(o4b, sc); sc += 8;
	#define F5U_STAGE4 xs1234a = vec_add(xs12a, xs34a); xs1234b = vec_add(xs12b, xs34b); x5a = vec_perm(msq5, lsq5a, mask5); x5b = vec_perm(lsq5a, lsq5b, mask5); msq5 = lsq5b;
	#define F5U_STAGE5 za = vec_add(xs1234a, x5a); zb = vec_add(xs1234b, x5b);
	#define F5U_STAGE6 maxa = vec_max(maxa, za); maxb = vec_max(maxb, zb); vec_st(za, 0, d); vec_st(zb, 16, d); d += 8;

#if 0
	if (0 && end >= 5*2*STAGE_ADVANCE) {
		o1b = off1 * sizeof(float) + 31;
		o2b = off2 * sizeof(float) + 31;
		o3b = off3 * sizeof(float) + 31;
		o4b = off4 * sizeof(float) + 31;

		maxb = vec_xor(maxb, maxb);
		taillim = vec_uisplat(((end - 1) & 0x7) + 1);
		
		F5U_STAGE1;
		F5U_STAGE2; F5U_STAGE1;
		F5U_STAGE3; F5U_STAGE2; F5U_STAGE1;
		F5U_STAGE4; F5U_STAGE3; F5U_STAGE2; F5U_STAGE1;
		F5U_STAGE5; F5U_STAGE4; F5U_STAGE3; F5U_STAGE2; F5U_STAGE1;
		while (i < end) {
			F5U_STAGE6; F5U_STAGE5; F5U_STAGE4; F5U_STAGE3; F5U_STAGE2; F5U_STAGE1;
		}
		F5U_STAGE6; F5U_STAGE5; F5U_STAGE4; F5U_STAGE3; F5U_STAGE2;
								F5U_STAGE6; F5U_STAGE5; F5U_STAGE4; F5U_STAGE3;
														F5U_STAGE6; F5U_STAGE5; F5U_STAGE4;
																				F5U_STAGE6; F5U_STAGE5;

		tailmaska = (vector unsigned int) vec_cmplt(tailelema, taillim);
		tailmaskb = (vector unsigned int) vec_cmplt(tailelemb, taillim);
		za = vec_and(za, (vector float) tailmaska);
		zb = vec_and(zb, (vector float) tailmaskb);
		maxa = vec_max(maxa, za);
		maxb = vec_max(maxb, zb);
		vec_st(za, 0, d);
		vec_st(zb, 16, d);

		maxa = vec_max(maxa, maxb);		
	} else 
#endif
	{
		taillim = vec_uisplat(((end - 1) & 0x3) + 1);

	  if (end >= 5*STAGE_ADVANCE) {
			F5_STAGE1;
			F5_STAGE2; F5_STAGE1;
			F5_STAGE3; F5_STAGE2; F5_STAGE1;
			F5_STAGE4; F5_STAGE3; F5_STAGE2; F5_STAGE1;
			F5_STAGE5; F5_STAGE4; F5_STAGE3; F5_STAGE2; F5_STAGE1;
			while (i < end) {
				F5_STAGE6; F5_STAGE5; F5_STAGE4; F5_STAGE3; F5_STAGE2; F5_STAGE1;
			}
			F5_STAGE6; F5_STAGE5; F5_STAGE4; F5_STAGE3; F5_STAGE2;
								 F5_STAGE6; F5_STAGE5; F5_STAGE4; F5_STAGE3;
													  F5_STAGE6; F5_STAGE5; F5_STAGE4;
																			 F5_STAGE6; F5_STAGE5;
		} else {
			while (i < end-STAGE_ADVANCE) {
				F5_STAGE1; F5_STAGE2; F5_STAGE3; F5_STAGE4; F5_STAGE5; F5_STAGE6;
			}
			F5_STAGE1; F5_STAGE2; F5_STAGE3; F5_STAGE4; F5_STAGE5;
		}

		tailmaska = (vector unsigned int) vec_cmplt(tailelema, taillim);
		za = vec_and(za, (vector float) tailmaska);
		maxa = vec_max(maxa, za);
		vec_st(za, 0, d);
	}
	
	maxa = vec_max(maxa, vec_sld(maxa, maxa, 8));
	max = vec_max(maxa, vec_sld(maxa, maxa, 4));
	
	float *maxp = (float *) &max;
	return *maxp;
}
#endif

#if 0 && OLD_FOLDARRAY
static float
foldArrayBy5SP(const float *PoT, int end, int off1, int off2, int off3,
               int off4, float *foldedPoT) {
  float max;
  int vEnd;
  vector float tempMaxV = ZERO, maxV;
  vector float msq2 = vec_ld(0, PoT + off1);
  vector float msq3 = vec_ld(0, PoT + off2);
  vector float msq4 = vec_ld(0, PoT + off3);
  vector float msq5 = vec_ld(0, PoT + off4);
  vector unsigned char mask2 = vec_add(vec_lvsl(-1L, PoT + off1), vec_splat_u8(1));
  vector unsigned char mask3 = vec_add(vec_lvsl(-1L, PoT + off2), vec_splat_u8(1));
  vector unsigned char mask4 = vec_add(vec_lvsl(-1L, PoT + off3), vec_splat_u8(1));
  vector unsigned char mask5 = vec_add(vec_lvsl(-1L, PoT + off4), vec_splat_u8(1));
  vector unsigned int tailelem = {0, 1, 2, 3};
  vector unsigned int taillim = vec_uisplat(((end-1)&0x3) + 1);
  vector unsigned int tailmask;
  int i = 0;
  
  const float *p1 = PoT, *p2 = PoT + off1, *p3 = PoT + off2, *p4 = PoT + off3, *p5 = PoT + off4;
  float *pst = foldedPoT;
  vector float lsq2, lsq3, lsq4, lsq5, x1, x2, x3, x4, x5, xs12, xs34, xs345, z;
  
#define F5_STAGE1 lsq3 = vec_ld(15, p3); lsq4 = vec_ld(15, p4); i += 4; p3 += 4; p4 += 4; lsq2 = vec_ld(15, p2); x3 = vec_perm(msq3, lsq3, mask3); x4 = vec_perm(msq4, lsq4, mask4); lsq5 = vec_ld(15, p5); p2 += 4; msq3 = lsq3; msq4 = lsq4; p5 += 4; x1 = vec_ld(0, p1); x2 = vec_perm(msq2, lsq2, mask2); xs34 = vec_add(x3, x4); x5 = vec_perm(msq5, lsq5, mask5); p1 += 4; msq2 = lsq2; msq5 = lsq5;
#define F5_STAGE2 xs12 = vec_add(x1, x2); xs345 = vec_add(xs34, x5);
#define F5_STAGE3 z = vec_add(xs12, xs345);
#define F5_STAGE4 tempMaxV = vec_max(tempMaxV, z); vec_st(z, 0, pst); pst += 4;
  
  // software pipelined main loop
  if (end-i >= 12) {
    F5_STAGE1;
    F5_STAGE2; F5_STAGE1;
    F5_STAGE3; F5_STAGE2; F5_STAGE1;
    while (i < end) {
      F5_STAGE4; F5_STAGE3; F5_STAGE2; F5_STAGE1;
    }
                 F5_STAGE4; F5_STAGE3; F5_STAGE2;
                            F5_STAGE4; F5_STAGE3;
//                                     F5_STAGE4; 
  } else {
    // main loop
    while (i < end-4) {
      F5_STAGE1; F5_STAGE2; F5_STAGE3; F5_STAGE4;
    }
    F5_STAGE1; F5_STAGE2; F5_STAGE3;
  }
    
#undef F5_STAGE1
#undef F5_STAGE2
#undef F5_STAGE3
#undef F5_STAGE4
  
  tailmask = (vector unsigned int) vec_cmplt(tailelem, taillim);
  z = vec_and(z, (vector float) tailmask);
  tempMaxV = vec_max(tempMaxV, z);
  vec_st(z, 0, pst);
  
  // take the maximum across max and maxV and return it
  tempMaxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 8));
  maxV = vec_max(tempMaxV, vec_sld(tempMaxV, tempMaxV, 4));
  float *maxVp = (float *) &maxV;
  max = *maxVp;
  
  return max;
}
#endif


#elif defined(__SSE2__)

#ifdef USE_I386_CORE2
#define USE_PALIGNED_PULSE
#warning using palign for pulse-finding
#endif

static float
foldArrayBy2A(float *PoT, int end, int off0, int off1, int offst) {
  float max;
  vFloat maxV = _mm_setzero_ps(), maxV1 = _mm_setzero_ps(), maxV2 = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT + off0, *p2 = PoT + off1;
  float *pst = PoT + offst;
  vFloat x1, x2, xs, zst;
  
#if 1
  while (i < end-8) {
    // unrolled 2X serially
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    zst = _mm_add_ps(x1, x2);
//    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV1 = _mm_max_ps(maxV1, zst);
    i += 4;
    
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    zst = _mm_add_ps(x1, x2);
//    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV2 = _mm_max_ps(maxV2, zst);    
    i += 4;
  }
  if (i < end-4) {
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    zst = _mm_add_ps(x1, x2);
//    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV1 = _mm_max_ps(maxV1, zst);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
//  x2 = _mm_loadu_ps(p2+i);
  x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
  zst = _mm_add_ps(x1, x2);
//  zst = _mm_mul_ps(xs, RECIP_TWO);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  zst = _mm_and_ps(zst, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV1, _mm_max_ps(maxV2, zst));
  _mm_store_ps(pst+i, zst);
#else
  while (i < end-4) {
    x1 = _mm_load_ps(p1+i);
    x2 = _mm_loadu_ps(p2+i);
    xs = _mm_add_ps(x1, x2);
    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV = _mm_max_ps(maxV, zst);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_loadu_ps(p2+i);
  xs = _mm_add_ps(x1, x2);
  zst = _mm_mul_ps(xs, RECIP_TWO);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  zst = _mm_and_ps(zst, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, zst);
  _mm_store_ps(pst+i, zst);
#endif
  
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x4e));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x39));
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x1));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
//  _mm_store_ss(&max, maxV);
  max = _mm_cvtss_f32(maxV);
  
  return max;
}


static float
foldArrayBy2AL(float *PoT, int end, int off0, int off1, int offst) {
  float max;
  vFloat maxV = _mm_setzero_ps(), maxV1 = _mm_setzero_ps(), maxV2 = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT + off0, *p2 = PoT + off1;
  float *pst = PoT + offst;
  vFloat x1, x2, xs, zst;
  
#if 1
  while (i < end-8) {
    // unrolled 2X serially
    x1 = _mm_load_ps(p1+i);
    x2 = _mm_load_ps(p2+i);
    zst = _mm_add_ps(x1, x2);
//    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV1 = _mm_max_ps(maxV1, zst);
    i += 4;

    x1 = _mm_load_ps(p1+i);
    x2 = _mm_load_ps(p2+i);
    zst = _mm_add_ps(x1, x2);
//    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV2 = _mm_max_ps(maxV2, zst);
    i += 4;    
  }
  if (i < end-4) {
    x1 = _mm_load_ps(p1+i);
    x2 = _mm_load_ps(p2+i);
    zst = _mm_add_ps(x1, x2);
//    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV1 = _mm_max_ps(maxV1, zst);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_load_ps(p2+i);
  zst = _mm_add_ps(x1, x2);
//  zst = _mm_mul_ps(xs, RECIP_TWO);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  zst = _mm_and_ps(zst, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV1, _mm_max_ps(maxV2,zst));
  _mm_store_ps(pst+i, zst);  
#else
  while (i < end-4) {
    x1 = _mm_load_ps(p1+i);
    x2 = _mm_load_ps(p2+i);
    xs = _mm_add_ps(x1, x2);
    zst = _mm_mul_ps(xs, RECIP_TWO);
    _mm_store_ps(pst+i, zst);
    maxV = _mm_max_ps(maxV, zst);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_load_ps(p2+i);
  xs = _mm_add_ps(x1, x2);
  zst = _mm_mul_ps(xs, RECIP_TWO);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  zst = _mm_and_ps(zst, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, zst);
  _mm_store_ps(pst+i, zst);
#endif
  
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x4e));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x39));
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x1));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
//  _mm_store_ss(&max, maxV);
  max = _mm_cvtss_f32(maxV);
  
  return max;
}

#ifdef USE_PALIGNED_PULSE
#define PALIGNFOLD2(foldfn, alignshift) \
static float                                                                    \
foldfn(float *PoT, int end, int off0, int off1, int offst) {                    \
  float max;                                                                    \
  vFloat maxV = _mm_setzero_ps(), maxV1 = _mm_setzero_ps(), maxV2 = _mm_setzero_ps();                               \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                 \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                     \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                    \
  vSInt32 tailmask;                                                             \
  int i = 0;                                                                    \
                                                                                \
  const float *p1 = PoT + off0;                                                 \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                  \
  float *pst = PoT + offst;                                                     \
                                                                                \
  vFloat x1, x2, xs, zst;                                                       \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3));                                \
  vFloat x2h;                                                                   \
                                                                                \
  while (i < end-4) {                                                           \
    x1 = _mm_load_ps(p1+i);                                                     \
    x2h = _mm_load_ps(p2+i);                                                    \
    x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift);      \
    zst = _mm_add_ps(x1, x2);                                                   \
    _mm_store_ps(pst+i, zst);                                                   \
    maxV = _mm_max_ps(maxV, zst);                                               \
    i += 4;                                                                     \
    x2l = x2h;                                                                  \
  }                                                                             \
  x1 = _mm_load_ps(p1+i);                                                       \
  x2h = _mm_load_ps(p2+i);                                                      \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift);        \
  zst = _mm_add_ps(x1, x2);                                                      \
                                                                                \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                \
  zst = _mm_and_ps(zst, (vFloat) tailmask);                                     \
  maxV = _mm_max_ps(maxV, zst);                                                 \
  _mm_store_ps(pst+i, zst);                                                     \
                                                                                \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                           \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                               \
  max = _mm_cvtss_f32(maxV);                                                    \
                                                                                \
  return max;                                                                   \
}

#define PALIGNFOLD2U(foldfn, alignshift) \
static inline float                                                             \
foldfn(float *PoT, int end, int off0, int off1, int offst) {                    \
  float max;                                                                    \
  vFloat maxV = _mm_setzero_ps(), maxV1 = _mm_setzero_ps(), maxV2 = _mm_setzero_ps();                               \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                 \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                     \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                    \
  vSInt32 tailmask;                                                             \
  int i = 0;                                                                    \
                                                                                \
  const float *p1 = PoT + off0;                                                 \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                  \
  float *pst = PoT + offst;                                                     \
                                                                                \
  vFloat x1, x2, xs, zst;                                                       \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3));                                \
  vFloat x2m, x2h;                                                              \
                                                                                \
  while (i < end-8) {                                                           \
    x1 = _mm_load_ps(p1+i);                                                     \
    x2m = _mm_load_ps(p2+i);                                                    \
    x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift);      \
    zst = _mm_add_ps(x1, x2);                                                   \
    _mm_store_ps(pst+i, zst);                                                   \
    maxV1 = _mm_max_ps(maxV1, zst);                                             \
    i += 4;                                                                     \
                                                                                \
    x1 = _mm_load_ps(p1+i);                                                     \
    x2l = _mm_load_ps(p2+i);                                                    \
    x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift);      \
    zst = _mm_add_ps(x1, x2);                                                   \
    _mm_store_ps(pst+i, zst);                                                   \
    maxV2 = _mm_max_ps(maxV2, zst);                                             \
    i += 4;                                                                     \
  }                                                                             \
    x1 = _mm_load_ps(p1+i);                                                     \
    x2h = _mm_load_ps(p2+i);                                                    \
    x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift);      \
    zst = _mm_add_ps(x1, x2);                                                   \
  if (i < end-4) {                                                              \
    _mm_store_ps(pst+i, zst);                                                   \
    maxV1 = _mm_max_ps(maxV1, zst);                                             \
    i += 4;                                                                     \
                                                                                \
    x1 = _mm_load_ps(p1+i);                                                       \
    x2l = _mm_load_ps(p2+i);                                                      \
    x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift);        \
    zst = _mm_add_ps(x1, x2);                                                      \
  }                                                                             \
                                                                                \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                \
  zst = _mm_and_ps(zst, (vFloat) tailmask);                                     \
  maxV = _mm_max_ps(maxV1, _mm_max_ps(maxV2, zst));                             \
  _mm_store_ps(pst+i, zst);                                                     \
                                                                                \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                           \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                               \
  max = _mm_cvtss_f32(maxV);                                                    \
                                                                                \
  return max;                                                                   \
}

PALIGNFOLD2U(foldArrayBy2A1, 4)
PALIGNFOLD2U(foldArrayBy2A2, 8)
PALIGNFOLD2U(foldArrayBy2A3, 12)

//float (*fold2fns[])(float *, int, int, int, int) = {foldArrayBy2AL, foldArrayBy2A1, foldArrayBy2A2, foldArrayBy2A3};
#endif

static float
foldArrayBy3(const float *PoT, int end, int off1, int off2, float *foldedPoT) {
  float max;
  vFloat maxV = _mm_setzero_ps(), maxV1 = _mm_setzero_ps(), maxV2 = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT, *p2 = PoT + off1, *p3 = PoT + off2;
  float *pst = foldedPoT;
  vFloat x1, x2, x3, xs12, xs, z;
  
#if 1
  while (i < end-8) {
    // unrolled 2X serially
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    xs12 = _mm_add_ps(x1, x2);
    z = _mm_add_ps(xs12, x3);
//    z = _mm_mul_ps(xs, RECIP_THREE);
    _mm_store_ps(pst+i, z);
    maxV1 = _mm_max_ps(maxV1, z);
    i += 4;
    
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    xs12 = _mm_add_ps(x1, x2);
    z = _mm_add_ps(xs12, x3);
//    z = _mm_mul_ps(xs, RECIP_THREE);
    _mm_store_ps(pst+i, z);
    maxV2 = _mm_max_ps(maxV2, z);
    i += 4;    
  }
  if (i < end-4) {
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    xs12 = _mm_add_ps(x1, x2);
    z = _mm_add_ps(xs12, x3);
//    z = _mm_mul_ps(xs, RECIP_THREE);
    _mm_store_ps(pst+i, z);
    maxV1 = _mm_max_ps(maxV1, z);
    i += 4;    
  }
  x1 = _mm_load_ps(p1+i);
//  x2 = _mm_loadu_ps(p2+i);
//  x3 = _mm_loadu_ps(p3+i);
  x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
  x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
  xs12 = _mm_add_ps(x1, x2);
  z = _mm_add_ps(xs12, x3);
//  z = _mm_mul_ps(xs, RECIP_THREE);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV1, _mm_max_ps(maxV2, z));
  _mm_store_ps(pst+i, z);
#else
  while (i < end-4) {
    x1 = _mm_load_ps(p1+i);
    x2 = _mm_loadu_ps(p2+i);
    x3 = _mm_loadu_ps(p3+i);
    xs12 = _mm_add_ps(x1, x2);
    xs = _mm_add_ps(xs12, x3);
    z = _mm_mul_ps(xs, RECIP_THREE);
    _mm_store_ps(pst+i, z);
    maxV = _mm_max_ps(maxV, z);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_loadu_ps(p2+i);
  x3 = _mm_loadu_ps(p3+i);
  xs12 = _mm_add_ps(x1, x2);
  xs = _mm_add_ps(xs12, x3);
  z = _mm_mul_ps(xs, RECIP_THREE);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, z);
  _mm_store_ps(pst+i, z);
#endif
  
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x4e));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x39));
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x1));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
//  _mm_store_ss(&max, maxV);
  max = _mm_cvtss_f32(maxV);
  
  return max;
}

#ifdef USE_PALIGNED_PULSE
#define PALIGNFOLD3(foldfn, alignshift1, alignshift2) \
static inline float                                                            \
foldfn(const float *PoT, int end, int off1, int off2, float *foldedPoT) {      \
  float max;                                                                   \
  vFloat maxV = _mm_setzero_ps();                                                          \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                    \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                   \
  vSInt32 tailmask;                                                            \
  int i = 0;                                                                   \
                                                                               \
  const float *p1 = PoT;                                                       \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                 \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                 \
  float *pst = foldedPoT;                                                      \
                                                                               \
  vFloat x1, x2, x3, xs12, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2h;                          \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3h;                          \
                                                                               \
  while (i < end-4) {                                                          \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2h = _mm_load_ps(p2+i);                                                      \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);       \
	x3h = _mm_load_ps(p3+i);                                                      \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
                                                                               \
	x2l = x2h;                                                                    \
	x3l = x3h;                                                                    \
  }                                                                            \
  x1 = _mm_load_ps(p1+i);                                                      \
  x2h = _mm_load_ps(p2+i);                                                     \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);      \
  x3h = _mm_load_ps(p3+i);                                                     \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);      \
  xs12 = _mm_add_ps(x1, x2);                                                   \
  z = _mm_add_ps(xs12, x3);                                                    \
                                                                               \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                               \
  z = _mm_and_ps(z, (vFloat) tailmask);                                        \
  maxV = _mm_max_ps(maxV, z);                                                  \
  _mm_store_ps(pst+i, z);                                                      \
                                                                               \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                          \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                              \
  max = _mm_cvtss_f32(maxV);                                                   \
                                                                               \
  return max;                                                                  \
}

#define PALIGNFOLD3U(foldfn, alignshift1, alignshift2) \
static inline float                                                            \
foldfn(const float *PoT, int end, int off1, int off2, float *foldedPoT) {      \
  float max;                                                                   \
  vFloat maxV = _mm_setzero_ps();                                                          \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                    \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                   \
  vSInt32 tailmask;                                                            \
  int i = 0;                                                                   \
                                                                               \
  const float *p1 = PoT;                                                       \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                 \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                 \
  float *pst = foldedPoT;                                                      \
                                                                               \
  vFloat x1, x2, x3, xs12, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                     \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                     \
                                                                               \
  while (i < end-8) {                                                          \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2m = _mm_load_ps(p2+i);                                                      \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);       \
	x3m = _mm_load_ps(p3+i);                                                      \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
                                                                               \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2l = _mm_load_ps(p2+i);                                                      \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);       \
	x3l = _mm_load_ps(p3+i);                                                      \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
  }                                                                            \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2h = _mm_load_ps(p2+i);                                                      \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);       \
	x3h = _mm_load_ps(p3+i);                                                      \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
  if (i < end-4) {                                                             \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
                                                                              \
  x1 = _mm_load_ps(p1+i);                                                      \
  x2l = _mm_load_ps(p2+i);                                                     \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);      \
  x3l = _mm_load_ps(p3+i);                                                     \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);      \
  xs12 = _mm_add_ps(x1, x2);                                                   \
  z = _mm_add_ps(xs12, x3);                                                    \
  }                                                                           \
                                                                               \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                               \
  z = _mm_and_ps(z, (vFloat) tailmask);                                        \
  maxV = _mm_max_ps(maxV, z);                                                  \
  _mm_store_ps(pst+i, z);                                                      \
                                                                               \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                          \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                              \
  max = _mm_cvtss_f32(maxV);                                                   \
                                                                               \
  return max;                                                                  \
}

#define PALIGNFOLD3U1A(foldfn, alignshift2)                                    \
static inline float                                                            \
foldfn(const float *PoT, int end, int off1, int off2, float *foldedPoT) {      \
  float max;                                                                   \
  vFloat maxV = _mm_setzero_ps();                                                          \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                    \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                   \
  vSInt32 tailmask;                                                            \
  int i = 0;                                                                   \
                                                                               \
  const float *p1 = PoT;                                                       \
  const float *p2 = PoT + off1;                                                \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                 \
  float *pst = foldedPoT;                                                      \
                                                                               \
  vFloat x1, x2, x3, xs12, xs, z;                                              \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                     \
                                                                               \
  while (i < end-8) {                                                          \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2 = _mm_load_ps(p2+i);                                                       \
	x3m = _mm_load_ps(p3+i);                                                      \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
	                                                                              \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2 = _mm_load_ps(p2+i);                                                       \
	x3l = _mm_load_ps(p3+i);                                                      \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
  }                                                                            \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2 = _mm_load_ps(p2+i);                                                       \
	x3h = _mm_load_ps(p3+i);                                                      \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
  if (i < end-4) {                                                             \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
	                                                                              \
  x1 = _mm_load_ps(p1+i);                                                      \
  x2 = _mm_load_ps(p2+i);                                                      \
  x3l = _mm_load_ps(p3+i);                                                     \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);      \
  xs12 = _mm_add_ps(x1, x2);                                                   \
  z = _mm_add_ps(xs12, x3);                                                    \
  }                                                                            \
                                                                               \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                               \
  z = _mm_and_ps(z, (vFloat) tailmask);                                        \
  maxV = _mm_max_ps(maxV, z);                                                  \
  _mm_store_ps(pst+i, z);                                                      \
                                                                               \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                          \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                              \
  max = _mm_cvtss_f32(maxV);                                                   \
                                                                               \
  return max;                                                                  \
}

#define PALIGNFOLD3U2A(foldfn, alignshift1)                                    \
static inline float                                                            \
foldfn(const float *PoT, int end, int off1, int off2, float *foldedPoT) {      \
  float max;                                                                   \
  vFloat maxV = _mm_setzero_ps();                                                          \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                    \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                   \
  vSInt32 tailmask;                                                            \
  int i = 0;                                                                   \
                                                                               \
  const float *p1 = PoT;                                                       \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                 \
  const float *p3 = PoT + off2;                                                \
  float *pst = foldedPoT;                                                      \
                                                                               \
  vFloat x1, x2, x3, xs12, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                     \
                                                                               \
  while (i < end-8) {                                                          \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2m = _mm_load_ps(p2+i);                                                      \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);       \
	x3 = _mm_load_ps(p3+i);                                                       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
	                                                                              \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2l = _mm_load_ps(p2+i);                                                      \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);       \
	x3 = _mm_load_ps(p3+i);                                                       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
  }                                                                            \
	x1 = _mm_load_ps(p1+i);                                                       \
	x2h = _mm_load_ps(p2+i);                                                      \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);       \
	x3 = _mm_load_ps(p3+i);                                                       \
	xs12 = _mm_add_ps(x1, x2);                                                    \
	z = _mm_add_ps(xs12, x3);                                                     \
  if (i < end-4) {                                                             \
	_mm_store_ps(pst+i, z);                                                       \
	maxV = _mm_max_ps(maxV, z);                                                   \
	i += 4;                                                                       \
	                                                                              \
  x1 = _mm_load_ps(p1+i);                                                      \
  x2l = _mm_load_ps(p2+i);                                                     \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);      \
  x3 = _mm_load_ps(p3+i);                                                      \
  xs12 = _mm_add_ps(x1, x2);                                                   \
  z = _mm_add_ps(xs12, x3);                                                    \
  }                                                                            \
                                                                               \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                               \
  z = _mm_and_ps(z, (vFloat) tailmask);                                        \
  maxV = _mm_max_ps(maxV, z);                                                  \
  _mm_store_ps(pst+i, z);                                                      \
                                                                               \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                          \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                              \
  max = _mm_cvtss_f32(maxV);                                                   \
                                                                               \
  return max;                                                                  \
}

#define PALIGNFOLD3URA(foldfn, alignshift)                                          \
static inline float                                                                 \
foldfn(const float *PoT, int end, int off1, int off2, float *foldedPoT) {           \
  float max;                                                                        \
  vFloat maxV = _mm_setzero_ps();                                                               \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                     \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                         \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                        \
  vSInt32 tailmask;                                                                 \
  int i = 0;                                                                        \
                                                                                    \
  const float *p1 = PoT;                                                            \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                      \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                      \
  float *pst = foldedPoT;                                                           \
                                                                                    \
  vFloat x1, xs23, xs, z;                                                           \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                          \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                          \
  vFloat xs23l = _mm_add_ps(x2l, x3l), xs23m, xs23h;                                \
                                                                                    \
  while (i < end-8) {                                                               \
	x1 = _mm_load_ps(p1+i);                                                            \
	x2m = _mm_load_ps(p2+i);                                                           \
	x3m = _mm_load_ps(p3+i);                                                           \
	xs23m = _mm_add_ps(x2m, x3m);                                                      \
	xs23 = (vFloat) _mm_alignr_epi8((vUInt8) xs23m, (vUInt8) xs23l, alignshift);       \
	z = _mm_add_ps(x1, xs23);                                                          \
	_mm_store_ps(pst+i, z);                                                            \
	maxV = _mm_max_ps(maxV, z);                                                        \
	i += 4;                                                                            \
	                                                                                   \
	x1 = _mm_load_ps(p1+i);                                                            \
	x2h = _mm_load_ps(p2+i);                                                           \
	x3h = _mm_load_ps(p3+i);                                                           \
	xs23h = _mm_add_ps(x2h, x3h);                                                      \
	xs23 = (vFloat) _mm_alignr_epi8((vUInt8) xs23h, (vUInt8) xs23m, alignshift);       \
	z = _mm_add_ps(x1, xs23);                                                          \
	_mm_store_ps(pst+i, z);                                                            \
	maxV = _mm_max_ps(maxV, z);                                                        \
	i += 4;                                                                            \
	                                                                                   \
	xs23l = xs23h;                                                                     \
  }                                                                                 \
  if (i < end-4) {                                                                  \
	x1 = _mm_load_ps(p1+i);                                                            \
	x2h = _mm_load_ps(p2+i);                                                           \
	x3h = _mm_load_ps(p3+i);                                                           \
	xs23h = _mm_add_ps(x2h, x3h);                                                      \
	xs23 = (vFloat) _mm_alignr_epi8((vUInt8) xs23h, (vUInt8) xs23l, alignshift);       \
	z = _mm_add_ps(x1, xs23);                                                          \
	_mm_store_ps(pst+i, z);                                                            \
	maxV = _mm_max_ps(maxV, z);                                                        \
	i += 4;                                                                            \
	                                                                                   \
	xs23l = xs23h;                                                                     \
  }                                                                                 \
  x1 = _mm_load_ps(p1+i);                                                           \
  x2h = _mm_load_ps(p2+i);                                                          \
  x3h = _mm_load_ps(p3+i);                                                          \
  xs23h = _mm_add_ps(x2h, x3h);                                                     \
  xs23 = (vFloat) _mm_alignr_epi8((vUInt8) xs23h, (vUInt8) xs23l, alignshift);      \
  z = _mm_add_ps(x1, xs23);                                                         \
                                                                                    \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                    \
  z = _mm_and_ps(z, (vFloat) tailmask);                                             \
  maxV = _mm_max_ps(maxV, z);                                                       \
  _mm_store_ps(pst+i, z);                                                           \
                                                                                    \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                               \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                   \
  max = _mm_cvtss_f32(maxV);                                                        \
                                                                                    \
  return max;                                                                       \
}

static inline float
foldArrayBy3AAA(const float *PoT, int end, int off1, int off2, float *foldedPoT) {
  float max;
  vFloat maxV = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT;
  const float *p2 = PoT + off1;
  const float *p3 = PoT + off2;
  float *pst = foldedPoT;
  
  vFloat x1, x2, x3, xs12, xs, z;
  
  while (i < end-8) {
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	xs12 = _mm_add_ps(x1, x2);
	z = _mm_add_ps(xs12, x3);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
	
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	xs12 = _mm_add_ps(x1, x2);
	z = _mm_add_ps(xs12, x3);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
  }
  if (i < end-4) {
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	xs12 = _mm_add_ps(x1, x2);
	z = _mm_add_ps(xs12, x3);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_load_ps(p2+i);
  x3 = _mm_load_ps(p3+i);
  xs12 = _mm_add_ps(x1, x2);
  z = _mm_add_ps(xs12, x3);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, z);
  _mm_store_ps(pst+i, z);
  
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
  max = _mm_cvtss_f32(maxV);
  
  return max;
}

PALIGNFOLD3U(foldArrayBy3A11, 4, 4)
PALIGNFOLD3U(foldArrayBy3A12, 4, 8)
PALIGNFOLD3U(foldArrayBy3A13, 4, 12)
PALIGNFOLD3U2A(foldArrayBy3A1A, 4)
PALIGNFOLD3U(foldArrayBy3A21, 8, 4)
PALIGNFOLD3U(foldArrayBy3A22, 8, 8)
PALIGNFOLD3U(foldArrayBy3A23, 8, 12)
PALIGNFOLD3U2A(foldArrayBy3A2A, 8)
PALIGNFOLD3U(foldArrayBy3A31, 12, 4)
PALIGNFOLD3U(foldArrayBy3A32, 12, 8)
PALIGNFOLD3U(foldArrayBy3A33, 12, 12)
PALIGNFOLD3U2A(foldArrayBy3A3A, 12)
PALIGNFOLD3U1A(foldArrayBy3AA1, 4)
PALIGNFOLD3U1A(foldArrayBy3AA2, 8)
PALIGNFOLD3U1A(foldArrayBy3AA3, 12)
#endif

static float
foldArrayBy4(const float *PoT, int end, int off1, int off2, int off3,
             float *foldedPoT) {
  float max;
  vFloat maxV = _mm_setzero_ps(), maxV1 = _mm_setzero_ps(), maxV2 = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT, *p2 = PoT + off1, *p3 = PoT + off2, *p4 = PoT + off3;
  float *pst = foldedPoT;
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;
  
#if 1
  while (i < end-8) {
    // unrolled 2X serially
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
//    x4 = _mm_loadu_ps(p4+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    z = _mm_add_ps(xs12, xs34);
//    z = _mm_mul_ps(xs, RECIP_FOUR);
    _mm_store_ps(pst+i, z);
    maxV1 = _mm_max_ps(maxV1, z);
    i += 4;
    
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
//    x4 = _mm_loadu_ps(p4+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    z = _mm_add_ps(xs12, xs34);
//    z = _mm_mul_ps(xs, RECIP_FOUR);
    _mm_store_ps(pst+i, z);
    maxV2 = _mm_max_ps(maxV2, z);
    i += 4;
  }
  if (i < end-4) {
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
//    x4 = _mm_loadu_ps(p4+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    z = _mm_add_ps(xs12, xs34);
//    z = _mm_mul_ps(xs, RECIP_FOUR);
    _mm_store_ps(pst+i, z);
    maxV1 = _mm_max_ps(maxV1, z);
    i += 4;    
  }
  x1 = _mm_load_ps(p1+i);
//  x2 = _mm_loadu_ps(p2+i);
//  x3 = _mm_loadu_ps(p3+i);
//  x4 = _mm_loadu_ps(p4+i);
  x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
  x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
  x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
  xs12 = _mm_add_ps(x1, x2);
  xs34 = _mm_add_ps(x3, x4);
  z = _mm_add_ps(xs12, xs34);
//  z = _mm_mul_ps(xs, RECIP_FOUR);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV1, _mm_max_ps(maxV2, z));
  _mm_store_ps(pst+i, z);
#else
  while (i < end-4) {
    x1 = _mm_load_ps(p1+i);
    x2 = _mm_loadu_ps(p2+i);
    x3 = _mm_loadu_ps(p3+i);
    x4 = _mm_loadu_ps(p4+i);
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    xs = _mm_add_ps(xs12, xs34);
    z = _mm_mul_ps(xs, RECIP_FOUR);
    _mm_store_ps(pst+i, z);
    maxV = _mm_max_ps(maxV, z);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_loadu_ps(p2+i);
  x3 = _mm_loadu_ps(p3+i);
  x4 = _mm_loadu_ps(p4+i);
  xs12 = _mm_add_ps(x1, x2);
  xs34 = _mm_add_ps(x3, x4);
  xs = _mm_add_ps(xs12, xs34);
  z = _mm_mul_ps(xs, RECIP_FOUR);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, z);
  _mm_store_ps(pst+i, z);
#endif
  
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x4e));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x39));
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x1));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
//  _mm_store_ss(&max, maxV);
  max = _mm_cvtss_f32(maxV);
  
  return max;
}


#ifdef USE_PALIGNED_PULSE
#define PALIGNFOLD4U(foldfn, alignshift1, alignshift2, alignshift3)                      \
static inline float                                                                      \
foldfn(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {      \
  float max;                                                                             \
  vFloat maxV = _mm_setzero_ps();                                                                    \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                          \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                              \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                             \
  vSInt32 tailmask;                                                                      \
  int i = 0;                                                                             \
                                                                                         \
  const float *p1 = PoT;                                                                 \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                           \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                           \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                           \
  float *pst = foldedPoT;                                                                \
                                                                                         \
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                               \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                               \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                               \
                                                                                         \
  while (i < end-8) {                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2m = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                 \
	x3m = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                 \
	x4m = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
                                                                                         \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2l = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                 \
	x3l = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                 \
	x4l = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
  }                                                                                      \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2h = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                 \
	x3h = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                 \
	x4h = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
  if (i < end-4) {                                                                       \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
                                                                                         \
  x1 = _mm_load_ps(p1+i);                                                                \
  x2l = _mm_load_ps(p2+i);                                                               \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                \
  x3l = _mm_load_ps(p3+i);                                                               \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                \
  x4l = _mm_load_ps(p4+i);                                                               \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                \
  xs12 = _mm_add_ps(x1, x2);                                                             \
  xs34 = _mm_add_ps(x3, x4);                                                             \
  z = _mm_add_ps(xs12, xs34);                                                            \
  }                                                                                      \
                                                                                         \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                         \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                  \
  maxV = _mm_max_ps(maxV, z);                                                            \
  _mm_store_ps(pst+i, z);                                                                \
                                                                                         \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                    \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                        \
  max = _mm_cvtss_f32(maxV);                                                             \
                                                                                         \
  return max;                                                                            \
}

#define PALIGNFOLD4U1A(foldfn, alignshift2, alignshift3)                                 \
static inline float                                                                      \
foldfn(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {      \
  float max;                                                                             \
  vFloat maxV = _mm_setzero_ps();                                                                    \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                          \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                              \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                             \
  vSInt32 tailmask;                                                                      \
  int i = 0;                                                                             \
                                                                                         \
  const float *p1 = PoT;                                                                 \
  const float *p2 = PoT + off1;                                                          \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                           \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                           \
  float *pst = foldedPoT;                                                                \
                                                                                         \
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;                                              \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                               \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                               \
                                                                                         \
  while (i < end-8) {                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3m = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                 \
	x4m = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3l = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                 \
	x4l = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
  }                                                                                      \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3h = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                 \
	x4h = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
  if (i < end-4) {                                                                       \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
  x1 = _mm_load_ps(p1+i);                                                                \
  x2 = _mm_load_ps(p2+i);                                                                \
  x3l = _mm_load_ps(p3+i);                                                               \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                \
  x4l = _mm_load_ps(p4+i);                                                               \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                \
  xs12 = _mm_add_ps(x1, x2);                                                             \
  xs34 = _mm_add_ps(x3, x4);                                                             \
  z = _mm_add_ps(xs12, xs34);                                                            \
  }                                                                                      \
                                                                                         \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                         \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                  \
  maxV = _mm_max_ps(maxV, z);                                                            \
  _mm_store_ps(pst+i, z);                                                                \
                                                                                         \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                    \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                        \
  max = _mm_cvtss_f32(maxV);                                                             \
                                                                                         \
  return max;                                                                            \
}

#define PALIGNFOLD4U2A(foldfn, alignshift1, alignshift3)								 \
static inline float                                                                      \
foldfn(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {      \
  float max;                                                                             \
  vFloat maxV = _mm_setzero_ps();                                                                    \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                          \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                              \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                             \
  vSInt32 tailmask;                                                                      \
  int i = 0;                                                                             \
                                                                                         \
  const float *p1 = PoT;                                                                 \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                           \
  const float *p3 = PoT + off2;                                                          \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                           \
  float *pst = foldedPoT;                                                                \
                                                                                         \
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                               \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                               \
                                                                                         \
  while (i < end-8) {                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2m = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4m = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2l = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4l = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
  }                                                                                      \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2h = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4h = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
  if (i < end-4) {                                                                       \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
  x1 = _mm_load_ps(p1+i);                                                                \
  x2l = _mm_load_ps(p2+i);                                                               \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                \
  x3 = _mm_load_ps(p3+i);                                                                \
  x4l = _mm_load_ps(p4+i);                                                               \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                \
  xs12 = _mm_add_ps(x1, x2);                                                             \
  xs34 = _mm_add_ps(x3, x4);                                                             \
  z = _mm_add_ps(xs12, xs34);                                                            \
  }                                                                                      \
                                                                                         \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                         \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                  \
  maxV = _mm_max_ps(maxV, z);                                                            \
  _mm_store_ps(pst+i, z);                                                                \
                                                                                         \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                    \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                        \
  max = _mm_cvtss_f32(maxV);                                                             \
                                                                                         \
  return max;                                                                            \
}

#define PALIGNFOLD4U3A(foldfn, alignshift1, alignshift2)                                 \
static inline float                                                                      \
foldfn(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {      \
  float max;                                                                             \
  vFloat maxV = _mm_setzero_ps();                                                                    \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                          \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                              \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                             \
  vSInt32 tailmask;                                                                      \
  int i = 0;                                                                             \
                                                                                         \
  const float *p1 = PoT;                                                                 \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                           \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                           \
  const float *p4 = PoT + off3;                                                          \
  float *pst = foldedPoT;                                                                \
                                                                                         \
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                               \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                               \
                                                                                         \
  while (i < end-8) {                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2m = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                 \
	x3m = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2l = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                 \
	x3l = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
  }                                                                                      \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2h = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                 \
	x3h = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
  if (i < end-4) {                                                                       \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
  x1 = _mm_load_ps(p1+i);                                                                \
  x2l = _mm_load_ps(p2+i);                                                               \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                \
  x3l = _mm_load_ps(p3+i);                                                               \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                \
  x4 = _mm_load_ps(p4+i);                                                                \
  xs12 = _mm_add_ps(x1, x2);                                                             \
  xs34 = _mm_add_ps(x3, x4);                                                             \
  z = _mm_add_ps(xs12, xs34);                                                            \
  }                                                                                      \
                                                                                         \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                         \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                  \
  maxV = _mm_max_ps(maxV, z);                                                            \
  _mm_store_ps(pst+i, z);                                                                \
                                                                                         \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                    \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                        \
  max = _mm_cvtss_f32(maxV);                                                             \
                                                                                         \
  return max;                                                                            \
}

#define PALIGNFOLD4U12A(foldfn, alignshift3)                                             \
static inline float                                                                      \
foldfn(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {      \
  float max;                                                                             \
  vFloat maxV = _mm_setzero_ps();                                                                    \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                          \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                              \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                             \
  vSInt32 tailmask;                                                                      \
  int i = 0;                                                                             \
                                                                                         \
  const float *p1 = PoT;                                                                 \
  const float *p2 = PoT + off1;                                                          \
  const float *p3 = PoT + off2;                                                          \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                           \
  float *pst = foldedPoT;                                                                \
                                                                                         \
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;                                              \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                               \
                                                                                         \
  while (i < end-8) {                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4m = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4l = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
  }                                                                                      \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4h = _mm_load_ps(p4+i);                                                                \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
  if (i < end-4) {                                                                       \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
  x1 = _mm_load_ps(p1+i);                                                                \
  x2 = _mm_load_ps(p2+i);                                                                \
  x3 = _mm_load_ps(p3+i);                                                                \
  x4l = _mm_load_ps(p4+i);                                                               \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                \
  xs12 = _mm_add_ps(x1, x2);                                                             \
  xs34 = _mm_add_ps(x3, x4);                                                             \
  z = _mm_add_ps(xs12, xs34);                                                            \
  }                                                                                      \
                                                                                         \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                         \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                  \
  maxV = _mm_max_ps(maxV, z);                                                            \
  _mm_store_ps(pst+i, z);                                                                \
                                                                                         \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                    \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                        \
  max = _mm_cvtss_f32(maxV);                                                             \
                                                                                         \
  return max;                                                                            \
}

#define PALIGNFOLD4U23A(foldfn, alignshift1)                                             \
static inline float                                                                      \
foldfn(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {      \
  float max;                                                                             \
  vFloat maxV = _mm_setzero_ps();                                                                    \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                          \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                              \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                             \
  vSInt32 tailmask;                                                                      \
  int i = 0;                                                                             \
                                                                                         \
  const float *p1 = PoT;                                                                 \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                           \
  const float *p3 = PoT + off2;                                                          \
  const float *p4 = PoT + off3;                                                          \
  float *pst = foldedPoT;                                                                \
                                                                                         \
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                               \
                                                                                         \
  while (i < end-8) {                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2m = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2l = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
  }                                                                                      \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2h = _mm_load_ps(p2+i);                                                                \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                 \
	x3 = _mm_load_ps(p3+i);                                                                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
  if (i < end-4) {                                                                       \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
  x1 = _mm_load_ps(p1+i);                                                                \
  x2l = _mm_load_ps(p2+i);                                                               \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                \
  x3 = _mm_load_ps(p3+i);                                                                \
  x4 = _mm_load_ps(p4+i);                                                                \
  xs12 = _mm_add_ps(x1, x2);                                                             \
  xs34 = _mm_add_ps(x3, x4);                                                             \
  z = _mm_add_ps(xs12, xs34);                                                            \
  }                                                                                      \
                                                                                         \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                         \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                  \
  maxV = _mm_max_ps(maxV, z);                                                            \
  _mm_store_ps(pst+i, z);                                                                \
                                                                                         \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                    \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                        \
  max = _mm_cvtss_f32(maxV);                                                             \
                                                                                         \
  return max;                                                                            \
}

#define PALIGNFOLD4U13A(foldfn, alignshift2)                                             \
static inline float                                                                      \
foldfn(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {      \
  float max;                                                                             \
  vFloat maxV = _mm_setzero_ps();                                                                    \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                          \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                              \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                             \
  vSInt32 tailmask;                                                                      \
  int i = 0;                                                                             \
                                                                                         \
  const float *p1 = PoT;                                                                 \
  const float *p2 = PoT + off1;                                                          \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                           \
  const float *p4 = PoT + off3;                                                          \
  float *pst = foldedPoT;                                                                \
                                                                                         \
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;                                              \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                               \
                                                                                         \
  while (i < end-8) {                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3m = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3l = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
  }                                                                                      \
	x1 = _mm_load_ps(p1+i);                                                                 \
	x2 = _mm_load_ps(p2+i);                                                                 \
	x3h = _mm_load_ps(p3+i);                                                                \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                 \
	x4 = _mm_load_ps(p4+i);                                                                 \
	xs12 = _mm_add_ps(x1, x2);                                                              \
	xs34 = _mm_add_ps(x3, x4);                                                              \
	z = _mm_add_ps(xs12, xs34);                                                             \
  if (i < end-4) {                                                                       \
	_mm_store_ps(pst+i, z);                                                                 \
	maxV = _mm_max_ps(maxV, z);                                                             \
	i += 4;                                                                                 \
	                                                                                        \
  x1 = _mm_load_ps(p1+i);                                                                \
  x2 = _mm_load_ps(p2+i);                                                                \
  x3l = _mm_load_ps(p3+i);                                                               \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                \
  x4 = _mm_load_ps(p4+i);                                                                \
  xs12 = _mm_add_ps(x1, x2);                                                             \
  xs34 = _mm_add_ps(x3, x4);                                                             \
  z = _mm_add_ps(xs12, xs34);                                                            \
  }                                                                                      \
                                                                                         \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                         \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                  \
  maxV = _mm_max_ps(maxV, z);                                                            \
  _mm_store_ps(pst+i, z);                                                                \
                                                                                         \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                    \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                        \
  max = _mm_cvtss_f32(maxV);                                                             \
                                                                                         \
  return max;                                                                            \
}

static inline float
foldArrayBy4AAAA(const float *PoT, int end, int off1, int off2, int off3, float *foldedPoT) {
  float max;
  vFloat maxV = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT;
  const float *p2 = PoT + off1;
  const float *p3 = PoT + off2;
  const float *p4 = PoT + off3;
  float *pst = foldedPoT;
  
  vFloat x1, x2, x3, x4, xs12, xs34, xs, z;
  
  while (i < end-8) {
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	x4 = _mm_load_ps(p4+i);
	xs12 = _mm_add_ps(x1, x2);
	xs34 = _mm_add_ps(x3, x4);
	z = _mm_add_ps(xs12, xs34);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
	
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	x4 = _mm_load_ps(p4+i);
	xs12 = _mm_add_ps(x1, x2);
	xs34 = _mm_add_ps(x3, x4);
	z = _mm_add_ps(xs12, xs34);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
  }
  if (i < end-4) {
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	x4 = _mm_load_ps(p4+i);
	xs12 = _mm_add_ps(x1, x2);
	xs34 = _mm_add_ps(x3, x4);
	z = _mm_add_ps(xs12, xs34);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_load_ps(p2+i);
  x3 = _mm_load_ps(p3+i);
  x4 = _mm_load_ps(p4+i);
  xs12 = _mm_add_ps(x1, x2);
  xs34 = _mm_add_ps(x3, x4);
  z = _mm_add_ps(xs12, xs34);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, z);
  _mm_store_ps(pst+i, z);
  
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
  max = _mm_cvtss_f32(maxV);
  
  return max;
}

PALIGNFOLD4U(foldArrayBy4A111, 4, 4, 4)
PALIGNFOLD4U(foldArrayBy4A112, 4, 4, 8)
PALIGNFOLD4U(foldArrayBy4A113, 4, 4, 12)
PALIGNFOLD4U3A(foldArrayBy4A11A, 4, 4)
PALIGNFOLD4U(foldArrayBy4A121, 4, 8, 4)
PALIGNFOLD4U(foldArrayBy4A122, 4, 8, 8)
PALIGNFOLD4U(foldArrayBy4A123, 4, 8, 12)
PALIGNFOLD4U3A(foldArrayBy4A12A, 4, 8)
PALIGNFOLD4U(foldArrayBy4A131, 4, 12, 4)
PALIGNFOLD4U(foldArrayBy4A132, 4, 12, 8)
PALIGNFOLD4U(foldArrayBy4A133, 4, 12, 12)
PALIGNFOLD4U3A(foldArrayBy4A13A, 4, 12)
PALIGNFOLD4U2A(foldArrayBy4A1A1, 4, 4)
PALIGNFOLD4U2A(foldArrayBy4A1A2, 4, 8)
PALIGNFOLD4U2A(foldArrayBy4A1A3, 4, 12)
PALIGNFOLD4U23A(foldArrayBy4A1AA, 4)
PALIGNFOLD4U(foldArrayBy4A211, 8, 4, 4)
PALIGNFOLD4U(foldArrayBy4A212, 8, 4, 8)
PALIGNFOLD4U(foldArrayBy4A213, 8, 4, 12)
PALIGNFOLD4U3A(foldArrayBy4A21A, 8, 4)
PALIGNFOLD4U(foldArrayBy4A221, 8, 8, 4)
PALIGNFOLD4U(foldArrayBy4A222, 8, 8, 8)
PALIGNFOLD4U(foldArrayBy4A223, 8, 8, 12)
PALIGNFOLD4U3A(foldArrayBy4A22A, 8, 8)
PALIGNFOLD4U(foldArrayBy4A231, 8, 12, 4)
PALIGNFOLD4U(foldArrayBy4A232, 8, 12, 8)
PALIGNFOLD4U(foldArrayBy4A233, 8, 12, 12)
PALIGNFOLD4U3A(foldArrayBy4A23A, 8, 12)
PALIGNFOLD4U2A(foldArrayBy4A2A1, 8, 4)
PALIGNFOLD4U2A(foldArrayBy4A2A2, 8, 8)
PALIGNFOLD4U2A(foldArrayBy4A2A3, 8, 12)
PALIGNFOLD4U23A(foldArrayBy4A2AA, 8)
PALIGNFOLD4U(foldArrayBy4A311, 12, 4, 4)
PALIGNFOLD4U(foldArrayBy4A312, 12, 4, 8)
PALIGNFOLD4U(foldArrayBy4A313, 12, 4, 12)
PALIGNFOLD4U3A(foldArrayBy4A31A, 12, 4)
PALIGNFOLD4U(foldArrayBy4A321, 12, 8, 4)
PALIGNFOLD4U(foldArrayBy4A322, 12, 8, 8)
PALIGNFOLD4U(foldArrayBy4A323, 12, 8, 12)
PALIGNFOLD4U3A(foldArrayBy4A32A, 12, 8)
PALIGNFOLD4U(foldArrayBy4A331, 12, 12, 4)
PALIGNFOLD4U(foldArrayBy4A332, 12, 12, 8)
PALIGNFOLD4U(foldArrayBy4A333, 12, 12, 12)
PALIGNFOLD4U3A(foldArrayBy4A33A, 12, 12)
PALIGNFOLD4U2A(foldArrayBy4A3A1, 12, 4)
PALIGNFOLD4U2A(foldArrayBy4A3A2, 12, 8)
PALIGNFOLD4U2A(foldArrayBy4A3A3, 12, 12)
PALIGNFOLD4U23A(foldArrayBy4A3AA, 12)
PALIGNFOLD4U1A(foldArrayBy4AA11, 4, 4)
PALIGNFOLD4U1A(foldArrayBy4AA12, 4, 8)
PALIGNFOLD4U1A(foldArrayBy4AA13, 4, 12)
PALIGNFOLD4U13A(foldArrayBy4AA1A, 4)
PALIGNFOLD4U1A(foldArrayBy4AA21, 8, 4)
PALIGNFOLD4U1A(foldArrayBy4AA22, 8, 8)
PALIGNFOLD4U1A(foldArrayBy4AA23, 8, 12)
PALIGNFOLD4U13A(foldArrayBy4AA2A, 8)
PALIGNFOLD4U1A(foldArrayBy4AA31, 12, 4)
PALIGNFOLD4U1A(foldArrayBy4AA32, 12, 8)
PALIGNFOLD4U1A(foldArrayBy4AA33, 12, 12)
PALIGNFOLD4U13A(foldArrayBy4AA3A, 12)
PALIGNFOLD4U12A(foldArrayBy4AAA1, 4)
PALIGNFOLD4U12A(foldArrayBy4AAA2, 8)
PALIGNFOLD4U12A(foldArrayBy4AAA3, 12)
//PALIGNFOLD4U12A(foldArrayBy4AAAA, 16)
#endif


static float
foldArrayBy5(const float *PoT, int end, int off1, int off2, int off3, int off4,
             float *foldedPoT) {
  float max;
  vFloat maxV = _mm_setzero_ps(), maxV1 = _mm_setzero_ps(), maxV2 = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT, *p2 = PoT + off1, *p3 = PoT + off2, *p4 = PoT + off3, *p5 = PoT + off4;
  float *pst = foldedPoT;
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;
  
#if 1
  while (i < end-8) {
    // unrolled 2X serially
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
//    x4 = _mm_loadu_ps(p4+i);
//    x5 = _mm_loadu_ps(p5+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
    x5 = (vFloat) _mm_lddqu_si128((__m128i *) (p5+i));
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    xs345 = _mm_add_ps(xs34, x5);
    z = _mm_add_ps(xs12, xs345);
//    z = _mm_mul_ps(xs, RECIP_FIVE);
    _mm_store_ps(pst+i, z);
    maxV1 = _mm_max_ps(maxV1, z);
    i += 4;

    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
//    x4 = _mm_loadu_ps(p4+i);
//    x5 = _mm_loadu_ps(p5+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
    x5 = (vFloat) _mm_lddqu_si128((__m128i *) (p5+i));
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    xs345 = _mm_add_ps(xs34, x5);
    z = _mm_add_ps(xs12, xs345);
//    z = _mm_mul_ps(xs, RECIP_FIVE);
    _mm_store_ps(pst+i, z);
    maxV2 = _mm_max_ps(maxV2, z);
    i += 4;
  }
  if (i < end-4) {
    x1 = _mm_load_ps(p1+i);
//    x2 = _mm_loadu_ps(p2+i);
//    x3 = _mm_loadu_ps(p3+i);
//    x4 = _mm_loadu_ps(p4+i);
//    x5 = _mm_loadu_ps(p5+i);
    x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
    x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
    x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
    x5 = (vFloat) _mm_lddqu_si128((__m128i *) (p5+i));
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    xs345 = _mm_add_ps(xs34, x5);
    z = _mm_add_ps(xs12, xs345);
//    z = _mm_mul_ps(xs, RECIP_FIVE);
    _mm_store_ps(pst+i, z);
    maxV1 = _mm_max_ps(maxV1, z);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
//  x2 = _mm_loadu_ps(p2+i);
//  x3 = _mm_loadu_ps(p3+i);
//  x4 = _mm_loadu_ps(p4+i);
//  x5 = _mm_loadu_ps(p5+i);
  x2 = (vFloat) _mm_lddqu_si128((__m128i *) (p2+i));
  x3 = (vFloat) _mm_lddqu_si128((__m128i *) (p3+i));
  x4 = (vFloat) _mm_lddqu_si128((__m128i *) (p4+i));
  x5 = (vFloat) _mm_lddqu_si128((__m128i *) (p5+i));
  xs12 = _mm_add_ps(x1, x2);
  xs34 = _mm_add_ps(x3, x4);
  xs345 = _mm_add_ps(xs34, x5);
  z = _mm_add_ps(xs12, xs345);
//  z = _mm_mul_ps(xs, RECIP_FIVE);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV1, _mm_max_ps(maxV2, z));
  _mm_store_ps(pst+i, z);
#else
  while (i < end-4) {
    x1 = _mm_load_ps(p1+i);
    x2 = _mm_loadu_ps(p2+i);
    x3 = _mm_loadu_ps(p3+i);
    x4 = _mm_loadu_ps(p4+i);
    x5 = _mm_loadu_ps(p5+i);
    xs12 = _mm_add_ps(x1, x2);
    xs34 = _mm_add_ps(x3, x4);
    xs345 = _mm_add_ps(xs34, x5);
    xs = _mm_add_ps(xs12, xs345);
    z = _mm_mul_ps(xs, RECIP_FIVE);
    _mm_store_ps(pst+i, z);
    maxV = _mm_max_ps(maxV, z);
    i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_loadu_ps(p2+i);
  x3 = _mm_loadu_ps(p3+i);
  x4 = _mm_loadu_ps(p4+i);
  x5 = _mm_loadu_ps(p5+i);
  xs12 = _mm_add_ps(x1, x2);
  xs34 = _mm_add_ps(x3, x4);
  xs345 = _mm_add_ps(xs34, x5);
  xs = _mm_add_ps(xs12, xs345);
  z = _mm_mul_ps(xs, RECIP_FIVE);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, z);
  _mm_store_ps(pst+i, z);
#endif
  
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x4e));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x39));
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
//  maxV = _mm_max_ps(maxV, _mm_shuffle_ps(maxV, maxV, 0x1));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
//  _mm_store_ss(&max, maxV);
  max = _mm_cvtss_f32(maxV);
  
  return max;
}

#ifdef USE_PALIGNED_PULSE
#define PALIGNFOLD5U(foldfn, alignshift1, alignshift2, alignshift3, alignshift4)                    \
static inline float                                                                                 \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {       \
  float max;                                                                                        \
  vFloat maxV = _mm_setzero_ps();                                                                               \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                     \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                         \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                        \
  vSInt32 tailmask;                                                                                 \
  int i = 0;                                                                                        \
                                                                                                    \
  const float *p1 = PoT;                                                                            \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                                      \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                                      \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                                      \
  const float *p5 = PoT + ((off4 + 3) & ~0x3);                                                      \
  float *pst = foldedPoT;                                                                           \
                                                                                                    \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                              \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                                          \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                                          \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                                          \
  vFloat x5l = _mm_load_ps(PoT + (off4 & ~0x3)), x5m, x5h;                                          \
                                                                                                    \
  while (i < end-8) {                                                                               \
	x1 = _mm_load_ps(p1+i);                                                                            \
	x2m = _mm_load_ps(p2+i);                                                                           \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                            \
	x3m = _mm_load_ps(p3+i);                                                                           \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                            \
	x4m = _mm_load_ps(p4+i);                                                                           \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                            \
	x5m = _mm_load_ps(p5+i);                                                                           \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5m, (vUInt8) x5l, alignshift4);                            \
	xs12 = _mm_add_ps(x1, x2);                                                                         \
	xs34 = _mm_add_ps(x3, x4);                                                                         \
	xs345 = _mm_add_ps(xs34, x5);                                                                      \
	z = _mm_add_ps(xs12, xs345);                                                                       \
	_mm_store_ps(pst+i, z);                                                                            \
	maxV = _mm_max_ps(maxV, z);                                                                        \
	i += 4;                                                                                            \
                                                                                                    \
	x1 = _mm_load_ps(p1+i);                                                                            \
	x2l = _mm_load_ps(p2+i);                                                                           \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                            \
	x3l = _mm_load_ps(p3+i);                                                                           \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                            \
	x4l = _mm_load_ps(p4+i);                                                                           \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                            \
	x5l = _mm_load_ps(p5+i);                                                                           \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5m, alignshift4);                            \
	xs12 = _mm_add_ps(x1, x2);                                                                         \
	xs34 = _mm_add_ps(x3, x4);                                                                         \
	xs345 = _mm_add_ps(xs34, x5);                                                                      \
	z = _mm_add_ps(xs12, xs345);                                                                       \
	_mm_store_ps(pst+i, z);                                                                            \
	maxV = _mm_max_ps(maxV, z);                                                                        \
	i += 4;                                                                                            \
  }                                                                                                 \
	x1 = _mm_load_ps(p1+i);                                                                            \
	x2h = _mm_load_ps(p2+i);                                                                           \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                            \
	x3h = _mm_load_ps(p3+i);                                                                           \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                            \
	x4h = _mm_load_ps(p4+i);                                                                           \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                            \
	x5h = _mm_load_ps(p5+i);                                                                           \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5h, (vUInt8) x5l, alignshift4);                            \
	xs12 = _mm_add_ps(x1, x2);                                                                         \
	xs34 = _mm_add_ps(x3, x4);                                                                         \
	xs345 = _mm_add_ps(xs34, x5);                                                                      \
	z = _mm_add_ps(xs12, xs345);                                                                        \
  if (i < end-4) {                                                                                  \
	_mm_store_ps(pst+i, z);                                                                            \
	maxV = _mm_max_ps(maxV, z);                                                                        \
	i += 4;                                                                                            \
                                                                                                    \
  x1 = _mm_load_ps(p1+i);                                                                           \
  x2l = _mm_load_ps(p2+i);                                                                          \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                           \
  x3l = _mm_load_ps(p3+i);                                                                          \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                           \
  x4l = _mm_load_ps(p4+i);                                                                          \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                           \
  x5l = _mm_load_ps(p5+i);                                                                          \
  x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5h, alignshift4);                           \
  xs12 = _mm_add_ps(x1, x2);                                                                        \
  xs34 = _mm_add_ps(x3, x4);                                                                        \
  xs345 = _mm_add_ps(xs34, x5);                                                                     \
  z = _mm_add_ps(xs12, xs345);                                                                      \
  }                                                                                                 \
                                                                                                    \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                    \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                             \
  maxV = _mm_max_ps(maxV, z);                                                                       \
  _mm_store_ps(pst+i, z);                                                                           \
                                                                                                    \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                               \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                   \
  max = _mm_cvtss_f32(maxV);                                                                        \
                                                                                                    \
  return max;                                                                                       \
}

#define PALIGNFOLD5U1A(foldfn, alignshift2, alignshift3, alignshift4)                 \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + off1;                                                                    \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                                     \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                                     \
  const float *p5 = PoT + ((off4 + 3) & ~0x3);                                                     \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                                         \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                                         \
  vFloat x5l = _mm_load_ps(PoT + (off4 & ~0x3)), x5m, x5h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3m = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                           \
	x4m = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                           \
	x5m = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5m, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3l = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                           \
	x4l = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                           \
	x5l = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5m, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3h = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                           \
	x4h = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                           \
	x5h = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5h, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2 = _mm_load_ps(p2+i);                                                                          \
  x3l = _mm_load_ps(p3+i);                                                                         \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                          \
  x4l = _mm_load_ps(p4+i);                                                                         \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                          \
  x5l = _mm_load_ps(p5+i);                                                                         \
  x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5h, alignshift4);                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

#define PALIGNFOLD5U2A(foldfn, alignshift1, alignshift3, alignshift4)                              \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                                     \
  const float *p3 = PoT + off2;                                                                    \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                                     \
  const float *p5 = PoT + ((off4 + 3) & ~0x3);                                                     \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                                         \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                                         \
  vFloat x5l = _mm_load_ps(PoT + (off4 & ~0x3)), x5m, x5h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2m = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4m = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                           \
	x5m = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5m, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2l = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4l = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                           \
	x5l = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5m, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2h = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4h = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                           \
	x5h = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5h, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2l = _mm_load_ps(p2+i);                                                                         \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                          \
  x3 = _mm_load_ps(p3+i);                                                                          \
  x4l = _mm_load_ps(p4+i);                                                                         \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                          \
  x5l = _mm_load_ps(p5+i);                                                                         \
  x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5h, alignshift4);                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

#define PALIGNFOLD5U3A(foldfn, alignshift1, alignshift2, alignshift4)                              \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                                     \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                                     \
  const float *p4 = PoT + off3;                                                                    \
  const float *p5 = PoT + ((off4 + 3) & ~0x3);                                                     \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                                         \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                                         \
  vFloat x5l = _mm_load_ps(PoT + (off4 & ~0x3)), x5m, x5h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2m = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                           \
	x3m = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5m = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5m, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2l = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                           \
	x3l = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5l = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5m, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2h = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                           \
	x3h = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5h = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5h, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2l = _mm_load_ps(p2+i);                                                                         \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                          \
  x3l = _mm_load_ps(p3+i);                                                                         \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                          \
  x4 = _mm_load_ps(p4+i);                                                                          \
  x5l = _mm_load_ps(p5+i);                                                                         \
  x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5h, alignshift4);                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

#define PALIGNFOLD5U4A(foldfn, alignshift1, alignshift2, alignshift3)                              \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                                     \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                                     \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                                     \
  const float *p5 = PoT + off4;                                                                    \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                                         \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                                         \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2m = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                           \
	x3m = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                           \
	x4m = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2l = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                           \
	x3l = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                           \
	x4l = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2h = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                           \
	x3h = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                           \
	x4h = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2l = _mm_load_ps(p2+i);                                                                         \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                          \
  x3l = _mm_load_ps(p3+i);                                                                         \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                          \
  x4l = _mm_load_ps(p4+i);                                                                         \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                          \
  x5 = _mm_load_ps(p5+i);                                                                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

#define PALIGNFOLD5U234A(foldfn, alignshift1)                                                      \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + ((off1 + 3) & ~0x3);                                                     \
  const float *p3 = PoT + off2;                                                                    \
  const float *p4 = PoT + off3;                                                                    \
  const float *p5 = PoT + off4;                                                                    \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x2l = _mm_load_ps(PoT + (off1 & ~0x3)), x2m, x2h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2m = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2m, (vUInt8) x2l, alignshift1);                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2l = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2m, alignshift1);                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2h = _mm_load_ps(p2+i);                                                                          \
	x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2h, (vUInt8) x2l, alignshift1);                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2l = _mm_load_ps(p2+i);                                                                         \
  x2 = (vFloat) _mm_alignr_epi8((vUInt8) x2l, (vUInt8) x2h, alignshift1);                          \
  x3 = _mm_load_ps(p3+i);                                                                          \
  x4 = _mm_load_ps(p4+i);                                                                          \
  x5 = _mm_load_ps(p5+i);                                                                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

#define PALIGNFOLD5U134A(foldfn, alignshift2)                                                      \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + off1;                                                                    \
  const float *p3 = PoT + ((off2 + 3) & ~0x3);                                                     \
  const float *p4 = PoT + off3;                                                                    \
  const float *p5 = PoT + off4;                                                                    \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x3l = _mm_load_ps(PoT + (off2 & ~0x3)), x3m, x3h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3m = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3m, (vUInt8) x3l, alignshift2);                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3l = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3m, alignshift2);                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3h = _mm_load_ps(p3+i);                                                                          \
	x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3h, (vUInt8) x3l, alignshift2);                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2 = _mm_load_ps(p2+i);                                                                          \
  x3l = _mm_load_ps(p3+i);                                                                         \
  x3 = (vFloat) _mm_alignr_epi8((vUInt8) x3l, (vUInt8) x3h, alignshift2);                          \
  x4 = _mm_load_ps(p4+i);                                                                          \
  x5 = _mm_load_ps(p5+i);                                                                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

#define PALIGNFOLD5U124A(foldfn, alignshift3)                                                      \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + off1;                                                                    \
  const float *p3 = PoT + off2;                                                                    \
  const float *p4 = PoT + ((off3 + 3) & ~0x3);                                                     \
  const float *p5 = PoT + off4;                                                                    \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x4l = _mm_load_ps(PoT + (off3 & ~0x3)), x4m, x4h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4m = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4m, (vUInt8) x4l, alignshift3);                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4l = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4m, alignshift3);                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4h = _mm_load_ps(p4+i);                                                                          \
	x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4h, (vUInt8) x4l, alignshift3);                           \
	x5 = _mm_load_ps(p5+i);                                                                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2 = _mm_load_ps(p2+i);                                                                          \
  x3 = _mm_load_ps(p3+i);                                                                          \
  x4l = _mm_load_ps(p4+i);                                                                         \
  x4 = (vFloat) _mm_alignr_epi8((vUInt8) x4l, (vUInt8) x4h, alignshift3);                          \
  x5 = _mm_load_ps(p5+i);                                                                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

#define PALIGNFOLD5U123A(foldfn, alignshift4)                                                      \
static inline float                                                                                \
foldfn(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {      \
  float max;                                                                                       \
  vFloat maxV = _mm_setzero_ps();                                                                              \
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);                                                    \
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);                                        \
  taillim = _mm_shuffle_epi32(taillim, 0x0);                                                       \
  vSInt32 tailmask;                                                                                \
  int i = 0;                                                                                       \
                                                                                                   \
  const float *p1 = PoT;                                                                           \
  const float *p2 = PoT + off1;                                                                    \
  const float *p3 = PoT + off2;                                                                    \
  const float *p4 = PoT + off3;                                                                    \
  const float *p5 = PoT + ((off4 + 3) & ~0x3);                                                     \
  float *pst = foldedPoT;                                                                          \
                                                                                                   \
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;                                             \
  vFloat x5l = _mm_load_ps(PoT + (off4 & ~0x3)), x5m, x5h;                                         \
                                                                                                   \
  while (i < end-8) {                                                                              \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5m = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5m, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5l = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5m, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
  }                                                                                                \
	x1 = _mm_load_ps(p1+i);                                                                           \
	x2 = _mm_load_ps(p2+i);                                                                           \
	x3 = _mm_load_ps(p3+i);                                                                           \
	x4 = _mm_load_ps(p4+i);                                                                           \
	x5h = _mm_load_ps(p5+i);                                                                          \
	x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5h, (vUInt8) x5l, alignshift4);                           \
	xs12 = _mm_add_ps(x1, x2);                                                                        \
	xs34 = _mm_add_ps(x3, x4);                                                                        \
	xs345 = _mm_add_ps(xs34, x5);                                                                     \
	z = _mm_add_ps(xs12, xs345);                                                                      \
  if (i < end-4) {                                                                                 \
	_mm_store_ps(pst+i, z);                                                                           \
	maxV = _mm_max_ps(maxV, z);                                                                       \
	i += 4;                                                                                           \
	                                                                                                  \
  x1 = _mm_load_ps(p1+i);                                                                          \
  x2 = _mm_load_ps(p2+i);                                                                          \
  x3 = _mm_load_ps(p3+i);                                                                          \
  x4 = _mm_load_ps(p4+i);                                                                          \
  x5l = _mm_load_ps(p5+i);                                                                         \
  x5 = (vFloat) _mm_alignr_epi8((vUInt8) x5l, (vUInt8) x5h, alignshift4);                          \
  xs12 = _mm_add_ps(x1, x2);                                                                       \
  xs34 = _mm_add_ps(x3, x4);                                                                       \
  xs345 = _mm_add_ps(xs34, x5);                                                                    \
  z = _mm_add_ps(xs12, xs345);                                                                     \
  }                                                                                                \
                                                                                                   \
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);                                                   \
  z = _mm_and_ps(z, (vFloat) tailmask);                                                            \
  maxV = _mm_max_ps(maxV, z);                                                                      \
  _mm_store_ps(pst+i, z);                                                                          \
                                                                                                   \
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));                                              \
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));                                                  \
  max = _mm_cvtss_f32(maxV);                                                                       \
                                                                                                   \
  return max;                                                                                      \
}

static inline float
foldArrayBy5AAAAA(const float *PoT, int end, int off1, int off2, int off3, int off4, float *foldedPoT) {
  float max;
  vFloat maxV = _mm_setzero_ps();
  vSInt32 tailelem = _mm_set_epi32(3, 2, 1, 0);
  vSInt32 taillim = _mm_cvtsi32_si128(((end-1) & 0x3) + 1);
  taillim = _mm_shuffle_epi32(taillim, 0x0);
  vSInt32 tailmask;
  int i = 0;
  
  const float *p1 = PoT;
  const float *p2 = PoT + off1;
  const float *p3 = PoT + off2;
  const float *p4 = PoT + off3;
  const float *p5 = PoT + off4;
  float *pst = foldedPoT;
  
  vFloat x1, x2, x3, x4, x5, xs12, xs34, xs345, xs, z;
  
  while (i < end-8) {
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	x4 = _mm_load_ps(p4+i);
	x5 = _mm_load_ps(p5+i);
	xs12 = _mm_add_ps(x1, x2);
	xs34 = _mm_add_ps(x3, x4);
	xs345 = _mm_add_ps(xs34, x5);
	z = _mm_add_ps(xs12, xs345);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
	
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	x4 = _mm_load_ps(p4+i);
	x5 = _mm_load_ps(p5+i);
	xs12 = _mm_add_ps(x1, x2);
	xs34 = _mm_add_ps(x3, x4);
	xs345 = _mm_add_ps(xs34, x5);
	z = _mm_add_ps(xs12, xs345);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
  }
  if (i < end-4) {
	x1 = _mm_load_ps(p1+i);
	x2 = _mm_load_ps(p2+i);
	x3 = _mm_load_ps(p3+i);
	x4 = _mm_load_ps(p4+i);
	x5 = _mm_load_ps(p5+i);
	xs12 = _mm_add_ps(x1, x2);
	xs34 = _mm_add_ps(x3, x4);
	xs345 = _mm_add_ps(xs34, x5);
	z = _mm_add_ps(xs12, xs345);
	_mm_store_ps(pst+i, z);
	maxV = _mm_max_ps(maxV, z);
	i += 4;
  }
  x1 = _mm_load_ps(p1+i);
  x2 = _mm_load_ps(p2+i);
  x3 = _mm_load_ps(p3+i);
  x4 = _mm_load_ps(p4+i);
  x5 = _mm_load_ps(p5+i);
  xs12 = _mm_add_ps(x1, x2);
  xs34 = _mm_add_ps(x3, x4);
  xs345 = _mm_add_ps(xs34, x5);
  z = _mm_add_ps(xs12, xs345);
  
  tailmask = _mm_cmpgt_epi32(taillim, tailelem);
  z = _mm_and_ps(z, (vFloat) tailmask);
  maxV = _mm_max_ps(maxV, z);
  _mm_store_ps(pst+i, z);
  
  maxV = _mm_max_ps(maxV, _mm_movehl_ps(maxV, maxV));
  maxV = _mm_max_ss(maxV, _mm_movehdup_ps(maxV));
  max = _mm_cvtss_f32(maxV);
  
  return max;
}

PALIGNFOLD5U(foldArrayBy5A1111, 4, 4, 4, 4)
PALIGNFOLD5U(foldArrayBy5A1112, 4, 4, 4, 8)
PALIGNFOLD5U(foldArrayBy5A1113, 4, 4, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A111A, 4, 4, 4)
PALIGNFOLD5U(foldArrayBy5A1121, 4, 4, 8, 4)
PALIGNFOLD5U(foldArrayBy5A1122, 4, 4, 8, 8)
PALIGNFOLD5U(foldArrayBy5A1123, 4, 4, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A112A, 4, 4, 8)
PALIGNFOLD5U(foldArrayBy5A1131, 4, 4, 12, 4)
PALIGNFOLD5U(foldArrayBy5A1132, 4, 4, 12, 8)
PALIGNFOLD5U(foldArrayBy5A1133, 4, 4, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A113A, 4, 4, 12)
PALIGNFOLD5U3A(foldArrayBy5A11A1, 4, 4, 4)
PALIGNFOLD5U3A(foldArrayBy5A11A2, 4, 4, 8)
PALIGNFOLD5U3A(foldArrayBy5A11A3, 4, 4, 12)
PALIGNFOLD5U3A(foldArrayBy5A11AA, 4, 4, 16)
PALIGNFOLD5U(foldArrayBy5A1211, 4, 8, 4, 4)
PALIGNFOLD5U(foldArrayBy5A1212, 4, 8, 4, 8)
PALIGNFOLD5U(foldArrayBy5A1213, 4, 8, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A121A, 4, 8, 4)
PALIGNFOLD5U(foldArrayBy5A1221, 4, 8, 8, 4)
PALIGNFOLD5U(foldArrayBy5A1222, 4, 8, 8, 8)
PALIGNFOLD5U(foldArrayBy5A1223, 4, 8, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A122A, 4, 8, 8)
PALIGNFOLD5U(foldArrayBy5A1231, 4, 8, 12, 4)
PALIGNFOLD5U(foldArrayBy5A1232, 4, 8, 12, 8)
PALIGNFOLD5U(foldArrayBy5A1233, 4, 8, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A123A, 4, 8, 12)
PALIGNFOLD5U3A(foldArrayBy5A12A1, 4, 8, 4)
PALIGNFOLD5U3A(foldArrayBy5A12A2, 4, 8, 8)
PALIGNFOLD5U3A(foldArrayBy5A12A3, 4, 8, 12)
PALIGNFOLD5U3A(foldArrayBy5A12AA, 4, 8, 16)
PALIGNFOLD5U(foldArrayBy5A1311, 4, 12, 4, 4)
PALIGNFOLD5U(foldArrayBy5A1312, 4, 12, 4, 8)
PALIGNFOLD5U(foldArrayBy5A1313, 4, 12, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A131A, 4, 12, 4)
PALIGNFOLD5U(foldArrayBy5A1321, 4, 12, 8, 4)
PALIGNFOLD5U(foldArrayBy5A1322, 4, 12, 8, 8)
PALIGNFOLD5U(foldArrayBy5A1323, 4, 12, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A132A, 4, 12, 8)
PALIGNFOLD5U(foldArrayBy5A1331, 4, 12, 12, 4)
PALIGNFOLD5U(foldArrayBy5A1332, 4, 12, 12, 8)
PALIGNFOLD5U(foldArrayBy5A1333, 4, 12, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A133A, 4, 12, 12)
PALIGNFOLD5U3A(foldArrayBy5A13A1, 4, 12, 4)
PALIGNFOLD5U3A(foldArrayBy5A13A2, 4, 12, 8)
PALIGNFOLD5U3A(foldArrayBy5A13A3, 4, 12, 12)
PALIGNFOLD5U3A(foldArrayBy5A13AA, 4, 12, 16)
PALIGNFOLD5U2A(foldArrayBy5A1A11, 4, 4, 4)
PALIGNFOLD5U2A(foldArrayBy5A1A12, 4, 4, 8)
PALIGNFOLD5U2A(foldArrayBy5A1A13, 4, 4, 12)
PALIGNFOLD5U2A(foldArrayBy5A1A1A, 4, 4, 16)
PALIGNFOLD5U2A(foldArrayBy5A1A21, 4, 8, 4)
PALIGNFOLD5U2A(foldArrayBy5A1A22, 4, 8, 8)
PALIGNFOLD5U2A(foldArrayBy5A1A23, 4, 8, 12)
PALIGNFOLD5U2A(foldArrayBy5A1A2A, 4, 8, 16)
PALIGNFOLD5U2A(foldArrayBy5A1A31, 4, 12, 4)
PALIGNFOLD5U2A(foldArrayBy5A1A32, 4, 12, 8)
PALIGNFOLD5U2A(foldArrayBy5A1A33, 4, 12, 12)
PALIGNFOLD5U2A(foldArrayBy5A1A3A, 4, 12, 16)
PALIGNFOLD5U2A(foldArrayBy5A1AA1, 4, 16, 4)
PALIGNFOLD5U2A(foldArrayBy5A1AA2, 4, 16, 8)
PALIGNFOLD5U2A(foldArrayBy5A1AA3, 4, 16, 12)
PALIGNFOLD5U234A(foldArrayBy5A1AAA, 4)
PALIGNFOLD5U(foldArrayBy5A2111, 8, 4, 4, 4)
PALIGNFOLD5U(foldArrayBy5A2112, 8, 4, 4, 8)
PALIGNFOLD5U(foldArrayBy5A2113, 8, 4, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A211A, 8, 4, 4)
PALIGNFOLD5U(foldArrayBy5A2121, 8, 4, 8, 4)
PALIGNFOLD5U(foldArrayBy5A2122, 8, 4, 8, 8)
PALIGNFOLD5U(foldArrayBy5A2123, 8, 4, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A212A, 8, 4, 8)
PALIGNFOLD5U(foldArrayBy5A2131, 8, 4, 12, 4)
PALIGNFOLD5U(foldArrayBy5A2132, 8, 4, 12, 8)
PALIGNFOLD5U(foldArrayBy5A2133, 8, 4, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A213A, 8, 4, 12)
PALIGNFOLD5U3A(foldArrayBy5A21A1, 8, 4, 4)
PALIGNFOLD5U3A(foldArrayBy5A21A2, 8, 4, 8)
PALIGNFOLD5U3A(foldArrayBy5A21A3, 8, 4, 12)
PALIGNFOLD5U3A(foldArrayBy5A21AA, 8, 4, 16)
PALIGNFOLD5U(foldArrayBy5A2211, 8, 8, 4, 4)
PALIGNFOLD5U(foldArrayBy5A2212, 8, 8, 4, 8)
PALIGNFOLD5U(foldArrayBy5A2213, 8, 8, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A221A, 8, 8, 4)
PALIGNFOLD5U(foldArrayBy5A2221, 8, 8, 8, 4)
PALIGNFOLD5U(foldArrayBy5A2222, 8, 8, 8, 8)
PALIGNFOLD5U(foldArrayBy5A2223, 8, 8, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A222A, 8, 8, 8)
PALIGNFOLD5U(foldArrayBy5A2231, 8, 8, 12, 4)
PALIGNFOLD5U(foldArrayBy5A2232, 8, 8, 12, 8)
PALIGNFOLD5U(foldArrayBy5A2233, 8, 8, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A223A, 8, 8, 12)
PALIGNFOLD5U3A(foldArrayBy5A22A1, 8, 8, 4)
PALIGNFOLD5U3A(foldArrayBy5A22A2, 8, 8, 8)
PALIGNFOLD5U3A(foldArrayBy5A22A3, 8, 8, 12)
PALIGNFOLD5U3A(foldArrayBy5A22AA, 8, 8, 16)
PALIGNFOLD5U(foldArrayBy5A2311, 8, 12, 4, 4)
PALIGNFOLD5U(foldArrayBy5A2312, 8, 12, 4, 8)
PALIGNFOLD5U(foldArrayBy5A2313, 8, 12, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A231A, 8, 12, 4)
PALIGNFOLD5U(foldArrayBy5A2321, 8, 12, 8, 4)
PALIGNFOLD5U(foldArrayBy5A2322, 8, 12, 8, 8)
PALIGNFOLD5U(foldArrayBy5A2323, 8, 12, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A232A, 8, 12, 8)
PALIGNFOLD5U(foldArrayBy5A2331, 8, 12, 12, 4)
PALIGNFOLD5U(foldArrayBy5A2332, 8, 12, 12, 8)
PALIGNFOLD5U(foldArrayBy5A2333, 8, 12, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A233A, 8, 12, 12)
PALIGNFOLD5U3A(foldArrayBy5A23A1, 8, 12, 4)
PALIGNFOLD5U3A(foldArrayBy5A23A2, 8, 12, 8)
PALIGNFOLD5U3A(foldArrayBy5A23A3, 8, 12, 12)
PALIGNFOLD5U3A(foldArrayBy5A23AA, 8, 12, 16)
PALIGNFOLD5U2A(foldArrayBy5A2A11, 8, 4, 4)
PALIGNFOLD5U2A(foldArrayBy5A2A12, 8, 4, 8)
PALIGNFOLD5U2A(foldArrayBy5A2A13, 8, 4, 12)
PALIGNFOLD5U2A(foldArrayBy5A2A1A, 8, 4, 16)
PALIGNFOLD5U2A(foldArrayBy5A2A21, 8, 8, 4)
PALIGNFOLD5U2A(foldArrayBy5A2A22, 8, 8, 8)
PALIGNFOLD5U2A(foldArrayBy5A2A23, 8, 8, 12)
PALIGNFOLD5U2A(foldArrayBy5A2A2A, 8, 8, 16)
PALIGNFOLD5U2A(foldArrayBy5A2A31, 8, 12, 4)
PALIGNFOLD5U2A(foldArrayBy5A2A32, 8, 12, 8)
PALIGNFOLD5U2A(foldArrayBy5A2A33, 8, 12, 12)
PALIGNFOLD5U2A(foldArrayBy5A2A3A, 8, 12, 16)
PALIGNFOLD5U2A(foldArrayBy5A2AA1, 8, 16, 4)
PALIGNFOLD5U2A(foldArrayBy5A2AA2, 8, 16, 8)
PALIGNFOLD5U2A(foldArrayBy5A2AA3, 8, 16, 12)
PALIGNFOLD5U234A(foldArrayBy5A2AAA, 8)
PALIGNFOLD5U(foldArrayBy5A3111, 12, 4, 4, 4)
PALIGNFOLD5U(foldArrayBy5A3112, 12, 4, 4, 8)
PALIGNFOLD5U(foldArrayBy5A3113, 12, 4, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A311A, 12, 4, 4)
PALIGNFOLD5U(foldArrayBy5A3121, 12, 4, 8, 4)
PALIGNFOLD5U(foldArrayBy5A3122, 12, 4, 8, 8)
PALIGNFOLD5U(foldArrayBy5A3123, 12, 4, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A312A, 12, 4, 8)
PALIGNFOLD5U(foldArrayBy5A3131, 12, 4, 12, 4)
PALIGNFOLD5U(foldArrayBy5A3132, 12, 4, 12, 8)
PALIGNFOLD5U(foldArrayBy5A3133, 12, 4, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A313A, 12, 4, 12)
PALIGNFOLD5U3A(foldArrayBy5A31A1, 12, 4, 4)
PALIGNFOLD5U3A(foldArrayBy5A31A2, 12, 4, 8)
PALIGNFOLD5U3A(foldArrayBy5A31A3, 12, 4, 12)
PALIGNFOLD5U3A(foldArrayBy5A31AA, 12, 4, 16)
PALIGNFOLD5U(foldArrayBy5A3211, 12, 8, 4, 4)
PALIGNFOLD5U(foldArrayBy5A3212, 12, 8, 4, 8)
PALIGNFOLD5U(foldArrayBy5A3213, 12, 8, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A321A, 12, 8, 4)
PALIGNFOLD5U(foldArrayBy5A3221, 12, 8, 8, 4)
PALIGNFOLD5U(foldArrayBy5A3222, 12, 8, 8, 8)
PALIGNFOLD5U(foldArrayBy5A3223, 12, 8, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A322A, 12, 8, 8)
PALIGNFOLD5U(foldArrayBy5A3231, 12, 8, 12, 4)
PALIGNFOLD5U(foldArrayBy5A3232, 12, 8, 12, 8)
PALIGNFOLD5U(foldArrayBy5A3233, 12, 8, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A323A, 12, 8, 12)
PALIGNFOLD5U3A(foldArrayBy5A32A1, 12, 8, 4)
PALIGNFOLD5U3A(foldArrayBy5A32A2, 12, 8, 8)
PALIGNFOLD5U3A(foldArrayBy5A32A3, 12, 8, 12)
PALIGNFOLD5U3A(foldArrayBy5A32AA, 12, 8, 16)
PALIGNFOLD5U(foldArrayBy5A3311, 12, 12, 4, 4)
PALIGNFOLD5U(foldArrayBy5A3312, 12, 12, 4, 8)
PALIGNFOLD5U(foldArrayBy5A3313, 12, 12, 4, 12)
PALIGNFOLD5U4A(foldArrayBy5A331A, 12, 12, 4)
PALIGNFOLD5U(foldArrayBy5A3321, 12, 12, 8, 4)
PALIGNFOLD5U(foldArrayBy5A3322, 12, 12, 8, 8)
PALIGNFOLD5U(foldArrayBy5A3323, 12, 12, 8, 12)
PALIGNFOLD5U4A(foldArrayBy5A332A, 12, 12, 8)
PALIGNFOLD5U(foldArrayBy5A3331, 12, 12, 12, 4)
PALIGNFOLD5U(foldArrayBy5A3332, 12, 12, 12, 8)
PALIGNFOLD5U(foldArrayBy5A3333, 12, 12, 12, 12)
PALIGNFOLD5U4A(foldArrayBy5A333A, 12, 12, 12)
PALIGNFOLD5U3A(foldArrayBy5A33A1, 12, 12, 4)
PALIGNFOLD5U3A(foldArrayBy5A33A2, 12, 12, 8)
PALIGNFOLD5U3A(foldArrayBy5A33A3, 12, 12, 12)
PALIGNFOLD5U3A(foldArrayBy5A33AA, 12, 12, 16)
PALIGNFOLD5U2A(foldArrayBy5A3A11, 12, 4, 4)
PALIGNFOLD5U2A(foldArrayBy5A3A12, 12, 4, 8)
PALIGNFOLD5U2A(foldArrayBy5A3A13, 12, 4, 12)
PALIGNFOLD5U2A(foldArrayBy5A3A1A, 12, 4, 16)
PALIGNFOLD5U2A(foldArrayBy5A3A21, 12, 8, 4)
PALIGNFOLD5U2A(foldArrayBy5A3A22, 12, 8, 8)
PALIGNFOLD5U2A(foldArrayBy5A3A23, 12, 8, 12)
PALIGNFOLD5U2A(foldArrayBy5A3A2A, 12, 8, 16)
PALIGNFOLD5U2A(foldArrayBy5A3A31, 12, 12, 4)
PALIGNFOLD5U2A(foldArrayBy5A3A32, 12, 12, 8)
PALIGNFOLD5U2A(foldArrayBy5A3A33, 12, 12, 12)
PALIGNFOLD5U2A(foldArrayBy5A3A3A, 12, 12, 16)
PALIGNFOLD5U2A(foldArrayBy5A3AA1, 12, 16, 4)
PALIGNFOLD5U2A(foldArrayBy5A3AA2, 12, 16, 8)
PALIGNFOLD5U2A(foldArrayBy5A3AA3, 12, 16, 12)
PALIGNFOLD5U234A(foldArrayBy5A3AAA, 12)
PALIGNFOLD5U1A(foldArrayBy5AA111, 4, 4, 4)
PALIGNFOLD5U1A(foldArrayBy5AA112, 4, 4, 8)
PALIGNFOLD5U1A(foldArrayBy5AA113, 4, 4, 12)
PALIGNFOLD5U1A(foldArrayBy5AA11A, 4, 4, 16)
PALIGNFOLD5U1A(foldArrayBy5AA121, 4, 8, 4)
PALIGNFOLD5U1A(foldArrayBy5AA122, 4, 8, 8)
PALIGNFOLD5U1A(foldArrayBy5AA123, 4, 8, 12)
PALIGNFOLD5U1A(foldArrayBy5AA12A, 4, 8, 16)
PALIGNFOLD5U1A(foldArrayBy5AA131, 4, 12, 4)
PALIGNFOLD5U1A(foldArrayBy5AA132, 4, 12, 8)
PALIGNFOLD5U1A(foldArrayBy5AA133, 4, 12, 12)
PALIGNFOLD5U1A(foldArrayBy5AA13A, 4, 12, 16)
PALIGNFOLD5U1A(foldArrayBy5AA1A1, 4, 16, 4)
PALIGNFOLD5U1A(foldArrayBy5AA1A2, 4, 16, 8)
PALIGNFOLD5U1A(foldArrayBy5AA1A3, 4, 16, 12)
PALIGNFOLD5U134A(foldArrayBy5AA1AA, 4)
PALIGNFOLD5U1A(foldArrayBy5AA211, 8, 4, 4)
PALIGNFOLD5U1A(foldArrayBy5AA212, 8, 4, 8)
PALIGNFOLD5U1A(foldArrayBy5AA213, 8, 4, 12)
PALIGNFOLD5U1A(foldArrayBy5AA21A, 8, 4, 16)
PALIGNFOLD5U1A(foldArrayBy5AA221, 8, 8, 4)
PALIGNFOLD5U1A(foldArrayBy5AA222, 8, 8, 8)
PALIGNFOLD5U1A(foldArrayBy5AA223, 8, 8, 12)
PALIGNFOLD5U1A(foldArrayBy5AA22A, 8, 8, 16)
PALIGNFOLD5U1A(foldArrayBy5AA231, 8, 12, 4)
PALIGNFOLD5U1A(foldArrayBy5AA232, 8, 12, 8)
PALIGNFOLD5U1A(foldArrayBy5AA233, 8, 12, 12)
PALIGNFOLD5U1A(foldArrayBy5AA23A, 8, 12, 16)
PALIGNFOLD5U1A(foldArrayBy5AA2A1, 8, 16, 4)
PALIGNFOLD5U1A(foldArrayBy5AA2A2, 8, 16, 8)
PALIGNFOLD5U1A(foldArrayBy5AA2A3, 8, 16, 12)
PALIGNFOLD5U134A(foldArrayBy5AA2AA, 8)
PALIGNFOLD5U1A(foldArrayBy5AA311, 12, 4, 4)
PALIGNFOLD5U1A(foldArrayBy5AA312, 12, 4, 8)
PALIGNFOLD5U1A(foldArrayBy5AA313, 12, 4, 12)
PALIGNFOLD5U1A(foldArrayBy5AA31A, 12, 4, 16)
PALIGNFOLD5U1A(foldArrayBy5AA321, 12, 8, 4)
PALIGNFOLD5U1A(foldArrayBy5AA322, 12, 8, 8)
PALIGNFOLD5U1A(foldArrayBy5AA323, 12, 8, 12)
PALIGNFOLD5U1A(foldArrayBy5AA32A, 12, 8, 16)
PALIGNFOLD5U1A(foldArrayBy5AA331, 12, 12, 4)
PALIGNFOLD5U1A(foldArrayBy5AA332, 12, 12, 8)
PALIGNFOLD5U1A(foldArrayBy5AA333, 12, 12, 12)
PALIGNFOLD5U1A(foldArrayBy5AA33A, 12, 12, 16)
PALIGNFOLD5U1A(foldArrayBy5AA3A1, 12, 16, 4)
PALIGNFOLD5U1A(foldArrayBy5AA3A2, 12, 16, 8)
PALIGNFOLD5U1A(foldArrayBy5AA3A3, 12, 16, 12)
PALIGNFOLD5U134A(foldArrayBy5AA3AA, 12)
PALIGNFOLD5U1A(foldArrayBy5AAA11, 16, 4, 4)
PALIGNFOLD5U1A(foldArrayBy5AAA12, 16, 4, 8)
PALIGNFOLD5U1A(foldArrayBy5AAA13, 16, 4, 12)
PALIGNFOLD5U124A(foldArrayBy5AAA1A, 4)
PALIGNFOLD5U1A(foldArrayBy5AAA21, 16, 8, 4)
PALIGNFOLD5U1A(foldArrayBy5AAA22, 16, 8, 8)
PALIGNFOLD5U1A(foldArrayBy5AAA23, 16, 8, 12)
PALIGNFOLD5U124A(foldArrayBy5AAA2A, 8)
PALIGNFOLD5U1A(foldArrayBy5AAA31, 16, 12, 4)
PALIGNFOLD5U1A(foldArrayBy5AAA32, 16, 12, 8)
PALIGNFOLD5U1A(foldArrayBy5AAA33, 16, 12, 12)
PALIGNFOLD5U124A(foldArrayBy5AAA3A, 12)
PALIGNFOLD5U123A(foldArrayBy5AAAA1, 4)
PALIGNFOLD5U123A(foldArrayBy5AAAA2, 8)
PALIGNFOLD5U123A(foldArrayBy5AAAA3, 12)
//PALIGNFOLD5U1A(foldArrayBy5AAAAA, 16, 16, 16)

float (*fold5fns[])(const float *, int, int, int, int, int, float *) = {
  foldArrayBy5AAAAA, foldArrayBy5AAAA1, foldArrayBy5AAAA2, foldArrayBy5AAAA3,
  foldArrayBy5AAA1A, foldArrayBy5AAA11, foldArrayBy5AAA12, foldArrayBy5AAA13,
  foldArrayBy5AAA2A, foldArrayBy5AAA21, foldArrayBy5AAA22, foldArrayBy5AAA23,
  foldArrayBy5AAA3A, foldArrayBy5AAA31, foldArrayBy5AAA32, foldArrayBy5AAA33,
  foldArrayBy5AA1AA, foldArrayBy5AA1A1, foldArrayBy5AA1A2, foldArrayBy5AA1A3,
  foldArrayBy5AA11A, foldArrayBy5AA111, foldArrayBy5AA112, foldArrayBy5AA113,
  foldArrayBy5AA12A, foldArrayBy5AA121, foldArrayBy5AA122, foldArrayBy5AA123,
  foldArrayBy5AA13A, foldArrayBy5AA131, foldArrayBy5AA132, foldArrayBy5AA133,
  foldArrayBy5AA2AA, foldArrayBy5AA2A1, foldArrayBy5AA2A2, foldArrayBy5AA2A3,
  foldArrayBy5AA21A, foldArrayBy5AA211, foldArrayBy5AA212, foldArrayBy5AA213,
  foldArrayBy5AA22A, foldArrayBy5AA221, foldArrayBy5AA222, foldArrayBy5AA223,
  foldArrayBy5AA23A, foldArrayBy5AA231, foldArrayBy5AA232, foldArrayBy5AA233,
  foldArrayBy5AA3AA, foldArrayBy5AA3A1, foldArrayBy5AA3A2, foldArrayBy5AA3A3,
  foldArrayBy5AA31A, foldArrayBy5AA311, foldArrayBy5AA312, foldArrayBy5AA313,
  foldArrayBy5AA32A, foldArrayBy5AA321, foldArrayBy5AA322, foldArrayBy5AA323,
  foldArrayBy5AA33A, foldArrayBy5AA331, foldArrayBy5AA332, foldArrayBy5AA333,

  foldArrayBy5A1AAA, foldArrayBy5A1AA1, foldArrayBy5A1AA2, foldArrayBy5A1AA3,
  foldArrayBy5A1A1A, foldArrayBy5A1A11, foldArrayBy5A1A12, foldArrayBy5A1A13,
  foldArrayBy5A1A2A, foldArrayBy5A1A21, foldArrayBy5A1A22, foldArrayBy5A1A23,
  foldArrayBy5A1A3A, foldArrayBy5A1A31, foldArrayBy5A1A32, foldArrayBy5A1A33,
  foldArrayBy5A11AA, foldArrayBy5A11A1, foldArrayBy5A11A2, foldArrayBy5A11A3,
  foldArrayBy5A111A, foldArrayBy5A1111, foldArrayBy5A1112, foldArrayBy5A1113,
  foldArrayBy5A112A, foldArrayBy5A1121, foldArrayBy5A1122, foldArrayBy5A1123,
  foldArrayBy5A113A, foldArrayBy5A1131, foldArrayBy5A1132, foldArrayBy5A1133,
  foldArrayBy5A12AA, foldArrayBy5A12A1, foldArrayBy5A12A2, foldArrayBy5A12A3,
  foldArrayBy5A121A, foldArrayBy5A1211, foldArrayBy5A1212, foldArrayBy5A1213,
  foldArrayBy5A122A, foldArrayBy5A1221, foldArrayBy5A1222, foldArrayBy5A1223,
  foldArrayBy5A123A, foldArrayBy5A1231, foldArrayBy5A1232, foldArrayBy5A1233,
  foldArrayBy5A13AA, foldArrayBy5A13A1, foldArrayBy5A13A2, foldArrayBy5A13A3,
  foldArrayBy5A131A, foldArrayBy5A1311, foldArrayBy5A1312, foldArrayBy5A1313,
  foldArrayBy5A132A, foldArrayBy5A1321, foldArrayBy5A1322, foldArrayBy5A1323,
  foldArrayBy5A133A, foldArrayBy5A1331, foldArrayBy5A1332, foldArrayBy5A1333,

  foldArrayBy5A2AAA, foldArrayBy5A2AA1, foldArrayBy5A2AA2, foldArrayBy5A2AA3,
  foldArrayBy5A2A1A, foldArrayBy5A2A11, foldArrayBy5A2A12, foldArrayBy5A2A13,
  foldArrayBy5A2A2A, foldArrayBy5A2A21, foldArrayBy5A2A22, foldArrayBy5A2A23,
  foldArrayBy5A2A3A, foldArrayBy5A2A31, foldArrayBy5A2A32, foldArrayBy5A2A33,
  foldArrayBy5A21AA, foldArrayBy5A21A1, foldArrayBy5A21A2, foldArrayBy5A21A3,
  foldArrayBy5A211A, foldArrayBy5A2111, foldArrayBy5A2112, foldArrayBy5A2113,
  foldArrayBy5A212A, foldArrayBy5A2121, foldArrayBy5A2122, foldArrayBy5A2123,
  foldArrayBy5A213A, foldArrayBy5A2131, foldArrayBy5A2132, foldArrayBy5A2133,
  foldArrayBy5A22AA, foldArrayBy5A22A1, foldArrayBy5A22A2, foldArrayBy5A22A3,
  foldArrayBy5A221A, foldArrayBy5A2211, foldArrayBy5A2212, foldArrayBy5A2213,
  foldArrayBy5A222A, foldArrayBy5A2221, foldArrayBy5A2222, foldArrayBy5A2223,
  foldArrayBy5A223A, foldArrayBy5A2231, foldArrayBy5A2232, foldArrayBy5A2233,
  foldArrayBy5A23AA, foldArrayBy5A23A1, foldArrayBy5A23A2, foldArrayBy5A23A3,
  foldArrayBy5A231A, foldArrayBy5A2311, foldArrayBy5A2312, foldArrayBy5A2313,
  foldArrayBy5A232A, foldArrayBy5A2321, foldArrayBy5A2322, foldArrayBy5A2323,
  foldArrayBy5A233A, foldArrayBy5A2331, foldArrayBy5A2332, foldArrayBy5A2333,

  foldArrayBy5A3AAA, foldArrayBy5A3AA1, foldArrayBy5A3AA2, foldArrayBy5A3AA3,
  foldArrayBy5A3A1A, foldArrayBy5A3A11, foldArrayBy5A3A12, foldArrayBy5A3A13,
  foldArrayBy5A3A2A, foldArrayBy5A3A21, foldArrayBy5A3A22, foldArrayBy5A3A23,
  foldArrayBy5A3A3A, foldArrayBy5A3A31, foldArrayBy5A3A32, foldArrayBy5A3A33,
  foldArrayBy5A31AA, foldArrayBy5A31A1, foldArrayBy5A31A2, foldArrayBy5A31A3,
  foldArrayBy5A311A, foldArrayBy5A3111, foldArrayBy5A3112, foldArrayBy5A3113,
  foldArrayBy5A312A, foldArrayBy5A3121, foldArrayBy5A3122, foldArrayBy5A3123,
  foldArrayBy5A313A, foldArrayBy5A3131, foldArrayBy5A3132, foldArrayBy5A3133,
  foldArrayBy5A32AA, foldArrayBy5A32A1, foldArrayBy5A32A2, foldArrayBy5A32A3,
  foldArrayBy5A321A, foldArrayBy5A3211, foldArrayBy5A3212, foldArrayBy5A3213,
  foldArrayBy5A322A, foldArrayBy5A3221, foldArrayBy5A3222, foldArrayBy5A3223,
  foldArrayBy5A323A, foldArrayBy5A3231, foldArrayBy5A3232, foldArrayBy5A3233,
  foldArrayBy5A33AA, foldArrayBy5A33A1, foldArrayBy5A33A2, foldArrayBy5A33A3,
  foldArrayBy5A331A, foldArrayBy5A3311, foldArrayBy5A3312, foldArrayBy5A3313,
  foldArrayBy5A332A, foldArrayBy5A3321, foldArrayBy5A3322, foldArrayBy5A3323,
  foldArrayBy5A333A, foldArrayBy5A3331, foldArrayBy5A3332, foldArrayBy5A3333,
};
#endif
#endif


/**********************
 *
 * find_pulse - uses a folding algorithm to find repeating pulses
 *   Initially folds by prime number then by powers of two.
 *
 * Initial code by Eric Korpela
 * Major revisions and optimization by Ben Herndon
 *
 */
int find_pulse(const float * fp_PulsePot, int PulsePotLen,
               float pulse_thresh, int TOffset, int FOffset) {
  static float *div,*FoldedPOT;
  static float *divref;
//  float period;
  unsigned int ndivs;
  unsigned int i,dbins=PulsePotLen, nperiods;
  unsigned int p;
  float fp;
//  float cperiod;
  float max=0,maxd=0,avg=0,ravg,rms=0,maxp=0, snr=0, fthresh=0, mmax_scale;
  unsigned int num_adds;
  register float tmp_max,tmp_max_vec,t1,tmp_maxref;
  float stmp_max;
  int res=1;
  float pulse_display_thresh = (float) swi.analysis_cfg.pulse_display_thresh;
  float rPulsePotLen = 1.0f/PulsePotLen;
#if defined(__VEC__) && defined(USE_VECTORIZED_PULSE)
  vFloat vtempavg, vtemprms;
#endif
#if (defined(__VEC__) || defined(__SSE3__)) && defined(USE_VECTORIZED_PULSE)
  vFloat vavg, vrms;
#endif
  // debug possible heap corruption -- jeffc
#ifdef _WIN32
  BOINCASSERT(_CrtCheckMemory());
#endif

#ifdef DEBUG_PULSE_VERBOSE
  fprintf(stderr, "In find_pulse()...   PulsePotLen = %d   pulse_thresh = %f   TOffset = %d   PoT = %d\n",  PulsePotLen,  pulse_thresh, TOffset, FOffset);
#endif

  // boinc_worker_timer();
  
  init_t_funct();
  
  //  Create internal arrays....
  if (!div) {
    FoldedPOT=(float *)malloc_a((PoTInfo.PulseMax+1)*sizeof(float)/3,MEM_ALIGN);
    div=(float *)malloc_a(PoTInfo.PulseMax*3*sizeof(float),MEM_ALIGN);
    if ((!div) || (!FoldedPOT)) SETIERROR(MALLOC_FAILED, "(!div) || (!FoldedPOT)");
    if (t_funct(6,1000,0)<0) SETIERROR(MALLOC_FAILED, "t_funct(6,1000,0)<0");;
	
	divref=(float *)malloc_a(PoTInfo.PulseMax*3*sizeof(float),MEM_ALIGN);
  }

  //  Calculate average power /* and RMS deviation */
#if defined(__VEC__) && defined(USE_VECTORIZED_PULSE)
  vtempavg = ZERO;
  int vEnd = PulsePotLen - (PulsePotLen & 3);
  for (i = 0; i < vEnd ; i+=4) {
    const float *p = (const float *) (fp_PulsePot + i);
    vector float p1 = vec_ld(0, p);
    
    vtempavg = vec_add(vtempavg, p1);
  }
  vavg = vtempavg;
  for (;i<PulsePotLen;i++) {
    avg+=fp_PulsePot[i];
  }
  float *vpavg = (float *) &vavg;
  avg += (vpavg[0] + vpavg[1]) + (vpavg[2] + vpavg[3]);
  avg *= rPulsePotLen;
  ravg = 1.0f/avg;
#elif defined(__SSE3__) && defined(USE_VECTORIZED_PULSE)
  vavg = _mm_setzero_ps();
  int vEnd = PulsePotLen - (PulsePotLen & 3);
  for (i = 0; i < vEnd ; i+=4) {
    const float *p = (const float *) (fp_PulsePot + i);
    vFloat p1 = _mm_load_ps(p);
    
    vavg = _mm_add_ps(vavg, p1);
  }
//  vavg = _mm_hadd_ps(vavg, vavg);
//  vavg = _mm_hadd_ps(vavg, vavg);
  vavg = _mm_add_ps(vavg, _mm_movehl_ps(vavg, vavg));
  vavg = _mm_add_ss(vavg, _mm_movehdup_ps(vavg));
//  _mm_store_ss(&avg, vavg);
  avg = _mm_cvtss_f32(vavg);
  for (;i<PulsePotLen;i++) {
    avg+=fp_PulsePot[i];
  }
  avg *= rPulsePotLen;
  ravg = 1.0f/avg;
#elif defined(USE_VECTORIZED_PULSE)
  vDSP_meanv((float *) fp_PulsePot, 1, &avg, PulsePotLen);
  ravg = 1.0f/avg;
#else
  for (i=0;i<PulsePotLen;i++) {
    avg+=fp_PulsePot[i];
  }
  avg*=rPulsePotLen;
  ravg = 1.0f/avg;
#endif
  
//  float bailmax = 0.0;
//  for (i=0;i<PulsePotLen;i++) {
//    if (fp_PulsePot[i] > bailmax) bailmax = fp_PulsePot[i];
//  }
  
  for (i = 32, ndivs = 1; i <= PulsePotLen; ndivs++, i *= 2);
  // ndivs=(int)(log((float)PulsePotLen)/log(2.0)-3);

  int32_t thePotLen = PulsePotLen;
  nperiods =  (thePotLen*2)/3-(thePotLen*1)/2;
  nperiods += (thePotLen*3)/4-(thePotLen*3)/5;
  nperiods += (thePotLen*4)/5-(thePotLen*4)/6;
  nperiods *= ndivs;

  //  Periods from PulsePotLen/3 to PulsePotLen/4, and power of 2 fractions of.
  //   then (len/4 to len/5) and finally (len/5 to len/6)
  //
  int32_t firstP, lastP;
  for(num_adds = 3; num_adds <= 5; num_adds++) {
    float rnum_adds_minus1;
    unsigned int num_adds_minus1 = num_adds - 1;
    unsigned int roundbias = num_adds_minus1 / 2;
    
    switch(num_adds) {
      case 3: lastP = (thePotLen*2)/3;  firstP = (thePotLen*1)/2; rnum_adds_minus1 = 0.5f; break;
      case 4: lastP = (thePotLen*3)/4;  firstP = (thePotLen*3)/5; rnum_adds_minus1 = 1.0f / 3.0f; break;
      case 5: lastP = (thePotLen*4)/5;  firstP = (thePotLen*4)/6; rnum_adds_minus1 = 0.25f; break;
    }
    
//    unsigned int nonbails = 0;

    for (p = lastP, fp = lastP ; p > firstP ; p--, fp--) {
      unsigned int tmp0, tmp1, tmp2, tmp3;
      float cur_thresh , dis_thresh;
      float cpy_thresh;
      float thresh_scale;
      float max_scale;
      int tabofst = ndivs*3 - 1 + 3 - num_adds;
      dbins = PulsePotLen; // Output storage
      float cperiod = fp * rnum_adds_minus1;
      unsigned int di, offset;
//      cperiod = period;
      
      // float -> int conversion and int division are painful on PPC
      switch(num_adds) {
        case 3:
          tmp0 = (p + roundbias) / 2;
          tmp1 = (p*2 + roundbias) / 2;
          di = p / 2;
          max_scale = 1.0f / 3.0f;
          break;
        case 4:
          tmp0 = (p + roundbias) / 3;
          tmp1 = (p*2 + roundbias) / 3;
          di = p / 3;
          max_scale = 1.0f / 4.0f;
          break;
        case 5:
          tmp0 = (p + roundbias) / 4;
          tmp1 = (p*2 + roundbias) / 4;
          di = p / 4;
          max_scale = 1.0f / 5.0f;
          break;
      }
          
//      tmp0 = (int)(cperiod+0.5f);
//      tmp1 = (int)(cperiod*2.0f+0.5f);
//      di  = (int)cperiod;

//      if ((unsigned)di >= ((unsigned)PoTInfo.PulseMax)*2)
//        SETIERROR(FP_ERROR,"FPU failure in pulse_find");

      thresh_scale = t_funct(di,num_adds,di+tabofst);
      cur_thresh = thresh_scale*avg;
//      dis_thresh = (cur_thresh-avg)*pulse_display_thresh + avg;
//      cpy_thresh = cur_thresh+maxd;
	  dis_thresh = avg*(thresh_scale*pulse_display_thresh + (1 - pulse_display_thresh));
	  cpy_thresh = thresh_scale*avg+maxd;
      //      dis_thresh=(1.0+PULSE_DISPLAY_THRESH/sqrt(num_adds))*avg;
      
      int baildi = di >> (ndivs - 1);
      int bailtab = tabofst - 3 * (ndivs - 1);
      float lowest_thresh = t_funct(baildi, num_adds << (ndivs - 1), baildi + bailtab);

      float bail_scale = best_pulse->score;
      if (bail_scale > 1.0f) bail_scale = 1.0f;
      else if (bail_scale < pulse_display_thresh) bail_scale = pulse_display_thresh;
      float bail_thresh = avg*(lowest_thresh*bail_scale + (1 - bail_scale));
      
//      if (PulsePotLen < 200) printf("max = %f di = %d num_adds = %d bail_scale = %g thresh_scale = %g dis_thresh = %g\n", bailmax / avg, di, num_adds, lowest_thresh*bail_scale + (1 - bail_scale), thresh_scale, thresh_scale*pulse_display_thresh + (1 - pulse_display_thresh));
      
      switch(num_adds) {
	#define LOOP_THRESH 8
	#define LOOP_COUNT 5
        case 3:
          //   tmp_max = sum3tables(di, &fp_PulsePot[0], &fp_PulsePot[tmp0], &fp_PulsePot[tmp1], &fp_PulsePot[dbins]);
#if defined(__VEC__) && defined(USE_VECTORIZED_PULSE)
          tmp_max = foldArrayBy3SP(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
          
#elif defined(__SSE2__) && defined(USE_VECTORIZED_PULSE)
#ifdef USE_PALIGNED_PULSE
		  if (tmp0 & 0x2) {
			if (tmp0 & 0x1) {
			  if (tmp1 & 0x2) {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3A33(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3A32(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  } else {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3A31(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3A3A(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  }
			} else {
			  if (tmp1 & 0x2) {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3A23(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3A22(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  } else {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3A21(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3A2A(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  }
			}
		  } else{
			if (tmp0 & 0x1) {
			  if (tmp1 & 0x2) {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3A13(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3A12(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  } else {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3A11(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3A1A(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  }
			} else {
			  if (tmp1 & 0x2) {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3AA3(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3AA2(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  } else {
				if (tmp1 & 0x1) tmp_max = foldArrayBy3AA1(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
				else			tmp_max = foldArrayBy3AAA(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
			  }
			}			
		  }
//		  tmp_max = fold3fns[((tmp0 & 0x3) << 2) + (tmp1 & 0x3)](fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
//		  tmp_maxref = 0.0f;
//          for (i=0;i<di;i++) {
//            register float tmpfloat=(fp_PulsePot[i]+fp_PulsePot[tmp0+i]+fp_PulsePot[tmp1+i])/3;
//            divref[i]=tmpfloat;
//            if (tmpfloat>tmp_maxref) {
//              tmp_maxref=tmpfloat;
//            }
//          }
			
#else
// 	  if (di < LOOP_THRESH) {
//             tmp_max = 0.0f;
//             #pragma loop count (LOOP_COUNT)
//             for (i=0;i<di;i++) {
//               register float tmpfloat=(fp_PulsePot[i]+fp_PulsePot[tmp0+i]+fp_PulsePot[tmp1+i]);
//               div[i]=tmpfloat;
//               if (tmpfloat>tmp_max) {
//                 tmp_max=tmpfloat;
//               }
//             }
//             tmp_max *= max_scale;
// 	  } else {
            tmp_max = foldArrayBy3(fp_PulsePot,di,tmp0,tmp1,div) * max_scale;
//           }
#endif
#else
          tmp_max = 0.0f;
          for (i=0;i<di;i++) {
            register float tmpfloat=(fp_PulsePot[i]+fp_PulsePot[tmp0+i]+fp_PulsePot[tmp1+i])/3;
            div[i]=tmpfloat;
            if (tmpfloat>tmp_max) {
              tmp_max=tmpfloat;
            }
          }
#endif
//		  tmp_maxref = 0.0f;
//		           for (i=0;i<di;i++) {
//		             register float tmpfloat=(fp_PulsePot[i]+fp_PulsePot[tmp0+i]+fp_PulsePot[tmp1+i]);
//		             divref[i]=tmpfloat;
//		             if (REL_ERR(divref[i], div[i]) > 0.000001) printf("foldArrayBy3SP element failure (%d/%d)\n", i, di);
//		             if (tmpfloat>tmp_maxref) {
//		               tmp_maxref=tmpfloat;
//		             }
//		           }
//		             tmp_maxref *= max_scale;
//		           if (REL_ERR(tmp_maxref, tmp_max) > 0.000001) printf("foldArrayBy3SP failure (%p, %d, %d, %d, %p)\n", fp_PulsePot, di, tmp0, tmp1, div);

          break;
        case 4:
          //    tmp_max = sum4tables(di, &fp_PulsePot[0], &fp_PulsePot[tmp0], &fp_PulsePot[tmp1], &fp_PulsePot[tmp2], &fp_PulsePot[dbins]);
          tmp2 = (p*3 + roundbias) / 3;

          //          tmp2 = (int)(cperiod*3.0f+0.5f);
#if defined(__VEC__) && defined(USE_VECTORIZED_PULSE)
          tmp_max = foldArrayBy4SP(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
#elif defined(__SSE2__) && defined(USE_VECTORIZED_PULSE)
#ifdef USE_PALIGNED_PULSE
		  if (di & ~0x3f) {
			if (tmp0 & 0x2) {
			  if (tmp0 & 0x1) {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A333(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A332(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A331(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A33A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A323(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A322(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A321(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A32A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A313(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A312(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A311(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A31A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A3A3(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A3A2(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A3A1(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A3AA(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				}
			  } else {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A233(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A232(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A231(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A23A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A223(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A222(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A221(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A22A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A213(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A212(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A211(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A21A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A2A3(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A2A2(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A2A1(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A2AA(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				}
			  }
			} else {
			  if (tmp0 & 0x1) {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A133(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A132(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A131(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A13A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A123(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A122(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A121(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A12A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A113(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A112(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A111(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A11A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A1A3(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A1A2(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4A1A1(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4A1AA(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				}
			  } else {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AA33(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AA32(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AA31(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AA3A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AA23(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AA22(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AA21(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AA2A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AA13(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AA12(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AA11(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AA1A(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AAA3(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AAA2(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					} else {
					  if (tmp2 & 0x1) tmp_max = foldArrayBy4AAA1(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					  else			tmp_max = foldArrayBy4AAAA(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
					}						  
				  }
				}
			  }
			}			
		  } else
#endif
//           if (di < LOOP_THRESH) {
//             tmp_max = 0.0f;
//             #pragma loop count (LOOP_COUNT)
//             for (i=0;i<di;i++) {
//               register float tmpfloat=((fp_PulsePot[i]+fp_PulsePot[tmp0+i])+(fp_PulsePot[tmp1+i])+fp_PulsePot[tmp2+i]);
//               div[i]=tmpfloat;
//               if (tmpfloat>tmp_max) {
//                 tmp_max=tmpfloat;
//               }
//             }
//             tmp_max *= max_scale;
//           } else {
		  {
            tmp_max = foldArrayBy4(fp_PulsePot,di,tmp0,tmp1,tmp2,div) * max_scale;
		  }
//           }
#else
          tmp_max = 0.0f;
          for (i=0;i<di;i++) {
            register float tmpfloat=(fp_PulsePot[i]+fp_PulsePot[tmp0+i]+fp_PulsePot[tmp1+i]+fp_PulsePot[tmp2+i])/4;
            div[i]=tmpfloat;
            if (tmpfloat>tmp_max) {
              tmp_max=tmpfloat;
            }
          }
#endif
//		  tmp_maxref = 0.0f;
//		           for (i=0;i<di;i++) {
//		             register float tmpfloat=((fp_PulsePot[i]+fp_PulsePot[tmp0+i])+(fp_PulsePot[tmp1+i]+fp_PulsePot[tmp2+i]));
//		             divref[i]=tmpfloat;
//		             if (REL_ERR(divref[i], div[i]) > 0.000001) printf("foldArrayBy4SP element failure (%g %g) (%d/%d) (%d %d %d)\n", divref[i], div[i], i, di, tmp0, tmp1, tmp2);
//		             if (tmpfloat>tmp_maxref) {
//		               tmp_maxref=tmpfloat;
//		             }
//		           }
//		             tmp_maxref *= max_scale;
//		           if (REL_ERR(tmp_maxref, tmp_max) > 0.000001) printf("foldArrayBy4SP failure (%p, %d, %d, %d, %d, %p)\n", fp_PulsePot, di, tmp0, tmp1, tmp2, div);
          break;
        case 5:
          //    tmp_max = sum5tables(di, &fp_PulsePot[0], &fp_PulsePot[tmp0], &fp_PulsePot[tmp1], &fp_PulsePot[tmp2], &fp_PulsePot[tmp3], &fp_PulsePot[dbins]);
          tmp2 = (p*3 + roundbias) / 4;
          tmp3 = (p*4 + roundbias) / 4;

//          tmp2 = (int)(cperiod*3.0f+0.5f);
//          tmp3 = (int)(cperiod*4.0f+0.5f);
#if defined(__VEC__) && defined(USE_VECTORIZED_PULSE)
          tmp_max = foldArrayBy5SP(fp_PulsePot,di,tmp0,tmp1,tmp2,tmp3,div) * max_scale;
		  tmp_maxref = 0.0f;
#elif defined(__SSE2__) && defined(USE_VECTORIZED_PULSE)
#ifdef USE_PALIGNED_PULSE
#if 0
		tmp_max = fold5fns[((tmp0 & 0x3) << 6) + ((tmp1 & 0x3) << 4) + ((tmp2 & 0x3) << 2) + (tmp3 & 0x3)](fp_PulsePot,di,tmp0,tmp1,tmp2,tmp3,div) * max_scale;
#else
#define TMP3_DISPATCH(fna, fn1, fn2, fn3)                                                    \
		  {				                                                                                    \
			if (tmp3 & 0x2) {                                                                         \
			  if (tmp3 & 0x1) tmp_max = fn3(fp_PulsePot,di,tmp0,tmp1,tmp2,tmp3,div) * max_scale;      \
			  else			  tmp_max = fn2(fp_PulsePot,di,tmp0,tmp1,tmp2,tmp3,div) * max_scale;             \
			} else {                                                                                  \
			  if (tmp3 & 0x1) tmp_max = fn1(fp_PulsePot,di,tmp0,tmp1,tmp2,tmp3,div) * max_scale;      \
			  else			  tmp_max = fna(fp_PulsePot,di,tmp0,tmp1,tmp2,tmp3,div) * max_scale;             \
			}	                                                                                        \
		  }

		  if (di & ~0x3f) {
			if (tmp0 & 0x2) {
			  if (tmp0 & 0x1) {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A333A, foldArrayBy5A3331, foldArrayBy5A3332, foldArrayBy5A3333)
					  else			TMP3_DISPATCH(foldArrayBy5A332A, foldArrayBy5A3321, foldArrayBy5A3322, foldArrayBy5A3323)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A331A, foldArrayBy5A3311, foldArrayBy5A3312, foldArrayBy5A3313)
					  else			TMP3_DISPATCH(foldArrayBy5A33AA, foldArrayBy5A33A1, foldArrayBy5A33A2, foldArrayBy5A33A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A323A, foldArrayBy5A3231, foldArrayBy5A3232, foldArrayBy5A3233)
					  else			TMP3_DISPATCH(foldArrayBy5A322A, foldArrayBy5A3221, foldArrayBy5A3222, foldArrayBy5A3223)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A321A, foldArrayBy5A3211, foldArrayBy5A3212, foldArrayBy5A3213)
					  else			TMP3_DISPATCH(foldArrayBy5A32AA, foldArrayBy5A32A1, foldArrayBy5A32A2, foldArrayBy5A32A3)
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A313A, foldArrayBy5A3131, foldArrayBy5A3132, foldArrayBy5A3133)
					  else			TMP3_DISPATCH(foldArrayBy5A312A, foldArrayBy5A3121, foldArrayBy5A3122, foldArrayBy5A3123)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A311A, foldArrayBy5A3111, foldArrayBy5A3112, foldArrayBy5A3113)
					  else			TMP3_DISPATCH(foldArrayBy5A31AA, foldArrayBy5A31A1, foldArrayBy5A31A2, foldArrayBy5A31A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A3A3A, foldArrayBy5A3A31, foldArrayBy5A3A32, foldArrayBy5A3A33)
					  else			TMP3_DISPATCH(foldArrayBy5A3A2A, foldArrayBy5A3A21, foldArrayBy5A3A22, foldArrayBy5A3A23)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A3A1A, foldArrayBy5A3A11, foldArrayBy5A3A12, foldArrayBy5A3A13)
					  else			TMP3_DISPATCH(foldArrayBy5A3AAA, foldArrayBy5A3AA1, foldArrayBy5A3AA2, foldArrayBy5A3AA3)
					}						  
				  }
				}
			  } else {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A233A, foldArrayBy5A2331, foldArrayBy5A2332, foldArrayBy5A2333)
					  else			TMP3_DISPATCH(foldArrayBy5A232A, foldArrayBy5A2321, foldArrayBy5A2322, foldArrayBy5A2323)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A231A, foldArrayBy5A2311, foldArrayBy5A2312, foldArrayBy5A2313)
					  else			TMP3_DISPATCH(foldArrayBy5A23AA, foldArrayBy5A23A1, foldArrayBy5A23A2, foldArrayBy5A23A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A223A, foldArrayBy5A2231, foldArrayBy5A2232, foldArrayBy5A2233)
					  else			TMP3_DISPATCH(foldArrayBy5A222A, foldArrayBy5A2221, foldArrayBy5A2222, foldArrayBy5A2223)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A221A, foldArrayBy5A2211, foldArrayBy5A2212, foldArrayBy5A2213)
					  else			TMP3_DISPATCH(foldArrayBy5A22AA, foldArrayBy5A22A1, foldArrayBy5A22A2, foldArrayBy5A22A3)
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A213A, foldArrayBy5A2131, foldArrayBy5A2132, foldArrayBy5A2133)
					  else			TMP3_DISPATCH(foldArrayBy5A212A, foldArrayBy5A2121, foldArrayBy5A2122, foldArrayBy5A2123)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A211A, foldArrayBy5A2111, foldArrayBy5A2112, foldArrayBy5A2113)
					  else			TMP3_DISPATCH(foldArrayBy5A21AA, foldArrayBy5A21A1, foldArrayBy5A21A2, foldArrayBy5A21A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A2A3A, foldArrayBy5A2A31, foldArrayBy5A2A32, foldArrayBy5A2A33)
					  else			TMP3_DISPATCH(foldArrayBy5A2A2A, foldArrayBy5A2A21, foldArrayBy5A2A22, foldArrayBy5A2A23)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A2A1A, foldArrayBy5A2A11, foldArrayBy5A2A12, foldArrayBy5A2A13)
					  else			TMP3_DISPATCH(foldArrayBy5A2AAA, foldArrayBy5A2AA1, foldArrayBy5A2AA2, foldArrayBy5A2AA3)
					}						  
				  }
				}
			  }
			} else {
			  if (tmp0 & 0x1) {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A133A, foldArrayBy5A1331, foldArrayBy5A1332, foldArrayBy5A1333)
					  else			TMP3_DISPATCH(foldArrayBy5A132A, foldArrayBy5A1321, foldArrayBy5A1322, foldArrayBy5A1323)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A131A, foldArrayBy5A1311, foldArrayBy5A1312, foldArrayBy5A1313)
					  else			TMP3_DISPATCH(foldArrayBy5A13AA, foldArrayBy5A13A1, foldArrayBy5A13A2, foldArrayBy5A13A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A123A, foldArrayBy5A1231, foldArrayBy5A1232, foldArrayBy5A1233)
					  else			TMP3_DISPATCH(foldArrayBy5A122A, foldArrayBy5A1221, foldArrayBy5A1222, foldArrayBy5A1223)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A121A, foldArrayBy5A1211, foldArrayBy5A1212, foldArrayBy5A1213)
					  else			TMP3_DISPATCH(foldArrayBy5A12AA, foldArrayBy5A12A1, foldArrayBy5A12A2, foldArrayBy5A12A3)
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A113A, foldArrayBy5A1131, foldArrayBy5A1132, foldArrayBy5A1133)
					  else			TMP3_DISPATCH(foldArrayBy5A112A, foldArrayBy5A1121, foldArrayBy5A1122, foldArrayBy5A1123)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A111A, foldArrayBy5A1111, foldArrayBy5A1112, foldArrayBy5A1113)
					  else			TMP3_DISPATCH(foldArrayBy5A11AA, foldArrayBy5A11A1, foldArrayBy5A11A2, foldArrayBy5A11A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A1A3A, foldArrayBy5A1A31, foldArrayBy5A1A32, foldArrayBy5A1A33)
					  else			TMP3_DISPATCH(foldArrayBy5A1A2A, foldArrayBy5A1A21, foldArrayBy5A1A22, foldArrayBy5A1A23)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5A1A1A, foldArrayBy5A1A11, foldArrayBy5A1A12, foldArrayBy5A1A13)
					  else			TMP3_DISPATCH(foldArrayBy5A1AAA, foldArrayBy5A1AA1, foldArrayBy5A1AA2, foldArrayBy5A1AA3)
					}						  
				  }
				}
			  } else {
				if (tmp1 & 0x2) {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AA33A, foldArrayBy5AA331, foldArrayBy5AA332, foldArrayBy5AA333)
					  else			TMP3_DISPATCH(foldArrayBy5AA32A, foldArrayBy5AA321, foldArrayBy5AA322, foldArrayBy5AA323)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AA31A, foldArrayBy5AA311, foldArrayBy5AA312, foldArrayBy5AA313)
					  else			TMP3_DISPATCH(foldArrayBy5AA3AA, foldArrayBy5AA3A1, foldArrayBy5AA3A2, foldArrayBy5AA3A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AA23A, foldArrayBy5AA231, foldArrayBy5AA232, foldArrayBy5AA233)
					  else			TMP3_DISPATCH(foldArrayBy5AA22A, foldArrayBy5AA221, foldArrayBy5AA222, foldArrayBy5AA223)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AA21A, foldArrayBy5AA211, foldArrayBy5AA212, foldArrayBy5AA213)
					  else			TMP3_DISPATCH(foldArrayBy5AA2AA, foldArrayBy5AA2A1, foldArrayBy5AA2A2, foldArrayBy5AA2A3)
					}						  
				  }
				} else {
				  if (tmp1 & 0x1) {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AA13A, foldArrayBy5AA131, foldArrayBy5AA132, foldArrayBy5AA133)
					  else			TMP3_DISPATCH(foldArrayBy5AA12A, foldArrayBy5AA121, foldArrayBy5AA122, foldArrayBy5AA123)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AA11A, foldArrayBy5AA111, foldArrayBy5AA112, foldArrayBy5AA113)
					  else			TMP3_DISPATCH(foldArrayBy5AA1AA, foldArrayBy5AA1A1, foldArrayBy5AA1A2, foldArrayBy5AA1A3)
					}			
				  } else {
					if (tmp2 & 0x2) {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AAA3A, foldArrayBy5AAA31, foldArrayBy5AAA32, foldArrayBy5AAA33)
					  else			TMP3_DISPATCH(foldArrayBy5AAA2A, foldArrayBy5AAA21, foldArrayBy5AAA22, foldArrayBy5AAA23)
					} else {
					  if (tmp2 & 0x1) TMP3_DISPATCH(foldArrayBy5AAA1A, foldArrayBy5AAA11, foldArrayBy5AAA12, foldArrayBy5AAA13)
					  else			TMP3_DISPATCH(foldArrayBy5AAAAA, foldArrayBy5AAAA1, foldArrayBy5AAAA2, foldArrayBy5AAAA3)
					}						  
				  }
				}
			  }
			}
		  } else
#endif
#endif
//           if (di < LOOP_THRESH) {
//             tmp_max = 0.0f;
//             #pragma loop count (LOOP_COUNT)
//             for (i=0;i<di;i++) {
//               register float tmpfloat=((fp_PulsePot[i]+fp_PulsePot[tmp0+i])+(fp_PulsePot[tmp1+i]+fp_PulsePot[tmp2+i])+fp_PulsePot[tmp3+i]);
//               div[i]=tmpfloat;
//               if (tmpfloat>tmp_max) {
//                 tmp_max=tmpfloat;
//               }
//             }
//             tmp_max *= max_scale;
//           } else {
		  {
			tmp_max = foldArrayBy5(fp_PulsePot,di,tmp0,tmp1,tmp2,tmp3,div) * max_scale;
		  }
//           }
#else
          tmp_max = 0.0f;
          for (i=0;i<di;i++) {
            register float tmpfloat=(fp_PulsePot[i]+fp_PulsePot[tmp0+i]+fp_PulsePot[tmp1+i]+fp_PulsePot[tmp2+i]+fp_PulsePot[tmp3+i])/5;
            div[i]=tmpfloat;
            if (tmpfloat>tmp_max) {
              tmp_max=tmpfloat;
            }
          }
#endif
//          for (i=0;i<di;i++) {
//            register float tmpfloat=(((fp_PulsePot[i]+fp_PulsePot[tmp0+i])+(fp_PulsePot[tmp1+i]+fp_PulsePot[tmp2+i]))+fp_PulsePot[tmp3+i]);
//            divref[i]=tmpfloat;
//            if (REL_ERR(divref[i], div[i]) > 0.000001) printf("foldArrayBy5SP element failure (%d/%d)\n", i, di);
//            if (tmpfloat>tmp_maxref) {
//              tmp_maxref=tmpfloat;
//            }
//          }
//            tmp_maxref *= max_scale;
//          if (REL_ERR(tmp_maxref, tmp_max) > 0.000001) printf("foldArrayBy5SP failure (%p, %d, %d, %d, %d, %d, %p)\n", fp_PulsePot, di, tmp0, tmp1, tmp2, tmp3, div);
          break;
      }
                    
//      if (PulsePotLen < 200) printf(" observed = %g\n", tmp_max / avg);
                    
      if (tmp_max<bail_thresh) {
//        printf("bailing (1/%d): best=%g max=%g bail=%g dis=%g cpy=%g\n", ndivs, bail_scale, tmp_max/avg, bail_thresh/avg, dis_thresh/avg, cpy_thresh/avg);
        continue;
      }
                    
//      nonbails++;

      if (tmp_max>dis_thresh) {
        ReportPulseEvent(tmp_max*ravg,avg,cperiod*res,
                         TOffset+PulsePotLen/2,FOffset,
                         (tmp_max-avg)*sqrtf((float)num_adds)*ravg,
                         (cur_thresh-avg)*sqrtf((float)num_adds)*ravg, div, max_scale, 0);

        if (tmp_max>cpy_thresh) {
          maxp  = cperiod;
          maxd  = tmp_max - cur_thresh;
          max = tmp_max;
          snr = (float)((tmp_max-avg)*sqrt((float)num_adds)*ravg);
          fthresh=(float)((cur_thresh-avg)*sqrt((float)num_adds)*ravg);

          mmax_scale = max_scale;
          memcpy(FoldedPOT, div, di*sizeof(float));
        }		  
	  }

      offset =0;
      dbins =PulsePotLen/2;
      cperiod /=2.0f;
      unsigned int num_adds_2 = 2* num_adds;
      unsigned int num_adds_minus1_2 = 2 * num_adds_minus1;
      unsigned int stoffset = (offset + dbins + 3) & ~0x3;
//      stoffset += 4 - (stoffset & 3); // force alignment of destination
      
      for (unsigned int j = 1; j < ndivs ; j++) {
        unsigned int roundthresh = num_adds_minus1_2 - num_adds_minus1_2 / 2;
        
        tabofst -=3;
        di /= 2;
        if (p - di * num_adds_minus1_2 >= roundthresh) {
          tmp0 = di+1;
        } else {
          tmp0 = di;
        }
        
//		tmp0  = (int)(cperiod+0.5f);
//        di  = (int)cperiod;
//        if ((unsigned)di>=(unsigned)(PoTInfo.PulseMax*2))
//          SETIERROR(FP_ERROR,"FPU failure in pulse_find");
        thresh_scale = t_funct(di,num_adds_2,di+tabofst);
        cur_thresh= thresh_scale*avg;
//        dis_thresh= (cur_thresh-avg)*pulse_display_thresh + avg;
//		dis_thresh = (thresh_scale-1)*avg*pulse_display_thresh + avg;
		dis_thresh = avg*(thresh_scale*pulse_display_thresh + (1 - pulse_display_thresh));
        cpy_thresh = thresh_scale*avg+maxd;
        
//        if (PulsePotLen < 200) printf(" di = %d num_adds = %d thresh_scale = %g dis_thresh = %g\n", di, num_adds_2, thresh_scale, thresh_scale*pulse_display_thresh + (1 - pulse_display_thresh));
        
        max_scale *= 0.5f;

        //  tmp_max = sum2tables(di, &div[offset], &div[offset+tmp0], &div[stoffset]);
#if defined(__VEC__) && defined(USE_VECTORIZED_PULSE)
        if (tmp0 & 0x3)
          tmp_max = foldArrayBy2SPA(div,di,offset,offset+tmp0,stoffset) * max_scale;
        else
          tmp_max = foldArrayBy2SPAL(div,di,offset,offset+tmp0,stoffset) * max_scale;
       // tmp_maxref = 0.0f;
       // for (i=0;i<di;i++) {
       //   register float tmpfloat=(div[i+offset]+div[tmp0+i+offset]);
       //   divref[i+stoffset]=tmpfloat;
       //   if (divref[i+stoffset] != div[i+stoffset]) printf("foldArrayBy2SP element failure (%d/%d)\n", i, di);
       //   if (tmpfloat>tmp_maxref) {
       //     tmp_maxref=tmpfloat;
       //   }
       // }
       // tmp_maxref *= max_scale;
       // 
       // if (tmp_maxref != tmp_max) printf("foldArrayBy2 failure (%p, %d, %d, %d, %d)\n", div, di, offset, offset+tmp0, stoffset);
        
#elif defined(__SSE2__) && defined(USE_VECTORIZED_PULSE)
//         if (di < LOOP_THRESH) {
//           tmp_max = 0.0f;
//         #pragma loop count (LOOP_COUNT)
//           for (i=0;i<di;i++) {
//             register float tmpfloat=(div[i+offset]+div[tmp0+i+offset]);
//             div[i+stoffset]=tmpfloat;
//             if (tmpfloat>tmp_max) {
//               tmp_max=tmpfloat;
//             }
//           }
//           tmp_max *= max_scale;
//         } else if (tmp0 & 0x3)
#ifdef USE_PALIGNED_PULSE
//        tmp_max = fold2fns[tmp0 & 0x3](div, di, offset, offset+tmp0, stoffset) * max_scale;
        if (tmp0 & 0x2) {
          if (tmp0 & 0x1)
            tmp_max = foldArrayBy2A3(div,di,offset,offset+tmp0,stoffset) * max_scale;
          else
            tmp_max = foldArrayBy2A2(div,di,offset,offset+tmp0,stoffset) * max_scale;            
        } else {
          if (tmp0 & 0x1)
            tmp_max = foldArrayBy2A1(div,di,offset,offset+tmp0,stoffset) * max_scale;
          else
            tmp_max = foldArrayBy2AL(div,di,offset,offset+tmp0,stoffset) * max_scale;                      
        }
#else
		if (tmp0 & 0x3)
          tmp_max = foldArrayBy2A(div,di,offset,offset+tmp0,stoffset) * max_scale;
        else
          tmp_max = foldArrayBy2AL(div,di,offset,offset+tmp0,stoffset) * max_scale;
#endif
#else
        tmp_max = 0.0f;
        for (i=0;i<di;i++) {
          register float tmpfloat=(div[i+offset]+div[tmp0+i+offset])/2;
          div[i+stoffset]=tmpfloat;
          if (tmpfloat>tmp_max) {
            tmp_max=tmpfloat;
          }
        }
#endif
//        if (PulsePotLen < 200) printf(" observed = %g\n", tmp_max / avg);
		
        if (tmp_max<bail_thresh) {
//          printf("bailing (%d/%d): best=%g max=%g bail=%g dis=%g cpy=%g\n", j+1, ndivs, bail_scale, tmp_max/avg, bail_thresh/avg, dis_thresh/avg, cpy_thresh/avg);
          break;
        }
        
        if (tmp_max>dis_thresh) {
          ReportPulseEvent(tmp_max*ravg,avg,cperiod*res,
                           TOffset+PulsePotLen/2,FOffset,
                           (tmp_max-avg)*(float)sqrt((float)num_adds_2)*ravg,
                           (cur_thresh-avg)*(float)sqrt((float)num_adds_2)*ravg, div+stoffset, max_scale, 0);

    		  if (tmp_max>cpy_thresh) {
      			maxp = cperiod;
      			maxd = tmp_max - cur_thresh;
      			max  = tmp_max;
      			snr  = (tmp_max-avg)*(float)sqrt((float)num_adds_2)*ravg;
      			fthresh = (cur_thresh-avg)*(float)sqrt((float)num_adds_2)*ravg;
      			
            mmax_scale = max_scale;
            memcpy(FoldedPOT, div+stoffset, di*sizeof(float));
    		  }
        }

        cperiod  /=2.0f;
        num_adds_2 *=2;
        num_adds_minus1_2 *= 2;
        offset = stoffset;
        dbins   /= 2;
        stoffset = (offset + dbins + 3) & ~0x3;
//        stoffset += 4 - (stoffset & 3);
      }  // for (j = 1; j < ndivs
    } // for (p = lastP
                      
//    if (!nonbails) printf("skipped an entire series: bail_scale = %f num_adds = %d count(p) = %d\n", best_pulse->score, num_adds, lastP - firstP);
//    else printf("could not skip series: bail_scale = %f num_adds = %d count(p) = %d\n", best_pulse->score, num_adds, lastP - firstP);
  } // for(num_adds =

  analysis_state.FLOP_counter+=(PulsePotLen*0.1818181818182+400.0)*PulsePotLen;
  if (maxp!=0)
    ReportPulseEvent(max*ravg,avg,maxp*res,TOffset+PulsePotLen/2,FOffset,
                     snr, fthresh, FoldedPOT, mmax_scale, 1);

  // debug possible heap corruption -- jeffc
#ifdef _WIN32
  BOINCASSERT(_CrtCheckMemory());
#endif

  return 0;
}
